/*
  $Id: ApplicationContextImpl.java,v 1.6 2004/03/11 22:10:24 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.application;

import java.util.Properties;
import java.io.File;
import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;
import java.util.ArrayList;

public class ApplicationContextImpl implements ApplicationContext
{
  private HashMap properties = new HashMap(); // Map<String, ApplicationProperty>

  public void declareProperty(String key, PropertyKindEnum type, String help, String defaultValue) throws PropertyException
  {
    properties.put(key.toLowerCase(), new ApplicationPropertyImpl(key.toLowerCase(), type, help, defaultValue));
  }

  public void removeProperty(String key)
  {
    properties.remove(key);
  }

  public void loadPropertiesFromFile(File file) throws java.io.IOException, PropertyException
  {
    // ask java to load the file
    Properties javaProperties = new Properties();
    javaProperties.load(new java.io.FileInputStream(file));

    // set all the properties
    java.util.Enumeration enum = javaProperties.propertyNames();
    while (enum.hasMoreElements())
    {
      String key   = (String)enum.nextElement();
      String value = javaProperties.getProperty(key);

      setProperty(key, value);
    }
  }

  public void loadPropertiesFromEnv() throws PropertyException
  {
    Iterator it = properties.entrySet().iterator();
    while (it.hasNext())
    {
      Map.Entry entry = (Map.Entry)it.next();
      String key = (String)entry.getKey();
      String env_key = key.replace('.', '_');

      String env = System.getProperty(env_key);
      if (env != null)
        setProperty(key, env);
    }
  }

  public Iterator iterateProperties() { return properties.values().iterator(); }

  public Iterator iteratePropertiesByName()
  {
    java.util.TreeSet temp = new java.util.TreeSet(
     new java.util.Comparator() {
       public int compare(Object o1, Object o2)
       {
         ApplicationProperty p1 = (ApplicationProperty)o1;
         ApplicationProperty p2 = (ApplicationProperty)o2;

         return p1.getName().compareTo(p2.getName());
       }
       public boolean equals(Object o)
       {
         return false;
       }
     }
    );

    Iterator it = properties.values().iterator();
    while (it.hasNext())
    {
      temp.add(it.next());
    }

    return temp.iterator();
  }

  public void setProperty(String key, String value) throws PropertyException
  {
    ApplicationProperty property = (ApplicationProperty)properties.get(key.toLowerCase());
    if (property == null)
      throw new PropertyException("Key [" + key + "] not found");
    property.load(value);
  }

  private ApplicationProperty getProperty(String key, PropertyKindEnum type) throws PropertyException
  {
    ApplicationProperty property = (ApplicationProperty)properties.get(key.toLowerCase());
    if (property == null)
      throw new PropertyException("Property [" + key + "] not found");
    if (property.isEmpty())
      throw new PropertyException("Property [" + key + "] is empty");
    if (property.getType() != type)
      throw new PropertyException("Property [" + key + "] is not of type [" + type + "]");
    return property;
  }

  public String getPropertyString(String key) throws PropertyException
  {
    ApplicationProperty property = getProperty(key, PropertyKindEnum.StringValue);
    return property.getStringValue();
  }

  public int getPropertyInt(String key) throws PropertyException
  {
    ApplicationProperty property = getProperty(key, PropertyKindEnum.IntValue);
    return property.getIntValue();
  }

  public File getPropertyPath(String key) throws PropertyException
  {
    ApplicationProperty property = getProperty(key, PropertyKindEnum.PathValue);
    return property.getPathValue();
  }

  public ArrayList getPropertyPathList(String key) throws PropertyException
  {
    ApplicationProperty property = getProperty(key, PropertyKindEnum.PathListValue);
    return property.getPathListValue();
  }

  public boolean getPropertyBoolean(String key) throws PropertyException
  {
    ApplicationProperty property = getProperty(key, PropertyKindEnum.BooleanValue);
    return property.getBooleanValue();
  }

  public String getVersion()
  {
    try
    {
      return getPropertyString("version");
    }
    catch(PropertyException e)
    {
      throw new Error("Internal error : Version is not defined");
    }
  }

  public void terminate(String message)
  {
    System.err.println(message);

    System.exit(1);
  }

  // --- Singleton access

  static private ApplicationContextImpl theInstance = new ApplicationContextImpl();
  private ApplicationContextImpl() {}
  synchronized static public ApplicationContextImpl instance() { return theInstance; }
  synchronized static public void instanceReset() { theInstance = new ApplicationContextImpl(); }
}
