/*
  $Id: ApplicationPropertyImpl.java,v 1.5 2004/03/04 22:16:16 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.application;

import java.util.StringTokenizer;
import java.util.Iterator;
import java.util.ArrayList;
import java.io.File;

public class ApplicationPropertyImpl implements ApplicationProperty
{
  private String  name;
  private PropertyKindEnum type;
  private String  help;
  private String  stringValue;
  private int     intValue;
  private boolean booleanValue;
  private File    pathValue;
  private ArrayList  pathListValue;
  private boolean isEmpty;

  public ApplicationPropertyImpl(String name, PropertyKindEnum type, String help, String defaultValue) throws PropertyException
  {
    this.name  = name;
    this.type  = type;
    this.help  = help;

    load(defaultValue);
  }

  public String           getName() { return name; }
  public PropertyKindEnum getType() { return type; }
  public String           getHelp() { return help; }
  public String           getStringValue() { return stringValue; }
  public int              getIntValue() { return intValue; }
  public boolean          getBooleanValue() { return booleanValue; }
  public File             getPathValue() { return pathValue; }
  public ArrayList           getPathListValue() { return pathListValue; }
  public Iterator         iteratePathListValue() { return pathListValue.iterator(); }
  public boolean          isEmpty() { return isEmpty; }

  public void load(String value) throws PropertyException
  {
    if (value == null)
      isEmpty = true;
    else
    {
      isEmpty = false;

      stringValue = value;

      if (type == PropertyKindEnum.StringValue)
        return;
      else
      if (type == PropertyKindEnum.IntValue)
      {
        try
        {
          intValue = Integer.parseInt(value);
        }
        catch(NumberFormatException e)
        {
          throw new PropertyException("Property " + name + " : Type mismatch : is not of type int");
        }
      }
      else
      if (type == PropertyKindEnum.BooleanValue)
      {
        if (value.equals("true"))
          booleanValue = true;
        else
        if (value.equals("false"))
          booleanValue = false;
        else
          throw new PropertyException("Property " + name + " : Type mismatch : is not of type boolean");
      }
      else
      if (type == PropertyKindEnum.PathValue)
        pathValue = new File(value);
      else
      if (type == PropertyKindEnum.PathListValue)
      {
        pathListValue = new ArrayList();
        StringTokenizer st = new StringTokenizer(value, ",");
        while (st.hasMoreTokens())
        {
           String dir = st.nextToken();
           dir = dir.trim();
           File file = new File(dir);
           if (!file.exists())
             throw new PropertyException("Property " + name + " : File/Directory [" + dir + "] does not exist on disk");
           pathListValue.add(file);
        }
      }
      else
        throw new Error("Internal error : Unrecognized Property type");
    }
  }

  public String toString() { return stringValue; }
}