/*
  $Id: FileLoaderImpl.java,v 1.6 2004/03/04 22:16:16 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.application;

import net.nosica.application.PropertyException;

import net.nosica.parser.node.util.var.TypeName;
import net.nosica.parser.node.util.var.TypeNameComponent;
import net.nosica.util.Helper;

import java.util.Iterator;
import java.util.ArrayList;
import java.io.File;

public class FileLoaderImpl implements FileLoader
{
  private static FileLoaderImpl theInstance = new FileLoaderImpl();
  private FileLoaderImpl() {}
  public static FileLoaderImpl instance() { return theInstance; }
  public static void instanceReset() { theInstance = new FileLoaderImpl(); }

  private ApplicationContext context = ApplicationContextImpl.instance();

  public Iterator iterateSourcePaths()
  {
    try
    {
      Iterator it = context.getPropertyPathList("sourcepaths").iterator();

      if (!it.hasNext())
        throw new Error("property sourcepaths is defined but is empty");

      return it;
    }
    catch (PropertyException e)
    {
      context.terminate("Fatal : Property sourcepaths is not defined");
      throw new Error("Fatal : Property sourcepaths is not defined");
    }
  }

  public ArrayList findFiles(TypeName typeName)
  {
    // transforms into a non array type
    TypeName nonArrayType = Helper.createNonArrayTypeName(typeName);

    ArrayList result = new ArrayList();
    Iterator it = iterateSourcePaths();

    while (it.hasNext())
    {
      File file = (File)it.next();

      // nonArrayType is empty : import all sourcepaths
      if (nonArrayType.isEmpty())
        result.add(file);
      else
      {
        // iterates all components (except last one)
        if (nonArrayType.getFirstComponents() != null)
        {
          Iterator j = nonArrayType.getFirstComponents().iterateComponents();
          while (j.hasNext())
          {
            TypeNameComponent component = (TypeNameComponent)j.next();
            file = new File(file, component.getPackageElement());
          }
        }

        // look for a nosica file
        File temp = new File(file, nonArrayType.getLastComponent().getPackageElement() + ".nos");
        if (temp.exists())
          result.add(temp);
        else
        {
          // try with a directory
          file = new File(file, nonArrayType.getLastComponent().getPackageElement());
          if (file.isDirectory())
            result.add(file);
        }
      }
    }

    return result;
  }
}
