/*
  $Id: MessageService.java,v 1.5 2004/03/04 22:16:16 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.application;

import net.nosica.compiler.application.Filter;
import net.nosica.framework.Localizable;

/**
 * \brief Manages errors inside Nosica
 * This instance report errors to users when they are discovered and take appropriate action<BR>
 */

public interface MessageService
{
  /**
   * \brief report a message
   *
   * \param level level of the message
   * \param msgId a string identifying a message in the message table
   * \param params parameters of the message description
   * \param localizable an instance able to localize the message in the source tree
   *
   * \note as a side effect internal counters are incremented, message is printed
   */
  void report(MessageLevel level, String msgId, String[] params, Localizable localizable);

  /**
   * \brief report a message
   *
   * \param level level of the message
   * \param msgId a string identifying an message in the message table
   * \param param the parameter of the message description
   * \param localizable an instance able to localize the message in the source tree
   *
   * \note as a side effect internal counters are incremented, message is printed
   */
  void report(MessageLevel level, String msgId, String param, Localizable localizable);

  /**
   * \brief report a message
   *
   * \param level level of the message
   * \param msgId a string identifying a message in the message table
   * \param localizable an instance able to localize the message in the source tree
   *
   * \note as a side effect internal counters are incremented, message is printed
   */
  void report(MessageLevel level, String msgId, Localizable localizable);

  /**
   * \brief report a message
   *
   * \param level level of the message
   * \param msgId a string identifying a message in the message table
   *
   * \note as a side effect internal counters are incremented, message is printed
   */
  void report(MessageLevel level, String msgId);

  /**
   * \brief Get number of error
   *
   * \return the number of error
   */
  int getNbError();

  /**
   * \brief Get number of warning
   *
   * \return the number of warning
   */
  int getNbWarning();

  /**
   * \brief attach a filter to MessageService, so that only a set of messages are displayed
   *
   * \param filter the filter
   */
  void attachFilter(Filter filter);
}
