/*
  $Id: NosicaPropertySetup.java,v 1.7 2004/05/01 16:04:10 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.application;

/**
 * \brief an helper class to setup Nosica common properties
 */
public class NosicaPropertySetup
{
  /**
   * \brief create a default Nosica property configuration on given ApplicationContext
   *
   * \param context the context to setup
   */
  static public void setup(ApplicationContext context) throws PropertyException
  {
    // compiler flags
    context.declareProperty("error.max",          PropertyKindEnum.IntValue,      "maximum number of reported errors during compilation", "100");
    context.declareProperty("error.messages",     PropertyKindEnum.PathValue,     "location of the error messages file", null);
    context.declareProperty("code.language",      PropertyKindEnum.StringValue,   "language output (only <c> for now)", "c");
    context.declareProperty("code.output",        PropertyKindEnum.PathValue,     "base path for generated code", ".");
    context.declareProperty("source",             PropertyKindEnum.StringValue,   "name of the class to compile", "");
    context.declareProperty("sourcepaths",        PropertyKindEnum.PathListValue, "list of all possible source tree paths", null);

    // make flags
    context.declareProperty("MAKE",                             PropertyKindEnum.StringValue,    "make program", null);
    context.declareProperty("MAKE_CHANGE_DIR_FLAG",             PropertyKindEnum.StringValue,    "make flag to change ", null);

    // c compiler flags
    context.declareProperty("CC",                               PropertyKindEnum.StringValue,    "c compiler to use in back end", null);
    context.declareProperty("CC_INCLUDE_FLAG",                  PropertyKindEnum.StringValue,    "c compiler flag to add include path", null);
    context.declareProperty("CC_EXTRA_FLAG",                    PropertyKindEnum.StringValue,    "c compiler extra flags", "");
    context.declareProperty("CC_OUTPUT_FLAG",                   PropertyKindEnum.StringValue,    "c compiler flag to set output", null);
    context.declareProperty("CC_COMPIL_FLAG",                   PropertyKindEnum.StringValue,    "c compiler flag to compil only (do not link)", null);
    context.declareProperty("CC_LIB_PATH_FLAG",                 PropertyKindEnum.StringValue,    "c linker flag for adding lib path", null);
    context.declareProperty("CC_LIB_USE_FLAG",                  PropertyKindEnum.StringValue,    "c linker flag for using lib", null);
    context.declareProperty("CC_LIB_EXTRA_FLAG",                PropertyKindEnum.StringValue,    "c linker extra flags", null);

    // nosica native standard library
    context.declareProperty("NOSICA_NATIVESTDLIB_INCLUDE_PATH", PropertyKindEnum.StringValue,    "nosica standard library include path", null);
    context.declareProperty("NOSICA_NATIVESTDLIB_LIB_PATH",     PropertyKindEnum.StringValue,    "nosica standard library lib path", null);

    // debug flags
    // symbols : aka GenericType
    context.declareProperty("debug.symbol",         PropertyKindEnum.BooleanValue,  "debug symbols, default is false", "false");
    context.declareProperty("debug.symbol.type",    PropertyKindEnum.StringValue,   "debug symbols of this type only", null);
    // types : aka instanciated GenericType
    context.declareProperty("debug.type",           PropertyKindEnum.BooleanValue,  "debug types, default is false", "false");
    context.declareProperty("debug.type.type",      PropertyKindEnum.StringValue,   "debug this type only", null);
    // records : aka IR of types
    context.declareProperty("debug.record",         PropertyKindEnum.BooleanValue,  "debug records, default is false", "false");
    context.declareProperty("debug.record.type",    PropertyKindEnum.StringValue,   "debug record of this type only", null);
    // recordsPTN : aka PTN of types
    context.declareProperty("debug.recordptn",      PropertyKindEnum.BooleanValue,  "debug ptn of records, default is false", "false");
    context.declareProperty("debug.recordptn.type", PropertyKindEnum.StringValue,   "debug ptn of this record only", null);
    // ir : aka IR of methods
    context.declareProperty("debug.ir",             PropertyKindEnum.BooleanValue,  "debug ir methods, default is false", "false");
    context.declareProperty("debug.ir.type",        PropertyKindEnum.StringValue,   "debug ir methods of this type only", null);
    // ptn : aka PTN of methods
    context.declareProperty("debug.ptn",            PropertyKindEnum.BooleanValue,  "debug possibleTypeName, default is false", "false");
    context.declareProperty("debug.ptn.type",       PropertyKindEnum.StringValue,   "debug possibleTypeName of this type", null);
    //	ee : aka RuntimeException of methods
    context.declareProperty("debug.exception",            PropertyKindEnum.BooleanValue,  "debug runtime exception analysis, default is false", "false");
    context.declareProperty("debug.exception.type",       PropertyKindEnum.StringValue,   "debug runtime exception analysis of this type", null);
  }
}