/*
  $Id: SymbolEnvironment.java,v 1.4 2004/03/04 22:16:16 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.application;

import net.nosica.parser.node.util.var.TypeName;
import net.nosica.symbol.GenericType;
import net.nosica.symbol.SymbolException;

import java.util.Iterator;
import java.util.Map;

/**
 * \brief Manages symbol for all classes loaded by the compiler
 * Manages symbol for all classes loaded by compiler, a symbol is
 * <ul>
 *   <li>a TypeName that references a Type
 *   <li>a String that references a member inside a type
 * </ul>
 * <BR>
 * We separates Type representation from aliases valid in a type<BR>
 * Each type is associated with a particular AliasTable that contains all alias for a particular type<BR>
 * For example, a directive like import net.nosica.Test; will add a new alias from Test to net.nosica.Test that will
 * be valid for each types declared in a particular CompilationUnit, thus, this alias will be added to all AliasTable
 * that are associated with the types of the CompilatioUnit<BR>
 */

public interface SymbolEnvironment
{
  /**
   * \brief Add a type an its associated AliasTable to the global SymbolEnvironment
   *
   * \param type type to add
   * \throw SymbolException if a type with same name was already added to table
   */
  void addGenericType(GenericType type) throws SymbolException;

  /**
   * \brief Get a type by its TypeName
   *
   * \param typeName exact name of generic type to retrieve
   * \return the type referenced by typeName if it exists
   * \throw SymbolException if TypeName does not exists
   */
  GenericType getGenericType(TypeName typeName) throws SymbolException;

  /**
   * \brief get best suitable generic type
   * Since we can have several partial implementations of a type, this method
   * tries to best match the different partial implementations available and return it
   *
   * \param typeName name of generic type to retrieve
   * \param genericAssociation a map to give that will contain as result all generic association
   * \return the type that can be used to produce typeName
   */
  GenericType getBestSuitableGenericType(TypeName typeName, Map genericAssociation) throws SymbolException;

  /**
   * \brief Get all types that can be resolved by the non generic version of name typeName
   *
   * \return Map<TypeName, GenericType> of all types that have a non generic name typeName
   * \throws SymbolException if there is no such types
   */
  Map getGenericTypes(TypeName typeName) throws SymbolException;

  /**
   * \brief return an iterator on all TypeName contained in SymbolEnvironment
   *
   * \return Iterator<TypeName>
   */
  Iterator iterateTypeName();

  /**
   * \brief return true if a particular type exists
   *
   */
  boolean genericTypeExists(TypeName typeName);
}