/*
  $Id: TypeNameResolver.java,v 1.4 2004/03/04 22:16:16 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.application;

import net.nosica.parser.node.util.var.TypeName;
import net.nosica.symbol.SymbolException;
import net.nosica.symbol.AliasTable;

/**
 * \brief An interface to resolve name
 * a TypeName may be difficult to understand because it is dotted, may have genericity information
 * and/or array information, can be aliased, may refer to a nested class, so it can be difficult to
 * correclty resolved it. This is the purpose of this interface.
 */

public interface TypeNameResolver
{
  /**
   * \brief Resolve a TypeName using an AliasTable
   * Several cases :
   * <ul>
   *   <li>fully qualified name : net.nosica.lang.int
   *   <li>class name : int
   *   <li>access to nested classes : Toto.Titi (after an <import net.nosica.Toto;>)
   *   <li>array type : int[]
   *   <li>generic type : Tutu<int>
   * </ul>
   * <BR>
   * First case needs a FileLoader lookup to see if type exists<BR>
   * Second case needs an AliasTable lookup then a FileLoader lookup to see if type exists<BR>
   * Third case needs an AliasTable lookup with Toto that resolves to net.nosica.Toto, then a
   * FileLoader lookup to see if file exists, then an importSymbols to see if nested class Titi exists<BR>
   * Fourth case needs to transform first int[] to int, then resolve int to net.nosica.lang.int32 then
   * transform back result to array : net.nosica.lang.int32[]
   * Fifth case needs to transform first Tutu<int> to Tutu, then resolve Tutu to net.nosica.Tutu (for example),
   * then resolve all generic parameter (in this case, int to net.nosica.lang.int32) then reconstruct type.
   * Result is finally net.nosica.Tutu<net.nosica.lang.int32>
   *
   * \param typeName name of class to load
   * \param aliasTable aliasTable to use to resolve name
   * \param FileLoader file loader to use to look on disk
   * \return fully qualifiable name
   */
  TypeName resolve(TypeName typeName, AliasTable aliasTable, FileLoader fileLoader) throws SymbolException;

  /**
   * \brief Resolve a TypeName using an AliasTable like resolve, but will not resolve generic parameter if any
   *
   * \param typeName name of class to load
   * \param aliasTable alias table to use to resolve name
   * \param FileLoader file loader to use to look on disk
   * \return fully qualifiable name
   */
  TypeName resolveExceptGeneric(TypeName typeName, AliasTable aliasTable, FileLoader fileLoader) throws SymbolException;
}
