/*
  $Id: TypeNameResolverImpl.java,v 1.6 2004/03/04 22:16:16 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.application;

import net.nosica.parser.node.util.var.TypeName;
import net.nosica.parser.node.util.var.TypeNameImpl;
import net.nosica.parser.node.util.var.TypeNameComponent;
import net.nosica.parser.node.util.var.TypeNameComponentImpl;

import net.nosica.symbol.AliasTable;
import net.nosica.symbol.SymbolException;
import net.nosica.util.Helper;

import java.util.Iterator;
import java.util.ArrayList;

public class TypeNameResolverImpl implements TypeNameResolver
{
  private static TypeNameResolverImpl theInstance = new TypeNameResolverImpl();
  private TypeNameResolverImpl() {}
  public static TypeNameResolverImpl instance() { return theInstance; }
  public static void instanceReset() { theInstance = new TypeNameResolverImpl(); }

  public TypeName resolve(TypeName typeName, AliasTable aliasTable, FileLoader fileLoader) throws SymbolException
  {
    // first resolve radix using alias, nested, ...
    typeName = resolveExceptGeneric(typeName, aliasTable, fileLoader);

    if (typeName.isGeneric())
    {
      TypeName radix = Helper.createNonArrayTypeName(typeName);

      ArrayList components = new ArrayList();
      Iterator it = radix.getComponents().iterator();
      while (it.hasNext())
      {
        TypeNameComponent component = (TypeNameComponent)it.next();
        ArrayList resolvedGenericParameters = null;

        if (component.hasGenericExtension())
        {
          Iterator j = component.iterateGenericExtension();
          resolvedGenericParameters = new ArrayList();
          while (j.hasNext())
          {
            TypeName genericParm = (TypeName)j.next();
            genericParm = resolve(genericParm, aliasTable, fileLoader);
            resolvedGenericParameters.add(genericParm);
          }

          component = new TypeNameComponentImpl(component.getPackageElement(), resolvedGenericParameters);
        }

        components.add(component);
      }

      radix = new TypeNameImpl(components);

      return Helper.reconstructArrayTypeNameFromRadix(radix, typeName);
    }

    return typeName;
  }

  public TypeName resolveExceptGeneric(TypeName typeName, AliasTable aliasTable, FileLoader fileLoader) throws SymbolException
  {
    if (typeName.isEmpty())
      throw new SymbolException("Cannot resolve an empty typename");

    TypeName resolvedTypeName = null;
    TypeName model = typeName;
    TypeName radix = Helper.createRadixTypeName(model);

    try
    {
      resolvedTypeName = resolveRadix(radix, aliasTable, fileLoader);
    }
    catch (SymbolException e)
    {
      // try to decompose name
      Iterator it = radix.getComponents().iterator();
      ArrayList partialComponent = new ArrayList();
      partialComponent.add(it.next());
      model = new TypeNameImpl(partialComponent);
      radix = Helper.createRadixTypeName(model);

      // may throw if we can't resolve name
      resolvedTypeName = resolveRadix(radix, aliasTable, fileLoader);

      // reconstruct name
      resolvedTypeName = Helper.reconstructTypeNameFromRadix(resolvedTypeName, model);
      // add last part
      ArrayList completeComponents = new ArrayList();
      completeComponents.addAll(resolvedTypeName.getComponents());

      while (it.hasNext())
        completeComponents.add(it.next());

      return Helper.reconstructTypeNameFromRadix(new TypeNameImpl(completeComponents), typeName);
    }

    // reconstruct typeName
    return Helper.reconstructTypeNameFromRadix(resolvedTypeName, model);
  }

  /**
   * \brief resolve a radix type name
   * what is a radix type name ?<BR>
   * Well, this is a type name that isn't array nor generic.
   *
   * \param typeName radix type name to resolve
   * \param aliasTable aliasTable to use to resolve name
   * \param fileLoader fileLoader to use to look on disk
   * \return resolved TypeName
   * \throws SymbolException if typeName wasn't resolved
   */

  private TypeName resolveRadix(TypeName typeName, AliasTable aliasTable, FileLoader fileLoader) throws SymbolException
  {
    if (typeName.isArray())
    {
      ApplicationContextImpl.instance().terminate("resolveRadix cannot resolve array type");
    }
    if (typeName.isGeneric())
    {
      ApplicationContextImpl.instance().terminate("resolveRadix cannot resolve generic type");
    }

    // if file exists on disk
    if (!fileLoader.findFiles(typeName).isEmpty())
      return typeName;

    // file does not exists on disk
    // try to resolve it using an alias
    if (aliasTable.aliasExists(typeName))
    {
      TypeName resolved = aliasTable.resolveAlias(typeName);

      if (resolved.equals(typeName)) // generic parameters are added with a fake alias
          return resolved;

      try
      {
        // try to recursively resolve alias as it can be the alias of another alias
        return resolveExceptGeneric(resolved, aliasTable, fileLoader);
      }
      catch (SymbolException e)
      {
        return resolved;
      }
    }
    else
      throw new SymbolException("Could not resolve [" + typeName + "]");
  }
}
