/*
  $Id: PossibleTypeName.java,v 1.8 2004/03/04 22:16:16 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.compiler.analysis;

import net.nosica.parser.node.util.var.TypeName;

import java.util.Iterator;

/**
 * \brief collects all type an instance can take during its life
 */
public interface PossibleTypeName
{
  /**
   * \brief are multiple types stored here ?
   *
   * \return true if underlying PossibleType is a list of PossibleTypes
   */
  boolean hasMultiplePossibleTypes();

  /**
   * \brief return the number of underlying PossibleType stored in list
   *
   * \remarks need hasMultiplePossibleTypes() to return true
   * \return number of possible type in list
   */
  int nbPossibleTypes();

  /**
   * \brief returns static type stored
   * requires : !hasMultiplePossibleTypes()
   *
   * \return TypeName
   */
  TypeName getTypeName();

   /**
   * \brief change the static type
   *
   * \param typeName the new typeName of this PTN
   */
  void setTypeName(TypeName typeName);

  /**
   * \brief returns the possible types that can be taken by array element
   * requires : getTypeName().isArray()
   *
   * \return PossibleTypes of array element type
   */
  PossibleTypeName getArrayElementPossibleTypes();

  /**
   * \brief merge this representation with an other one
   *
   * \param rhs the representation to merge with
   * \return whether the list of possible type names has changed
   */
  boolean merge(PossibleTypeName rhs);

  /**
   * \brief diff this representation with an other one
   *
   * \param rhs the representation to diff against
   * \return differences between this and rhs
   * \note diff can be empty !!
   */
  PossibleTypeName diff(PossibleTypeName rhs);

  /**
   * \brief indicates if instance is empty
   *
   * \return true if instance is empty
   */
  boolean isEmpty();

  /**
   * \brief iterate over all possible types
   *
   * \return Iterator<PossibleTypeName>
   * \note all elements of Iterator are scalar
   */
  Iterator iteratePossibleType();

  /**
   * \brief is underlying type name primitive ?
   * for debug purpose only
   *
   * \return true if getTypeName is a primitive typeName
   */
  boolean isPrimitive();
}
