/*
  $Id: PossibleTypeNameAnalysisCallBack.java,v 1.5 2004/03/04 22:16:16 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.compiler.analysis;

import net.nosica.parser.node.util.var.TypeName;
import net.nosica.framework.VisitorException;
import net.nosica.compiler.intermediate.Function;

import java.util.Set;

/**
 * \brief a call back for IR analysis ...
 */
public interface PossibleTypeNameAnalysisCallBack
{
  /**
   * \brief schedule the function to be re-analysed
   *
   * \param function the function to schedule
   */
  void analyse(Function function) throws VisitorException;

  /**
   * \brief schedule the direct callers to be re-analysed
   */
  void analyseCallers() throws VisitorException;

  /**
   * \brief set the active function as caller of f
   *
   * \param f the function that has a new caller
   */
  void addCaller(Function f) throws VisitorException;

  /**
   * \brief schedule the active function to be re-analysed
   */
  void analyseActive() throws VisitorException;

  /**
   * \brief schedule all methods of type to be analysed
   *
   * \param typeName type to analyse
   */
  void analyseType(TypeName typeName) throws VisitorException;

  /**
   * \brief merge the result returned by a function
   *
   * \param ptn description of the run time type returned as result by this function
   * \return whether the list of possible type names has changed
   *
   */
  boolean mergeFunctionResult(PossibleTypeName ptn);

  /**
   * \brief add to active function's result a list of field array source
   *
   * \param fieldArraySource Set<Field> set of array Field
   */
  void addFieldArraySourceToActive(Set fieldArraySource);
}
