/*
  $Id: PossibleTypeNameImpl.java,v 1.12 2004/03/04 22:16:16 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.compiler.analysis;

import net.nosica.parser.node.util.var.TypeName;
import net.nosica.compiler.typechecker.Type;
import net.nosica.symbol.SymbolException;
import net.nosica.application.NativeTypes;
import net.nosica.compiler.application.Environment;

import java.util.ArrayList;
import java.util.Iterator;

public class PossibleTypeNameImpl implements PossibleTypeName
{
  private TypeName typeName;
  private boolean isPrimitive;
  private PossibleTypeNameImpl arrayElement;
  private ArrayList possibleTypes;               // ArrayList<PossibleTypeName>

  private PossibleTypeNameImpl() {}

  private PossibleTypeNameImpl(PossibleTypeName PTN)
  {
    this.typeName = PTN.getTypeName();
    this.isPrimitive = PTN.isPrimitive();

    if (PTN.hasMultiplePossibleTypes())
    {
      possibleTypes = new ArrayList();
      Iterator it = PTN.iteratePossibleType();
      while (it.hasNext())
      {
        PossibleTypeName PTN2 = (PossibleTypeName)it.next();
        possibleTypes.add(new PossibleTypeNameImpl(PTN2));
      }
    }
    else
    if (typeName.isArray())
      arrayElement = new PossibleTypeNameImpl(PTN.getArrayElementPossibleTypes());
  }

  static public PossibleTypeNameImpl createScalar(TypeName typeName)
  {
    PossibleTypeNameImpl result = new PossibleTypeNameImpl();

    result.typeName = typeName;

    try
    {
      Type type = Environment.instance().getTE().getType(typeName);
      result.isPrimitive = type.isPrimitive();
    }
    catch (SymbolException e)
    {
      Environment.instance().terminate("Internal error " + e.getMessage());
    }

    if (typeName.isArray())
      result.arrayElement = createList(typeName.getArrayElementType());

    return result;
  }

  static public PossibleTypeNameImpl createList(TypeName typeName)
  {
    PossibleTypeNameImpl result = new PossibleTypeNameImpl();

    result.typeName = typeName;
    result.possibleTypes = new ArrayList();

    try
    {
      Type type = Environment.instance().getTE().getType(typeName);
      result.isPrimitive = type.isPrimitive();
    }
    catch (SymbolException e)
    {
      Environment.instance().terminate("Internal error " + e.getMessage());
    }

    return result;
  }

  static public PossibleTypeNameImpl copy(PossibleTypeName PTN)
  {
    return new PossibleTypeNameImpl(PTN);
  }

  static public PossibleTypeNameImpl filterDerivedTypes(TypeName parent, PossibleTypeName PTN)
  {
    PossibleTypeNameImpl result = createList(PTN.getTypeName());

    try
    {
      if (PTN.hasMultiplePossibleTypes())
      {
        Iterator it = PTN.iteratePossibleType();
        while (it.hasNext())
        {
          PossibleTypeName ptn = (PossibleTypeName) it.next();
          if (
               Environment.instance().getTE().doesInherits(
                 ptn.getTypeName(),
                 parent
               ) ||
               ptn.getTypeName().equals(parent)
             )
          {
            result.merge(ptn);
          }
        }
      }
      else
      {
        if (
             Environment.instance().getTE().doesInherits(
               PTN.getTypeName(),
               parent
             ) ||
             PTN.getTypeName().equals(parent)
           )
        {
          result.merge(PTN);
        }
      }
    }
    catch (SymbolException e)
    {
      Environment.instance().terminate("Internal error : " + e.getMessage());
    }

    return result;
  }

  public boolean isPrimitive() { return isPrimitive; }
  public Iterator iteratePossibleType()
  {
    if (possibleTypes != null)
      return possibleTypes.iterator();
    throw new UnsupportedOperationException();
  }
  public boolean isEmpty()
  {
    if (possibleTypes != null)
      return possibleTypes.isEmpty();
    throw new UnsupportedOperationException();
  }
  public PossibleTypeName diff(PossibleTypeName rhs)
  {
    // verify integrity
    if (
        !rhs.getTypeName().equals(getTypeName()) ||
        rhs.isPrimitive() != isPrimitive
       )
      throw new UnsupportedOperationException();

    // if other is scalar, create a list out of it and diff it
    if (!rhs.hasMultiplePossibleTypes())
    {
      PossibleTypeName temp = createList(rhs.getTypeName());
      temp.merge(rhs);
      return diff(temp);
    }

    // if this is scalar, create a list out of it and diff it
    if (!hasMultiplePossibleTypes())
    {
      PossibleTypeName temp = createList(getTypeName());
      temp.merge(this);
      return temp.diff(rhs);
    }

    // at this point this and other are list
    // create diff of elements that are in this and not in other
    PossibleTypeName result1 = privateDiff(this, rhs);
    // create diff of elements that are in other and not in this
    PossibleTypeName result2 = privateDiff(rhs, this);

    // merge results
    result1.merge(result2);

    return result1;
  }
  /**
   * \brief will diff rhs and lhs and return elements that are in rhs and not in lhs
   */
  private static PossibleTypeName privateDiff(PossibleTypeName rhs, PossibleTypeName lhs)
  {
    // prepare result
    PossibleTypeName result = createList(rhs.getTypeName());

    // iterates elements of rhs
    Iterator it = rhs.iteratePossibleType();
    while (it.hasNext())
    {
      PossibleTypeName PTN1 = (PossibleTypeName)it.next();

      // say that this elements does not belongs to lhs
      boolean doNotExistInOther = true;
      PossibleTypeName PTN3 = new PossibleTypeNameImpl(PTN1);

      // iterates elements of lhs
      Iterator jt = lhs.iteratePossibleType();
      while (jt.hasNext())
      {
        PossibleTypeName PTN2 = (PossibleTypeName)jt.next();
        if (PTN2.getTypeName().equals(PTN1.getTypeName()))
        {
          // this element belongs to lhs too
          doNotExistInOther = false;

          // if array
          if (PTN2.getTypeName().isArray())
          {
            // let's modify PTN3 so that it reflects the differences between PTN1 and PTN2
            // diff PTN3 and PTN4
            PossibleTypeName PTN4 = PTN3.getArrayElementPossibleTypes().diff(PTN2.getArrayElementPossibleTypes());
            // prepare temp result
            PossibleTypeNameImpl temp = createScalar(PTN3.getTypeName());
            // set array element of temp
            temp.arrayElement.merge(PTN4);
            // now PTN3 contains diff between PTN2 and PTN1
            PTN3 = temp;

            if (!PTN3.getArrayElementPossibleTypes().isEmpty())
              doNotExistInOther = true;
          }

          break;
        }
      }

      // this element has passed all tests
      if (doNotExistInOther)
        result.merge(PTN3);
    }

    return result;
  }

  public boolean merge(PossibleTypeName rhs)
  {
    if (rhs.isPrimitive() != isPrimitive)
    {
      throw new UnsupportedOperationException();
    }

    if (
        !rhs.getTypeName().equals(NativeTypes.get_net_nosica_lang_None()) &&
        !getTypeName().equals(NativeTypes.get_net_nosica_lang_None()) &&
        rhs.getTypeName().isArray() != getTypeName().isArray()
       )
    {
      // rhs != None &&
      // lhs != None &&
      // rhs.isArray != lhs.isArray
      throw new UnsupportedOperationException();
    }

    if (Environment.instance().debug)
    {
      if (isPrimitive)
      {
        if (!rhs.getTypeName().equals(typeName))
        {
          Environment.instance().terminate("Can't add different primitive types to a primitive PTN");
        }
      }
    }

    boolean modified = false;

    // if this is a list !
    if (hasMultiplePossibleTypes())
    {
      // if other is a list
      if (rhs.hasMultiplePossibleTypes())
      {
        // for each element of other merge element
        Iterator it = rhs.iteratePossibleType();
        while (it.hasNext())
        {
          PossibleTypeName PTN = (PossibleTypeName)it.next();
          if (merge(PTN)) modified=true;
        }
      }
      else
      {
        // other contains only one element
        boolean foundSameType = false;

        // loop over all elements of this to find other.typeName
        Iterator it = iteratePossibleType();
        while (it.hasNext())
        {
          PossibleTypeName PTN = (PossibleTypeName)it.next();
          if (PTN.getTypeName().equals(rhs.getTypeName()))
          {
            // yep, found it : no need to add it
            if (PTN.getTypeName().isArray())
            {
              // merge array element if needed
              if (PTN.getArrayElementPossibleTypes().merge(rhs.getArrayElementPossibleTypes()))
                modified=true;
            }

            foundSameType = true;
            break;
          }
        }

        if (!foundSameType)
        {
          // type is not registered ... add it !
          possibleTypes.add(new PossibleTypeNameImpl(rhs));
          modified=true;

          if (Environment.instance().debug)
          {
            if (!isPrimitive)
            {
              try
              {
                if (!Environment.instance().getTE().doesInherits(rhs.getTypeName(), typeName) &&
                    !rhs.getTypeName().equals(typeName)
                   )
                {
                  Environment.instance().terminate("Can't add a reference type that do not inherits from this PTN");
                }
              }
              catch (SymbolException e)
              {
                Environment.instance().terminate("Internal error : " + e.getMessage());
              }
            }
          }
        }
      }
    }
    else
    {
      // only PossibleTypeNameList supports the merge operation
      throw new UnsupportedOperationException();
    }

    return modified;
  }

  public PossibleTypeName getArrayElementPossibleTypes()
  {
    if (arrayElement != null)
      return arrayElement;
    throw new UnsupportedOperationException();
  }
  public TypeName getTypeName() { return typeName; }
  public void setTypeName(TypeName typeName) { this.typeName = typeName; }
  public boolean hasMultiplePossibleTypes() { return possibleTypes != null; }
  public int nbPossibleTypes() { return possibleTypes.size(); }

  public String toString()
  {
    StringBuffer sb = new StringBuffer();
    display(sb, this);
    return sb.toString();
  }

  private static void display(StringBuffer sb, PossibleTypeName PTN)
  {
    sb.append("{");
    sb.append(PTN.getTypeName());
    sb.append("|");
    if (PTN.hasMultiplePossibleTypes())
      displayMPTN(sb, PTN);
    else
      displayPTN(sb, PTN);
    sb.append("}");
  }


  private static void displayMPTN(StringBuffer sb, PossibleTypeName PTN)
  {
    if (!PTN.hasMultiplePossibleTypes())
      Environment.instance().terminate("Intenal error : PTN should be multiple");

    Iterator i = PTN.iteratePossibleType();
    while (i.hasNext())
    {
      PossibleTypeName PTN2 = (PossibleTypeName)i.next();
      displayPTN(sb, PTN2);

      if (i.hasNext())
        sb.append(", ");
    }
  }

  private static void displayPTN(StringBuffer sb, PossibleTypeName PTN)
  {
    if (PTN.hasMultiplePossibleTypes())
      Environment.instance().terminate("Intenal error : PTN should not be multiple");

    sb.append(PTN.getTypeName());

    if (PTN.getTypeName().isArray())
      display(sb, PTN.getArrayElementPossibleTypes());
  }
}

