/*
  $Id: TypeNameEnvironmentDisplayer.java,v 1.13 2004/03/04 22:16:16 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.compiler.analysis;

import net.nosica.framework.VisitorException;

import net.nosica.compiler.application.Environment;

import net.nosica.compiler.intermediate.Record;
import net.nosica.compiler.intermediate.Function;
import net.nosica.compiler.intermediate.Field;
import net.nosica.compiler.intermediate.Statement;
import net.nosica.compiler.intermediate.Expression;
import net.nosica.compiler.intermediate.IRVisitor;
import net.nosica.compiler.intermediate.Call;
import net.nosica.compiler.intermediate.MemberAccess;
import net.nosica.compiler.intermediate.Literal;
import net.nosica.compiler.intermediate.Sequence;
import net.nosica.compiler.intermediate.StatementExpression;
import net.nosica.compiler.intermediate.Variable;
import net.nosica.compiler.intermediate.Move;
import net.nosica.compiler.intermediate.ExtendedSequence;
import net.nosica.compiler.intermediate.Jump;
import net.nosica.compiler.intermediate.LabelName;
import net.nosica.compiler.intermediate.ConditionalJump;
import net.nosica.compiler.intermediate.Cast;
import net.nosica.compiler.intermediate.InstanceOf;
import net.nosica.compiler.intermediate.RawAllocation;
import net.nosica.compiler.intermediate.Init;
import net.nosica.compiler.intermediate.StatementList;
import net.nosica.compiler.intermediate.ArrayGet;
import net.nosica.compiler.intermediate.ArraySet;
import net.nosica.compiler.intermediate.ConstraintError;
import net.nosica.compiler.intermediate.TryRegister;
import net.nosica.compiler.intermediate.TryUnregister;
import net.nosica.compiler.intermediate.Throw;
import net.nosica.compiler.intermediate.VariableScopeLeave;
import net.nosica.compiler.intermediate.Return;

import net.nosica.compiler.production.c.intermediate.CheckNull;
import net.nosica.compiler.production.c.intermediate.DecRef;
import net.nosica.compiler.production.c.intermediate.IncRef;
import net.nosica.compiler.typechecker.Method;

import net.nosica.util.IndentedPrintWriter;
import net.nosica.util.Pair;

import net.nosica.parser.node.util.var.TypeName;

import java.util.Iterator;
import java.util.Map;

public class TypeNameEnvironmentDisplayer implements IRVisitor
{
  private IndentedPrintWriter writer;
  private Environment env = Environment.instance();

  /**
   * \brief creates a new displayer
   *
   * \param writer an IndentedPrintWriter
   */
  public TypeNameEnvironmentDisplayer(IndentedPrintWriter writer) { this.writer = writer; }

  public void displayRecords() throws VisitorException
  {
    Iterator i = env.getIRRE().iterateRecords();
    while (i.hasNext())
    {
      Record record = (Record)i.next();
      display(record);
      writer.println();
    }

    writer.flush();
  }

  public void displayFunctions() throws VisitorException
  {
    Iterator i = env.getIRFE().iterateFunctions();

    while (i.hasNext())
    {
      Function function = (Function)i.next();
      display(function);
      writer.println();
    }

    writer.flush();
  }

  public void display(Record record) throws VisitorException
  {
    writer.println("RECORD " + record.getName() + " {");
    writer.addIndent();

    Iterator it = record.iterateFields();
    while (it.hasNext())
    {
      Field field = (Field)it.next();

      PossibleTypeName ptnField = env.getPTNE().getPTN(field);

      writer.println(field.getName() + " : " + ptnField);
    }

    writer.decIndent();
    writer.println("}");
  }

  public void display(Function function) throws VisitorException
  {
    Method method = function.getMethod();
    Statement statement = function.getStatement();

    // display method
    if (!method.isSub())
    {
      writer.print(method.getResultType().getTypeName() + " ");
    }

    writer.println(method.getType() + "." + method.getName() + "(");
    writer.addIndent();
    Iterator j = function.iterateArgs();
    while (j.hasNext())
    {
      Variable var = (Variable)j.next();

      writer.print(var.getName() + " : " + var.getTypeName() + " ");

      if (j.hasNext())
        writer.print(",");

      writer.println();
    }
    writer.decIndent();
    writer.println(")");

    if (!method.isSub())
    {
      PossibleTypeName ptn = env.getPTNE().getPTN(function);

      writer.println("RESULT : " + ptn.toString());
    }

    if (!method.isStatic())
    {
      PossibleTypeName ptn = env.getPTNE().getPTN(function.getThisInfo());

      writer.println("THIS : " + ptn.toString());
    }

    writer.println("{");
    writer.addIndent();

    if (statement != null)
      statement.accept(this);
    else
      writer.println("EMPTY");

    writer.decIndent();
    writer.println("}");
    writer.println();
  }

  public void visit(Call call) throws VisitorException
  {
    if (call.hasSubscript())
      call.getSubscript().accept(this);
    else
      writer.print(call.getMethod().getType().toString());

    writer.print(".");

    writer.println(call.getMethod().getName() + "(");
    writer.addIndent();

    Iterator i = call.iterateArgs();
    while (i.hasNext())
    {
      Expression exp = (Expression)i.next();
      exp.accept(this);
      if (i.hasNext())
        writer.println(",");
    }
    writer.println();
    writer.decIndent();
    writer.print(")");

    if (!call.getMethod().isSub())
    {
      PossibleTypeName ptn = env.getPTNE().getPTN(call);

      writer.println("RESULT : " + ptn.toString());
    }
  }

  public void visit(ArrayGet arrayGet) throws VisitorException
  {
    arrayGet.getSubscript().accept(this);
    writer.print("[");

    Iterator it = arrayGet.iterateArgs();
    while (it.hasNext())
    {
      Expression exp = (Expression)it.next();
      exp.accept(this);

      if (it.hasNext())
        writer.print(", ");
    }

    writer.print("]");

    PossibleTypeName ptn = env.getPTNE().getPTN(arrayGet);
    writer.println("ArrayGetPTN : " + ptn.toString());
  }

  public void visit(ArraySet arraySet) throws VisitorException
  {
    arraySet.getSubscript().accept(this);
    writer.print("[");

    Iterator it = arraySet.iterateArgs();
    while (it.hasNext())
    {
      Expression exp = (Expression)it.next();
      exp.accept(this);

      if (it.hasNext())
        writer.print(", ");
    }

    writer.print("] = ");
    arraySet.getRhs().accept(this);
  }

  public void visit(MemberAccess memberAccess) throws VisitorException
  {
    if (memberAccess.hasSubscript())
      memberAccess.getSubscript().accept(this);
    else
      writer.print(memberAccess.getField().getEnclosing().toString());

    PossibleTypeName ptn = env.getPTNE().getPTN(memberAccess);
    writer.print("." + memberAccess.getName() + " {" + ptn.toString() + "} ");
  }

  public void visit(Literal literal) throws VisitorException
  {
    PossibleTypeName ptn = env.getPTNE().getPTN(literal);
    writer.print(literal.getStrValue() + " {" + ptn.toString() + "} ");
  }

  public void visit(Sequence sequence) throws VisitorException
  {
    writer.println("SEQ(");
    writer.addIndent();
    sequence.getLeft().accept(this);
    writer.println(",");
    sequence.getRight().accept(this);
    writer.println();
    writer.decIndent();
    writer.print(")");
  }

  public void visit(StatementExpression statementExpression) throws VisitorException
  {
    writer.println("EXP(");
    writer.addIndent();
    statementExpression.getExpression().accept(this);
    writer.println();
    writer.decIndent();
    writer.print(")");
  }

  public void visit(Variable variable) throws VisitorException
  {
    PossibleTypeName ptn = env.getPTNE().getPTN(variable);
    writer.print(variable.getName() + " {" + ptn.toString() + "} ");
  }

  public void visit(ExtendedSequence exSeq) throws VisitorException
  {
    writer.println("ESEQ(");
    writer.addIndent();
    exSeq.getStatement().accept(this);
    writer.println(",");
    exSeq.getExpression().accept(this);
    writer.println();
    writer.decIndent();
    writer.print(")");
  }

  public void visit(Move move) throws VisitorException
  {
    writer.println("MOVE(");
    writer.addIndent();
    move.getVariable().accept(this);
    writer.println(",");
    move.getRhs().accept(this);
    writer.println();
    writer.decIndent();
    writer.print(")");
  }

  public void visit(Jump jump) throws VisitorException
  {
    writer.print("JUMP (" + jump.getLabel().getName() + ")");
  }

  public void visit(LabelName labelName) throws VisitorException
  {
    writer.print("LABEL (" + labelName.getLabel().getName() + ")");
  }

  public void visit(ConditionalJump conditionalJump) throws VisitorException
  {
    writer.println("CONDJUMP(");
    writer.addIndent();
    conditionalJump.getCondition().accept(this);
    writer.println(",");
    writer.println(conditionalJump.getTrueLabel().getName() + ",");
    writer.println(conditionalJump.getFalseLabel().getName());
    writer.decIndent();
    writer.print(")");

  }

  public void visit(Cast castExp) throws VisitorException
  {
    writer.println("CAST (");
    writer.addIndent();
    castExp.getExpression().accept(this);
    writer.println(",");
    writer.println(castExp.getTypeName().toString());
    writer.decIndent();
    writer.print(")");

    PossibleTypeName ptn = env.getPTNE().getPTN(castExp);
    writer.println("CASTPTN : " + ptn.toString());
  }

  public void visit(InstanceOf instanceOfExp) throws VisitorException
  {
    writer.println("INSTANCEOF (");
    writer.addIndent();
    instanceOfExp.getExpression().accept(this);
    writer.println(",");
    writer.println(instanceOfExp.getRhsTypeName().toString());
    writer.decIndent();
    writer.print(")");

    PossibleTypeName ptn = env.getPTNE().getPTN(instanceOfExp);
    writer.println("INSTANCEOFPTN : " + ptn.toString());
  }

  public void visit(RawAllocation rawAllocation) throws VisitorException
  {
    writer.print("RAW_ALLOCATION(");
    writer.print(rawAllocation.getTypeName().toString());
    writer.println(")");

    PossibleTypeName ptn = env.getPTNE().getPTN(rawAllocation);
    writer.println("RAW_ALLOCATIONPTN : " + ptn.toString());
  }

  public void visit(IncRef incRef) throws VisitorException
  {
    writer.print("INCREF(");
    incRef.getVariable().accept(this);
    writer.println(")");
  }

  public void visit(DecRef decRef) throws VisitorException
  {
    writer.print("DECREF(");
    decRef.getVariable().accept(this);
    writer.println(")");
  }

  public void visit(Init init) throws VisitorException
  {
    writer.println("INIT(");
    writer.addIndent();
    init.getVariable().accept(this);
    writer.println(",");
    init.getRhs().accept(this);
    writer.println();
    writer.decIndent();
    writer.print(")");
  }

  public void visit(StatementList statementList) throws VisitorException
  {
    writer.println("SEQLIST(");
    writer.addIndent();
    Iterator it = statementList.iterateStatementList();
    while (it.hasNext())
    {
      Statement statement = (Statement)it.next();
      statement.accept(this);

      if (it.hasNext())
        writer.print(",");
      writer.println();
    }
    writer.decIndent();
    writer.println(")");
  }

  public void visit(CheckNull checkNull) throws VisitorException
  {
    writer.println("CHECKNULL(");
    writer.addIndent();
    checkNull.getExpression().accept(this);
    writer.println();
    writer.decIndent();
    writer.println(")");
  }
  public void visit(ConstraintError constraintError) throws VisitorException
  {
    writer.println("CONSTRAINTERROR(" + constraintError.getErrorString() + ")");
  }
  public void visit(TryRegister tryRegister) throws VisitorException
  {
    writer.println("TRYREGISTER(");

    writer.addIndent();
    // display exceptions sites and exception variables
    Iterator it = tryRegister.getExceptionMap().entrySet().iterator();
    while (it.hasNext())
    {
      Map.Entry entry = (Map.Entry)it.next();
      TypeName exceptionTypeName = (TypeName)entry.getKey();
      Pair pair = (Pair)entry.getValue();
      net.nosica.compiler.typechecker.Label label = (net.nosica.compiler.typechecker.Label)pair.first;
      Variable var = (Variable)pair.second;

      writer.println(exceptionTypeName + " => " + label.getName() + " : " + var.getName());
    }

    if (tryRegister.hasFinallyStatement())
    {
      writer.println("Finally : ");
      tryRegister.getFinallyStatement().accept(this);
    }
    writer.decIndent();
  }
  public void visit(TryUnregister tryUnregister) throws VisitorException
  {
    writer.println("TRYUNREGISTER");
  }
  public void visit(Throw throwStatement) throws VisitorException
  {
    writer.print("THROW ");
    throwStatement.getExpression().accept(this);
  }
  public void visit(VariableScopeLeave varScopeLeave) throws VisitorException
  {
    writer.println("SCOPELEAVE(" + varScopeLeave.getVariable().getName() + ")");
  }
  public void visit(Return returnStatement) throws VisitorException
  {
    writer.println("RETURN(");
    writer.addIndent();
    returnStatement.getResult().accept(this);
    writer.decIndent();
  }
}
