/*
  $Id: GenericitySolver.java,v 1.7 2004/03/04 22:16:16 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.compiler.application;

import net.nosica.parser.node.util.var.TypeName;
import net.nosica.symbol.GenericType;
import net.nosica.symbol.GenericMethod;
import net.nosica.symbol.AliasTable;
import net.nosica.compiler.typechecker.Type;
import net.nosica.compiler.typechecker.Method;

import java.util.Map;

/**
 * \brief Deals with all problem linked to genericity
 * ie, infer generic types to use to instantiate a generic type<BR>
 * infer which partial generic type to use ion case of many<BR>
 * instanciate a GenericType in a Type
 */

public interface GenericitySolver
{
  /**
   * \brief instantiate a GenericType
   * Given a Map<TypeName, TypeName> of generic parameters to use and a GenericParameter
   * this method instantiate the type, that is, it replaces all generic parameters it can see
   * with is real type.
   *
   * \param type the generic type to instantiate
   * \param genericAssociation Map<TypeName, TypeName>
   * \param target the target typeName
   * \return an instantiated type
   */
  Type instantiateType(GenericType type, Map genericAssociation, TypeName target) throws GenericityException;

  /**
   * \brief instantiate method given a signature
   *
   * \param method the method to instantiate
   * \param signature the target signature
   * \param enclosingType name of type that defines generic method method
   * \param aliasTable alias table of enclosing type
   * \return instantiated method if it was possible
   */
  Method instantiateMethod(
                            GenericMethod method,
                            Method signature,
                            TypeName enclosingType,
                            AliasTable aliasTable
                          ) throws GenericityException;
}