/*
  $Id: GenericitySolverImpl.java,v 1.21 2004/04/18 00:52:03 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.compiler.application;

import net.nosica.parser.node.util.var.FormalParametersImpl;
import net.nosica.parser.node.util.var.TypeName;
import net.nosica.parser.node.util.var.TypeNameImpl;
import net.nosica.parser.node.util.var.TypeNameListImpl;
import net.nosica.parser.node.util.var.TypeNameComponent;
import net.nosica.parser.node.util.var.TypeNameComponentImpl;
import net.nosica.parser.node.util.generic.GenericParameter;
import net.nosica.parser.node.typedeclaration.ConstructorDeclarationImpl;
import net.nosica.parser.node.typedeclaration.DestructorDeclarationImpl;
import net.nosica.parser.node.typedeclaration.Initializer;
import net.nosica.parser.node.typedeclaration.InitializerImpl;
import net.nosica.parser.node.typedeclaration.Deinitializer;
import net.nosica.parser.node.typedeclaration.DeinitializerImpl;
import net.nosica.parser.node.typedeclaration.statement.BlockImpl;

import net.nosica.symbol.GenericType;
import net.nosica.symbol.GenericMethod;
import net.nosica.symbol.GenericMethodImpl;
import net.nosica.symbol.MethodKindEnum;
import net.nosica.symbol.SymbolException;
import net.nosica.symbol.AliasTable;
import net.nosica.symbol.AliasTableImpl;
import net.nosica.symbol.ResultType;
import net.nosica.symbol.ResultTypeImpl;
import net.nosica.symbol.Argument;

import net.nosica.application.NativeTypes;
import net.nosica.application.MessageLevel;
import net.nosica.application.TypeNameResolver;
import net.nosica.compiler.typechecker.Type;
import net.nosica.compiler.typechecker.TypeImpl;
import net.nosica.compiler.typechecker.Method;
import net.nosica.compiler.typechecker.MethodImpl;
import net.nosica.compiler.typechecker.Field;
import net.nosica.compiler.typechecker.FieldImpl;

import net.nosica.util.Helper;

import java.util.Map;
import java.util.HashMap;
import java.util.Set;
import java.util.HashSet;
import java.util.ArrayList;
import java.util.Iterator;

public class GenericitySolverImpl implements GenericitySolver
{
  private static GenericitySolverImpl theInstance = new GenericitySolverImpl();
  private GenericitySolverImpl() {}
  public static GenericitySolverImpl instance() { return theInstance; }
  public static void instanceReset() { theInstance = new GenericitySolverImpl(); }

  private Environment env = Environment.instance();
  private TypeNameResolver typeNameResolver = env.getTR();

  public Type instantiateType(GenericType type, Map genericAssociation, TypeName target) throws GenericityException
  {
    AliasTable aliasTable = new AliasTableImpl(type.getAliasTable());

    completeGenericAliasTable(type, genericAssociation, target, aliasTable);
    completeAliasTable(target, aliasTable);

    // instantiate extends list
    Set instantiatedExtendsList = instantiateExtendsList(type.iterateExtends(), aliasTable);
    // instantiate implements list
    Set instantiatedImplementsList = instantiateImplementsList(type.iterateImplements(), aliasTable);

    // add default parents if needed ...
    if (
        type.isClass() &&
        !type.isPrimitive() &&
        instantiatedExtendsList.isEmpty() &&
        !target.equals(NativeTypes.get_net_nosica_lang_Object())
       )
    {
      instantiatedExtendsList.add(NativeTypes.get_net_nosica_lang_Object());
    }

    ArrayList instantiatedFields = instantiateFields(type, target, aliasTable);
    Map instantiatedOverloadables = instantiateOverloadables(type, target, aliasTable);

    // if type is net.nosica.lang.Object, add it built-in methods
    if (target.equals(NativeTypes.get_net_nosica_lang_Object()))
    {
      completeNetNosicaLangObject(aliasTable, instantiatedOverloadables);
    }

    if (type.isInterface())
    {
      // change protection of method : until now methods looked like they were package
      // but in fact, they are public in an interface
      Iterator i = instantiatedOverloadables.values().iterator();
      while (i.hasNext())
      {
        ArrayList methods = (ArrayList)i.next();
        Iterator j = methods.iterator();
        while (j.hasNext())
        {
          Method m = (Method)j.next();
          m.setPublic();
        }
      }
    }

    Map instantiatedCastOperators = instantiateCastOperators(type, target, aliasTable);

    Method instantiatedDestructor    = instantiateDestructor(type, target, aliasTable);
    Method instantiatedInitializer   = instantiateInitializer(type, target, aliasTable);
    Method instantiatedDeinitializer = instantiateDeinitializer(type, target, aliasTable);

    // method alias table instanciation
    Map methodAliasTable = type.getMethodAliasTable();
    methodAliasTable = instantiateMethodAliasTable(methodAliasTable, aliasTable);

    return new TypeImpl(
      type,
      target,
      instantiatedExtendsList,
      instantiatedImplementsList,
      instantiatedFields,
      instantiatedCastOperators,
      instantiatedOverloadables,
      instantiatedDestructor,
      instantiatedInitializer,
      instantiatedDeinitializer,
      type.isNested() ? instantiateTypeName(type.getEnclosing(), aliasTable) : null,
      aliasTable,
      methodAliasTable
    );
  }

  private void completeGenericAliasTable(
    GenericType type,
    Map genericAssociation,
    TypeName target,
    AliasTable aliasTable
  ) throws GenericityException
  {
    if (!type.getName().isGeneric())
      return;

    // now create new AliasTable for type by inserting generic parameters in AliasTable
    try
    {
      Iterator it = genericAssociation.entrySet().iterator();
      while (it.hasNext())
      {
        Map.Entry entry = (Map.Entry)it.next();
        TypeName genericParm = (TypeName)entry.getKey();
        TypeName value = (TypeName)entry.getValue();

        aliasTable.replaceAlias(genericParm, value);
      }

      // also replace this generic alias by its target name
      ArrayList symbolNameComponent = new ArrayList();
      symbolNameComponent.add(new TypeNameComponentImpl(target.getLastComponent().getPackageElement(), null));

      aliasTable.replaceAlias(new TypeNameImpl(symbolNameComponent), target);

      // now loop over all generic import to resolve them
      it = type.getAliasTable().iterateAliases();
      while (it.hasNext())
      {
        Map.Entry entry = (Map.Entry)it.next();
        TypeName aliasedType = (TypeName)entry.getValue();

        if (aliasedType.isGeneric())
          aliasTable.replaceAlias((TypeName)entry.getKey(), env.getTR().resolve(aliasedType, aliasTable, env.getFL()));
        else
        {
          // see if aliasedType refers target as its enclosing type
          Iterator j = target.iterateComponents();
          Iterator k = aliasedType.iterateComponents();
          boolean isEnclosed = true;
          while (j.hasNext() && k.hasNext())
          {
            String targetComp = ((TypeNameComponent)j.next()).getPackageElement();
            String aliasedComp = ((TypeNameComponent)k.next()).getPackageElement();

            if (!targetComp.equals(aliasedComp))
            {
              isEnclosed = false;
              break;
            }
          }

          if (!j.hasNext() && isEnclosed)
          {
            ArrayList resolvedTypeName = new ArrayList();
            resolvedTypeName.addAll(target.getComponents());
            while (k.hasNext())
              resolvedTypeName.add(k.next());

            aliasTable.replaceAlias((TypeName)entry.getKey(), new TypeNameImpl(resolvedTypeName));
          }
        }
      }
    }
    catch (SymbolException e)
    {
      throw new GenericityException(e.getMessage());
    }
  }

  private void completeAliasTable(TypeName target, AliasTable aliasTable) throws GenericityException
  {
    try
    {
      // add alias This on target
      ArrayList thisComponents = new ArrayList();
      thisComponents.add(new TypeNameComponentImpl("This", null));
      TypeName aliasThis = new TypeNameImpl(thisComponents);

      if (aliasTable.aliasExists(aliasThis))
        aliasTable.replaceAlias(aliasThis, target);
      else
        aliasTable.addAlias(aliasThis, target);
    }
    catch (SymbolException e)
    {
      throw new GenericityException("Alias This is already defined");
    }
  }

  private ArrayList instantiateFields(GenericType type, TypeName target, AliasTable aliasTable) throws GenericityException
  {
    // instantiate fields
    ArrayList instantiatedFields = new ArrayList();
    Iterator fieldsIt = type.iterateFields();
    while (fieldsIt.hasNext())
    {
      net.nosica.symbol.Field field = (net.nosica.symbol.Field)fieldsIt.next();
      Field instantiatedField = instantiateField(field, aliasTable, target);
      instantiatedFields.add(instantiatedField);
    }

    return instantiatedFields;
  }

  private Map instantiateOverloadables(GenericType type, TypeName target, AliasTable aliasTable) throws GenericityException
  {
    createDefaultConstructorIfNeeded(type, target, aliasTable);

    // instantiate overloadables
    Map instantiatedOverloadables = new HashMap();
    // iterate through methods
    Iterator overloadablesIt = type.getOverloadables().values().iterator();
    while (overloadablesIt.hasNext())
    {
      ArrayList methods = (ArrayList)overloadablesIt.next();
      Iterator methodsIt = methods.iterator();

      ArrayList instantiatedMethods = new ArrayList();    // ArrayList<Method>
      String methodName = null;
      while (methodsIt.hasNext())
      {
        // get generic method
        net.nosica.symbol.GenericMethod method = (net.nosica.symbol.GenericMethod)methodsIt.next();

        if (methodName == null)
          methodName = method.getName();

        // can it be instantiated ?
        if (!method.hasGenericParameter())
        {
          // yep : there is no new generic parameters
          Method instantiatedMethod = instantiateNonGenericMethod(method, target, aliasTable);

          // check there is no method with same signature
          // note : this check is delayed here because during symbol extraction
          // we have not resolved yet typename
          Iterator it = instantiatedMethods.iterator();
          boolean error = false;
          while (it.hasNext())
          {
            Method m = (Method)it.next();
            if (instantiatedMethod.hasSameSignature(m))
            {
              error = true;
              env.getMS().report(MessageLevel.ERR, "Symbol025", new String[]{m.getName(), m.getModel().getNode().localization()}, method.getNode());
              break;
            }
          }

          if (!error)
          {
            instantiatedMethods.add(instantiatedMethod);
          }
        }
      }

      if (!instantiatedMethods.isEmpty())
        instantiatedOverloadables.put(methodName, instantiatedMethods);
    }

    return instantiatedOverloadables;
  }

  private void createDefaultConstructorIfNeeded(GenericType type, TypeName target, AliasTable aliasTable)
  {
    if (type.getOverloadables().get("constructor") == null)
    {
      // no default constructor : create one and add it to generic type
      GenericMethod constructor = new GenericMethodImpl(
        new ConstructorDeclarationImpl(
          null,
          new FormalParametersImpl(new ArrayList()),
          new TypeNameListImpl(new ArrayList()),
          null,
          new ArrayList(),
          true, false, false
        ),
        true, false, false, new ArrayList(), type.getName(),
        false, false, false, false, null, MethodKindEnum.constructor,
        "constructor", new ArrayList(), java.util.Collections.EMPTY_SET, !type.isNative()
      );
      type.addMethod(constructor);
    }
  }

  private Map instantiateCastOperators(GenericType type, TypeName target, AliasTable aliasTable) throws GenericityException
  {
    // iterate through cast operators
    Map instantiatedCastOperators = new HashMap();
    Iterator castOperatorsIt = type.getCastOperators().values().iterator();
    while (castOperatorsIt.hasNext())
    {
      // get generic method
      net.nosica.symbol.GenericMethod method = (net.nosica.symbol.GenericMethod)castOperatorsIt.next();

      // can it be instantiated ?
      if (isGenericMethodInstantiable(method, aliasTable))
      {
        // yep : there is no new generic parameters
        Method instantiatedMethod = instantiateNonGenericMethod(method, target, aliasTable);

        if (instantiatedCastOperators.containsKey(instantiatedMethod.getResultType().getTypeName()))
        {
          Method oldMethod = (Method)instantiatedCastOperators.get(instantiatedMethod.getResultType().getTypeName());
          env.getMS().report(MessageLevel.ERR, "Symbol025", new String[]{instantiatedMethod.getResultType().getTypeName().toString(), oldMethod.getModel().getNode().localization()}, method.getNode());
        }

        instantiatedCastOperators.put(instantiatedMethod.getResultType().getTypeName(), instantiatedMethod);
      }
      else
        throw new GenericityException("A cast operator cannot introduce new generic parameters");
    }

    return instantiatedCastOperators;
  }

  private Method instantiateDestructor(GenericType type, TypeName target, AliasTable aliasTable) throws GenericityException
  {
    Method instantiatedDestructor = null;
    if (type.hasDestructor())
      instantiatedDestructor = instantiateNonGenericMethod(type.getDestructor(), target, aliasTable);
    else  // create an empty destructor !!
    {
      instantiatedDestructor = new MethodImpl(
        new GenericMethodImpl(
          new DestructorDeclarationImpl(
            new BlockImpl(new ArrayList())
          ),
          true,
          false,
          false,
          new ArrayList(),
          type.getName(),
          false,
          false,
          false,
          false,
          null,
          MethodKindEnum.destructor,
          "destructor",
          new ArrayList(),
          java.util.Collections.EMPTY_SET,
          !type.isNative()
        ),
        null,
        new ArrayList(),
        java.util.Collections.EMPTY_SET,
        target,
        aliasTable
      );
    }

    return instantiatedDestructor;
  }

  private Method instantiateInitializer(GenericType type, TypeName target, AliasTable aliasTable) throws GenericityException
  {
    // create initializer method
    Method instantiatedInitializer = null;
    if (type.hasInitializer())
    {
      instantiatedInitializer = instantiateNonGenericMethod(type.getInitializer(), target, aliasTable);
    }
    else
    {
      Initializer initializer = new InitializerImpl(new BlockImpl(new ArrayList()));
      instantiatedInitializer = new MethodImpl(
        new GenericMethodImpl(
          initializer,
          true,
          false,
          false,
          new ArrayList(),
          type.getName(),
          true,
          false,
          true,
          false,
          null,
          MethodKindEnum.initializer,
          "initializer",
          new ArrayList(),
          java.util.Collections.EMPTY_SET,
          !type.isNative()
        ),
        null,
        new ArrayList(),
        java.util.Collections.EMPTY_SET,
        target,
        aliasTable
      );
    }

    return instantiatedInitializer;
  }

  private Method instantiateDeinitializer(GenericType type, TypeName target, AliasTable aliasTable) throws GenericityException
  {
    // create deinitializer method
    Method instantiatedDeinitializer = null;
    if (type.hasDeinitializer())
    {
      instantiatedDeinitializer = instantiateNonGenericMethod(type.getDeinitializer(), target, aliasTable);
    }
    else
    {
      Deinitializer deinitializer = new DeinitializerImpl(new BlockImpl(new ArrayList()));
      instantiatedDeinitializer = new MethodImpl(
        new GenericMethodImpl(
          deinitializer,
          true,
          false,
          false,
          new ArrayList(),
          type.getName(),
          true,
          false,
          true,
          false,
          null,
          MethodKindEnum.deinitializer,
          "deinitializer",
          new ArrayList(),
          java.util.Collections.EMPTY_SET,
          !type.isNative()
        ),
        null,
        new ArrayList(),
        java.util.Collections.EMPTY_SET,
        target,
        aliasTable
      );
    }

    return instantiatedDeinitializer;
  }

  private void completeNetNosicaLangObject(AliasTable aliasTable, Map instantiatedOverloadables) throws GenericityException
  {
    // add a value comparison operator ==
  ArrayList genericRefEqArgs = new ArrayList();
    genericRefEqArgs.add(
    new net.nosica.symbol.ArgumentImpl(
      true,
      NativeTypes.get_net_nosica_lang_Object(),
      "obj"
    )
    );

    ArrayList refEqOperator = new ArrayList();
    refEqOperator.add(
    new MethodImpl(
      new GenericMethodImpl(
      new net.nosica.parser.node.typedeclaration.OperatorDeclarationImpl(
        null,
        true,
        false,
        false,
        null,
        null,
        null,
        true,
        null,
        new BlockImpl(new ArrayList()),
        true,
        false,
        false,
        true
      ),
      true,
      false,
      false,
      new ArrayList(),
      NativeTypes.get_net_nosica_lang_Object(),
      false,
      true,
      true,
      false,
      new ResultTypeImpl(
        false,
        NativeTypes.get_net_nosica_lang_bool()
      ),
      MethodKindEnum.infix,
      "==",
      genericRefEqArgs,
      java.util.Collections.EMPTY_SET,
      false
      ),
      new net.nosica.compiler.typechecker.ResultTypeImpl(
      NativeTypes.get_net_nosica_lang_bool(),
      false
      ),
      instantiateArguments(genericRefEqArgs, aliasTable),
      java.util.Collections.EMPTY_SET,
      NativeTypes.get_net_nosica_lang_Object(),
      aliasTable
    )
    );

    instantiatedOverloadables.put(
    "==",
    refEqOperator
    );

    //	add a value comparison operator !=
    ArrayList genericRefNeArgs = new ArrayList();
    genericRefNeArgs.add(
    new net.nosica.symbol.ArgumentImpl(
      true,
      NativeTypes.get_net_nosica_lang_Object(),
      "obj"
    )
    );

    ArrayList refNeOperator = new ArrayList();
    refNeOperator.add(
    new MethodImpl(
      new GenericMethodImpl(
      new net.nosica.parser.node.typedeclaration.OperatorDeclarationImpl(
        null,
        true,
        false,
        false,
        null,
        null,
        null,
        true,
        null,
        new BlockImpl(new ArrayList()),
        true,
        false,
        false,
        true
      ),
      true,
      false,
      false,
      new ArrayList(),
      NativeTypes.get_net_nosica_lang_Object(),
      false,
      true,
      true,
      false,
      new ResultTypeImpl(
        false,
        NativeTypes.get_net_nosica_lang_bool()
      ),
      MethodKindEnum.infix,
      "!=",
      genericRefNeArgs,
      java.util.Collections.EMPTY_SET,
      false
      ),
      new net.nosica.compiler.typechecker.ResultTypeImpl(
      NativeTypes.get_net_nosica_lang_bool(),
      false
      ),
      instantiateArguments(genericRefNeArgs, aliasTable),
      java.util.Collections.EMPTY_SET,
      NativeTypes.get_net_nosica_lang_Object(),
      aliasTable
    )
    );

    instantiatedOverloadables.put(
    "!=",
    refNeOperator
    );
  }

  public Method instantiateMethod(GenericMethod method, Method signature, TypeName enclosingType, AliasTable aliasTable) throws GenericityException
  {
    // try first to infer generic parameters ...
    Iterator i = method.iterateArguments();
    Iterator j = signature.iterateArguments();

    AliasTable methodAliasTable = new AliasTableImpl(aliasTable);

    while (i.hasNext() && j.hasNext())
    {
      Argument arg = (Argument)i.next();
      net.nosica.compiler.typechecker.Argument argSignature = (net.nosica.compiler.typechecker.Argument)j.next();

      inferMethodAliasTableFromTypeName(
                                  arg.getTypeName(),
                                  argSignature.getTypeName(),
                                  method.getGenericParameters(),
                                  methodAliasTable
      );
    }

    // at this point, if no exception was thrown, then it is possible that types are
    // compatible
    // anyway, let's complete the method alias table
    i = method.iterateGenericParameters();
    while (i.hasNext())
    {
      GenericParameter gp = (GenericParameter)i.next();
      if (!methodAliasTable.aliasExists(gp.getName()))
      {
        if (gp.hasDefaultType())
        {
          try
          {
            methodAliasTable.addAlias(gp.getName(), gp.getDefaultType());
          }
          catch (SymbolException e)
          {
            env.terminate("Internal error : a generic parameter shadow a generic parameter of enclosing type : this should have been detected earlier");
          }
        }
        else
          throw new GenericityException("Missing generic parameter");
      }
    }

    Method result = instantiateNonGenericMethod(method, enclosingType, methodAliasTable);

    result.setAliasTable(methodAliasTable);

    return result;
  }

  private void inferMethodAliasTableFromTypeName(
                                                  TypeName arg,
                                                  TypeName signature,
                                                  ArrayList genericDeclaration,
                                                  AliasTable aliasTable
                                                )
                                                throws GenericityException
  {
    // same type
    if (arg.equals(signature))
      return;

    // arg is declared in generic declaration : we have found it's type
    if (isGenericParameter(arg, genericDeclaration))
    {
      try
      {
        aliasTable.addAlias(arg, signature);
        return;
      }
      catch (SymbolException e)
      {
        env.terminate("Internal error : method generic parameter shadow existing one in enclosing type, this should have been detected earlier");
      }
    }

    // look closer into structure of type name
    // first let's get non array type
    while (arg.isArray() && signature.isArray())
    {
      arg = arg.getArrayElementType();
      signature = signature.getArrayElementType();
    }

    if (arg.isArray() || signature.isArray())
    {
      // seems they are not compatible
      throw new GenericityException("Argument is not compatible");
    }

    // arg is declared in generic declaration : we have found it's type
    if (isGenericParameter(arg, genericDeclaration))
    {
      try
      {
        aliasTable.addAlias(arg, signature);
        return;
      }
      catch (SymbolException e)
      {
        env.terminate("Internal error : method generic parameter shadow existing one in enclosing type, this should have been detected earlier");
      }
    }

    // we can have A<T>, and signature is package.A<T> ==> We have to resolve A as package.A
    TypeName nonGenericArg = Helper.createNonGenericTypeName(arg);
    try
    {
      nonGenericArg = env.getTR().resolve(nonGenericArg, aliasTable, env.getFL());
      arg = Helper.reconstructGenericTypeNameFromRadix(nonGenericArg, arg);
    }
    catch (SymbolException e)
    {
    }

    // no ... we will have to go through all components of type name ...
    Iterator i = arg.iterateComponents();
    Iterator j = signature.iterateComponents();

    while (i.hasNext() && j.hasNext())
    {
      TypeNameComponent argComp = (TypeNameComponent)i.next();
      TypeNameComponent signatureComp = (TypeNameComponent)j.next();

      if (argComp.hasGenericExtension() != signatureComp.hasGenericExtension())
      {
        // seems they are not compatible
        throw new GenericityException("Argument is not compatible");
      }

      if (argComp.hasGenericExtension())
      {
        Iterator k = argComp.iterateGenericExtension();
        Iterator l = signatureComp.iterateGenericExtension();

        while (k.hasNext() && l.hasNext())
          inferMethodAliasTableFromTypeName((TypeName)k.next(), (TypeName)l.next(), genericDeclaration, aliasTable);

        if (k.hasNext() || l.hasNext())
        {
          // seems they are not compatible
          throw new GenericityException("Argument is not compatible");
        }
      }
    }

    if (i.hasNext() || j.hasNext())
    {
      // seems they are not compatible
      throw new GenericityException("Argument is not compatible");
    }
  }

  // genericDeclaration : ArrayList<GenericParameter>
  private boolean isGenericParameter(TypeName typeName, ArrayList genericDeclaration)
  {
    Iterator i = genericDeclaration.iterator();
    while (i.hasNext())
    {
      GenericParameter genericParm = (GenericParameter)i.next();
      if (genericParm.getName().equals(typeName))
        return true;
    }

    return false;
  }

  /**
   * \brief indicates if a method is instantiable (using AliasTable)
   *
   * \param method the method to look for
   * \param aliasTable the alias table to use
   * \return true if it is possible to instantiate method using aliasTable
   */
  private boolean isGenericMethodInstantiable(GenericMethod method, AliasTable aliasTable)
  {
    if (!method.isSub())
    {
      if (!isResultTypeInstantiable(method.getResultType(), aliasTable))
        return false;
    }

    Iterator argsIt = method.iterateArguments();
    while (argsIt.hasNext())
    {
      net.nosica.symbol.Argument arg = (net.nosica.symbol.Argument)argsIt.next();

      if (!isArgumentInstantiable(arg, aliasTable))
        return false;
    }

    Iterator throwsIt = method.iterateThrowList();
    while (throwsIt.hasNext())
    {
      TypeName throwName = (TypeName)throwsIt.next();
      if (!isTypeNameInstantiable(throwName, aliasTable))
        return false;
    }

    return true;
  }

  /**
   * \brief indicates if a TypeName is instantiable (using aliasTable)
   *
   * \param typeName the TypeName to look for
   * \param aliasTable the alias table to use
   * \return true if it is possible to instantiate typeName
   */
  private boolean isTypeNameInstantiable(TypeName typeName, AliasTable aliasTable)
  {
    try
    {
      typeNameResolver.resolve(typeName, aliasTable, env.getFL());
      return true;
    }
    catch (SymbolException e)
    {
      return false;
    }
  }

  /**
   * \brief indicates if a ResultType is instantiable (using aliasTable)
   *
   * \param resultType the result type to look for
   * \param aliasTable the alias table to use
   * \return true if it is possible to instantiate resultType
   */
  private boolean isResultTypeInstantiable(net.nosica.symbol.ResultType resultType, AliasTable aliasTable)
  {
    return isTypeNameInstantiable(resultType.getTypeName(), aliasTable);
  }

  /**
   * \brief indicates if an argument is instantiable (using aliasTable)
   *
   * \param arg the argument to look for
   * \param aliasTable the alias table to use
   * \return true if it is possible to instantiate arg
   */
  private boolean isArgumentInstantiable(net.nosica.symbol.Argument arg, AliasTable aliasTable)
  {
    return isTypeNameInstantiable(arg.getTypeName(), aliasTable);
  }

  /**
   * \brief instantiates a non generic method
   * that is, instantiates a method that does not introduce new generic parameters
   *
   * \param method thje method to instantiate
   * \param type enclosing type
   * \param aliasTable the aliasTable to attach to method
   * \param uid unique identifier of method among type
   * \return instantiated method
   */
  private Method instantiateNonGenericMethod(
                    GenericMethod method,
                    TypeName type,
                    AliasTable aliasTable
                 ) throws GenericityException
  {
    return new MethodImpl(
                           method,
                           method.isSub() ? null : instantiateResultType(method.getResultType(), aliasTable),
                           instantiateArguments(method.getArguments(), aliasTable),
                           instantiateTypeNameSet(method.iterateThrowList(), aliasTable),
                           type,
                           aliasTable
                         );
  }

  /**
   * \brief instantiates a TypeName
   *
   * \param typeName typeName to instantiate
   * \param aliasTable alias table to use to instantiate type
   * \return instantiated TypeName
   */
  private TypeName instantiateTypeName(TypeName typeName, AliasTable aliasTable) throws GenericityException
  {
    try
    {
      return typeNameResolver.resolve(typeName, aliasTable, env.getFL());
    }
    catch (SymbolException e)
    {
      env.getMS().report(MessageLevel.ERR, "Symbol031", typeName.toString(), typeName);
      return typeName;
    }
  }

  private net.nosica.compiler.typechecker.ResultType instantiateResultType(ResultType resultType, AliasTable aliasTable) throws GenericityException
  {
    TypeName instantiatedTypeName = instantiateTypeName(resultType.getTypeName(), aliasTable);

    // if result type is a primitive typeName
    // then signature of returned type is always const
    return new net.nosica.compiler.typechecker.ResultTypeImpl(
                               instantiatedTypeName,
                               resultType.isConst()
                             );
  }

  private net.nosica.compiler.typechecker.Argument instantiateArgument(Argument arg, AliasTable aliasTable) throws GenericityException
  {
    return new net.nosica.compiler.typechecker.ArgumentImpl(
                             arg.isConst(),
                             instantiateTypeName(arg.getTypeName(), aliasTable),
                             arg.getName()
                           );
  }

  private ArrayList instantiateArguments(ArrayList args, AliasTable aliasTable) throws GenericityException
  {
    ArrayList result = new ArrayList();

    Iterator it = args.iterator();
    while (it.hasNext())
    {
      Argument arg = (Argument)it.next();
      result.add(instantiateArgument(arg, aliasTable));
    }

    return result;
  }

  /**
   * \brief instantiates a set of TypeName
   *
   * \param typeNameIt a Iterator<TypeName>
   * \param aliasTable the alias table to use to instantiate this set
   * \return a Set<TypeName> of instantiated TypeName
   */
  private Set instantiateTypeNameSet(Iterator typeNameIt, AliasTable aliasTable) throws GenericityException
  {
    Set result = new HashSet();
    while (typeNameIt.hasNext())
    {
      TypeName typeName = (TypeName)typeNameIt.next();
      typeName = instantiateTypeName(typeName, aliasTable);
      result.add(typeName);
    }

    return result;
  }

  /**
   * \brief instantiates a set of extends TypeName
   *
   * \param extendsIt Iterator<TypeName>
   * \param aliasTable aliasTable to use to instantiate extends list
   * \return a Set<TypeName> of instantiated TypeName
   */
  private Set instantiateExtendsList(Iterator extendsIt, AliasTable aliasTable) throws GenericityException
  {
    return instantiateTypeNameSet(extendsIt, aliasTable);
  }

  /**
   * \brief instantiates a set of implements TypeName
   *
   * \param implementsIt a Iterator<TypeName>
   * \param aliasTable alias table to use to instantiate implementsList
   * \return a Set<TypeName> of instantiated TypeName
   */
  private Set instantiateImplementsList(Iterator implementsIt, AliasTable aliasTable) throws GenericityException
  {
    return instantiateTypeNameSet(implementsIt, aliasTable);
  }

  /**
   * \brief instantiates a Field
   *
   * \param field a possibly generic Field
   * \param aliasTable the aliasTable to use to instantiate Field
   * \param target the target typeName of field
   * \return an instantiated Field
   */
  private Field instantiateField(
    net.nosica.symbol.Field field,
    AliasTable aliasTable,
    TypeName target
  ) throws GenericityException
  {
    return new FieldImpl(
                         field,
                         instantiateTypeName(field.getTypeName(), aliasTable),
                         target
                         );
  }

  /**
   * \brief verify that associated generic parameters really implements their declared interface
   * a generic parameter must implements a declared interface to be valid, else, this is a compil time
   * error, and this method will throw
   *
   * \param genericAssociation a Map<TypeName, TypeName> such as the one returned by method associate
   * \param genericDeclaration a ArrayList<GenericParameter>
   */
  private void verifyGenericImplements(Map genericAssociation, ArrayList genericDeclaration) throws GenericityException
  {
    Iterator it = genericDeclaration.iterator();

    while (it.hasNext())
    {
      GenericParameter genericDecl = (GenericParameter)it.next();
      TypeName genericName = genericDecl.getName();
      TypeName interfaceToImplement = genericDecl.getImplements();

      if (false)  // @TODO@ : implements test using compil time type information
        throw new GenericityException("Generic parameter [" + genericName + "] does not implements interface [" + interfaceToImplement + "]");
    }
  }

  private Map instantiateMethodAliasTable(Map methodAliasTable, AliasTable aliasTable) throws GenericityException
  {
    Map result = new HashMap();

    Iterator it = methodAliasTable.entrySet().iterator();
    while (it.hasNext())
    {
      Map.Entry entry = (Map.Entry)it.next();

      String alias = (String)entry.getKey();
      TypeName typeName = (TypeName)entry.getValue();
      TypeName type = typeName.getFirstComponents();
      type = instantiateTypeName(type, aliasTable);

      ArrayList components = new ArrayList();
      components.addAll(type.getComponents());
      components.add(typeName.getLastComponent());

      result.put(alias, new TypeNameImpl(components));
    }

    return result;
  }
}