/*
  $Id: IRRecordEnvironment.java,v 1.6 2004/03/04 22:16:16 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.compiler.application;

import net.nosica.parser.node.util.var.TypeName;
import net.nosica.symbol.SymbolException;
import net.nosica.compiler.intermediate.Record;
import net.nosica.compiler.typechecker.Type;

import java.util.Iterator;

/**
 * \brief repository for all IR types
 * At this points, types are no longer types.<br>
 * They are just simple record (as in pascal, or struct in C)
 */
public interface IRRecordEnvironment
{
  /**
   * \brief does record named typeName was already registered ?
   *
   * \param typeName name of record to search for
   * \return true if record exists
   */
  boolean recordExists(TypeName typeName);

  /**
   * \brief add a record to repository
   *
   * \param record record to add to repository
   * \throw SymbolException if record already exists in repository
   */
  void addRecord(Record record) throws SymbolException;

  /**
   * \brief get a record from repository
   *
   * \param typeName name of record to retrieve
   * \return the record
   * \throw SymbolException if record doesn't exist in repository
   */
  Record getRecord(TypeName typeName) throws SymbolException;

  /**
   * \brief return an iterator over all records
   *
   * \return Iterator<Record>
   */
  Iterator iterateRecords();

   /**
   * \brief translate a Type into a Record suitable for code generation
   *
   * \param type the type to translate
   * \return translated record
   */
  Record translateType(Type type);

  /**
   * \brief return the number of record in environment
   *
   * \return number of record
   */
  int nbRecord();
}