/*
  $Id: IRRecordEnvironmentDisplayer.java,v 1.8 2004/03/04 22:16:16 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.compiler.application;

import net.nosica.compiler.intermediate.Record;
import net.nosica.compiler.intermediate.Field;
import net.nosica.util.IndentedPrintWriter;

import java.util.Iterator;

/**
 * \brief a class to display content of IRRecordEnvironment
 *
 * Main purpose of class is debugging
 */
public class IRRecordEnvironmentDisplayer
{
  private IndentedPrintWriter writer;
  private Environment env = Environment.instance();

  /**
   * \brief creates a new displayer
   *
   * \param writer an IndentedPrintWriter
   */
  public IRRecordEnvironmentDisplayer(IndentedPrintWriter writer) { this.writer = writer; }

  public void display()
  {
    Iterator i = env.getIRRE().iterateRecords();
    while (i.hasNext())
    {
      Record record = (Record)i.next();

      display(record);
    }
  }

  public void display(Record record)
  {
    writer.print("-" + record.getName());
    if (record.hasExtends())
      writer.print(" extends " + record.getExtends());
    writer.println();

    writer.addIndent();
    Iterator j = record.iterateFields();
    while (j.hasNext())
    {
      Field field = (Field)j.next();
      if (field.isStatic())
        writer.print("static ");
      writer.println(field.getTypeName() + " " + field.getName());
    }
    writer.decIndent();

    writer.println();
  }
}