/*
  $Id: IRRecordEnvironmentImpl.java,v 1.8 2004/03/04 22:16:16 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.compiler.application;

import net.nosica.parser.node.util.var.TypeName;
import net.nosica.symbol.SymbolException;
import net.nosica.compiler.intermediate.Record;
import net.nosica.compiler.intermediate.RecordImpl;
import net.nosica.compiler.intermediate.Field;
import net.nosica.compiler.intermediate.FieldImpl;
import net.nosica.compiler.typechecker.Type;

import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;
import java.util.ArrayList;

public class IRRecordEnvironmentImpl implements IRRecordEnvironment
{
  private static IRRecordEnvironmentImpl theInstance = new IRRecordEnvironmentImpl();
  private IRRecordEnvironmentImpl() {}
  public static IRRecordEnvironmentImpl instance() { return theInstance; }
  public static void instanceReset() { theInstance = new IRRecordEnvironmentImpl(); }

  private Map records = new HashMap();
  private int uniqueTypeCode = 0;

  public int nbRecord() { return records.size(); }
  public boolean recordExists(TypeName typeName) { return records.containsKey(typeName); }
  public void addRecord(Record record) throws SymbolException
  {
    if (records.containsKey(record.getName()))
      throw new SymbolException("Record [" + record.getName() + "] already exists");
    records.put(record.getName(), record);
  }
  public Record getRecord(TypeName typeName) throws SymbolException
  {
    if (!records.containsKey(typeName))
      throw new SymbolException("Record [" + typeName + "] does not exists");
    return (Record)records.get(typeName);
  }
  public Iterator iterateRecords() { return records.values().iterator(); }

  public Record translateType(Type type)
  {
    Iterator it = type.iterateExtends();

    TypeName extendsTypeName = null;
    if (it.hasNext())
      extendsTypeName = (TypeName)it.next();

    ArrayList fields = new ArrayList();

    if (extendsTypeName != null)
    {
      Field field =
        new FieldImpl(
          false,
          extendsTypeName,         // well, super is access to super type, but instance is really sub type
          "super",
          type.getName()
        );

      fields.add(field);
    }

    TypeName enclosingTypeName = null;

    if (type.isInner())
    {
      enclosingTypeName = type.getEnclosing();

      Field field =
        new FieldImpl(
          false,
          enclosingTypeName,
          "___enclosing",
          type.getName()
        );

      fields.add(field);
    }

    it = type.iterateFields();
    while (it.hasNext())
    {
      net.nosica.compiler.typechecker.Field field =
        (net.nosica.compiler.typechecker.Field)it.next();

      fields.add(
        new FieldImpl(
          field.isStatic(),
          field.getTypeName(),
          field.getName(),
          type.getName()
        )
      );
    }

    return new RecordImpl(type.getName(), extendsTypeName, enclosingTypeName, fields, ++uniqueTypeCode);
  }
}
