/*
  $Id: LabelRepository.java,v 1.8 2004/03/04 22:16:16 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.compiler.application;

import net.nosica.compiler.typechecker.Label;
import net.nosica.compiler.typechecker.TypeCheckerException;

import java.util.ArrayList;

/**
 * \brief deals with label generated by a method
 */
public interface LabelRepository
{
  /**
   * \brief enter a scope : save the labels state
   */
  void save();

  /**
   * \brief leave a scope : restore the labels state
   */
  ArrayList restore();

  /**
   * \brief return the number of active scope
   *
   * \return number of active scope
   */
  int getNumberOfActiveScope();

  /**
   * \brief return the labels accessible from current scope up to nbScope scope
   *
   * \param nbScope the number of scope to examine (must be <= getNumberOfActiveScope)
   * \return ArrayList<Label>
   * \throws TypeCheckerException if nbScope > getNumberOfActiveScope
   */
  ArrayList labelInScope(int nbScope) throws TypeCheckerException;

  /**
   * \brief add a new label to the repository
   *
   * \param label the new label to add to repository
   * \throws TypeCheckerException if label shadow another label's name
   */
  void add(Label label) throws TypeCheckerException;

  /**
   * \brief return a label by name
   *
   * \param name the name of the label to retrieve
   * \return the label, or null if it does not exist
   */
  Label get(String name);

  /**
   * \brief generate a new unique label
   *
   * \return a new label
   */
  Label generateLabel();
}