/*
  $Id: LabelRepositoryImpl.java,v 1.9 2004/03/04 22:16:16 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.compiler.application;

import net.nosica.compiler.typechecker.TypeCheckerException;
import net.nosica.compiler.typechecker.Label;
import net.nosica.compiler.typechecker.LabelImpl;

import java.util.Map;
import java.util.HashMap;
import java.util.Stack;
import java.util.ArrayList;

public class LabelRepositoryImpl implements LabelRepository
{
  // generation of labels
  private int uid = 0;
  private String seedString;

  // scope stuff
  private Map labels      = new HashMap();  // Map<String, Label>
  private Stack scopes    = new Stack();    // Stack<Integer>
  private Stack undoStack = new Stack();    // Stack<String>
  private int nbModificationsSinceLastSave = 0;

  public LabelRepositoryImpl(String seedString)
  {
    this.seedString = seedString;
  }

  public void save()
  {
    scopes.push(new Integer(nbModificationsSinceLastSave));
    nbModificationsSinceLastSave = 0;
  }

  public ArrayList restore()
  {
    ArrayList removedLabels = new ArrayList();

    for (int i = 0; i < nbModificationsSinceLastSave; ++i)
    {
      String labelName = (String)undoStack.pop();
      removedLabels.add(labels.remove(labelName));
    }

    nbModificationsSinceLastSave = ((Integer)scopes.pop()).intValue();

    return removedLabels;
  }

  public int getNumberOfActiveScope()
  {
    return scopes.size();
  }

  public ArrayList labelInScope(int nbScope) throws TypeCheckerException
  {
    ArrayList removedLabels = new ArrayList();
    int tmpNbMod = nbModificationsSinceLastSave;
    int index = 0;

    if (nbScope > scopes.size())
      throw new TypeCheckerException("Too much scope asked : asked [" + nbScope + "], got [" + scopes.size() + "] at most");

    for (int i = 0; i < nbScope; ++i)
    {
      for (int j = 0; j < tmpNbMod; ++j, ++index)
      {
        String labelName = (String)undoStack.get(undoStack.size() - 1 - index);
        removedLabels.add(labels.get(labelName));
      }

      tmpNbMod = ((Integer)scopes.get(scopes.size() - 1 - i)).intValue();
    }

    return removedLabels;
  }

  public void add(Label label) throws TypeCheckerException
  {
    // check duplicate names
    if (labels.containsKey(label.getName()))
      throw new TypeCheckerException("A label with same name [" + label.getName() + "] is already active");

    ++nbModificationsSinceLastSave;

    undoStack.push(label.getName());
    labels.put(label.getName(), label);
  }

  public Label get(String name)
  {
    return (Label)labels.get(name);
  }

  public Label generateLabel()
  {
    String labelName = seedString + uid++;

    return new LabelImpl(labelName);
  }
}