/*
  $Id: PTNEnvironment.java,v 1.11 2004/03/04 22:16:16 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.compiler.application;

import net.nosica.compiler.intermediate.Expression;
import net.nosica.compiler.intermediate.Function;
import net.nosica.compiler.intermediate.Field;
import net.nosica.compiler.intermediate.Call;

import net.nosica.compiler.analysis.PossibleTypeName;

import java.util.Set;
import java.util.Map;

/**
 * \brief PossibleTypeName environment for all IR expression
 *
 * Repository for dynamic analysis
 */
public interface PTNEnvironment
{
  /**
   * \brief returns the ptn associated with expression
   *
   * if no ptn previously existed, a new one is created and bound with it
   *
   * \param exp the expression bound to ptn
   * \return ptn associated with exp
   */
  PossibleTypeName getPTN(Expression exp);

  /**
   * \brief returns the ptn associated with field
   *
   * if no ptn previously existed, a new one is created and bound with it
   *
   * \param f the field bound to ptn
   * \return ptn associated with f
   */
  PossibleTypeName getPTN(Field f);

  /**
   * \brief returns the ptn associated with function (the result type)
   *
   * if no ptn previously existed, a new one is created and bound with it
   *
   * \param f the function bound to ptn
   * \return ptn associated with f
   */
  PossibleTypeName getPTN(Function f);

  /**
   * \brief tells wether expression has an assigned field array source
   *
   * \param exp the expression to look for
   * \return true if exp has a field array source
   */
  boolean hasFieldArraySource(Expression exp);

  /**
   * \brief returns the list of array field that are possibly sources of exp
   *
   * if no ptn previously existed, a new one is created and bound with it
   *
   * \param exp the expression to search for
   * \return Set<Field> as the list of field that can be rhs of a move/init (and exp as lhs)
   */
  Set getFieldArraySource(Expression exp);

  /**
   * \brief tells wether function result has an assigned field array source
   *
   * \param f the function to look for
   * \return true if f has a field array source
   */
  boolean hasFieldArraySource(Function f);

  /**
   * \brief returns the list of array field that are possibly sources of f result
   *
   * if no ptn previously existed, a new one is created and bound with it
   *
   * \param f the function to search for
   * \return Set<Field> as the list of field that can be returned by f
   */
  Set getFieldArraySource(Function f);

  /**
   * \brief return resolved method for this call
   *
   * \param call the call as the key from which to return resolved method
   * \return resolved methods as a Map<TypeName, Function>
   */
  Map getResolvedMethod(Call call);
}