/*
  $Id: PTNEnvironmentImpl.java,v 1.13 2004/03/04 22:16:16 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.compiler.application;

import net.nosica.compiler.intermediate.Expression;
import net.nosica.compiler.intermediate.Function;
import net.nosica.compiler.intermediate.Field;
import net.nosica.compiler.intermediate.Call;

import net.nosica.compiler.analysis.PossibleTypeName;
import net.nosica.compiler.analysis.PossibleTypeNameImpl;

import java.util.Set;
import java.util.HashSet;
import java.util.Map;
import java.util.HashMap;

public class PTNEnvironmentImpl implements PTNEnvironment
{
  static private PTNEnvironmentImpl instance = new PTNEnvironmentImpl();
  private PTNEnvironmentImpl() {}
  static public PTNEnvironmentImpl instance() { return instance; }

  private Map expRepository = new HashMap();    // Map<Expression, PossibleTypeName>
  private Map funcRepository = new HashMap();   // Map<Function, PossibleTypeName>
  private Map fieldRepository = new HashMap();  // Map<Field, PossibleTypeName>

  private Map expFasRepository = new HashMap(); // Map<Expression, Set<Field> > FieldArraySource
  private Map fFasRepository = new HashMap();   // Map<Function, Set<Field> > FieldArraySource

  // Map<Call, Map<TypeName, Function> >
  private Map resolvedMethodRepository = new HashMap();

  public PossibleTypeName getPTN(Expression exp)
  {
    PossibleTypeName ptn = (PossibleTypeName)expRepository.get(exp);

    if (ptn == null)
    {
      ptn = PossibleTypeNameImpl.createList(exp.getTypeName());

      expRepository.put(exp, ptn);
    }

    return ptn;
  }

  public PossibleTypeName getPTN(Field f)
  {
    PossibleTypeName ptn = (PossibleTypeName)fieldRepository.get(f);

    if (ptn == null)
    {
      ptn = PossibleTypeNameImpl.createList(f.getTypeName());
      if (ptn.isPrimitive())
      {
        PossibleTypeName primitivePtn = PossibleTypeNameImpl.createScalar(ptn.getTypeName());
        ptn.merge(primitivePtn);
      }

      fieldRepository.put(f, ptn);
    }

    return ptn;
  }

  public PossibleTypeName getPTN(Function f)
  {
    PossibleTypeName ptn = (PossibleTypeName)funcRepository.get(f);

    if (ptn == null)
    {
      ptn = PossibleTypeNameImpl.createList(f.getMethod().getResultType().getTypeName());

      funcRepository.put(f, ptn);
    }

    return ptn;
  }

  public boolean hasFieldArraySource(Expression exp)
  {
    return expFasRepository.containsKey(exp);
  }

  public Set getFieldArraySource(Expression exp)
  {
    Set expFas = (Set)expFasRepository.get(exp);

    if (expFas == null)
    {
      expFas = new HashSet();

      expFasRepository.put(exp, expFas);
    }

    return expFas;
  }

  public boolean hasFieldArraySource(Function f)
  {
    return fFasRepository.containsKey(f);
  }

  public Set getFieldArraySource(Function f)
  {
    Set fFas = (Set)fFasRepository.get(f);

    if (fFas == null)
    {
      fFas = new HashSet();

      fFasRepository.put(f, fFas);
    }

    return fFas;
  }

  public Map getResolvedMethod(Call call)
  {
    Map resolvedMethods = (Map)resolvedMethodRepository.get(call);

    if (resolvedMethods == null)
    {
      resolvedMethods = new HashMap();

      resolvedMethodRepository.put(call, resolvedMethods);
    }

    return resolvedMethods;
  }
}