/*
  $Id: TypeEnvironment.java,v 1.6 2004/03/04 22:16:16 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.compiler.application;

import net.nosica.parser.node.util.var.TypeName;
import net.nosica.compiler.typechecker.Type;
import net.nosica.compiler.typechecker.Method;
import net.nosica.symbol.SymbolException;

import java.util.Iterator;
import java.util.SortedSet;

/**
 * \brief repository for types
 * This is the place where one can dispose Types.
 */

public interface TypeEnvironment
{
  /**
   * \brief Get a type by it's TypeName
   *
   * \param typeName complete type name of type to retrieve
   * \return the type named typeName
   */
  Type getType(TypeName typeName) throws SymbolException;

  /**
   * \brief Add a type
   *
   * \param type the type to add
   */
  void addType(Type type) throws SymbolException;

  /**
   * \brief Does this type exists in repository ?
   *
   * \param typeName the name of the type to look for
   * \return true if type exists
   */
  boolean typeExists(TypeName typeName);

  /**
   * \brief return an iterator on all type name
   *
   * \return Iterator<TypeName>
   */
  Iterator iterateTypeName();

  /**
   * \brief explores inheritance tree to see if son derives from parent
   *
   * \param son the derived typename
   * \param parent the father typeName
   * \return true if it exists a chain of inheritance from son to parent
   */
  boolean doesInherits(TypeName son, TypeName parent) throws SymbolException;

  /**
   * \brief returns all parents of this typeName sorted by ineritance order
   * direct parents are placed first, Object is placed last
   *
   * \param typeName name of type to get parents
   * \return SortedSet<Pair<Integer, TypeName> >
   */
  SortedSet getParents(TypeName typeName) throws SymbolException, GenericityException;

  /**
   * \brief returns the nearest parent of two types
   * It will ultimately be Object or General
   *
   * \param type1 first type name
   * \param type2 second type name
   * \return nearest type name of type1 and type2
   */
  TypeName getNearestParent(TypeName type1, TypeName type2) throws SymbolException, GenericityException;

  /**
   * \brief looks if method m can implement method signature
   *
   * \param m the method to validate
   * \param signature the signature to validate against
   * \param compareName if set to true, name of method will be compared in test
   * \return true if m implements signature, false otherwise
   */
  boolean doesImplements(Method m, Method signature, boolean compareName) throws SymbolException;
}