/*
  $Id: TypeLoader.java,v 1.8 2004/03/04 22:16:16 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.compiler.application;

import net.nosica.parser.node.util.var.TypeName;
import net.nosica.parser.node.typedeclaration.ClassBody;
import net.nosica.compiler.typechecker.Type;
import net.nosica.symbol.SymbolException;

/**
 * \brief load type
 * That is, import symbols, resolve genericity and instantiates Type.
 * When the process is over, you have a full Type ready for type checker.
 */

public interface TypeLoader
{
  /**
   * \brief Loads a type if it does not exists
   * The type is first looked in the TypeEnvironment<BR>
   * If it is not found, is is looked in the SymbolEnvironment<BR>
   * If it not found, this is an error<BR>
   * Else, GenericType is retrieved, instanciated using the GenericitySolver<BR>
   * finally, type is returned
   *
   * \param typeName name of the type to retrieve
   * \return the type if it exists
   */
  Type getType(TypeName typeName) throws SymbolException, GenericityException;

  /**
   * \brief Loads a type from a ClassBody
   * This is used to parse anonymous classes
   *
   * \param enclosing name of enclosing type
   * \param interfaceName interface name and eventually generic parameter of this anonymous class
   * \param classBody parser node of anonymous class
   * \return the type if it was loaded successfully
   */
  Type getType(TypeName enclosing, TypeName interfaceName, ClassBody classBody) throws SymbolException, GenericityException;

  /**
   * \brief Loads symbols from a TypeName
   *
   * \param fullTypeName full type name
   * \note as a side effect SymbolEnvironment now contains symbols of fullTypeName or an exception is thrown
   * \throws SymbolException if fullTypeName does not exists on disk
   */
  void loadSymbols(TypeName fullTypeName) throws SymbolException;

  /**
   * \brief looks in repositories to see if a particular type exists
   *
   * \param typeName name of the type to retrieve
   * \return true if type exists
   */
  boolean typeExists(TypeName typeName);

  /**
   * \brief load all type needed by type to get instantiated
   * This method loads all types needed by method an field encountered in type
   * Beware that a check is done to ensure that native type only use native type
   * During loading all non primitive type are converted to Referenced type
   *
   * \param type the type to load
   */
  void loadDepends(Type type) throws SymbolException, GenericityException;

  /**
   * \brief verify that type implements all its interfaces
   *
   * \param type the type to verify
   */
  void verifyInterface(Type type);

  /**
   * \brief verify integrity of type
   *
   * \param type the type to verify
   */
  void verifyType(Type type);
}