/*
  $Id: TypeLoaderImpl.java,v 1.31 2004/04/01 21:27:55 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.compiler.application;

import net.nosica.parser.node.util.var.TypeName;
import net.nosica.parser.node.util.var.TypeNameImpl;
import net.nosica.parser.node.util.var.TypeNameComponent;
import net.nosica.parser.node.util.var.TypeNameComponentImpl;
import net.nosica.parser.node.util.var.TypeNameListImpl;
import net.nosica.parser.node.util.generic.GenericParameter;
import net.nosica.parser.node.util.generic.GenericParameterImpl;
import net.nosica.parser.node.typedeclaration.ClassDeclaration;
import net.nosica.parser.node.typedeclaration.ClassDeclarationImpl;
import net.nosica.parser.node.typedeclaration.NestedClassDeclaration;
import net.nosica.parser.node.typedeclaration.NestedClassDeclarationImpl;
import net.nosica.parser.node.typedeclaration.DestructorDeclarationImpl;
import net.nosica.parser.node.typedeclaration.ClassBody;
import net.nosica.parser.node.typedeclaration.statement.BlockImpl;
import net.nosica.parser.NosicaParser;
import net.nosica.parser.node.filestructure.CompilationUnit;

import net.nosica.symbol.FeedSymbolVisitor;
import net.nosica.symbol.SymbolException;
import net.nosica.symbol.AliasTable;
import net.nosica.symbol.GenericType;
import net.nosica.symbol.GenericTypeImpl;
import net.nosica.symbol.GenericMethodImpl;
import net.nosica.symbol.MethodKindEnum;

import net.nosica.compiler.symbol.CompilerSymbolCallBackImpl;

import net.nosica.application.*;
import net.nosica.application.NativeTypes;
import net.nosica.application.PropertyException;
import net.nosica.application.MessageLevel;
import net.nosica.compiler.typechecker.Type;
import net.nosica.compiler.typechecker.TypeImpl;
import net.nosica.compiler.typechecker.Field;
import net.nosica.compiler.typechecker.Method;
import net.nosica.compiler.typechecker.MethodImpl;
import net.nosica.compiler.typechecker.Argument;
import net.nosica.compiler.typechecker.ArgumentImpl;
import net.nosica.compiler.typechecker.ResultType;
import net.nosica.compiler.typechecker.ResultTypeImpl;

import net.nosica.util.IndentedPrintWriter;

import java.util.Iterator;
import java.util.ArrayList;
import java.util.Map;
import java.util.HashMap;
import java.util.Set;
import java.util.HashSet;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;

public class TypeLoaderImpl implements TypeLoader
{
  private static TypeLoaderImpl theInstance = new TypeLoaderImpl();
  public static TypeLoaderImpl instance() { return theInstance; }
  public static void instanceReset() { theInstance = new TypeLoaderImpl(); }

  private static NosicaParser nosicaParser = null;
  private int anonymousId = 0;
  private Environment env = Environment.instance();

  private boolean debugSymbol = false;
  private boolean debugType = false;
  private String debugSymbolOfType;
  private String debugTypeOfType;

  private TypeLoaderImpl()
  {
    try
    {
      debugSymbol = env.getAC().getPropertyBoolean("debug.symbol");
    }
    catch (PropertyException e)
    {
    }

    try
    {
      debugSymbolOfType = env.getAC().getPropertyString("debug.symbol.type");
    }
    catch (PropertyException e)
    {
    }

    try
    {
      debugType = env.getAC().getPropertyBoolean("debug.type");
    }
    catch (PropertyException e)
    {
    }

    try
    {
      debugTypeOfType = env.getAC().getPropertyString("debug.type.type");
    }
    catch (PropertyException e)
    {
    }
  }

  public Type getType(TypeName typeName) throws SymbolException, GenericityException
  {
    // if type was already loaded
    if (env.getTE().typeExists(typeName))
      return env.getTE().getType(typeName);

    // no ! if array, intercept call
    if (typeName.isArray())
    {
      // force loading of net.nosica.lang.Array
      getType(NativeTypes.get_net_nosica_lang_Array());
      Type type = getType(typeName.getArrayElementType());
      type = createArrayType(typeName, type.getAliasTable());
      env.getTE().addType(type);
      loadDepends(type);

      return type;
    }

    // type is not an array !
    // load symbols if not previously loaded
    if (!env.getSE().genericTypeExists(typeName))
      loadSymbols(typeName);

    // get model
    Map genericAssociation = new HashMap();
    GenericType genericType = env.getSE().getBestSuitableGenericType(typeName, genericAssociation);

    if (debugSymbol)
    {
      SymbolEnvironmentDisplayer displayer = new SymbolEnvironmentDisplayer(
        new IndentedPrintWriter(System.out)
      );

      displayer.display(genericType);
    }
    else
    if (debugSymbolOfType != null)
    {
      if (genericType.getName().toString().equals(debugSymbolOfType))
      {
        SymbolEnvironmentDisplayer displayer = new SymbolEnvironmentDisplayer(
          new IndentedPrintWriter(System.out)
        );

        displayer.display(genericType);
      }
    }

    // and instantiate it !
    GenericitySolver genericitySolver = env.getGS();
    Type type = genericitySolver.instantiateType(genericType, genericAssociation, typeName);

    if (debugType)
    {
      TypeEnvironmentDisplayer displayer = new TypeEnvironmentDisplayer(
        new IndentedPrintWriter(System.out)
      );

      displayer.display(type);
    }
    else
    if (debugTypeOfType != null)
    {
      if (type.getName().toString().equals(debugTypeOfType))
      {
        TypeEnvironmentDisplayer displayer = new TypeEnvironmentDisplayer(
          new IndentedPrintWriter(System.out)
        );

        displayer.display(type);
      }
    }

    // add it to Type Environment
    env.getTE().addType(type);

    // load all types from which depends type
    loadDepends(type);

    // verify it implements its interface
    if (type.isClass() && !type.isAbstract())
      verifyInterface(type);

    // verify some other details
    verifyType(type);

    return type;
  }

  public Type getType(TypeName enclosing, TypeName interfaceName, ClassBody classBody) throws SymbolException, GenericityException
  {
    TypeName typeName = loadSymbols(enclosing, interfaceName, classBody);

    // get model
    Map genericAssociation = new HashMap();
    GenericType genericType = env.getSE().getBestSuitableGenericType(typeName, genericAssociation);

    if (debugSymbol)
    {
      SymbolEnvironmentDisplayer displayer = new SymbolEnvironmentDisplayer(
        new IndentedPrintWriter(System.out)
      );

      displayer.display(genericType);
    }

    // and instantiate it !
    GenericitySolver genericitySolver = env.getGS();
    Type type = genericitySolver.instantiateType(genericType, genericAssociation, typeName);

    if (debugType)
    {
      TypeEnvironmentDisplayer displayer = new TypeEnvironmentDisplayer(
        new IndentedPrintWriter(System.out)
      );

      displayer.display(type);
    }

    // add it to Type Environment
    env.getTE().addType(type);

    // load all types from which depends type
    loadDepends(type);

    // verify it implements its interface
    if (type.isClass() && !type.isAbstract())
      verifyInterface(type);

    // verify some other details
    verifyType(type);

    return type;
  }

  public void loadSymbols(TypeName fullTypeName) throws SymbolException
  {
    try
    {
      Iterator it = env.getFL().findFiles(fullTypeName).iterator();

      if (!it.hasNext())
        throw new SymbolException("Could not load type [" + fullTypeName + "]");

      while (it.hasNext())
      {
        File file = (File)it.next();

        if (file.isDirectory())
          continue;

        if (nosicaParser == null)
        {
          nosicaParser = new NosicaParser(new FileInputStream(file));
          NosicaParser.messageService = env.getMS();
        }
        else
          NosicaParser.ReInit(new FileInputStream(file));

        NosicaParser.setCurrentFile(file);

        try
        {
          CompilationUnit compilUnit = NosicaParser.CompilationUnit();
          FeedSymbolVisitor visitor = new FeedSymbolVisitor(
            env.getFL(),
            new CompilerSymbolCallBackImpl(env.getMS(), fullTypeName)
          );
          compilUnit.accept(visitor);

          Map reconstructedTypeName = new HashMap(); // Map<TypeName, TypeName>

          // iterate over extracted type to resolve
          // their generic parameter when possible and their typeName
          Iterator itTypes = visitor.getTypes().iterator();
          while (itTypes.hasNext())
          {
            GenericType type = (GenericType)itTypes.next();
            AliasTable aliasTable = type.getAliasTable();

            resolveGenericParameters(type, aliasTable, reconstructedTypeName);
          }
        }
        catch (net.nosica.parser.ParseException e)
        {
          env.getMS().report(MessageLevel.ERR, "Parse", new String[]{file.getAbsolutePath() , e.getMessage()}, null);

          throw new SymbolException();
        }
        catch (net.nosica.parser.node.EOFParseException e)
        {
          env.getMS().report(MessageLevel.ERR, "Parse", new String[]{file.getAbsolutePath(), "EOF encountered while parsing nested comment"}, null);

          throw new SymbolException();
        }
      }
    }
    catch (FileNotFoundException e)
    {
      throw new SymbolException(e.getMessage());
    }
  }

  private TypeName loadSymbols(TypeName enclosing, TypeName interfaceName, ClassBody classBody) throws SymbolException
  {
    // create name of anonymous class
    ArrayList anonymousComponents = new ArrayList();
    anonymousComponents.addAll(enclosing.getComponents());
    anonymousComponents.add(
      new TypeNameComponentImpl(
        new Integer(++anonymousId).toString(),
        interfaceName.getLastComponent().getGenericExtension()
      )
    );

    TypeName anonymousTypeName = new TypeNameImpl(anonymousComponents);

    // create a fake NestedClassDeclaration
    ArrayList implementList = new ArrayList();
    implementList.add(interfaceName);

    ClassDeclaration classDecl = new ClassDeclarationImpl(
      anonymousTypeName.getLastComponent().getPackageElement(),
      null,
      null,
      new TypeNameListImpl(implementList),
      classBody
    );

    classDecl.setPrivate(true);
    classDecl.setInner(true);

    NestedClassDeclaration nestedClassDecl = new NestedClassDeclarationImpl(
      classDecl
    );

    // now feed symbols from ClassDeclaration
    FeedSymbolVisitor visitor = new FeedSymbolVisitor(
      env.getFL(),
      new CompilerSymbolCallBackImpl(env.getMS(), anonymousTypeName),
      env.getTE().getType(enclosing).getModel()
    );

    nestedClassDecl.accept(visitor);
    if (env.getMS().getNbError() > 0)
      env.terminate("-- Correct above errors");

    Map reconstructedTypeName = new HashMap(); // Map<TypeName, TypeName>

    // iterate over extracted type to resolve
    // their generic parameter when possible and their typeName
    Iterator itTypes = visitor.getTypes().iterator();
    while (itTypes.hasNext())
    {
      GenericType type = (GenericType)itTypes.next();
      AliasTable aliasTable = type.getAliasTable();

      resolveGenericParameters(type, aliasTable, reconstructedTypeName);
    }

    return anonymousTypeName;
  }

  private void resolveGenericParameters(GenericType type, AliasTable aliasTable, Map reconstructedTypeName) throws SymbolException
  {
    TypeName typeName = type.getName();

    ArrayList genericParameters = type.getGenericParameters();
    Set partialNames = new HashSet();

    // first iteration to add generic parameter to alias table
    Iterator itGenericParms = genericParameters.iterator();
    while (itGenericParms.hasNext())
    {
      GenericParameter genericParm = (GenericParameter)itGenericParms.next();

      TypeName name = genericParm.getName();
      // try to resolve it
      try
      {
        name = env.getTR().resolve(name, aliasTable, env.getFL());
        // resolver has resolved it !
        // this has to be a partial generic name (such as class Toto<net.nosica.lang.int32>)

        // verify classes exist on disk
        if (type.isNested() && env.getFL().findFiles(name).isEmpty())
        {
          // well in fact, this has to be a generic parameter defined in parent
          // let's verify it :
          try
          {
            aliasTable.replaceAlias(name, name);
          }
          catch (SymbolException e)
          {
            // this should not throw
            env.terminate("Internal error : " + e.getMessage());
          }

          env.getMS().report(MessageLevel.ERR, "Symbol036", name.toString(), genericParm);
        }
        else
          partialNames.add(name);
      }
      catch (SymbolException e)
      {
        // resolver could not resolve it
        // this has to be a full generic parameter (such as class Toto<T>)
        // add an alias from generic parameter to himself
        aliasTable.addAlias(name, name);
      }
    }

    // now that we have addded all generic parameters that could be added,
    // we should be able to resolve any generic parameter
    // constructs list of resolved generic parameter
    ArrayList resolvedGenericParameter = new ArrayList();
    itGenericParms = genericParameters.iterator();
    while (itGenericParms.hasNext())
    {
      GenericParameter genericParm = (GenericParameter)itGenericParms.next();

      TypeName name = genericParm.getName();
      TypeName implement = genericParm.getImplements();
      TypeName defaultName = genericParm.getDefaultType();

      name = env.getTR().resolve(name, aliasTable, env.getFL());
      if (genericParm.hasImplements())
        implement = env.getTR().resolve(implement, aliasTable, env.getFL());
      else
        implement = null;
      if (genericParm.hasDefaultType())
        defaultName = env.getTR().resolve(defaultName, aliasTable, env.getFL());
      else
        defaultName = null;

      resolvedGenericParameter.add(new GenericParameterImpl(name, implement, defaultName));
    }

    // constructs real type name
    typeName = constructRealTypeName(
      typeName,
      (TypeName)(type.isNested() ? reconstructedTypeName.get(type.getEnclosing()) : null),
      resolvedGenericParameter
    );

    // for example, Test -> Test<T>
    reconstructedTypeName.put(type.getName(), typeName);

    // set name
    type.setName(typeName);

    // now it is time to definitly remove partial name from generic parameter
    // because they are not generic anymore
    ArrayList nonPartialResolvedGenericParameter = null;
    if (partialNames.isEmpty())
      nonPartialResolvedGenericParameter = resolvedGenericParameter;
    else
    {
      nonPartialResolvedGenericParameter = new ArrayList();
      itGenericParms = resolvedGenericParameter.iterator();
      while (itGenericParms.hasNext())
      {
        GenericParameter genericParm = (GenericParameter)itGenericParms.next();
        if (!partialNames.contains(genericParm.getName()))
          nonPartialResolvedGenericParameter.add(genericParm);
      }

      type.setPartial(true);
    }

    type.setGenericParameters(nonPartialResolvedGenericParameter);

    try
    {
      env.getSE().addGenericType(type);
    }
    catch (SymbolException e)
    {
      env.getMS().report(MessageLevel.ERR, "Symbol013", type.getName().toString(), null);
    }
  }

  private TypeName constructRealTypeName(
    TypeName fakeTypeName,
    TypeName enclosing,
    ArrayList genericParameterList
  )
  {
    if (enclosing != null)
    {
      for (int i = 0; i < enclosing.getComponents().size(); ++i)
        fakeTypeName.getComponents().set(i, enclosing.getComponents().get(i));
    }

    if (!genericParameterList.isEmpty())
    {
      // create real TypeName
      ArrayList components = new ArrayList();
      components.addAll(fakeTypeName.getComponents());

      TypeNameComponent lastComponent = (TypeNameComponent)components.get(components.size() - 1);
      ArrayList genericDeclaration = new ArrayList();
      Iterator it = genericParameterList.iterator();
      while (it.hasNext())
      {
        GenericParameter genericParm = (GenericParameter)it.next();
        genericDeclaration.add(genericParm.getName());
      }

      components.set(
        components.size() - 1,
        new TypeNameComponentImpl(lastComponent.getPackageElement(), genericDeclaration)
        );

      // recreate a new TypeName, to create a good cache
      fakeTypeName = new TypeNameImpl(components);
    }

    return fakeTypeName;
  }

  public boolean typeExists(TypeName typeName)
  {
    try
    {
      getType(typeName);

      return true;
    }
    catch (SymbolException e)
    {
    }
    catch (GenericityException e)
    {
    }

    return false;
  }

  public void loadDepends(Type type) throws SymbolException, GenericityException
  {

    Iterator itFields = type.iterateFields();

    // native types do not have any field
    if (type.isNative() && itFields.hasNext())
      env.getMS().report(MessageLevel.ERR, "Symbol033", new String[]{type.getName().toString()}, type.getModel().getNode());

    // iterate fields
    while (itFields.hasNext())
    {
      Field field = (Field)itFields.next();
      TypeName typeName = field.getTypeName();

      // load type of field
      getType(typeName);
    }

    // iterate cast operators
    Iterator itCastOperator = type.iterateCastOperators();
    while (itCastOperator.hasNext())
    {
      Method method = (Method)itCastOperator.next();

      loadDepends(method, type);
    }

    // iterate overloadables
    Iterator itOverloadables = type.iterateOverloadables();
    while (itOverloadables.hasNext())
    {
      Iterator itMethods = ((ArrayList)itOverloadables.next()).iterator();

      while (itMethods.hasNext())
      {
        Method method = (Method)itMethods.next();

        loadDepends(method, type);
      }
    }

    // iterate extends
    Iterator extendsIt = type.iterateExtends();
    while (extendsIt.hasNext())
    {
      TypeName extendsTypeName = (TypeName)extendsIt.next();
      getType(extendsTypeName);
    }

    // iterate implements
    Iterator implementsIt = type.iterateImplements();
    while (implementsIt.hasNext())
    {
      TypeName implementsTypeName = (TypeName)implementsIt.next();
      getType(implementsTypeName);
    }
  }

  public void verifyInterface(Type type)
  {
    try
    {
      // first get all methods in all interfaces of type
      Map interfaceMethods = new HashMap();  // Map<String, ArrayList<Method> >
      Iterator it = type.iterateImplements();
      while (it.hasNext())
      {
        TypeName implementsName = (TypeName)it.next();
        recCollectMethods(implementsName, interfaceMethods);
      }

      // then all methods implemented by type and all its super type
      Map implementedMethods = new HashMap(); // Map<String, ArrayList<Method> >
      recCollectMethods(type.getName(), implementedMethods);

      // now begin comparison.
      // first check alias methods
      it = type.getMethodAliasTable().entrySet().iterator();
      while (it.hasNext())
      {
        Map.Entry entry = (Map.Entry)it.next();
        String aliasName = (String)entry.getKey();
        TypeName alias = (TypeName)entry.getValue();

        TypeName methodInterfaceTypeName = alias.getFirstComponents();
        String methodInterfaceName = alias.getLastComponent().getPackageElement();

        // if no methods use alias, error !
        if (!implementedMethods.containsKey(aliasName))
        {
          env.getMS().report(MessageLevel.ERR, "TypeCheck007", aliasName, type.getModel().getNode());
          continue;
        }

        // retrieve those methods
        ArrayList methods = (ArrayList)implementedMethods.get(aliasName);

        // if no interface define this alias name, error !
        if (!interfaceMethods.containsKey(methodInterfaceName))
        {
          env.getMS().report(MessageLevel.ERR, "TypeCheck008", new String[]{aliasName, methodInterfaceName}, type.getModel().getNode());
          continue;
        }

        // retrieve those methods
        boolean found = false;
        Iterator j = ((ArrayList)interfaceMethods.get(methodInterfaceName)).iterator();
        while (j.hasNext())
        {
          Method m = (Method)j.next();

          // if alias typeName is method type name
          if (!m.getType().equals(methodInterfaceTypeName))
            continue;

          // find a method in implemented methods that match
          Iterator k = methods.iterator();
          while (k.hasNext())
          {
            Method m2 = (Method)k.next();

            if (env.getTE().doesImplements(m2, m, false))
            {
              // found ! BINGO ! Bravissimo !

              // additional checks :
              // protection and set list of inherited exceptions ...
              if (!m2.isPublic())
                env.getMS().report(MessageLevel.ERR, "TypeCheck011", m2.toString(), m.getModel().getNode());

              Set inherited = new HashSet();
              inherited.addAll(m2.getThrowList());
              inherited.addAll(m2.getInheritedException());

              m.setInheritedException(inherited);

              k.remove();
              j.remove();
              found = true;
              break;
            }
          }
        }

        if (!found)
          env.getMS().report(
            MessageLevel.ERR,
            "TypeCheck034",
            new String[]{methodInterfaceName, methodInterfaceTypeName.toString()},
            type.getModel().getNode()
          );
      }

      // now check all remaining methods ...
      it = interfaceMethods.entrySet().iterator();
      while (it.hasNext())
      {
        Map.Entry entry = (Map.Entry)it.next();
        String methodName = (String)entry.getKey();
        ArrayList methods = (ArrayList)entry.getValue();

        // can be empty because checking aliases remove some methods ...
        if (methods.isEmpty())
          continue;

        if (!implementedMethods.containsKey(methodName))
        {
          env.getMS().report(MessageLevel.ERR, "TypeCheck035", new String[]{type.getName().toString(), methodName}, type.getModel().getNode());
          continue;
        }

        ArrayList iMethods = (ArrayList)implementedMethods.get(methodName);

        Iterator j = methods.iterator();
        while (j.hasNext())
        {
          Method m = (Method)j.next();

          Iterator k = iMethods.iterator();
          boolean found = false;
          while (k.hasNext())
          {
            Method m2 = (Method)k.next();
            if (env.getTE().doesImplements(m2, m, true))
            {
              // additional checks :
              // protection and set list of inherited exceptions ...
              if (m.isPublic() && !m2.isPublic())
                env.getMS().report(MessageLevel.ERR, "TypeCheck011", m2.toString(), m.getModel().getNode());
              if (m.isProtected() && !m2.isProtected())
                env.getMS().report(MessageLevel.ERR, "TypeCheck012", m2.toString(), m.getModel().getNode());
              if (m.isPackage() && !m2.isPackage())
                env.getMS().report(MessageLevel.ERR, "TypeCheck013", m2.toString(), m.getModel().getNode());

              Set inherited = new HashSet();
              inherited.addAll(m2.getThrowList());
              inherited.addAll(m2.getInheritedException());

              m.setInheritedException(inherited);

              found = true;
              break;
            }
          }

          if (!found)
            env.getMS().report(MessageLevel.ERR, "TypeCheck009", new String[]{type.getName().toString(), m.getType().toString(), methodName}, type.getModel().getNode());
        }
      }
    }
    catch (SymbolException e)
    {
      env.terminate("Internal error : type should exist");
    }
  }

  private boolean verifyInheritance(
    TypeName typeName,
    Iterator iteratorTypeNames,
    boolean checkClass,
    String messageId,
    net.nosica.framework.Localizable localization
  )
  {
    boolean foundError = false;

    Iterator it = iteratorTypeNames;
    while (it.hasNext())
    {
      TypeName parentTypeName = (TypeName)it.next();

      try
      {
        Type parent = env.getTE().getType(parentTypeName);

        if (parent.isClass() != checkClass)
        {
          env.getMS().report(
            MessageLevel.ERR,
            messageId,
            new String[] {typeName.toString(), parentTypeName.toString()},
            localization
          );

          foundError = true;
        }
      }
      catch (SymbolException e)
      {
        env.terminate("Internal error : " + e.getMessage());
      }
    }

    return foundError;
  }

  public void verifyType(Type type)
  {
    boolean foundError = false;

    if (type.isClass())
    {
      // verify all extends are classes
      if (verifyInheritance(type.getName(), type.iterateExtends(), true, "TypeCheck038", type.getModel().getNode()))
        foundError = true;
      // verify all implements are interfaces
      if (verifyInheritance(type.getName(), type.iterateImplements(), false, "TypeCheck039", type.getModel().getNode()))
       foundError= true;
    }
    else
    {
      // verify all extends are interfaces
      if (verifyInheritance(type.getName(), type.iterateExtends(), false, "TypeCheck040", type.getModel().getNode()))
        foundError = true;
    }

    if (foundError)
      throw new TypeCheckerErrorExpressionException();
  }

  private void recCollectMethods(TypeName typeName, Map methods)
  {
    try
    {
      Type type = env.getTE().getType(typeName);

      Iterator it = type.iterateMethods();
      while (it.hasNext())
      {
        Method m = (Method)it.next();

        ArrayList methodsSameName = null;
        if (methods.containsKey(m.getName()))
          methodsSameName = (ArrayList)methods.get(m.getName());
        else
        {
          methodsSameName= new ArrayList();
          methods.put(m.getName(), methodsSameName);
        }

        methodsSameName.add(m);
      }

      it = type.iterateExtends();
      while (it.hasNext())
      {
        TypeName implementsName = (TypeName)it.next();
        recCollectMethods(implementsName, methods);
      }
    }
    catch (SymbolException e)
    {
      env.terminate("Internal error : type should exist");
    }
  }

  private void loadDepends(Method method, Type enclosing) throws SymbolException, GenericityException
  {
    if (!method.isSub())
    {
      ResultType result = method.getResultType();
      TypeName typeName = result.getTypeName();

      getType(typeName);

      /* Authorised after all for callbacks from Native code to Nosica code
      if (enclosing.isNative() && !resultType.isNative())
        env.getMS().report("Symbol026", new String[]{enclosing.getName().toString(), typeName.toString()}, method.getModel().getNode());
      */
    }

    Iterator i = method.iterateArguments();
    while (i.hasNext())
    {
      Argument argument = (Argument)i.next();
      TypeName typeName = argument.getTypeName();

      getType(typeName);

      /* Authorised after all for callbacks from Native code to Nosica code
      if (enclosing.isNative() && !argumentType.isNative() && !typeName.isArray())
        env.getMS().report("Symbol026", new String[]{enclosing.getName().toString(), typeName.toString()}, method.getModel().getNode());
      */
    }
  }

  private Type createArrayType(TypeName typeName, AliasTable aliasTable) throws SymbolException, GenericityException
  {
    TypeName elementType = typeName.getArrayElementType();

    if (elementType.isArray())
      getType(elementType);

    // the map of overloadables
    Map overloadables = new HashMap();

    ArrayList properties = new ArrayList();
    Method propertyGetArray      = createPropertyGetArray(elementType, 1, typeName, false);
    properties.add(propertyGetArray);
    Method propertyGetArrayConst = createPropertyGetArray(elementType, 1, typeName, true);
    properties.add(propertyGetArrayConst);
    Method propertySetArray      = createPropertySetArray(elementType, 1, typeName);
    properties.add(propertySetArray);

    if (typeName.getDim() > 1)
    {
      Method multiPropertyGetArray      = createPropertyGetArray(elementType, typeName.getDim(), typeName, false);
      properties.add(multiPropertyGetArray);
      Method multiPropertyGetArrayConst = createPropertyGetArray(elementType, typeName.getDim(), typeName, true);
      properties.add(multiPropertyGetArrayConst);
      Method multiPropertySetArray      = createPropertySetArray(elementType, typeName.getDim(), typeName);
      properties.add(multiPropertySetArray);
    }

    overloadables.put(null, properties);

    ArrayList constructors = new ArrayList();
    constructors.add(createConstructor(typeName));

    overloadables.put("constructor", constructors);

    ArrayList lengthMethods = new ArrayList();
    lengthMethods.add(createLengthMethod(typeName));
    lengthMethods.add(createLengthProperty(typeName));

    overloadables.put("length", lengthMethods);

    ArrayList dimensionMethods = new ArrayList();
    dimensionMethods.add(createDimensionProperty(typeName));

    overloadables.put("dimension", dimensionMethods);

    Set extendsList = new HashSet();
    extendsList.add(NativeTypes.get_net_nosica_lang_Object());
    Set implementsList = new HashSet();
    implementsList.add(NativeTypes.get_net_nosica_lang_Array());

    GenericType genericType = new GenericTypeImpl(
      null,
      true,
      false,
      false,
      new ArrayList(),
      typeName,
      false,
      true,
      false,
      false,
      typeName,
      extendsList,
      implementsList,
      aliasTable,
      java.util.Collections.EMPTY_MAP
    );

    // @TODO@ : merge creation of default destructor with
    // the one of GenericitySolverImpl
    Method destructor = new MethodImpl(
        new GenericMethodImpl(
          new DestructorDeclarationImpl(
            new BlockImpl(
              new ArrayList()
            )
          ),
          true,
          false,
          false,
          new ArrayList(),
          typeName,
          false,
          false,
          false,
          false,
          null,
          MethodKindEnum.destructor,
          "destructor",
          new ArrayList(),
          java.util.Collections.EMPTY_SET,
          true
        ),
        null,
        new ArrayList(),
        java.util.Collections.EMPTY_SET,
        typeName,
        aliasTable
      );

    return new TypeImpl(
      genericType,
      typeName,
      extendsList,
      implementsList,
      new ArrayList(),
      java.util.Collections.EMPTY_MAP,
      overloadables,
      destructor,
      null,
      null,
      null,
      aliasTable,
      java.util.Collections.EMPTY_MAP
    );
  }

  private Method createConstructor(TypeName typeName)
  {
    ArrayList arguments = new ArrayList();
    for (int i = 0; i < typeName.getDim(); ++i)
      arguments.add(
        new ArgumentImpl(
          true,
          NativeTypes.get_net_nosica_lang_word32(),
          "dim" + i
        )
      );

    return new MethodImpl(
      null,
      "constructor",
      MethodKindEnum.constructor,
      arguments,
      false,
      typeName,
      null
    );
  }

  private Method createLengthMethod(TypeName typeName)
  {
    ArrayList arguments = new ArrayList();
    arguments.add(new ArgumentImpl(true, NativeTypes.get_net_nosica_lang_word32(), "dim"));

    return new MethodImpl(
      new ResultTypeImpl(NativeTypes.get_net_nosica_lang_word32(), true),
      "length",
      MethodKindEnum.method,
      arguments,
      true,
      typeName,
      null
    );
  }

  private Method createLengthProperty(TypeName typeName)
  {
    return new MethodImpl(
      new ResultTypeImpl(NativeTypes.get_net_nosica_lang_word32(), true),
      "length",
      MethodKindEnum.property,
      new ArrayList(),
      true,
      typeName,
      null
    );
  }

  private Method createDimensionProperty(TypeName typeName)
  {
    return new MethodImpl(
      new ResultTypeImpl(NativeTypes.get_net_nosica_lang_word32(), true),
      "dimension",
      MethodKindEnum.property,
      new ArrayList(),
      true,
      typeName,
      null
    );
  }

  private Method createPropertyGetArray(TypeName elementType, int nbDim, TypeName enclosing, boolean isResultConst) throws SymbolException, GenericityException
  {
    ArrayList arguments = new ArrayList();
    for (int i = 0; i < nbDim; ++i)
      arguments.add(new ArgumentImpl(true, NativeTypes.get_net_nosica_lang_word32(), "dim" + i));

    return new MethodImpl(
      new ResultTypeImpl(elementType, isResultConst),
      null,
      MethodKindEnum.propertyArray,
      arguments,
      isResultConst,
      enclosing,
      null
    );
  }

  private Method createPropertySetArray(TypeName elementType, int nbDim, TypeName enclosing) throws SymbolException, GenericityException
  {
    ArrayList arguments = new ArrayList();
    for (int i = 0; i < nbDim; ++i)
      arguments.add(new ArgumentImpl(true, NativeTypes.get_net_nosica_lang_word32(), "dim" + i));

    arguments.add(new ArgumentImpl(true, elementType, "data"));

    return new MethodImpl(
      null,
      null,
      MethodKindEnum.propertyArray,
      arguments,
      false,
      enclosing,
      null
    );
  }
}