/*
  $Id: ArrayGet.java,v 1.8 2004/03/04 22:16:16 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.compiler.intermediate;

import java.util.Iterator;

/**
 * \brief represent a read/write access to an element of an array
 *
 * In fact, this should represent a read only access to an array, but as we can use
 * expressions like :
 * <pre>
 *   Toto[] toto = new Toto[10];
 *   ++toto[0];
 * </pre>
 * It is also used in some write access.<br>
 * ArrayGet must act as a Variable.<br>
 * <br>
 * When producing code, we produce ArrayGet with is list of arg (for array access).<br>
 * Later, with ExSeqRemoverVisitor, we create an index that is the linear
 * counterpart of this list of args. Then we can safely clear args.<br>
 * We do this because almost all languages support only mono dimensional array<br>
 * And provided that we use one of this language as the output code, we need
 * to provide support for it.
 */
public interface ArrayGet extends Variable, SubscriptableExpression
{
  /**
   * \brief iterate arguments for array access
   *
   * All Expression should have a TypeName of net.nosica.lang.word32
   *
   * \return Iterator<Expression>
   */
  Iterator iterateArgs();
}