/*
  $Id: ArraySet.java,v 1.9 2004/03/04 22:16:16 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.compiler.intermediate;

import java.util.ArrayList;
import java.util.Iterator;

/**
 * \brief represent a write only access to an array element
 *
 * That is, expressions like :<
 * <pre>
 * Toto[] toto = new Toto[10];
 * toto[0] = 1;
 * </pre>
 * Assignment make us think this IR node is a statement.<br>
 * Like Move or Init.<br>
 * As it has special meanings at the Dynamic Analysis of code, it deserves a special node.<br>
 * (as ArrayGet)<br>
 * Later, as ArrayGet, we will transform list of args into the equivalent linear index.
 */
public interface ArraySet extends Statement, Subscriptable
{
  /**
   * \brief get arguments
   *
   * \return ArrayList<Expression> as list of arguments
   */
  ArrayList getArgs();

  /**
   * \brief iterate arguments of array access
   *
   * all TypeName of arguments should be net.nosica.lang.word32
   *
   * \return Iterator<Expression>
   */
  Iterator iterateArgs();

  /**
   * \brief return the right expression in the assignment
   *
   * \return right expression
   */
  Expression getRhs();

  /**
   * \brief set the new expression to use in the assignment
   *
   * \param newRhs the new expression to use
   */
  void setRhs(Expression newRhs);
}