/*
  $Id: BasicBlockVisitor.java,v 1.9 2004/03/04 22:16:16 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.compiler.intermediate;

import net.nosica.compiler.production.c.intermediate.*;
import net.nosica.framework.VisitorException;

import java.util.ArrayList;
import java.util.Stack;
import java.util.Iterator;

public class BasicBlockVisitor implements IRVisitorRewriter
{
  private Statement rewrittenStatement = null;
  private ArrayList blocks                = null;         // ArrayList<StatementList>
  private Stack  blockStack            = new Stack();  // Stack<ArrayList<StatementList>>
  private ArrayList currentBlockStatement = new ArrayList(); // ArrayList<Statement>

  public Statement getRewrittenStatement() { return rewrittenStatement; }

  public void visit(Call call) throws VisitorException
  {
    // should not go here
  }

  public void visit(MemberAccess memberAccess) throws VisitorException
  {
    // should not go here
  }

  public void visit(Literal literal) throws VisitorException
  {
    // should not go here
  }

  public void visit(Sequence sequence) throws VisitorException
  {
    throw new UnsupportedOperationException();
  }

  public void visit(StatementExpression statementExpression) throws VisitorException
  {
    currentBlockStatement.add(statementExpression);
  }

  public void visit(Variable variable) throws VisitorException
  {
    // should not go here
  }

  public void visit(Move move) throws VisitorException
  {
    currentBlockStatement.add(move);
  }

  public void visit(ExtendedSequence exSeq) throws VisitorException
  {
    throw new UnsupportedOperationException();
  }

  public void visit(Jump jump) throws VisitorException
  {
    // end current block, begin a new one
    currentBlockStatement.add(jump);

    blocks.add(new StatementListImpl(currentBlockStatement));
    currentBlockStatement = new ArrayList();
  }

  public void visit(LabelName labelName) throws VisitorException
  {
    // begin a new one, end the current block
    if (!currentBlockStatement.isEmpty())
    {
      Statement s = (Statement)currentBlockStatement.get(currentBlockStatement.size() - 1);

      if (
           !(s instanceof Jump) &&
           !(s instanceof ConditionalJump)
         )
      {
        // add a jump in currentBlockStatement to new created one !
        currentBlockStatement.add(new JumpImpl(labelName.getLabel()));
      }

      blocks.add(new StatementListImpl(currentBlockStatement));
      currentBlockStatement = new ArrayList();
    }

    // now that block is ended, and a new one has begun, add Label to it
    currentBlockStatement.add(labelName);
  }

  public void visit(ConditionalJump conditionalJump) throws VisitorException
  {
    // end current block, begin a new one
    currentBlockStatement.add(conditionalJump);

    blocks.add(new StatementListImpl(currentBlockStatement));
    currentBlockStatement = new ArrayList();
  }

  public void visit(Cast castExp) throws VisitorException
  {
    // should not go here
  }

  public void visit(InstanceOf instanceOfExp) throws VisitorException
  {
    // should not go here
  }

  public void visit(RawAllocation rawAllocation) throws VisitorException
  {
    // should not go here
  }

  public void visit(DecRef decRef) throws VisitorException
  {
    currentBlockStatement.add(decRef);
  }

  public void visit(IncRef incRef) throws VisitorException
  {
    currentBlockStatement.add(incRef);
  }

  public void visit(Init init) throws VisitorException
  {
    currentBlockStatement.add(init);
  }

  public void visit(StatementList statementList) throws VisitorException
  {
    blocks = new ArrayList();
    blockStack.push(blocks);

    Iterator it = statementList.iterateStatementList();
    while (it.hasNext())
    {
      Statement s = (Statement)it.next();
      s.accept(this);
    }

    statementList.setStatementList(blocks);
    blocks = (ArrayList)blockStack.pop();

    if (blockStack.isEmpty())
      rewrittenStatement = statementList;
  }

  public void visit(ArraySet arraySet) throws VisitorException
  {
    currentBlockStatement.add(arraySet);
  }

  public void visit(ArrayGet arrayGet) throws VisitorException
  {
    // should not go here
  }

  public void visit(CheckNull checkNull) throws VisitorException
  {
    currentBlockStatement.add(checkNull);
  }

  public void visit(ConstraintError constraintError) throws VisitorException
  {
    currentBlockStatement.add(constraintError);
  }

  public void visit(TryRegister tryRegister) throws VisitorException
  {
    currentBlockStatement.add(tryRegister);

    if (tryRegister.hasFinallyStatement())
      tryRegister.getFinallyStatement().accept(this);
  }

  public void visit(TryUnregister tryUnregister) throws VisitorException
  {
    currentBlockStatement.add(tryUnregister);
  }

  public void visit(Throw throwStatement) throws VisitorException
  {
    currentBlockStatement.add(throwStatement);
  }

  public void visit(VariableScopeLeave varScopeLeave) throws VisitorException
  {
    currentBlockStatement.add(varScopeLeave);
  }

  public void visit(Return returnStatement) throws VisitorException
  {
    currentBlockStatement.add(returnStatement);
  }
}