/*
  $Id: CallImpl.java,v 1.11 2004/03/04 22:16:16 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.compiler.intermediate;

import net.nosica.framework.VisitorException;
import net.nosica.parser.node.util.var.TypeName;
import net.nosica.compiler.typechecker.Method;

import java.util.ArrayList;
import java.util.Iterator;

public class CallImpl implements Call
{
  private Expression subscript;                     // subscript (if method is not static)
  private Method method;                            // signature of method
  private ArrayList args;                              // ArrayList<Expression> arg of call
  private String localization = null;

  public CallImpl(Expression subscript, Method method, ArrayList args)
  {
    this.method = method;

    setArgs(args);
    setSubscript(subscript);
  }

  public boolean isSuper() { return false; }
  public boolean isThis() { return false; }
  public boolean isStatic() { return method.isStatic(); }

  public boolean hasSubscript() { return subscript != null; }
  public void setSubscript(Expression subscript) { this.subscript = subscript; }
  public Expression getSubscript() { return subscript; }

  public Method getMethod() { return method; }

  public boolean hasArgs() { return args != null;}
  public void setArgs(ArrayList args) { this.args = args; }
  public Iterator iterateArgs() { return args.iterator(); }

  public TypeName getTypeName()
  {
    if (method.isSub())
      return null;
    else
      return method.getResultType().getTypeName();
  }

  public void accept(IRVisitor visitor) throws VisitorException
  {
    visitor.visit(this);
  }

  public boolean isArgument() { return false; }

  public String localization()
  {
    return localization;
  }

  public void setLocalization(String localization)
  {
    this.localization = localization;
  }
}
