/*
  $Id: Function.java,v 1.8 2004/03/04 22:16:16 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.compiler.intermediate;

import net.nosica.compiler.typechecker.Method;

import java.util.ArrayList;
import java.util.Iterator;

/**
 * \brief IR of a method
 */
public interface Function
{
  /**
   * \brief return the underlying method
   *
   * \return method
   */
  Method getMethod();

  /**
   * \brief return the IR representation of method
   *
   * \return IR tree
   */
  Statement getStatement();

  /**
   * \brief set the IR representation of method
   *
   * \param statement new IR tree
   */
  void setStatement(Statement statement);

  /**
   * \brief iterate each arguments
   *
   * \return Iterator<Variable>
   */
  Iterator iterateArgs();
  
  /**
   * \brief return arguments of this function
   * 
   * \return arguments of this function
   */
  ArrayList getArgs();

  /**
   * \brief access the dynamic information about 'this' for this function
   */
  Variable getThisInfo();
}
