/*
  $Id: Record.java,v 1.6 2004/03/04 22:16:16 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.compiler.intermediate;

import net.nosica.parser.node.util.var.TypeName;
import net.nosica.symbol.SymbolException;

import java.util.Iterator;

/**
 * \brief intermediate representation of a type
 */
public interface Record
{
  /**
   * \brief return the name of this record
   *
   * \return name of record
   */
  TypeName getName();

  /**
   * \brief has this type an extends ?
   *
   * \return true if type has an extends
   */
  boolean hasExtends();

  /**
   * \brief return name of extends
   * requires : hasExtends()
   *
   * \return name of extends
   */
  TypeName getExtends();

  /**
   * \brief has this type an enclosing type ?
   *
   * \return true if type has an enclosing type
   */
  boolean hasEnclosing();

  /**
   * \brief return name of enclosing type
   * requires : hasEnclosing
   *
   * \return name of enclosing
   */
  TypeName getEnclosing();

  /**
   * \brief iterate over all fields
   *
   * \return Iterator<Field>
   */
  Iterator iterateFields();

  /**
   * \brief get a field by name
   *
   * if record has a super type, a field named "super" is defined
   * if record is an inner type, a field names "___enclosing" is defined
   *
   * \param name name of field to retrieve
   * \return field named name
   */
  Field getField(String name) throws SymbolException;

  /**
   * \brief return an unique number associated with type
   *
   * \return unique number of type
   */
  int getTypeCode();
}