/*
  $Id: TryRegister.java,v 1.8 2004/03/04 22:16:16 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.compiler.intermediate;

import java.util.Map;

/**
 * \brief handle an exception handler registering, variable clean up, and catch block branching
 *
 * this IR node must
 * 1 - register an exception handler
 * 2 - gives control to the try block
 * 3 - in case of an exception
 *   a - clean up live variables
 *   b - execute finally block if any
 *   c - gives control to the correct exception block if any
 *   d - rethrow exception if no catch block can handle exception
 */
public interface TryRegister extends Statement
{
  /**
   * \brief does this try block has an attached finally statement ?
   *
   * \return true if try block has a finally statement
   */
  boolean hasFinallyStatement();
  /**
   * \brief return the attached finally statement
   * requires : hasFinallyStatement
   *
   * \return finally block
   */
  Statement getFinallyStatement();
  /**
   * \brief set the finally statement of this try block
   *
   * \param finallyStatement the block to attach
   */
  void setFinallyStatement(Statement finallyStatement);
  /**
   * \brief return the exception map
   *
   * return information about how to handle exceptions<br>
   * Basically for an exception of type eTypeName, find the entry in the map
   * that has the same type.
   * After having cleaning up, give control to catch block at label "Label" in variable "Variable"
   *
   * \return Map<TypeName, Pair<Label, Variable> >
   */
  Map getExceptionMap();
}