/*
  $Id: CCodeProductionImpl.java,v 1.31 2004/05/06 19:47:00 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.compiler.production.c;

import net.nosica.application.MessageLevel;
import net.nosica.application.NativeTypes;

import net.nosica.compiler.production.CodeProduction;
import net.nosica.compiler.production.c.intermediate.InsertCheckNullVisitor;
import net.nosica.compiler.production.c.intermediate.InsertRefCountVisitor;
import net.nosica.compiler.production.c.intermediate.InsertTemporaryVisitor;
import net.nosica.compiler.production.c.intermediate.CIRFunctionEnvironmentDisplayer;

import net.nosica.compiler.intermediate.Record;
import net.nosica.compiler.intermediate.Function;
import net.nosica.compiler.intermediate.Statement;
import net.nosica.compiler.intermediate.Field;
import net.nosica.compiler.intermediate.IRVisitor;
import net.nosica.compiler.intermediate.IRVisitorRewriter;

import net.nosica.application.ApplicationContext;
import net.nosica.application.PropertyException;

import net.nosica.compiler.application.Environment;

import net.nosica.compiler.typechecker.Type;
import net.nosica.compiler.typechecker.Method;
import net.nosica.compiler.typechecker.Argument;

import net.nosica.symbol.MethodKindEnum;
import net.nosica.symbol.SymbolException;

import net.nosica.parser.node.util.var.TypeName;
import net.nosica.framework.VisitorException;

import net.nosica.util.IndentedPrintWriter;
import net.nosica.util.SettableInteger;

import java.util.Map;
import java.util.HashMap;
import java.util.Set;
import java.util.HashSet;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.ListIterator;

import java.io.FileOutputStream;
import java.io.File;
import java.io.IOException;

public class CCodeProductionImpl implements CodeProduction
{
  private Environment env = Environment.instance();
  private CNameConverter cConverter = new CNameConverterImpl();
  private Type mainType;
  private File codeOutputPath = null;
  private File codeOutputFile = null;

  private boolean debug_ir = false;
  private String debug_ir_type = null;

  public CCodeProductionImpl(Type mainType)
  {
    this.mainType = mainType;

    try
    {
      debug_ir = env.getAC().getPropertyBoolean("debug.ir");
    }
    catch (PropertyException e)
    {
    }

    try
    {
      debug_ir_type = env.getAC().getPropertyString("debug.ir.type");
    }
    catch (PropertyException e)
    {
    }
  }

  public void produce()
  {
    try
    {
      codeOutputPath = env.getAC().getPropertyPath("code.output");
    }
    catch (PropertyException e)
    {
      env.getMS().report(MessageLevel.NFO, "property code.output is not defined : reverting to default");
      codeOutputPath = new File(".");
    }

    IndentedPrintWriter printer = null;

    try
    {
      codeOutputFile = new File(codeOutputPath, cConverter.cSourceFileName(mainType.getName()));

      printer = new IndentedPrintWriter(new FileOutputStream(codeOutputFile));
    }
    catch (java.io.FileNotFoundException e)
    {
      env.terminate("Error : code.output [" + codeOutputPath + "] points to an invalid path");
    }

    try
    {
      rewriteIR();

      Function mainFunction = getMainFunction();

      produceNosicaHeader(printer);

      ArrayList records = getRecordsAndSortByInheritance();
      produceTypeForwardDeclaration(records, printer);

      produceHeaders(printer);

      produceTypeDeclaration(records, printer);
      produceFunction(printer);
      produceClassDescription(printer);

      produceInitMethod(printer, records);
      produceExitMethod(printer, records);
      produceMain(printer, mainFunction);
    }
    catch(VisitorException e)
    {
      env.terminate("Internal Error : " + e.getMessage());
    }
    finally
    {
      printer.flush();
      printer.close();
    }

    try
    {
      //  now launch compilation by invoking c compiler
      launchCCompiler();
    }
    catch (VisitorException e)
    {
      env.terminate("Internal Error : " + e.getMessage());
    }
  }

  private void rewriteIR() throws SymbolException, VisitorException
  {
    Iterator it = env.getIRFE().iterateFunctions();
    while (it.hasNext())
    {
      Function f = (Function)it.next();

      rewriteFunction(f);
    }
  }

  private void rewriteFunction(Function f) throws SymbolException, VisitorException
  {
    Statement statement = f.getStatement();

    if (statement == null)
      return;

    statement = rewrite(
      statement,
      new InsertTemporaryVisitor()
    );

    statement = rewrite(
      statement,
      new InsertRefCountVisitor(f)
    );

    statement = rewrite(
      statement,
      new InsertCheckNullVisitor()
    );

    f.setStatement(statement);

    if (debug_ir)
    {
      display(f);
    }
    else
    if (debug_ir_type != null && debug_ir_type.equals(f.getMethod().getType().toString()))
    {
      display(f);
    }
  }

  private Statement rewrite(Statement statement, IRVisitorRewriter visitor) throws VisitorException
  {
    statement.accept(visitor);
    statement = visitor.getRewrittenStatement();
    return statement;
  }

  private void display(Function f) throws VisitorException
  {
    System.out.print(f.getMethod().getType().toString());
    System.out.print(".");

    if (f.getMethod().getName() != null)
    {
      System.out.print(f.getMethod().getName());
    }
    else
    {
      System.out.print("property []");
    }
    System.out.println(" :");

    if (f.getStatement() == null)
      return;

    CIRFunctionEnvironmentDisplayer displayer = new CIRFunctionEnvironmentDisplayer(
      new IndentedPrintWriter(System.out)
    );

    f.getStatement().accept(displayer);
  }

  private void launchCCompiler() throws SymbolException
  {
    // iterate over types and search native classes
    Set nativePackages = new HashSet(); // Set<TypeName>

    Iterator it = env.getIRRE().iterateRecords();
    while (it.hasNext())
    {
      Record r = (Record)it.next();
      Type type = env.getTE().getType(r.getName());
      if (type.isNative())
        nativePackages.add(type.getName().getFirstComponents());
    }

    ApplicationContext ac = env.getAC();

    // generate a Makefile
    generateMakefile(nativePackages);

    try
    {
      launchCommand(new String[]{ac.getPropertyString("MAKE"), ac.getPropertyString("MAKE_CHANGE_DIR_FLAG"), codeOutputPath.toString()});
    }
    catch (PropertyException e)
    {
      env.terminate("Internal error : " + e.getMessage());
    }
  }

  private void generateMakefile(Set nativePackages)
  {
    ApplicationContext ac = env.getAC();

    File makeFileOutputFile = new File(codeOutputPath, "Makefile");
    IndentedPrintWriter printer = null;

    try
    {
      printer = new IndentedPrintWriter(new FileOutputStream(makeFileOutputFile));
      printer.setIndentPattern("\t");
    }
    catch (java.io.FileNotFoundException e)
    {
      env.terminate("Internal error : " + e.getMessage());
    }

    try
    {
      // CC
      printer.println("CC=" + ac.getPropertyString("CC"));

      // CFLAGS
      String ccI           = ac.getPropertyString("CC_INCLUDE_FLAG");
      String nativeInclude = ac.getPropertyString("NOSICA_NATIVESTDLIB_INCLUDE_PATH");

      StringBuffer cflags = new StringBuffer();
      cflags.append("CFLAGS=")
            .append(ccI).append(nativeInclude).append(" ")
            .append(ccI).append(nativeInclude).append("/include ")
            .append(ccI).append(nativeInclude).append("/").append("native").append(" ")
            .append(ac.getPropertyString("CC_EXTRA_FLAG"));
      printer.println(cflags.toString());

      // LFLAGS
      StringBuffer lflags = new StringBuffer();
      lflags.append("LFLAGS=").append(ac.getPropertyString("CC_LIB_PATH_FLAG")).append(" ")
              .append(ac.getPropertyString("NOSICA_NATIVESTDLIB_LIB_PATH"));

      String CC_LIB_USE_FLAG = ac.getPropertyString("CC_LIB_USE_FLAG");
      Iterator it = nativePackages.iterator();
      while (it.hasNext())
      {
        TypeName packageName = (TypeName)it.next();
        lflags.append(" ").append(CC_LIB_USE_FLAG).append(cConverter.cNativeLibraryName(packageName));
      }
      String CC_LIB_EXTRA_FLAG = ac.getPropertyString("CC_LIB_EXTRA_FLAG");
      lflags.append(" ").append(CC_LIB_EXTRA_FLAG);

      printer.println(lflags.toString());

      // EXEC
      printer.println("EXEC="+ mainType.getName().getLastComponent().getPackageElement());

      // OBJECT
      printer.println("OBJECT="+ cConverter.cObjFileName(mainType.getName()));

      // RULE c compilation target
      printer.println("%.o : %.c");
      printer.addIndent();
      printer.println("$(CC) $(CFLAGS) -c $< -o $@");
      printer.decIndent();

      // RULE : all target
      printer.println("all : $(OBJECT)");
      printer.addIndent();
      printer.println("$(CC) $(OBJECT) $(LFLAGS) -o $(EXEC)");
      printer.decIndent();

      // RULE : clean target
      printer.println("clean :");
      printer.addIndent();
      printer.println("rm -f $(OBJECT)");
      printer.println("rm -f $(EXEC)");
      printer.decIndent();
    }
    catch (PropertyException e)
    {
      env.terminate("Internal error : " + e.getMessage());
    }
    finally
    {
      printer.flush();
      printer.close();
    }
  }

  private void launchCommand(String[] command)
  {
    StringBuffer sb = new StringBuffer();
    for (int i = 0; i < command.length; ++i)
    {
      sb.append(command[i]);
      if (i < command.length)
        sb.append(" ");
    }

    env.getMS().report(MessageLevel.NFO, "BackEnd001", sb.toString(), null);

    try
    {
      Process p = Runtime.getRuntime().exec(command);
      int errorCode = p.waitFor();
      if (errorCode != 0)
      {
        // by convention if error code != 0, an error occured in process
        env.getMS().report(MessageLevel.ERR, "BackEnd002");

        // dump input stream to a log (output)
        java.io.InputStream is = p.getInputStream();
        dumpInputStreamToFile(is, mainType.getName().getLastComponent().getPackageElement() + ".output.log");
        java.io.InputStream es = p.getErrorStream();
        dumpInputStreamToFile(es, mainType.getName().getLastComponent().getPackageElement() + ".error.log");
      }
    }
    catch (IOException e)
    {
      env.terminate("While invoking c compiler : " + e.getMessage());
    }
    catch (InterruptedException e)
    {
      env.terminate("Internal error : " + e.getMessage());
    }
  }

  private void dumpInputStreamToFile(java.io.InputStream is, String fileName)
  {
    try
    {
      FileOutputStream os = new FileOutputStream(new File(codeOutputPath, fileName));

      try
      {
        byte[] line = new byte[80]; // read 80 char by 80 char
        int available;

        while ((available = is.available()) != 0)
        {
          if (available > 80) available = 80;
          is.read(line);
          os.write(line, 0, available);
        }
      }
      catch (IOException e)
      {
        env.terminate(e.getMessage());
      }
      finally
      {
        os.flush();
        os.close();
      }
    }
    catch (java.io.FileNotFoundException e)
    {
      env.terminate(e.getMessage());
    }
    catch (IOException e)
    {
      env.terminate(e.getMessage());
    }
  }

  private Function getMainFunction() throws SymbolException
  {
    Method mainMethod = env.getMR().findMainMethod(mainType.getName());
    if (mainMethod != null)
      return env.getIRFE().getFunction(mainMethod);

    return null;
  }

  private void produceNosicaHeader(IndentedPrintWriter printer)
  {
    printer.println("// This file was produced by Nosica compiler");
    printer.println("// do not edit !");
    printer.println();
  }

  private void produceHeaders(IndentedPrintWriter printer) throws VisitorException
  {
    printer.println("#include <types.h>");
    Iterator it = env.getIRRE().iterateRecords();

    while (it.hasNext())
    {
      Record record = (Record)it.next();
      Type type = env.getTE().getType(record.getName());
      if (type.isNative())
        printer.println("#include <" + cConverter.cIncludeTypeName(record.getName()) + ">");
    }

    printer.println();
  }

  private void printInheritTypeCode(IndentedPrintWriter printer, Type type) throws SymbolException
  {
    boolean needComa = false;

    // special case for None
    if (type.getName() == NativeTypes.get_net_nosica_lang_None())
    {
      // None inherits from all other types ...
      Record none = env.getIRRE().getRecord(type.getName());
      int noneTypeCode = none.getTypeCode();

      for (int i = 1; i < env.getIRRE().nbRecord() + 1; ++i)
      {
          if (i != noneTypeCode)
          {
            if (needComa) printer.print(", ");
            printer.print(i);
            needComa = true;
          }
      }

      if (needComa) printer.print(", ");
      printer.print("0");

      return;
    }

    Iterator it = type.iterateExtends();
    while (it.hasNext())
    {
      TypeName typeName = (TypeName)it.next();
      Record record = env.getIRRE().getRecord(typeName);

      if (needComa) printer.print(", ");
      printer.print(record.getTypeCode());
      needComa = true;
    }

    it = type.iterateImplements();
    while (it.hasNext())
    {
      TypeName typeName = (TypeName) it.next();
      Record record = env.getIRRE().getRecord(typeName);

      if (needComa)
        printer.print(", ");
      printer.print(record.getTypeCode());
      needComa = true;
    }

    if (needComa) printer.print(", ");
    printer.print("0");
  }

  private void produceClassDescription(IndentedPrintWriter printer) throws VisitorException
  {
    // first sort type by typeCode
    Iterator it = env.getIRRE().iterateRecords();
    Record[] records = new Record[env.getIRRE().nbRecord()];

    while (it.hasNext())
    {
      Record record = (Record)it.next();
      records[record.getTypeCode() - 1] = record;
    }

    // emit RTTI structure
    printer.println("___RTTI rtti[] = {");
    printer.addIndent();

    for (int i = 0; i < records.length; ++i)
    {
      Type type = env.getTE().getType(records[i].getName());

      printer.print("{");
      printer.print(records[i].getTypeCode() + ", "); // typeCode

      if (
        env.getIRFE().functionExists(type.getDestructor()) &&
        !type.isPrimitive() &&
        !type.getName().equals(NativeTypes.get_net_nosica_lang_Object())
      )
      {
        printer.print("(pdtor)" + cConverter.cMethodName(type.getDestructor())); // and pdtor
        printer.print(", ");
        printer.print("\"" + type.getName().toString() + "\""); // type name
        printer.print(", ");
        printer.print("sizeof(" + cConverter.cTypeName(type.getName()) + ")"); // size
      }
      else
      {
        printer.print("0, \"\", 0");
      }
      printer.print("}");

      printer.println(",");
    }
    printer.println("{0, 0, \"\", 0}");

    printer.decIndent();
    printer.println("};");
    printer.println();

    // emit inherit data structure
    for (int i = 0; i < records.length; ++i)
    {
      Record record = records[i];

      printer.print("word32 ___" + cConverter.cRawTypeName(record.getName()) +
                    "_inherit_list[] = {");
      Type type = env.getTE().getType(record.getName());
      printInheritTypeCode(printer, type);
      printer.println("};");
    }

    printer.println();

    printer.println("word32 *___inheritData[] = {");
    printer.addIndent();
    printer.println("0,");
    for (int i = 0; i < records.length; ++i)
    {
      Record record = records[i];
      printer.println("___" + cConverter.cRawTypeName(record.getName()) +
                      "_inherit_list,");
    }
    printer.decIndent();
    printer.println("};");

    printer.println();

    printer.println(
      "word32 ___inheritDataEntries = sizeof(___inheritData) / sizeof(word32 *);");

    printer.println();

    printer.println(cConverter.cRawTypeName(NativeTypes.get_net_nosica_lang_Object()) +
                    " *" + cConverter.getExceptionVariableName() + " = 0;");

    printer.println();
  }

  private ArrayList getRecordsAndSortByInheritance() throws VisitorException
  {
    // first sort type by inheritance
    ArrayList records = new ArrayList();
    Iterator it = env.getIRRE().iterateRecords();

    while (it.hasNext())
    {
      Record record = (Record)it.next();
      addRecordAndSortByInheritance(record, records);
    }

    return records;
  }

  private void produceTypeForwardDeclaration(ArrayList records, IndentedPrintWriter printer) throws VisitorException
  {
    // emit structures forward declarations
    Iterator it = records.iterator();
    while(it.hasNext())
    {
      Record record = (Record)it.next();
      Type type = env.getTE().getType(record.getName());

      if (type.isInterface()) continue;
      if (record.getName().equals(NativeTypes.get_net_nosica_lang_Object())) continue;

      printer.println("struct ___" + cConverter.cTypeName(record.getName()) + ";");
      printer.println("typedef struct ___" + cConverter.cTypeName(record.getName()) + " " + cConverter.cTypeName(record.getName()) + ";");
    }
    printer.println();
  }

  private void produceTypeDeclaration(ArrayList records, IndentedPrintWriter printer) throws VisitorException
  {
    // emit structure definitions
    Iterator it = records.iterator();
    while (it.hasNext())
    {
      Record record = (Record)it.next();
      Type type = env.getTE().getType(record.getName());

      if (type.isInterface()) continue;
      if (record.getName().equals(NativeTypes.get_net_nosica_lang_Object())) continue;

      printer.println("// type [" + record.getName() + "]");

      // First produce static fields
      Iterator j = record.iterateFields();
      while (j.hasNext())
      {
        Field field = (Field)j.next();
        if (field.isStatic())
          printer.println(cConverter.cMemberTypeName(field) + " " + cConverter.cMemberName(field) + ";");
      }

      printer.println("struct ___"+cConverter.cTypeName(record.getName())+" {");
      printer.addIndent();

      if (record.getName().isArray())
      {
        // array types are somehow special !! :-)
        printer.println(cConverter.cTypeName(NativeTypes.get_net_nosica_lang_Object()) + " super;");

        TypeName arrayElement = record.getName().getArrayElementType();

        boolean isPrimitive = true;

        if (!arrayElement.isArray())
        {
          Type arrayElementType = env.getTE().getType(arrayElement);
          if (arrayElementType.isInterface())
          {
            arrayElement = NativeTypes.get_net_nosica_lang_Object();
          }

          if (!arrayElementType.isPrimitive())
          {
            isPrimitive = false;
          }
        }
        else
        {
          isPrimitive = false;
        }

        printer.print(cConverter.cTypeName(arrayElement) + " ");
        if (!isPrimitive)
          printer.print("*");
        printer.println("*data; // linear array");
        printer.println(cConverter.cTypeName(NativeTypes.get_net_nosica_lang_word32()) + " size; // total number of elements");
        printer.println(cConverter.cTypeName(NativeTypes.get_net_nosica_lang_word32()) + " nbDim; // total number of dimension");
        printer.println(cConverter.cTypeName(NativeTypes.get_net_nosica_lang_word32()) + " sizeOfDim[" + record.getName().getDim() + "]; // total number of bytes per dimension");
      }
      else
      {
        j = record.iterateFields();
        while (j.hasNext())
        {
          Field field = (Field)j.next();
          if (!field.isStatic())
            printer.println(cConverter.cMemberTypeName(field) + " " + cConverter.cMemberName(field) + ";");
        }
      }

      printer.decIndent();
      printer.println("};");
      printer.println();
    }
  }

  /**
   * \brief add all record to produce in correct order
   * if record has a super type, then super type must be added before record
   * if record has an enclosing type, then enclosing type must be added before record
   * if record is an array, then array element must be added before record,
   *   except if array element is an interface, in which case it must not be added at all
   * if record uses primitive typed values, those types must be added before record
   * Native type must not be added
   *
   *  \param record record to add
   *  \param records ArrayList<Record> sorted in right order for C code production
   */
  private void addRecordAndSortByInheritance(Record record, ArrayList records) throws VisitorException
  {
    if (!records.contains(record))
    {
      // super type
      if (record.hasExtends())
      {
        Record extend = env.getIRRE().getRecord(record.getExtends());
        addRecordAndSortByInheritance(extend, records);
      }

      // array element of array
      if (record.getName().isArray())
      {
        // it is possible that we have an array of interface
        // in this case, don't load interface (as it will not be in IRRE)
        if (!env.getTE().getType(record.getName().getArrayElementType()).isInterface())
        {
          Record array = env.getIRRE().getRecord(record.getName().getArrayElementType());
          addRecordAndSortByInheritance(array, records);
        }
      }

      // enclosing type
      if (record.hasEnclosing())
      {
        Record enclosing = env.getIRRE().getRecord(record.getEnclosing());
        addRecordAndSortByInheritance(enclosing, records);
      }

      // primitive typed members
      Iterator i=record.iterateFields();
      while(i.hasNext())
      {
        Field f = (Field)i.next();

        Type  t = env.getTE().getType(f.getTypeName());

        if (!t.getName().equals(record.getName()) && t.isPrimitive())
        {
          Record member = env.getIRRE().getRecord(f.getTypeName());
          addRecordAndSortByInheritance(member, records);
        }
      }

      // produce only non native type
      if (!env.getTE().getType(record.getName()).isNative())
        records.add(record);
    }
  }

  private void produceFunction(IndentedPrintWriter printer) throws VisitorException
  {
    // PASS 1: function's declarations
    printer.println();
    printer.println("// --- Method's declarations");
    printer.println();

    // a set to produce only once the accessors for arrays
    Set processedArrayAccessors = new HashSet(); // Set<TypeName>

    Iterator it = env.getIRFE().iterateFunctions();
    while (it.hasNext())
    {
      Function function = (Function)it.next();

      if (function.getMethod().getType().equals(NativeTypes.get_net_nosica_lang_Object()))
        continue;

      // don't produce native functions
      Type type = env.getTE().getType(function.getMethod().getType());
      if (type.isNative() && !function.getMethod().getModel().getMethod().hasBlock())
        continue;

      if (function.getMethod().getType().isArray())
      {
        TypeName typeName = function.getMethod().getType();

        if (!processedArrayAccessors.contains(typeName))
        {
          printer.print(cConverter.cArrayGetSignature(typeName, true));
          printer.println(";");
          printer.print(cConverter.cArraySetSignature(typeName, true));
          printer.println(";");

          processedArrayAccessors.add(typeName);
        }
      }

      printer.print(cConverter.cMethodSignature(function.getMethod()));
      printer.println(";");
    }

    // PASS 2: polymorphic call sites
    printer.println();
    printer.println("// --- Polymorphic sites functions");
    printer.println();

    Map polymorphicCallSitesIds = new HashMap(); // Map<Call, Integer>
    Map polymorphicInstanceOfSitesIds = new HashMap(); // Map<InstanceOf, Integer>
    SettableInteger polymorphicCallSitesCounter = new SettableInteger(0);

    it = env.getIRFE().iterateFunctions();
    while (it.hasNext())
    {
      Function function = (Function)it.next();
      if (function.getMethod().getType().equals(NativeTypes.get_net_nosica_lang_Object())) continue;
      if (function.getMethod().getType().isArray()) continue;
      if (function.getStatement() == null) continue;

      IRVisitor visitor = new PolymorphicCallSiteProducerVisitor(
        printer,
        cConverter,
        polymorphicCallSitesIds,
        polymorphicInstanceOfSitesIds,
        polymorphicCallSitesCounter
      );

      function.getStatement().accept(visitor);
    }

    // PASS 3: string literal
    printer.println();
    printer.println("// --- String literals");
    printer.println();

    Map stringLiteralMap = new HashMap(); // Map<String, Integer>
    SettableInteger stringLiteralCounter = new SettableInteger(0);
    boolean specialMethodProduced = false;

    it = env.getIRFE().iterateFunctions();
    while (it.hasNext())
    {
      Function function = (Function)it.next();
      if (function.getMethod().getType().equals(NativeTypes.get_net_nosica_lang_Object())) continue;
      if (function.getStatement() == null) continue;

      StringLiteralProducerVisitor visitor = new StringLiteralProducerVisitor(
        printer,
        stringLiteralMap,
        stringLiteralCounter,
        cConverter,
        specialMethodProduced
      );
      function.getStatement().accept(visitor);
      specialMethodProduced = visitor.getSpecialMethodProduced();
    }

    // PASS 4: function's bodies
    printer.println();
    printer.println("// --- Method's bodies");
    printer.println();

    processedArrayAccessors.clear();

    it = env.getIRFE().iterateFunctions();
    while (it.hasNext())
    {
      Function function = (Function)it.next();

      if (function.getMethod().getType().isArray())
      {
        TypeName typeName = function.getMethod().getType();
        if (!processedArrayAccessors.contains(typeName))
        {
          produceArrayAccessors(typeName, printer);
          processedArrayAccessors.add(typeName);
        }

        produceArrayMethod(function, printer);
        continue;
      }

      if (function.getMethod().getType().equals(NativeTypes.get_net_nosica_lang_Object())) continue;
      if (!function.getMethod().isImplemented()) continue;

      printer.print(cConverter.cMethodSignature(function.getMethod()));
      printer.println(" {");
      printer.addIndent();

      Type type = env.getTE().getType(function.getMethod().getType());

      if (
          function.getMethod().getKind() == MethodKindEnum.constructor &&
          type.isInner()
         )
        printer.println("this->___enclosing = ___enclosing;");

      if (function.getStatement() != null)
      {
        ArrayList args = new ArrayList();
        args.add("this");

        Iterator j = function.getMethod().iterateArguments();
        while (j.hasNext())
        {
          Argument arg = (Argument)j.next();
          args.add(arg.getName());
        }

        VariableProducerVisitor visitor1 = new VariableProducerVisitor(printer, cConverter, args);
        function.getStatement().accept(visitor1);

        FunctionProducerVisitor visitor2 = new FunctionProducerVisitor(
          printer,
          cConverter,
          polymorphicCallSitesIds,
          stringLiteralMap,
          type,
          function.getArgs()
        );
        function.getStatement().accept(visitor2);

        if (function.getMethod().isSub())
          printer.println("return;");

        visitor2.emitExceptionHandlingEpilogue();
      }

      printer.decIndent();
      printer.println("}");
      printer.println();
    }
  }

  private void produceArrayAccessors(TypeName typeName, IndentedPrintWriter printer) throws VisitorException
  {
    // 1 - array get
    printer.print(cConverter.cArrayGetSignature(typeName, false));
    printer.println(" {");
    printer.addIndent();

    printer.println("word32 index = dim0;");
    for (int i = 1; i < typeName.getDim(); ++i)
    {
      printer.print("index = index * array->sizeOfDim[" + (i - 1) + "] + dim" + i);
    }

    printer.println("return array->data[index];");

    printer.decIndent();
    printer.println("}");
    printer.println();

    // 2 - array set
    printer.print(cConverter.cArraySetSignature(typeName, false));
    printer.println(" {");
    printer.addIndent();

    printer.println("word32 index = dim0;");
    for (int i = 1; i < typeName.getDim(); ++i)
    {
      printer.print("index = index * array->sizeOfDim[" + (i - 1) + "] + dim" + i);
    }

    printer.println("array->data[index] = data;");

    printer.decIndent();
    printer.println("}");
    printer.println();
  }

  private void produceArrayMethod(Function function, IndentedPrintWriter printer) throws VisitorException
  {
    if (function.getMethod().getKind() == MethodKindEnum.method)
    {
      if (function.getMethod().getName().equals("length"))
      {
        produceArrayLengthMethod(function, printer);
      }
      else
      if (function.getMethod().getName().equals("copy"))
      {
        produceArrayCopyMethod(function, printer);
      }
      else
        env.terminate("Internal error : used function of array type [" + function.getMethod().getType() + "] should not exist");
    }
    else
    if (function.getMethod().getKind() == MethodKindEnum.constructor)
    {
      produceArrayConstructor(function, printer);
    }
    else
    if (function.getMethod().getKind() == MethodKindEnum.destructor)
    {
      produceArrayDestructor(function, printer);
    }
    else
    if (function.getMethod().getKind() == MethodKindEnum.property)
    {
      if (function.getMethod().getName().equals("length"))
      {
        produceArrayLengthProperty(function, printer);
      }
      else
      if (function.getMethod().getName().equals("dimension"))
      {
        produceArrayDimensionProperty(function, printer);
      }
      else
        env.terminate("Internal error : used function of array type [" + function.getMethod().getType() + "] should not exist");
    }
    else
    if (function.getMethod().getKind() != MethodKindEnum.propertyArray)
      env.terminate("Internal error : used function of array type [" + function.getMethod().getType() + "] should not exist");
  }

  private void produceArrayLengthMethod(Function function, IndentedPrintWriter printer) throws VisitorException
  {
    printer.print(cConverter.cMethodSignature(function.getMethod()));
    printer.println(" {");
    printer.addIndent();

    String argName = null;
    Iterator it = function.getMethod().iterateArguments();
    if (it.hasNext())
    {
      Argument arg = (Argument)it.next();
      argName = arg.getName();
    }
    else
      env.terminate("Internal error : missing argument to function");

    printer.println("ASSERT(this != null, \"in Array.length: array instance is null\");");
    printer.println("ASSERT(" + argName + " < this->nbDim, \"dimension number is out of bound\");");
    printer.println(
        "return this->sizeOfDim[" +
        argName +
        "] / sizeof(" +
        cConverter.cTypeName(function.getMethod().getType().getArrayElementType()) +
        ");"
    );

    printer.decIndent();
    printer.println("}");
    printer.println();
  }

  private void produceArrayCopyMethod(Function function, IndentedPrintWriter printer) throws VisitorException
  {
    // method is static
    printer.print(cConverter.cMethodSignature(function.getMethod()));
    printer.println(" {");
    printer.addIndent();

    String dstName = null;
    String dstOffsetName = null;
    String srcName = null;
    String srcOffsetName = null;
    String sizeName = null;

    Iterator it = function.getMethod().iterateArguments();

    if (it.hasNext())
    {
      Argument arg = (Argument)it.next();
      dstName = arg.getName();
    }
    else
      env.terminate("Internal error : missing argument to function");

    if (it.hasNext())
    {
      Argument arg = (Argument)it.next();
      dstOffsetName = arg.getName();
    }
    else
      env.terminate("Internal error : missing argument to function");

    if (it.hasNext())
    {
      Argument arg = (Argument)it.next();
      srcName = arg.getName();
    }
    else
      env.terminate("Internal error : missing argument to function");

    if (it.hasNext())
    {
      Argument arg = (Argument)it.next();
      dstOffsetName = arg.getName();
    }
    else
      env.terminate("Internal error : missing argument to function");

    if (it.hasNext())
    {
      Argument arg = (Argument)it.next();
      sizeName = arg.getName();
    }
    else
      env.terminate("Internal error : missing argument to function");

    TypeName arrayElementType = function.getMethod().getType().getArrayElementType();

    Type type = env.getTE().getType(arrayElementType);

    if (!type.isPrimitive())
    {
      printer.println("word32 i;");
      printer.println();
    }

    printer.println("ASSERT(" + srcName + " != null, \"in Array.copy(): source array instance is null\");");
    printer.println("ASSERT(" + dstName + " != null, \"in Array.copy(): destination array instance is null\");");
    printer.println("ASSERT(" + srcOffsetName + " + " + sizeName + " < " + srcName + "->sizeOfDim[0], \"in Array.copy(): dimensions mismatch\");");
    printer.println("ASSERT(" + dstOffsetName + " + " + sizeName + " < " + dstName + "->sizeOfDim[0], \"in Array.copy(): dimensions mismatch\");");
    printer.println();

    if (type.isPrimitive())
    {
      printer.println("memcpy(");
      printer.addIndent();
      printer.println(dstName + "->data + " + dstOffsetName + " * sizeof(" + cConverter.cTypeName(arrayElementType) + "),");
      printer.println(srcName + "->data + " + srcOffsetName + " * sizeof(" + cConverter.cTypeName(arrayElementType) + "),");
      printer.println(sizeName + " * sizeof(" + cConverter.cTypeName(arrayElementType) + ")");
      printer.decIndent();
      printer.println(");");
    }
    else
    {
      printer.println("for i = 0; i < sizeName; ++i) {");
      printer.addIndent();
      printer.println(dstName + "->data[i + " + dstOffsetName + "] = " + srcName + "->data[i + " + srcOffsetName + "];");
      printer.println("++" + dstName + "->super.___nbRef;");
      printer.decIndent();
      printer.println("}");
    }

    printer.decIndent();
    printer.println("}");
    printer.println();
  }

  private void produceArrayConstructor(Function function, IndentedPrintWriter printer) throws VisitorException
  {
    TypeName arrayElementType = function.getMethod().getType().getArrayElementType();

    printer.print(cConverter.cMethodSignature(function.getMethod()));
    printer.println(" {");
    printer.addIndent();
    printer.println("word32 i;");

    printer.println("ASSERT(this != null, \"in Array.ctor(): instance to create is null\");");

    printer.println("net_nosica_lang_Object_constructor(&this->super);");
    printer.print("this->size = ");
    int nbArg = function.getMethod().nbArg();
    Iterator it = function.getMethod().iterateArguments();
    while (it.hasNext())
    {
      Argument arg = (Argument)it.next();
      printer.print(arg.getName());
      if (it.hasNext())
        printer.print(" * ");
    }
    printer.print(" * sizeof(" + cConverter.cTypeName(arrayElementType) + ")");
    printer.println(";");

    printer.println(
            "this->data = (" +
            cConverter.cTypeName(arrayElementType) +
            "*)Malloc(this->size, __FILE__, __LINE__);"
    );

    it = function.getMethod().iterateArguments();
    int i = 0;
    while (it.hasNext())
    {
      Argument arg = (Argument)it.next();
      printer.println(
          "this->sizeOfDim[" +
          i +
          "] = sizeof(" +
          cConverter.cTypeName(arrayElementType) +
          ") * " +
          arg.getName() +
          ";"
      );
      ++i;
    }
    printer.println("this->nbDim = " + nbArg + ";");

    // call constructors for primitive types or reset references to null
    Type type = env.getTE().getType(arrayElementType);
    printer.print("for (i = 0; i < this->size / sizeof(");
    printer.println(cConverter.cTypeName(arrayElementType) + "); ++i)");
    if (!type.isPrimitive())
      printer.println("  this->data[i] = 0;");
    else
    {
      Method defaultCtor = env.getMR().getDefaultConstructor(arrayElementType, function.getMethod().getType(), new ArrayList());
      printer.print("  "+cConverter.cMethodName(defaultCtor)+"(");
      if (!type.isNative()) printer.print("&");
      printer.println("this->data[i]);");
    }
    printer.decIndent();
    printer.println("}");
    printer.println();
  }

  private void produceArrayDestructor(Function function, IndentedPrintWriter printer) throws VisitorException
  {
    printer.print(cConverter.cMethodSignature(function.getMethod()));
    printer.println(" {");
    printer.addIndent();

    printer.println("word32 i;");

    TypeName arrayElementType = function.getMethod().getType().getArrayElementType();
    Type type = env.getTE().getType(arrayElementType);

    printer.println("ASSERT(this != null, \"in Array.dtor(): instance to destroy is null\");");

    printer.print("for (i = 0; i < this->size / sizeof(");
    printer.print(cConverter.cTypeName(arrayElementType) + "); ++i) ");
    printer.println("{");
    printer.addIndent();

    if (type.isPrimitive())
    {
      Method dtor = type.getDestructor();
      printer.println(cConverter.cMethodName(dtor) + "(&this->data[i]);");
    }
    else
      printer.println("DECREF(this->data[i]);");

    printer.decIndent();
    printer.println("}");
    printer.println("Free(this->data);");

    printer.decIndent();
    printer.println("}");
    printer.println();
  }

  private void produceArrayLengthProperty(Function function, IndentedPrintWriter printer) throws VisitorException
  {
    printer.print(cConverter.cMethodSignature(function.getMethod()));
    printer.println(" {");
    printer.addIndent();

    printer.println("ASSERT(this != null, \"in Array.length: instance is null\");");
    printer.println(
        "return this->size / sizeof(" +
        cConverter.cTypeName(function.getMethod().getType().getArrayElementType()) +
        ");"
    );

    printer.decIndent();
    printer.println("}");
    printer.println();
  }

  private void produceArrayDimensionProperty(Function function, IndentedPrintWriter printer) throws VisitorException
  {
    printer.print(cConverter.cMethodSignature(function.getMethod()));
    printer.println(" {");
    printer.addIndent();

    printer.println("ASSERT(this != null, \"in Array.dimension[]: instance is null\");");
    printer.println("return this->nbDim;");

    printer.decIndent();
    printer.println("}");
    printer.println();
  }

  private void produceInitMethod(IndentedPrintWriter printer, ArrayList records) throws VisitorException
  {
    printer.println("void ___initializer()");
    printer.println("{");
    printer.addIndent();

    Iterator it = records.iterator();
    while (it.hasNext())
    {
      Record r = (Record)it.next();

      Type type = env.getTE().getType(r.getName());

      // don't produc initializer for interfaces
      if (type.isInterface())
      continue;

      // don't produce initializer call for array types (they don't have one)
      if (r.getName().isArray())
        continue;

      // don't produce initializer call to Object
      if (r.getName().equals(NativeTypes.get_net_nosica_lang_Object()))
        continue;

      Method initializer = env.getTE().getType(r.getName()).getInitializer();
      printer.println(cConverter.cMethodName(initializer) + "();");
    }

    printer.decIndent();
    printer.println("}");
  }

  private void produceExitMethod(IndentedPrintWriter printer, ArrayList records) throws VisitorException
  {
    printer.println("void ___deinitializer()");
    printer.println("{");
    printer.addIndent();

    ListIterator it = records.listIterator(records.size());
    while (it.hasPrevious())
    {
      Record r = (Record)it.previous();

    Type type = env.getTE().getType(r.getName());

    // don't produc initializer for interfaces
    if (type.isInterface())
        continue;

      // don't produce deinitializer call for array types (they don't have one)
      if (r.getName().isArray())
        continue;

      // don't produce deinitializer call to Object
      if (r.getName().equals(NativeTypes.get_net_nosica_lang_Object()))
        continue;

      Method deinitializer = env.getTE().getType(r.getName()).getDeinitializer();
      printer.println(cConverter.cMethodName(deinitializer) + "();");
    }

    printer.decIndent();
    printer.println("}");
  }

  private void produceMain(IndentedPrintWriter printer, Function mainFunction) throws VisitorException
  {
    Record stringArray      = env.getIRRE().getRecord(NativeTypes.get_net_nosica_lang_string_array());
    Record charArray        = env.getIRRE().getRecord(NativeTypes.get_net_nosica_lang_char_array());

    String mangledStringArray = cConverter.cTypeName(NativeTypes.get_net_nosica_lang_string_array());
    String mangledStringArrayConstructor = getMangledStringArrayConstructor();

    String mangledCharArray = cConverter.cTypeName(NativeTypes.get_net_nosica_lang_char_array());
    String mangledCharArrayConstructor = getMangledCharArrayConstructor();

    String mangledString = cConverter.cTypeName(NativeTypes.get_net_nosica_lang_string());
    String mangledChar = cConverter.cTypeName(NativeTypes.get_net_nosica_lang_char());

    printer.println("int main(int argc, char **argv) {");
    printer.addIndent();
    printer.println(mangledStringArray + " *args;");
    printer.println("int i; word32 j;");
    printer.println();
    printer.println("memMallocInit();");
    printer.println("___initializer();");
    printer.println();
    printer.println("// create the string[] args");
    printer.println("args = (" + mangledStringArray + "*)mallocAndInit(sizeof(" + mangledStringArray + "), "
                  + stringArray.getTypeCode() + ",__FILE__,__LINE__);");
    printer.println("INCREF(args);");
    printer.println(mangledStringArrayConstructor + "(args, argc);");
    printer.println();

    printer.println("// create the strings within the args");
    printer.println("for(i=0; i<argc; ++i)");
    printer.println("{");
    printer.addIndent();
    printer.println(mangledCharArray + " *buffer;");
    printer.println(mangledString + "* str;");
    printer.println("word32 length;");
    printer.println();

    printer.println("length = (word32)strlen(argv[i]);");
    printer.println("str    = &ARRAY_DATA(args, " + mangledString + ")[i];");
    printer.println();
    printer.println("// create the char[] buffer");
    printer.println("buffer = (" + mangledCharArray + "*)mallocAndInit(sizeof(" + mangledCharArray + "), "
                   + charArray.getTypeCode() + ",__FILE__,__LINE__);");
    printer.println("INCREF(buffer);");
    printer.println(mangledCharArrayConstructor + "(buffer, length);");
    printer.println("// copy the string from C to UNICODE");
    printer.println("for(j=0; j<length; ++j) ARRAY_DATA(buffer, " + mangledChar + ")[j] = htonl(argv[i][j]);");
    printer.println("// change the string content to this new buffer");
    printer.println("DECREF(str->_buffer);");
    printer.println("str->_buffer=buffer;");
    printer.println("//INCREF(str._buffer);");
    printer.println("//DECREF(buffer);");
    printer.println("//DECREF(str->_writable);");
    printer.println("//str->_writable=0;");
    printer.decIndent();
    printer.println("}");

    printer.println();
    printer.println("// call the actual main");
    printer.println(cConverter.cMethodName(mainFunction.getMethod()) + "(args);");
    printer.println();

    printer.println("// handle uncaught exception");
    printer.println("if (" + cConverter.getExceptionVariableName() + ") {");
    printer.addIndent();
    printer.println("printf(\"Uncaught exception : terminating program\\n\");");
    printer.println("DECREF(" + cConverter.getExceptionVariableName() + ");");
    printer.println(cConverter.getExceptionVariableName() + " = 0;");
    printer.decIndent();
    printer.println("}");

    printer.println();
    printer.println("// free the string[] args");
    printer.println("DECREF(args);");

    printer.println();
    printer.println("___deinitializer();");
    printer.println("memMallocDone();");
    printer.println("return 0;");
    printer.decIndent();
    printer.println("}");
  }

  private String getMangledStringArrayConstructor() throws SymbolException
  {
    Type stringArrayType = env.getTE().getType(NativeTypes.get_net_nosica_lang_string_array());
    Iterator it = stringArrayType.iterateMethodsByName("constructor");

    Method m = (Method)it.next();

    return cConverter.cMethodName(m);
  }

  private String getMangledCharArrayConstructor() throws SymbolException
  {
    Type stringArrayType = env.getTE().getType(NativeTypes.get_net_nosica_lang_char_array());
    Iterator it = stringArrayType.iterateMethodsByName("constructor");

    Method m = (Method)it.next();

    return cConverter.cMethodName(m);
  }
}
