/*
  $Id: CNameConverterImpl.java,v 1.24 2004/04/01 21:27:55 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.compiler.production.c;

import net.nosica.parser.node.util.var.TypeName;
import net.nosica.parser.node.util.var.TypeNameComponent;
import net.nosica.parser.node.util.operator.OperatorEnum;

import net.nosica.application.NativeTypes;
import net.nosica.compiler.application.Environment;
import net.nosica.framework.VisitorException;

import net.nosica.compiler.intermediate.Field;
import net.nosica.compiler.intermediate.Expression;

import net.nosica.symbol.MethodKindEnum;

import net.nosica.compiler.typechecker.Type;
import net.nosica.compiler.typechecker.Method;
import net.nosica.compiler.typechecker.Argument;
import net.nosica.compiler.typechecker.ArgumentImpl;
import net.nosica.compiler.typechecker.ResultType;
import net.nosica.compiler.typechecker.ResultTypeImpl;

import java.util.ArrayList;
import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;

public class CNameConverterImpl implements CNameConverter
{
  private Map methodNameCache     = new HashMap();   // Map<Method, String>
  private Map memberNameCache     = new HashMap();   // Map<Field, String>
  private Map arrayGetAccessCache = new HashMap();   // Map<TypeName, String>
  private Map arraySetAccessCache = new HashMap();   // Map<TypeName, String>
  private Map typeNameCache       = new HashMap();   // Map<TypeName, String>
  private Environment env = Environment.instance();

  private static Map operatorNameEquivalence = new HashMap(); // Map<String, String>

  static
  {
    operatorNameEquivalence.put("*=",  "mulassign");
    operatorNameEquivalence.put("/=",  "divassign");
    operatorNameEquivalence.put("+=",  "addassign");
    operatorNameEquivalence.put("-=",  "subassign");
    operatorNameEquivalence.put("%=",  "modassign");
    operatorNameEquivalence.put("^=",  "powassign");
    operatorNameEquivalence.put("=",   "assign");
    operatorNameEquivalence.put("~",   "iassign");
    operatorNameEquivalence.put("==",  "eq");
    operatorNameEquivalence.put("~~",  "ieq");
    operatorNameEquivalence.put("!=",  "ne");
    operatorNameEquivalence.put("!~",  "ine");
    operatorNameEquivalence.put(">",   "gt");
    operatorNameEquivalence.put("<",   "lt");
    operatorNameEquivalence.put(">=",  "ge");
    operatorNameEquivalence.put("<=",  "le");
    operatorNameEquivalence.put("!",   "not");
    operatorNameEquivalence.put("++",  "inc");
    operatorNameEquivalence.put("--",  "dec");
    operatorNameEquivalence.put("+",   "add");
    operatorNameEquivalence.put("-",   "sub");
    operatorNameEquivalence.put("*",   "mul");
    operatorNameEquivalence.put("/",   "div");
    operatorNameEquivalence.put("%",   "mod");
    operatorNameEquivalence.put("^",   "pow");
    operatorNameEquivalence.put("<<",  "lsh");
    operatorNameEquivalence.put(">>",  "rsh");
  }

  private int uniqueCounter = 0;

  public String cMethodName(Method method)
  {
    if (methodNameCache.containsKey(method))
      return (String)methodNameCache.get(method);

    String methodName = decorateMethod(method);
    methodNameCache.put(method, methodName);

    return methodName.toString();
  }

  public String cPolymorphicMethodName(Method method, int counter)
  {
    return cMethodName(method) + "_polymorphicCallSite_" + counter;
  }

  private String getOperatorMangling(String operatorName)
  {
    String operatorMangling = (String)operatorNameEquivalence.get(operatorName);
    if (operatorMangling == null)
    {
      env.terminate("Internal error : no mangling found for operator " + operatorName);
    }
    return operatorMangling;
  }

  public String decorateMethod(Method m)
  {
    ArrayList components = new ArrayList(); // ArrayList<String>

    components.add(cRawTypeName(m.getType()));
    components.add(m.getKind().toString());

    if (m.getName() != null)
    {
      if (
           m.getKind() == MethodKindEnum.infix ||
           m.getKind() == MethodKindEnum.postfix ||
           m.getKind() == MethodKindEnum.prefix
         )
      {
        components.add(getOperatorMangling(m.getName()));
      }
      else
      if (m.getKind() == MethodKindEnum.castOperator)
      {
        components.add(cRawTypeName(m.getResultType().getTypeName()));
      }
      else
      if (m.getKind() == MethodKindEnum.property || m.getKind() == MethodKindEnum.method)
      {
        components.add(m.getName());
      }
    }

    Iterator it = m.iterateArguments();
    while (it.hasNext())
    {
      Argument arg = (Argument)it.next();
      if (arg.isConst())
      {
        components.add("const");
      }

      components.add(cRawTypeName(arg.getTypeName()));
    }

    if (m.isConst())
    {
      components.add("const");
    }
    else
    if (m.isStatic())
    {
      components.add("static");
    }

    StringBuffer decoratedName = new StringBuffer();
    it = components.iterator();
    while (it.hasNext())
    {
      decoratedName.append((String)it.next());
      if (it.hasNext())
      {
        decoratedName.append("_");
      }
    }

    return decoratedName.toString();
  }

  public String cMethodSignature(Method method) throws VisitorException
  {
    StringBuffer result = new StringBuffer();

    // produce signature of method
    result.append("// type [" + method.getType() + "]\n");
    result.append("// method [");

    if (method.isStatic())
      result.append("static ");

    if (method.isSub())
      result.append("sub ");
    else
    {
      if (method.getResultType().isConst())
        result.append("const ");
      result.append(method.getResultType().getTypeName() + " ");
    }

    if (
        method.getKind() == MethodKindEnum.property ||
        method.getKind() == MethodKindEnum.propertyArray
       )
      result.append("property ");

    if (method.getName() != null)
      result.append(method.getName() + " ");

    if (method.getKind() == MethodKindEnum.propertyArray)
      result.append("[]");

    result.append("(");
    Iterator j = method.iterateArguments();
    while (j.hasNext())
    {
      Argument arg = (Argument)j.next();
      if (arg.isConst())
        result.append("const ");
      result.append(arg.getTypeName() + " ");
      result.append(arg.getName());

      if (j.hasNext())
        result.append(", ");
    }
    result.append(")]\n");

    // now produce real C signature
    result.append(cMethodSignatureLine(method, cMethodName(method)));
    return result.toString();
  }

  public String cMethodSignatureLine(Method method, String methodName) throws VisitorException
  {
    StringBuffer result = new StringBuffer();

    if (method.isSub())
      result.append("void ");
    else
    {
      result.append(cTypeName(method.getResultType().getTypeName()));
      if (isResultTypeTranslatedIntoPointer(method.getResultType()))
        result.append(" *");
      result.append(" ");
    }

    result.append(methodName + "(\n");

    if (!method.isStatic())
    {
      // add a hidden ___enclosing variable if necessary
      if (method.getKind() == MethodKindEnum.constructor)
      {
        Type methodType = env.getTE().getType(method.getType());

        if (methodType.isInner())
        {
          // add an 'enclosing' variable
          result.append("  " + cRawTypeName(methodType.getEnclosing()) + " *___enclosing,\n");
        }
      }

      // add a this variable (always passed as a pointer)
      result.append("  " + cTypeName(method.getType()) + " *this");
    }

    Iterator j = method.iterateArguments();
    if (!method.isStatic() && j.hasNext())
    {
      result.append(",\n");
    }

    while (j.hasNext())
    {
      Argument arg = (Argument)j.next();
      result.append("  " + cTypeName(arg.getTypeName()) + " ");

      if (isArgTranslatedIntoPointer(arg))
        result.append("*");

      result.append(arg.getName());

      if (j.hasNext())
        result.append(",\n");
    }

    result.append("\n)");

    return result.toString();
  }

  public String cArrayGetSignature(TypeName typeName, boolean forwardDeclaration) throws VisitorException
  {
    StringBuffer sb = new StringBuffer();

    sb.append(" // " + cRawTypeName(typeName) + " ARRAY GET\n");
    sb.append(cArrayGetSignatureLine(typeName, forwardDeclaration));

    return sb.toString();
  }

  public String cArrayGetSignatureLine(TypeName typeName, boolean forwardDeclaration) throws VisitorException
  {
    StringBuffer sb = new StringBuffer();

    if (forwardDeclaration)
      sb.append("inline ");
    sb.append(cTypeName(typeName.getArrayElementType()));
    sb.append(" ");
    if (isResultTypeTranslatedIntoPointer(new ResultTypeImpl(typeName.getArrayElementType(), false)))
      sb.append("*");
    sb.append(cArrayGetName(typeName));
    sb.append("(\n");
    sb.append("\t" + cTypeName(typeName) + " ");
    if (isArgTranslatedIntoPointer(new ArgumentImpl(true, typeName, "")))
      sb.append("*");
    sb.append("array,\n");
    for (int i = 0; i < typeName.getDim(); ++i)
    {
      sb.append("\tword32 dim" + i);
      if (i < typeName.getDim() - 1)
        sb.append(",\n");
    }
    sb.append("\n)");

    return sb.toString();
  }

  public String cArrayGetName(TypeName typeName)
  {
    if (arrayGetAccessCache.containsKey(typeName))
      return (String)arrayGetAccessCache.get(typeName);

    StringBuffer sb = new StringBuffer();
    sb.append(cRawTypeName(typeName));
    sb.append("___array_get");

    arrayGetAccessCache.put(typeName, sb.toString());
    return sb.toString();
  }

  public String cArraySetSignature(TypeName typeName, boolean forwardDeclaration) throws VisitorException
  {
    StringBuffer sb = new StringBuffer();

    sb.append(" // " + cRawTypeName(typeName) + " ARRAY SET\n");
    sb.append(cArraySetSignatureLine(typeName, forwardDeclaration));

    return sb.toString();
  }

  public String cArraySetSignatureLine(TypeName typeName, boolean forwardDeclaration) throws VisitorException
  {
    StringBuffer sb = new StringBuffer();

    if (forwardDeclaration)
      sb.append("inline ");
    sb.append("void ");
    sb.append(cArraySetName(typeName));
    sb.append("(\n");
    sb.append("\t" + cTypeName(typeName) + " ");
    if (isArgTranslatedIntoPointer(new ArgumentImpl(true, typeName, "")))
      sb.append(" *");
    sb.append("array,\n");
    for (int i = 0; i < typeName.getDim(); ++i)
    {
      sb.append("\tword32 dim" + i + ",\n");
    }
    sb.append("\t" + cTypeName(typeName.getArrayElementType()) + " ");
    if (isArgTranslatedIntoPointer(new ArgumentImpl(true, typeName.getArrayElementType(), "")))
      sb.append("*");
    sb.append("data\n");
    sb.append(")");

    return sb.toString();
  }

  public String cArraySetName(TypeName typeName)
  {
    if (arraySetAccessCache.containsKey(typeName))
      return (String)arraySetAccessCache.get(typeName);

    StringBuffer sb = new StringBuffer();
    sb.append(cRawTypeName(typeName));
    sb.append("___array_set");

    arraySetAccessCache.put(typeName, sb.toString());
    return sb.toString();
  }

  public String cTypeName(TypeName typeName) throws VisitorException
  {
    Type type = env.getTE().getType(typeName);

    if (type.isInterface())
    {
      // we always convert an interface to a pointer to net.nosica.lang.Object
      // we let the caller add the '*', as returned by
      // isExpTranslatedIntoPointer, and isArgTranslatedIntoPointer
      return cRawTypeName(NativeTypes.get_net_nosica_lang_Object());
    }

    return cRawTypeName(typeName);
  }

  public String cMemberName(Field field)
  {
    if (memberNameCache.containsKey(field))
      return (String)memberNameCache.get(field);

    StringBuffer result = new StringBuffer();

    if (field.isStatic())
      result.append(cRawTypeName(field.getEnclosing())).append("_").append(field.getName());
    else
      result.append(field.getName());

    memberNameCache.put(field, result.toString());

    return result.toString();
  }

  public String cMemberTypeName(Field field) throws VisitorException
  {
    boolean needStar = false;

    if (!field.getName().equals("super"))
      needStar = true;

    Type type = env.getTE().getType(field.getTypeName());

    if (type.isInterface())
      return cRawTypeName(NativeTypes.get_net_nosica_lang_Object()) + "*";

    if (type.isPrimitive())
      return cRawTypeName(field.getTypeName());
    else
    if (needStar)
      return cRawTypeName(field.getTypeName()) + "*";
    else
      return cRawTypeName(field.getTypeName());
  }

  public String cRawTypeName(TypeName typeName)
  {
    // if name is already in cache, return it !
    if (typeNameCache.containsKey(typeName))
      return (String)typeNameCache.get(typeName);

    // if name is an array
    if (typeName.isArray())
    {
      // get array element representation
      StringBuffer result = new StringBuffer(cRawTypeName(typeName.getArrayElementType()));
      // append array part
      result.insert(0, "array_" + typeName.getDim() + "d___");

      // write it in cache
      typeNameCache.put(typeName, result.toString());

      // adn return it !
      return result.toString();
    }

    // else
    StringBuffer result = new StringBuffer();
    boolean needUniqueCounter = false;

    // iterate package components
    Iterator it = typeName.iterateComponents();
    while (it.hasNext())
    {
      TypeNameComponent tnc = (TypeNameComponent)it.next();

      // append package
      result.append(tnc.getPackageElement());

      // if component is generic, only mark it (to avoid having very very long name !)
      if (tnc.hasGenericExtension())
        needUniqueCounter = true;

      // add separator if there is component left
      if (it.hasNext())
        result.append("_");
    }

    // if name was generic, append a uid
    if (needUniqueCounter)
      result.append("___" + ++uniqueCounter);

    // write it in cache
    typeNameCache.put(typeName, result.toString());

    // and return it !
    return result.toString();
  }

  public String cRadixFileName(TypeName typeName)
  {
    if (typeName.isArray() || typeName.isGeneric())
      throw new UnsupportedOperationException();

    StringBuffer result = new StringBuffer();
    Iterator it = typeName.iterateComponents();
    while (it.hasNext())
    {
      TypeNameComponent tnc = (TypeNameComponent)it.next();
      result.append(tnc.getPackageElement());
      if (it.hasNext())
        result.append("/");
    }

    return result.toString();
  }

  public String cIncludeTypeName(TypeName typeName)
  {
    return cRadixFileName(typeName) + ".h";
  }

  public String cCharValue(char ch)
  {
    StringBuffer sb = new StringBuffer();
    sb.append(ch << 24);
    return sb.toString();
  }

  public boolean isExpTranslatedIntoPointer(Expression exp) throws VisitorException
  {
    // handle special case for 'this' pointer
    if (exp.isThis())
      return true;

    // handle special case for 'super' field
    if (exp.isSuper())
      return false;

    boolean isPrimitive = env.getTE().getType(exp.getTypeName()).isPrimitive();

    // handle reference type
    if (!isPrimitive)
      return true;

    // handle primitive type
    if (exp.isArgument())
    {
      // primitive non const argument are passed as pointers ...
      net.nosica.compiler.intermediate.Argument arg =
        (net.nosica.compiler.intermediate.Argument)exp;

      if (!arg.isConst())
        return true;

      return false;
    }

    return false;
  }

  public boolean isArgTranslatedIntoPointer(Argument arg) throws VisitorException
  {
    boolean isPrimitive = env.getTE().getType(arg.getTypeName()).isPrimitive();

    if (isPrimitive && arg.isConst())
      return false;

    return true;
  }

  public boolean isResultTypeTranslatedIntoPointer(ResultType resultType) throws VisitorException
  {
    boolean isPrimitive = env.getTE().getType(resultType.getTypeName()).isPrimitive();

    if (isPrimitive)
      return false;

    return true;
  }

  public String cCleanUpPointLabelName(int cleanUpPoint)
  {
    return "cleanUpPoint" + cleanUpPoint;
  }

  public String getExceptionVariableName()
  {
    return "___e";
  }

  public String cSourceFileName(TypeName file)
  {
    return "source.c";
  }

  public String cObjFileName(TypeName file)
  {
    return "source.o";
  }

  public String cNativeLibraryName(TypeName packageName)
  {
    StringBuffer sb = new StringBuffer();

//    sb.append("lib");
    sb.append("nos___");
    Iterator it = packageName.iterateComponents();
    while (it.hasNext())
    {
      TypeNameComponent tnc = (TypeNameComponent)it.next();
      sb.append(tnc.getPackageElement());
      if (it.hasNext())
        sb.append("_");
    }

//    sb.append(".a");

    return sb.toString();
  }

  public String cMacroDefineName(TypeName typeName)
  {
    StringBuffer result = new StringBuffer();

    Iterator it = typeName.iterateComponents();
    while (it.hasNext())
    {
      TypeNameComponent component = (TypeNameComponent)it.next();

      result.append("_").append(component.getPackageElement().toUpperCase());
    }

    result.append("_HEADER_");

    return result.toString();
  }
}
