/*
  $Id: FunctionProducerVisitor.java,v 1.30 2004/03/04 22:16:16 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.compiler.production.c;

import net.nosica.parser.node.util.var.TypeName;
import net.nosica.compiler.application.Environment;
import net.nosica.application.NativeTypes;
import net.nosica.framework.VisitorException;

import net.nosica.symbol.MethodKindEnum;

import net.nosica.compiler.production.c.intermediate.CIRVisitor;

import net.nosica.compiler.intermediate.Function;
import net.nosica.compiler.intermediate.Record;
import net.nosica.compiler.intermediate.Expression;
import net.nosica.compiler.intermediate.Call;
import net.nosica.compiler.intermediate.MemberAccess;
import net.nosica.compiler.intermediate.Sequence;
import net.nosica.compiler.intermediate.StatementExpression;
import net.nosica.compiler.intermediate.Literal;
import net.nosica.compiler.intermediate.Variable;
import net.nosica.compiler.intermediate.Move;
import net.nosica.compiler.intermediate.ExtendedSequence;
import net.nosica.compiler.intermediate.Jump;
import net.nosica.compiler.intermediate.LabelName;
import net.nosica.compiler.intermediate.ConditionalJump;
import net.nosica.compiler.intermediate.Cast;
import net.nosica.compiler.intermediate.InstanceOf;
import net.nosica.compiler.intermediate.RawAllocation;
import net.nosica.compiler.intermediate.Init;
import net.nosica.compiler.intermediate.Statement;
import net.nosica.compiler.intermediate.StatementList;
import net.nosica.compiler.intermediate.ArrayGet;
import net.nosica.compiler.intermediate.ArrayGetImpl;
import net.nosica.compiler.intermediate.ArraySet;
import net.nosica.compiler.intermediate.ConstraintError;
import net.nosica.compiler.intermediate.TryRegister;
import net.nosica.compiler.intermediate.TryUnregister;
import net.nosica.compiler.intermediate.Throw;
import net.nosica.compiler.intermediate.VariableScopeLeave;
import net.nosica.compiler.intermediate.Return;

import net.nosica.compiler.production.c.intermediate.CheckNull;
import net.nosica.compiler.production.c.intermediate.DecRef;
import net.nosica.compiler.production.c.intermediate.DecRefImpl;
import net.nosica.compiler.production.c.intermediate.IncRef;
import net.nosica.compiler.typechecker.Type;
import net.nosica.compiler.typechecker.Method;
import net.nosica.compiler.typechecker.Label;

import net.nosica.util.IndentedPrintWriter;
import net.nosica.util.Pair;

import java.util.Iterator;
import java.util.ListIterator;
import java.util.Map;
import java.util.TreeMap;
import java.util.Set;
import java.util.HashSet;
import java.util.ArrayList;
import java.util.Stack;

public class FunctionProducerVisitor implements CIRVisitor
{
  private IndentedPrintWriter printer;
  private CNameConverter cConverter;
  private Environment env = Environment.instance();
  private Map polymorphicCallSitesIds;  // Map<Call, Integer>
  private Map stringLiteralMap; // Map<LIteral, Integer>
  private Type thisType;
  private ArrayList functionArgs; // ArrayList<Variable>

  // exception handling data
  private ArrayList liveVariables = new ArrayList(); // ArrayList<Variable>
  private ArrayList liveVariablesByCleanUpPoint = new ArrayList(); // ArrayList<ArrayList<Variable> >
  private boolean emitExceptionHandlingCode = false;

  private Stack activeTryRegister = new Stack();                    // Stack<TryRegister> this one is used when visiting to keep track of active try site
  private Set handledTryRegister = new HashSet();               // Set<TryRegister> this one is used to keep track of the assigned TryRegister by cleanUpPoint
  private Map tryRegisterByCleanUpPoint = new TreeMap(); // Map<Integer, TryRegister> This one keep track of which cleanUpPoint have an associated TryRegister

  private Return lastReturn = null;

  public FunctionProducerVisitor(
          IndentedPrintWriter printer,
          CNameConverter cConverter,
          Map polymorphicCallSitesIds,
          Map stringLiteralMap,
          Type thisType,
          ArrayList functionArgs
  )
  {
    this.printer = printer;
    this.cConverter = cConverter;
    this.polymorphicCallSitesIds = polymorphicCallSitesIds;
    this.stringLiteralMap = stringLiteralMap;
    this.thisType = thisType;
    this.functionArgs = functionArgs;
  }

  /**
   * \brief return the list of cleanUpPoint along with the variables to destroy at each cleanUpPoint
   *
   * \return Map<Integer, Set<Variable> >
   */
  public ArrayList getLiveVariablesByCleanUpPoint() { return liveVariablesByCleanUpPoint; }

  public void visit(Call call) throws VisitorException
  {
    if (call.hasSubscript())
    {
      Map resolvedMethods = env.getPTNE().getResolvedMethod(call);

      if (call.getMethod().getKind() == MethodKindEnum.constructor)
      {
        // a constructor is never polymorphic, just use resolved function
        emitConstructorCall(
          call.getSubscript(),
          call.getMethod(),
          call.iterateArgs()
        );

        emitExceptionHandling(env.getIRFE().getFunction(call.getMethod()));

        return;
      }
      else
      if (call.getMethod().getKind() == MethodKindEnum.destructor)
      {
        emitDestructorCall(call.getSubscript(), call.getMethod());

        return;
      }

      Type type = env.getTE().getType(call.getSubscript().getTypeName());

      if (type.isPrimitive())
      {
        // In fact, as type is primitive, it is not polymorphic at all !
        emitNonPolymorphicCall(
          call.getSubscript(),
          call.getMethod(),
          call.iterateArgs()
        );

        emitExceptionHandling(env.getIRFE().getFunction(call.getMethod()));

        return;
      }

      if (resolvedMethods.size() == 1)
      {
        // -- Non polymorphic call (this is a loop, because it can contain
        //    the actual "non-polymorphic" type and None and we don't know
        //    a-priori the position of None in the list
        Iterator it = resolvedMethods.entrySet().iterator();
        while (it.hasNext())
        {
          Map.Entry entry = (Map.Entry)it.next();
          Function function = (Function)entry.getValue();

          if (function == null) continue;

          // now call the function
          emitNonPolymorphicCall(
            call.getSubscript(),
            function.getMethod(),
            call.iterateArgs()
          );

          emitExceptionHandling(function);
        }

        return;
      }
      else
      if (resolvedMethods.size() > 1)
      {
        // -- polymorphic call
        Integer counter = (Integer)polymorphicCallSitesIds.get(call);

        emitPolymorphicCall(
          call.getSubscript(),
          call.getMethod(),
          counter.intValue(),
          call.iterateArgs()
        );

        emitExceptionHandling(resolvedMethods);

        return;
      }
      else
      {
        // means resolvedMethods.size() == 0
        // this is an internal error
        env.terminate("Internal error :  no methods resolved");
      }
    }

    Function f = env.getIRFE().getFunction(call.getMethod());

    // static call
    emitStaticCall(
      f.getMethod(),
      call.iterateArgs()
    );

    emitExceptionHandling(f);
  }

  private void emitSubscript(
    Expression subscript,
    TypeName targetTypeName
  ) throws VisitorException
  {
    printer.print("(" + cConverter.cTypeName(targetTypeName) + "*)");
    if (!cConverter.isExpTranslatedIntoPointer(subscript))
      printer.print("&");

    // emit subscript passing
    subscript.accept(this);
  }

  private void emitArgument(
    Expression arg,                                     // argument to pass
    net.nosica.compiler.typechecker.Argument signature  // signature of argument in called method
  ) throws VisitorException
  {
    boolean argIsPointer = cConverter.isExpTranslatedIntoPointer(arg);
    boolean signatureIsPointer = cConverter.isArgTranslatedIntoPointer(signature);

    if (signatureIsPointer)
    {
      printer.print("(");
      printer.print(cConverter.cTypeName(signature.getTypeName()));
      printer.print(" *)");
    }

    if (argIsPointer && !signatureIsPointer)
      printer.print("*");
    else
    if (!argIsPointer && signatureIsPointer)
      printer.print("&");

    // emit argument passing
    printer.print("(");
    arg.accept(this);
    printer.print(")");
  }

  private void emitConstructorCall(
    Expression subscript,   // subscript
    Method constructor,     // called constructor
    Iterator args           // Iterator<Expression>
  ) throws VisitorException
  {
    // emit method name
    printer.print(cConverter.cMethodName(constructor) + "(");

    Type type = env.getTE().getType(constructor.getType());

    // constructing an inner type
    if (type.isInner())
    {
      // that is, add the enclosing pointer to the list of variable to pass ...

      // this method's enclosing type is not an inner type
      if (!thisType.isInner())
        printer.print("this, ");
      else
      {
        Iterator it = thisType.iterateExtends();
        while (it.hasNext())
        {
          TypeName typeName = (TypeName)it.next();

          // class A
          // {
          //   SomeInterface createInner {
          //     return new C();
          //   }
          //
          //   inner class B {
          //   }
          //
          //   inner class C extends B {
          //     constructor {
          //       super();     // this is a call to super.constructor(), we
          //     }              // have to pass ___enclosing as enclosing variable
          //   }
          // }
          if (typeName.equals(type.getName()))
            printer.print("___enclosing, ");
          else
            printer.print("this, ");
        }
      }
    }

    // emit subscript (the cached 'this' variable)
    emitSubscript(subscript, constructor.getType());

    // now with arguments
    Iterator signatureArgs = constructor.iterateArguments();
    while (args.hasNext() && signatureArgs.hasNext())
    {
      if (args.hasNext())
        printer.print(", ");

      Expression exp = (Expression)args.next();
      net.nosica.compiler.typechecker.Argument signature =
        (net.nosica.compiler.typechecker.Argument)signatureArgs.next();

      emitArgument(exp, signature);
    }
    printer.print(")");
  }

  private void emitDestructorCall(
    Expression subscript,   // subscript
    Method destructor      // called destructor
  ) throws VisitorException
  {
    // a destructor called directly. Either a primitive one, or
    // a parent destructor called from a derived one
    emitNonPolymorphicCall(
      subscript,
      destructor,
      new ArrayList().iterator()
    );
  }

  private void emitNonPolymorphicCall(
    Expression subscript,   // subscript
    Method method,          // called method
    Iterator args           // Iterator<Expression>
  ) throws VisitorException
  {
    // emit method name
    printer.print(cConverter.cMethodName(method) + "(");

    // emit subscript (the cached 'this' variable)
    emitSubscript(subscript, method.getType());

    // now emit arguments
    Iterator signatureArgs = method.iterateArguments();
    while (args.hasNext() && signatureArgs.hasNext())
    {
      if (args.hasNext())
        printer.print(", ");

      Expression exp = (Expression)args.next();
      net.nosica.compiler.typechecker.Argument signature =
        (net.nosica.compiler.typechecker.Argument)signatureArgs.next();

      emitArgument(exp, signature);
    }
    printer.print(")");
  }

  private void emitPolymorphicCall(
    Expression subscript, // subscript
    Method method,        // polymorphic called method
    int counter,          // polymorphic identifier
    Iterator args         // Iterator<Expression>
  ) throws VisitorException
  {
    // emit method name
    printer.print(cConverter.cPolymorphicMethodName(method, counter) + "(");

    // emit subscript (the cached 'this' variable)
    emitSubscript(subscript, method.getType());

    // now emit arguments
    Iterator signatureArgs = method.iterateArguments();
    while (args.hasNext() && signatureArgs.hasNext())
    {
      if (args.hasNext())
        printer.print(", ");

      Expression exp = (Expression)args.next();
      net.nosica.compiler.typechecker.Argument signature =
        (net.nosica.compiler.typechecker.Argument)signatureArgs.next();

      emitArgument(exp, signature);
    }
    printer.print(")");
  }

  private void emitStaticCall(
    Method method,
    Iterator args // Iterator<Expression>
  ) throws VisitorException
  {
    // emit method name
    printer.print(cConverter.cMethodName(method) + "(");

    // don't emit subscript (the cached 'this' variable)

    // now emit arguments
    Iterator signatureArgs = method.iterateArguments();
    while (args.hasNext() && signatureArgs.hasNext())
    {
      Expression exp = (Expression)args.next();
      net.nosica.compiler.typechecker.Argument signature =
        (net.nosica.compiler.typechecker.Argument)signatureArgs.next();

      emitArgument(exp, signature);

      if (args.hasNext())
        printer.print(", ");
    }
    printer.print(")");
  }

  public void visit(ArrayGet arrayGet) throws VisitorException
  {
    // print macro for array access
    printer.print(cConverter.cArrayGetName(arrayGet.getSubscript().getTypeName()) + "(");
    // print subscript
    arrayGet.getSubscript().accept(this);

    // now dimensions
    printer.print(", ");

    Iterator it = arrayGet.iterateArgs();
    while (it.hasNext())
    {
      Expression exp = (Expression)it.next();
      exp.accept(this);

      if (it.hasNext())
        printer.print(", ");
    }

    printer.print(")");
  }

  public void visit(ArraySet arraySet) throws VisitorException
  {
    // print macro for array access
    printer.print(cConverter.cArraySetName(arraySet.getSubscript().getTypeName()) + "(");
    // print subscript
    arraySet.getSubscript().accept(this);

    // now dimensions
    printer.print(", ");

    Iterator it = arraySet.iterateArgs();
    while (it.hasNext())
    {
      Expression exp = (Expression)it.next();
      exp.accept(this);
      printer.print(", ");
    }

    // and finally rhs
    emitRhsOfAssignment(new ArrayGetImpl(arraySet.getSubscript(), arraySet.getArgs()), arraySet.getRhs());

    // array set is a statement, don't forget to add a ";"
    printer.println(");");
  }

  public void visit(MemberAccess memberAccess) throws VisitorException
  {
    if (memberAccess.hasSubscript() && !memberAccess.getField().isStatic())
    {
      // print underlying member access
      memberAccess.getSubscript().accept(this);

      // handle super
/*      if (!memberAccess.isSuper())
      {
*/
        boolean needPointer = !env.getPTNE().getPTN(memberAccess.getSubscript()).isPrimitive();
        if (memberAccess.getSubscript().isThis())
          needPointer = true;
        else
        if (memberAccess.getSubscript().isSuper())
          needPointer = false;

        if (needPointer)
          printer.print("->");
        else
          printer.print(".");

        // print field access
        printer.print(cConverter.cMemberName(memberAccess.getField()));
//      }
    }
    else
    {
      // access to a static member
      printer.print(cConverter.cMemberName(memberAccess.getField()));
    }
  }

  public void visit(Literal literal) throws VisitorException
  {
    if (literal.getTypeName().equals(NativeTypes.get_net_nosica_lang_char()))
    {
      // convert char literal
      printer.print(cConverter.cCharValue(literal.getStrValue().charAt(0)));
    }
    else
    if (literal.getTypeName().equals(NativeTypes.get_net_nosica_lang_string()))
    {
      // convert string literal
      int counter = ((Integer)stringLiteralMap.get(literal.getStrValue())).intValue();
      printer.print("STRING_LITERAL_"+counter);
    }
    else
    {
      // numeric one or null : just print it
      printer.print(literal.getStrValue());
    }
  }

  public void visit(Sequence sequence) throws VisitorException
  {
    // should have been removed long ago
    throw new UnsupportedOperationException();
  }

  public void visit(StatementExpression statementExpression) throws VisitorException
  {
    // print contained expression
    statementExpression.getExpression().accept(this);
    // and finish it !
    printer.println(";");
  }

  public void visit(Variable variable) throws VisitorException
  {
    // just print variable name
    printer.print(variable.getName());

    // register this variable in list of live variable for exception handling
    if (!functionArgs.contains(variable) && !liveVariables.contains(variable))
      liveVariables.add(variable);
  }

  private void emitRhsOfAssignment(
    Expression lhs,
    Expression rhs
  ) throws VisitorException
  {
    boolean lhsIsPointer = cConverter.isExpTranslatedIntoPointer(lhs);
    boolean rhsIsPointer = cConverter.isExpTranslatedIntoPointer(rhs);

    if (lhsIsPointer && !rhsIsPointer)
      printer.print("&");
    else
    if (!lhsIsPointer && rhsIsPointer)
      printer.print("*(" + cConverter.cTypeName(lhs.getTypeName()) + " *)");
    else
    if (lhsIsPointer && rhsIsPointer)
    {
      printer.print("(" + cConverter.cTypeName(lhs.getTypeName()) + " *)");
    }

    // now on with the rhs
    rhs.accept(this);
  }

  public void visit(Move move) throws VisitorException
  {
    // write the lhs
    move.getVariable().accept(this);

    printer.print(" = ");

    emitRhsOfAssignment(move.getVariable(), move.getRhs());

    // a move is a statement, let's finish c line
    printer.println(";");
  }

  public void visit(ExtendedSequence exSeq) throws VisitorException
  {
    // should have been removed long ago
    throw new UnsupportedOperationException();
  }

  public void visit(Jump jump) throws VisitorException
  {
    // do a c jump to label
    printer.println("goto " + jump.getLabel().getName() + ";");
  }

  public void visit(LabelName labelName) throws VisitorException
  {
    // print a c label name
    printer.println(labelName.getLabel().getName() + ":");
  }

  public void visit(ConditionalJump conditionalJump) throws VisitorException
  {
    // real final implementation should present only one if branch ...
    printer.print("if (");
    conditionalJump.getCondition().accept(this);
    printer.println(")");
    printer.addIndent();
    printer.println("goto " + conditionalJump.getTrueLabel().getName() + ";");
    printer.decIndent();
    printer.println("else");
    printer.addIndent();
    printer.println("goto " + conditionalJump.getFalseLabel().getName() + ";");
    printer.decIndent();
  }

  public void visit(Cast castExp) throws VisitorException
  {
    // (only if analysis has shown it is possible)
    castExp.getExpression().accept(this);
  }

  public void visit(InstanceOf instanceOfExp) throws VisitorException
  {
    printer.print("___inherits(");
    printer.print("((" + cConverter.cTypeName(NativeTypes.get_net_nosica_lang_Object()) + "*)");
    instanceOfExp.getExpression().accept(this);
    Record record = env.getIRRE().getRecord(instanceOfExp.getRhsTypeName());
    printer.print(")->___rtti->___typeCode, " + record.getTypeCode() + ")");
  }

  public void visit(RawAllocation rawAllocation) throws VisitorException
  {
    // allocation left to c code
    Record record = env.getIRRE().getRecord(rawAllocation.getTypeName());

    printer.print("(" + cConverter.cTypeName(rawAllocation.getTypeName()) + " *)mallocAndInit("
                      + "sizeof(" + cConverter.cRawTypeName(rawAllocation.getTypeName()) + "), "
                      + record.getTypeCode()+", "
                      + "__FILE__,__LINE__)");
  }

  public void visit(IncRef incRef) throws VisitorException
  {
    // INCREF implementation left to c code
    printer.print("INCREF(");
    incRef.getVariable().accept(this);
    printer.println(");");
  }

  public void visit(DecRef decRef) throws VisitorException
  {
    // DECREF implementation left to c code
    printer.print("DECREF(");
    decRef.getVariable().accept(this);
    printer.println(");");
  }

  public void visit(Init init) throws VisitorException
  {
    // delegate implementation to the Move statement ...
    // do we really need an Init statement ?
    visit((Move)init);
  }

  public void visit(StatementList statementList) throws VisitorException
  {
    // just iterates all statements
    Iterator it = statementList.iterateStatementList();
    while (it.hasNext())
    {
      Statement statement = (Statement)it.next();
      statement.accept(this);

      if (emitExceptionHandlingCode)
      {
        emitExceptionHandlingCode = false;
        emitExceptionHandlingCode(false);
      }
    }
  }
  public void visit(CheckNull checkNull) throws VisitorException
  {
    // @TODO@ : insert filename and line number of the original nosica source code
    printer.print("if (");
    checkNull.getExpression().accept(this);
    printer.println(" == null) {");
    printer.addIndent();
    printer.println("ASSERT(false, \"instance is null\");");
    printer.decIndent();
    printer.println("}");
  }
  public void visit(ConstraintError constraintError) throws VisitorException
  {
    // while we don't have exceptions, we use an assert
    printer.println("ASSERT(false, \"" + constraintError.getErrorString() + "\");");
  }
  public void visit(TryRegister tryRegister) throws VisitorException
  {
    activeTryRegister.push(tryRegister);
  }
  public void visit(TryUnregister tryUnregister) throws VisitorException
  {
    activeTryRegister.pop();
  }
  public void visit(Throw throwStatement) throws VisitorException
  {
    printer.print(cConverter.getExceptionVariableName() + " = ");
    printer.print("(");
    printer.print(cConverter.cTypeName(NativeTypes.get_net_nosica_lang_Object()));
    printer.print(" *)");
    throwStatement.getExpression().accept(this);
    printer.println(";");
    printer.println("INCREF(" + cConverter.getExceptionVariableName() + ");");

    emitExceptionHandlingCode(true);
  }
  public void visit(VariableScopeLeave varScopeLeave) throws VisitorException
  {
    // nothing to do, just a marker for garbage collecting implementation

    // for exception handling, to maintain list of live variables
    liveVariables.remove(varScopeLeave.getVariable());
  }
  public void visit(Return returnStatement) throws VisitorException
  {
    printer.print("return ");
    returnStatement.getResult().accept(this);
    printer.println(";");

    lastReturn = returnStatement;
  }

  /**
   *
   * \param throwing indicates if the caller is visit(ThrowException)
   */
  private void emitExceptionHandlingCode(boolean throwing)
  {
    int cleanUpPoint = liveVariablesByCleanUpPoint.size();

    if (throwing)
    {
      printer.println("goto " + cConverter.cCleanUpPointLabelName(cleanUpPoint) + ";");
    }
    else
    {
      printer.println("if (" + cConverter.getExceptionVariableName() + ") goto " + cConverter.cCleanUpPointLabelName(cleanUpPoint) + ";");
    }

    ArrayList copyOfLiveVariables = new ArrayList(liveVariables);
    liveVariablesByCleanUpPoint.add(copyOfLiveVariables);

    if (!activeTryRegister.empty())
    {
      TryRegister tryRegister = (TryRegister)activeTryRegister.peek();
      if (!handledTryRegister.contains(tryRegister))
      {
        handledTryRegister.add(tryRegister);
        tryRegisterByCleanUpPoint.put(new Integer(cleanUpPoint), tryRegister);
      }
    }
  }

  private void emitExceptionHandling(Function f)
  {
    if (!f.getMethod().getThrowList().isEmpty() || !env.getEE().getExceptions(f).isEmpty())
    {
      emitExceptionHandlingCode = true;
    }
  }
  /**
   *
   * \param resolvedMethods Map<TypeName, Function>
   */
  private void emitExceptionHandling(Map resolvedMethods)
  {
    Iterator it = resolvedMethods.entrySet().iterator();
    while (it.hasNext())
    {
      Map.Entry entry = (Map.Entry)it.next();
      Function f = (Function)entry.getValue();

      if (!f.getMethod().getThrowList().isEmpty() || !env.getEE().getExceptions(f).isEmpty())
      {
        emitExceptionHandlingCode = true;
        return;
      }
    }
  }

  public void emitExceptionHandlingEpilogue() throws VisitorException
  {
    ListIterator i = liveVariablesByCleanUpPoint.listIterator(liveVariablesByCleanUpPoint.size());
    while (i.hasPrevious())
    {
      int index = i.previousIndex();
      printer.println(cConverter.cCleanUpPointLabelName(index) + ":");

      // take the last element of the current list of live variables
      ArrayList liveVariables = (ArrayList)i.previous();

      ListIterator j = liveVariables.listIterator(liveVariables.size());
      while (j.hasPrevious())
      {
        Variable var = (Variable)j.previous();

         // let's try to find var in the preceding liveVariables
         int index2 = -1;

         while (i.hasPrevious())
         {
           index2 = i.previousIndex();
           ArrayList liveVariables2 = (ArrayList)i.previous();
           if (liveVariables2.contains(var))
           {
             break;
           }
         }

         // don't forget to rewind to previous iterator after the search
         i = liveVariablesByCleanUpPoint.listIterator(index);

         if (index2 != -1)
         {
           // we have found this variable to be destroyed in another cleanUpPoint, just use this one
           printer.println("goto " + cConverter.cCleanUpPointLabelName(index2) + ";");
           break;
         }

        // destroy this variable
        Type type = env.getTE().getType(var.getTypeName());
        if (type.isPrimitive())
        {
          emitNonPolymorphicCall(var, type.getDestructor(), new ArrayList().iterator());
          printer.println(";");
        }
        else
        {
          visit(new DecRefImpl(var));
        }
      }

      // install here tryRegister
      TryRegister tryRegister = (TryRegister)tryRegisterByCleanUpPoint.get(new Integer(index));
      if (tryRegister != null)
      {
        if (tryRegister.hasFinallyStatement())
        {
          // emit finally statement here
          tryRegister.getFinallyStatement().accept(this);
        }

        Iterator it = tryRegister.getExceptionMap().entrySet().iterator();
        while (it.hasNext())
        {
          Map.Entry entry           = (Map.Entry)it.next();
          TypeName typeName = (TypeName)entry.getKey();
          Pair pair                      = (Pair)entry.getValue();
          Label catchLabel        = (Label)pair.first;
          Variable exVar            = (Variable)pair.second;

          // we have to see if the exception is being catched
          Record record = env.getIRRE().getRecord(typeName);
          printer.println("if (___inherits(" + cConverter.getExceptionVariableName() + "->___rtti->___typeCode, " + record.getTypeCode() + ")) {");
          printer.addIndent();
          printer.println(exVar.getName() + "=" + cConverter.getExceptionVariableName() + ";");
          printer.println(cConverter.getExceptionVariableName() + " = 0;");
          printer.println("goto " + catchLabel.getName() + ";");
          printer.decIndent();
          printer.println("}");
        }
      }
    }
    // don't forget to add a decref for arguments of method
    Iterator it = functionArgs.iterator();
    while (it.hasNext())
    {
      Variable var = (Variable)it.next();

      Type type = env.getTE().getType(var.getTypeName());
      if (!type.isPrimitive())
      {
        visit(new DecRefImpl(var));
      }
    }

    // don't forget to emit a return statement if method return a value ...
    if (lastReturn != null)
    {
      lastReturn.accept(this);
    }
  }
}