/*
  $Id: PolymorphicCallSiteProducerVisitor.java,v 1.17 2004/03/04 22:16:16 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.compiler.production.c;

import net.nosica.parser.node.util.var.TypeName;
import net.nosica.compiler.application.Environment;
import net.nosica.application.NativeTypes;
import net.nosica.framework.VisitorException;

import net.nosica.compiler.analysis.PossibleTypeName;

import net.nosica.compiler.production.c.intermediate.CIRVisitor;

import net.nosica.compiler.intermediate.Function;
import net.nosica.compiler.intermediate.Record;
import net.nosica.compiler.intermediate.Call;
import net.nosica.compiler.intermediate.MemberAccess;
import net.nosica.compiler.intermediate.Sequence;
import net.nosica.compiler.intermediate.StatementExpression;
import net.nosica.compiler.intermediate.Literal;
import net.nosica.compiler.intermediate.Variable;
import net.nosica.compiler.intermediate.Move;
import net.nosica.compiler.intermediate.ExtendedSequence;
import net.nosica.compiler.intermediate.Jump;
import net.nosica.compiler.intermediate.LabelName;
import net.nosica.compiler.intermediate.ConditionalJump;
import net.nosica.compiler.intermediate.Cast;
import net.nosica.compiler.intermediate.InstanceOf;
import net.nosica.compiler.intermediate.RawAllocation;
import net.nosica.compiler.intermediate.Init;
import net.nosica.compiler.intermediate.Statement;
import net.nosica.compiler.intermediate.StatementList;
import net.nosica.compiler.intermediate.ArrayGet;
import net.nosica.compiler.intermediate.ArraySet;
import net.nosica.compiler.intermediate.ConstraintError;
import net.nosica.compiler.intermediate.TryRegister;
import net.nosica.compiler.intermediate.TryUnregister;
import net.nosica.compiler.intermediate.Throw;
import net.nosica.compiler.intermediate.VariableScopeLeave;
import net.nosica.compiler.intermediate.Return;
import net.nosica.compiler.intermediate.Expression;

import net.nosica.symbol.SymbolException;

import net.nosica.compiler.production.c.intermediate.CheckNull;
import net.nosica.compiler.production.c.intermediate.DecRef;
import net.nosica.compiler.production.c.intermediate.IncRef;
import net.nosica.compiler.typechecker.Type;

import net.nosica.util.IndentedPrintWriter;
import net.nosica.util.SettableInteger;

import java.util.Iterator;
import java.util.Map;

public class PolymorphicCallSiteProducerVisitor implements CIRVisitor
{
  private IndentedPrintWriter printer;
  private CNameConverter cConverter;
  private Environment env = Environment.instance();
  private Map callIds;  // Map<Call, Integer>
  private Map instanceOfIds; // Map<InstanceOfExpression, Integer>

  private SettableInteger counter;

  public PolymorphicCallSiteProducerVisitor(
    IndentedPrintWriter printer,
    CNameConverter cConverter,
    Map polymorphicCallSitesIds,
    Map polymorphicInstanceOfSitesIds,
    SettableInteger counter
  )
  {
    this.printer = printer;
    this.cConverter = cConverter;
    this.callIds = polymorphicCallSitesIds;
    this.instanceOfIds = polymorphicInstanceOfSitesIds;
    this.counter = counter;
  }

  public void visit(Call call) throws VisitorException
  {
    // arguments
    Iterator it = call.iterateArgs();
    while (it.hasNext())
    {
      Expression exp = (Expression)it.next();
      exp.accept(this);
    }

    if (call.hasSubscript())
    {
      Map resolvedMethods = env.getPTNE().getResolvedMethod(call);

      if (resolvedMethods.size() <= 1)
        return;

      // register polymorphic call
      int counterValue = counter.intValue();
      counter.inc();
      callIds.put(call, new Integer(counterValue));

      // emit polymorphic call site function declaration
      printer.println(
        cConverter.cMethodSignatureLine(
          call.getMethod(),
          cConverter.cMethodName(call.getMethod()) + "_polymorphicCallSite_" + counterValue
        )
      );
      printer.println("{");

      // emit polymorphic call site function body
      printer.println("  switch(((" + cConverter.cTypeName(NativeTypes.get_net_nosica_lang_Object()) + " *)this)->___rtti->___typeCode) {");
      it = resolvedMethods.entrySet().iterator();
      while (it.hasNext())
      {
        Map.Entry entry = (Map.Entry)it.next();
        Function function = (Function)entry.getValue();
        TypeName tn = (TypeName)entry.getKey();

        Record record = env.getIRRE().getRecord(tn);
        printer.print("    case " + record.getTypeCode() + ": ");

        TypeName castTypeName = argCastTypeName(function.getMethod().getType());

        if (call.getMethod().isSub())
        {
          emitCall(
            castTypeName,
            cConverter.cMethodName(function.getMethod()),
            function.iterateArgs(),
            function.getMethod().iterateArguments()
          );
          printer.println("; break;");
        }
        else
        {
          printer.print("return ");
          emitCall(
            castTypeName,
            cConverter.cMethodName(function.getMethod()),
            function.iterateArgs(),
            function.getMethod().iterateArguments()
          );
          printer.println(";");
        }
      }
      printer.println("    default: abort();");
      printer.println("  }");
      printer.println("}");
    }
  }

  private void emitCall(
    TypeName subscriptCast,
    String methodName,
    Iterator argsIterator,
    Iterator methodArgs
  ) throws VisitorException
  {
    printer.print(methodName + "((" + cConverter.cTypeName(subscriptCast) + " *)this");
    if (argsIterator.hasNext()) printer.print(", ");

    while (argsIterator.hasNext() && methodArgs.hasNext())
    {
      Variable arg = (Variable)argsIterator.next();
      net.nosica.compiler.typechecker.Argument methodArg = (net.nosica.compiler.typechecker.Argument)methodArgs.next();

      TypeName castTypeName = argCastTypeName(methodArg.getTypeName());
      if (castTypeName != null)
        printer.print("("+cConverter.cTypeName(castTypeName)+" *)");

      printer.print(arg.getName());

      if (argsIterator.hasNext())
        printer.print(", ");
    }
    printer.print(")");
  }

  private TypeName argCastTypeName(TypeName signatureTypeName) throws SymbolException
  {
    Type type = env.getTE().getType(signatureTypeName);

    if (type.isInterface())
      return NativeTypes.get_net_nosica_lang_Object();

    if (type.isPrimitive())
      return null;

    return signatureTypeName;
  }

  public void visit(ArrayGet arrayGet) throws VisitorException
  {
    arrayGet.getSubscript().accept(this);
    Iterator it = arrayGet.iterateArgs();
    while (it.hasNext())
    {
      Expression exp = (Expression)it.next();
      exp.accept(this);
    }
  }

  public void visit(ArraySet arraySet) throws VisitorException
  {
    arraySet.getSubscript().accept(this);
    Iterator it = arraySet.iterateArgs();
    while (it.hasNext())
    {
      Expression exp = (Expression)it.next();
      exp.accept(this);
    }

    arraySet.getRhs().accept(this);
  }

  public void visit(MemberAccess memberAccess) throws VisitorException
  {
    if (memberAccess.hasSubscript())
    {
      memberAccess.getSubscript().accept(this);
    }
  }
  public void visit(Literal literal) throws VisitorException {}
  public void visit(Sequence sequence) throws VisitorException { throw new UnsupportedOperationException(); }
  public void visit(StatementExpression statementExpression) throws VisitorException
  {
    statementExpression.getExpression().accept(this);
  }
  public void visit(Variable variable) throws VisitorException {}
  public void visit(Move move) throws VisitorException
  {
    move.getRhs().accept(this);
  }
  public void visit(ExtendedSequence exSeq) throws VisitorException { throw new UnsupportedOperationException(); }
  public void visit(Jump jump) throws VisitorException {}
  public void visit(LabelName labelName) throws VisitorException {}
  public void visit(ConditionalJump conditionalJump) throws VisitorException { conditionalJump.getCondition().accept(this); }
  public void visit(Cast castExp)  throws VisitorException { castExp.getExpression().accept(this); }
  public void visit(InstanceOf instanceOfExp) throws VisitorException
  {
    instanceOfExp.getExpression().accept(this);

    PossibleTypeName instanceOfPTN = env.getPTNE().getPTN(instanceOfExp);

    // need to emit a instanceof call site
    Environment.check(instanceOfPTN.nbPossibleTypes() != 0, "possible types list should contain at list one element");

    if (instanceOfPTN.nbPossibleTypes() == 1)
      return;

    int counterValue = counter.intValue();
    counter.inc();
    instanceOfIds.put(instanceOfExp, new Integer(counterValue));

    printer.print(cConverter.cTypeName(NativeTypes.get_net_nosica_lang_bool()));
    printer.println(" instanceOfCallSite" + counterValue + "(int typeCode) {");
    printer.addIndent();
    printer.println("switch (typeCode) {");

    Iterator it = instanceOfPTN.iteratePossibleType();
    while (it.hasNext())
    {
      PossibleTypeName possibleTypeName = (PossibleTypeName)it.next();
      TypeName typeName = possibleTypeName.getTypeName();

      if (typeName.equals(NativeTypes.get_net_nosica_lang_None()))
        continue;

      Record record = env.getIRRE().getRecord(typeName);
      printer.println("case " + record.getTypeCode() + " :");
      if (
          typeName.equals(instanceOfExp.getRhsTypeName()) ||
          env.getTE().doesInherits(typeName, instanceOfExp.getRhsTypeName()) ||
          env.getTE().doesInherits(instanceOfExp.getRhsTypeName(), typeName)
         )
        printer.println("  return true;");
      else
        printer.println("  return false;");
      printer.println("  break;");
      printer.println();
    }

    printer.println("default :");
    printer.println("  abort();");
    printer.println("  break;");
    printer.println("}");

    printer.decIndent();
    printer.println("}");
  }
  public void visit(RawAllocation rawAllocation) throws VisitorException {}
  public void visit(IncRef incRef) throws VisitorException {}
  public void visit(DecRef decRef) throws VisitorException {}

  public void visit(Init init) throws VisitorException
  {
    init.getRhs().accept(this);
  }

  public void visit(StatementList statementList) throws VisitorException
  {
    Iterator it = statementList.iterateStatementList();
    while (it.hasNext())
    {
      Statement statement = (Statement)it.next();
      statement.accept(this);
    }
  }
  public void visit(CheckNull checkNull) throws VisitorException
  {
    checkNull.getExpression().accept(this);
  }
  public void visit(ConstraintError constraintError) throws VisitorException {}
  public void visit(TryRegister tryRegister) throws VisitorException
  {
    if (tryRegister.hasFinallyStatement())
    {
      tryRegister.getFinallyStatement().accept(this);
    }
  }
  public void visit(TryUnregister tryUnregister) throws VisitorException {}
  public void visit(Throw throwStatement) throws VisitorException
  {
    throwStatement.getExpression().accept(this);
  }
  public void visit(VariableScopeLeave varScopeLeave) throws VisitorException {}
  public void visit(Return returnStatement) throws VisitorException
  {
  }
}

