/*
  $Id: StringLiteralProducerVisitor.java,v 1.13 2004/03/04 22:16:16 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.compiler.production.c;

import net.nosica.compiler.application.Environment;
import net.nosica.application.NativeTypes;
import net.nosica.framework.VisitorException;

import net.nosica.compiler.production.c.intermediate.CIRVisitor;

import net.nosica.compiler.intermediate.Record;
import net.nosica.compiler.intermediate.Expression;
import net.nosica.compiler.intermediate.Call;
import net.nosica.compiler.intermediate.MemberAccess;
import net.nosica.compiler.intermediate.Sequence;
import net.nosica.compiler.intermediate.StatementExpression;
import net.nosica.compiler.intermediate.Literal;
import net.nosica.compiler.intermediate.Variable;
import net.nosica.compiler.intermediate.Move;
import net.nosica.compiler.intermediate.ExtendedSequence;
import net.nosica.compiler.intermediate.Jump;
import net.nosica.compiler.intermediate.LabelName;
import net.nosica.compiler.intermediate.ConditionalJump;
import net.nosica.compiler.intermediate.Cast;
import net.nosica.compiler.intermediate.InstanceOf;
import net.nosica.compiler.intermediate.RawAllocation;
import net.nosica.compiler.intermediate.Init;
import net.nosica.compiler.intermediate.Statement;
import net.nosica.compiler.intermediate.StatementList;
import net.nosica.compiler.intermediate.ArrayGet;
import net.nosica.compiler.intermediate.ArraySet;
import net.nosica.compiler.intermediate.ConstraintError;
import net.nosica.compiler.intermediate.TryRegister;
import net.nosica.compiler.intermediate.TryUnregister;
import net.nosica.compiler.intermediate.Throw;
import net.nosica.compiler.intermediate.VariableScopeLeave;
import net.nosica.compiler.intermediate.Return;
import net.nosica.compiler.production.c.intermediate.CheckNull;
import net.nosica.compiler.production.c.intermediate.DecRef;
import net.nosica.compiler.production.c.intermediate.IncRef;

import net.nosica.util.IndentedPrintWriter;
import net.nosica.util.SettableInteger;

import java.util.Map;
import java.util.Iterator;

public class StringLiteralProducerVisitor implements CIRVisitor
{
  private IndentedPrintWriter printer;
  private Map stringLiteralMap;
  private SettableInteger stringLiteralCounter;
  private CNameConverter cConverter;
  private boolean specialMethodProduced;
  private Environment env = Environment.instance();
  private int charArrayTypeCode;

  public StringLiteralProducerVisitor(
            IndentedPrintWriter printer,
            Map stringLiteralMap,
            SettableInteger stringLiteralCounter,
            CNameConverter cConverter,
            boolean specialMethodProduced
  ) throws VisitorException
  {
    this.printer = printer;
    this.stringLiteralMap = stringLiteralMap;
    this.stringLiteralCounter = stringLiteralCounter;
    this.cConverter = cConverter;
    this.specialMethodProduced = specialMethodProduced;

    Record record = env.getIRRE().getRecord(NativeTypes.get_net_nosica_lang_char_array());
    this.charArrayTypeCode = record.getTypeCode();
  }

  public boolean getSpecialMethodProduced() { return specialMethodProduced; }

  public void visit(Call call) throws VisitorException
  {
    if (call.hasSubscript())
      call.getSubscript().accept(this);

    Iterator it = call.iterateArgs();
    while (it.hasNext())
      ((Expression)it.next()).accept(this);
  }

  public void visit(ArrayGet arrayGet) throws VisitorException
  {
    arrayGet.getSubscript().accept(this);
  }

  public void visit(ArraySet arraySet) throws VisitorException
  {
    arraySet.getSubscript().accept(this);
    arraySet.getRhs().accept(this);
  }

  public void visit(MemberAccess memberAccess) throws VisitorException
  {
    if (memberAccess.hasSubscript())
      memberAccess.getSubscript().accept(this);
  }

  public void visit(Literal literal) throws VisitorException
  {
    if (literal.getTypeName().equals(NativeTypes.get_net_nosica_lang_string()))
    {
      String value = literal.getStrValue();
      if (!stringLiteralMap.containsKey(value))
      {
        // get counter value, and prepare for next literal
        int counter = stringLiteralCounter.intValue();
        stringLiteralCounter.inc();
        // put the association in the map
        stringLiteralMap.put(value, new Integer(counter));

        String literalDisplay = value;
        literalDisplay = literalDisplay.replace('\n', '.');
        printer.println("// \""+literalDisplay+"\"");

        // emit the characters buffer
        int length = value.length();
        if (length == 0)
        {
          printer.println("net_nosica_lang_char STRING_LITERAL_DATA_" + counter + "[0] = {};");
        }
        else
        {
          printer.print("net_nosica_lang_char STRING_LITERAL_DATA_" + counter + "[" + length + "] = {");
          for (int i = 0; i < length; ++i)
          {
            if (i != 0) printer.print(", ");
            printer.print(cConverter.cCharValue(value.charAt(i)));
          }
          printer.println("};");
        }
        // emit the char[] instance
        printer.print("array_1d___net_nosica_lang_char STRING_LITERAL_ARRAY_" + counter + " = {{");
        printer.print("1,");                                      // char[].super.nbRef
        printer.print("rtti + " + (charArrayTypeCode - 1) + "},"); // char[].super.rtti
        printer.print("STRING_LITERAL_DATA_" + counter + ",");    // char[].data
        printer.print(length*4 + ",");                            // char[].size
        printer.print("1,");                                      // char[].nbDim
        printer.println("{"+length*4+"}};");                      // char[].sizeOfDim
        // emit the string instance
        printer.print("net_nosica_lang_string STRING_LITERAL_" + counter + " = {");
        printer.print("&STRING_LITERAL_ARRAY_" + counter + ", "); // string.buffer_
        printer.println("0};");                                   // string.writable_
      }
    }
  }

  public void visit(Sequence sequence) throws VisitorException
  {
    throw new UnsupportedOperationException();
  }

  public void visit(StatementExpression statementExpression) throws VisitorException
  {
    statementExpression.getExpression().accept(this);
  }

  public void visit(Variable variable) throws VisitorException
  {
    // nothing to do
  }

  public void visit(Move move) throws VisitorException
  {
    move.getRhs().accept(this);

    move.getVariable().accept(this);
  }

  public void visit(ExtendedSequence exSeq) throws VisitorException
  {
    throw new UnsupportedOperationException();
  }

  public void visit(Jump jump) throws VisitorException
  {
    // nothing to do
  }

  public void visit(LabelName labelName) throws VisitorException
  {
    // nothing to do
  }

  public void visit(ConditionalJump conditionalJump) throws VisitorException
  {
    conditionalJump.getCondition().accept(this);
  }

  public void visit(Cast castExp) throws VisitorException
  {
    castExp.getExpression().accept(this);
  }

  public void visit(InstanceOf instanceOfExp) throws VisitorException
  {
    instanceOfExp.getExpression().accept(this);
  }

  public void visit(RawAllocation rawAllocation) throws VisitorException
  {
    // nothing to do
  }

  public void visit(IncRef incRef) throws VisitorException
  {
    incRef.getVariable().accept(this);
  }

  public void visit(DecRef decRef) throws VisitorException
  {
    decRef.getVariable().accept(this);
  }

  public void visit(Init init) throws VisitorException
  {
    visit((Move)init);
  }

  public void visit(StatementList statementList) throws VisitorException
  {
    Iterator it = statementList.iterateStatementList();
    while (it.hasNext())
      ((Statement)it.next()).accept(this);
  }

  public void visit(CheckNull checkNull) throws VisitorException
  {
    checkNull.getExpression().accept(this);
  }

  public void visit(ConstraintError constraintError) throws VisitorException
  {
  }

  public void visit(TryRegister tryRegister) throws VisitorException
  {
    if (tryRegister.hasFinallyStatement())
      tryRegister.getFinallyStatement().accept(this);
  }

  public void visit(TryUnregister tryUnregister) throws VisitorException
  {
  }

  public void visit(Throw throwStatement) throws VisitorException
  {
    throwStatement.getExpression().accept(this);
  }

  public void visit(VariableScopeLeave varScopeLeave) throws VisitorException
  {
  }

  public void visit(Return returnStatement) throws VisitorException
  {
  }
}
