/*
  $Id: InsertCheckNullVisitor.java,v 1.4 2004/03/04 22:16:16 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.compiler.production.c.intermediate;

import net.nosica.framework.VisitorException;
import net.nosica.compiler.intermediate.AbstractIRVisitorRewriter;
import net.nosica.compiler.intermediate.ArrayGet;
import net.nosica.compiler.intermediate.ArraySet;
import net.nosica.compiler.intermediate.Call;
import net.nosica.compiler.intermediate.Expression;
import net.nosica.compiler.intermediate.IRVisitorRewriter;
import net.nosica.compiler.intermediate.MemberAccess;
import net.nosica.compiler.typechecker.MethodHelper;

import java.util.Iterator;

/**
 * \brief insert CheckNull nodes before each dereferencement
 *
 * A CheckNull node is inserted before each
 * <ul>
 *   <li>non static Call
 *   <li>non static MemberAccess
 *   <li>ArraySet
 *   <li>ArrayGet
 * </ul>
 */
public class InsertCheckNullVisitor extends AbstractIRVisitorRewriter implements CIRVisitor, IRVisitorRewriter
{
  public void visit(Call call) throws VisitorException
  {
    if (call.hasSubscript())
    {
      if (!MethodHelper.isRefComparison(call.getMethod()))
      {
        // something like "instance.super == expression"
        // we don't want the MemberAccess to add a CheckNull
        call.getSubscript().accept(this);
      }

      // insert a CheckNull
      if (needCheckNull(call.getSubscript()))
      {
        insertBefore(
          new CheckNullImpl(
          call.getSubscript()
          )
        );
      }
    }

    Iterator it = call.iterateArgs();
    while (it.hasNext())
    {
      Expression exp = (Expression)it.next();
      exp.accept(this);
    }
  }

  public void visit(MemberAccess memberAccess) throws VisitorException
  {
    if (memberAccess.hasSubscript())
    {
      memberAccess.getSubscript().accept(this);

      // insert a checkNull
      if (needCheckNull(memberAccess.getSubscript()))
      {
        insertBefore(
          new CheckNullImpl(
            memberAccess.getSubscript()
          )
        );
      }
    }
  }

  public void visit(ArrayGet arrayGet) throws VisitorException
  {
    insertBefore(
      new CheckNullImpl(
        arrayGet.getSubscript()
      )
    );

    arrayGet.getSubscript().accept(this);
  }

  public void visit(ArraySet arraySet) throws VisitorException
  {
    insertBefore(
      new CheckNullImpl(
        arraySet.getSubscript()
      )
    );

    arraySet.getSubscript().accept(this);
  }

  private boolean needCheckNull(Expression subscript) throws VisitorException
  {
    return !isPrimitive(subscript.getTypeName()) &&
           !subscript.isThis() &&
           !subscript.isSuper();
  }
}
