/*
  $Id: InsertTemporaryVisitor.java,v 1.5 2004/03/04 22:16:16 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.compiler.production.c.intermediate;

import net.nosica.compiler.intermediate.AbstractIRVisitorRewriter;
import net.nosica.compiler.intermediate.ArrayGet;
import net.nosica.compiler.intermediate.Call;
import net.nosica.compiler.intermediate.Expression;
import net.nosica.compiler.intermediate.IRVisitorRewriter;
import net.nosica.compiler.intermediate.InitImpl;
import net.nosica.compiler.intermediate.Literal;
import net.nosica.compiler.intermediate.MemberAccess;
import net.nosica.compiler.intermediate.Statement;
import net.nosica.compiler.intermediate.SubscriptableExpression;
import net.nosica.compiler.intermediate.Variable;
import net.nosica.compiler.intermediate.VariableImpl;
import net.nosica.compiler.intermediate.VariableScopeLeaveImpl;
import net.nosica.framework.VisitorException;

import java.util.Iterator;
import java.util.ArrayList;

public class InsertTemporaryVisitor extends AbstractIRVisitorRewriter implements CIRVisitor, IRVisitorRewriter
{
  public void visit(Call call) throws VisitorException
  {
    // visitor pattern implementation
    if (call.hasSubscript())
    {
      call.getSubscript().accept(this);
    }

    ArrayList newArgs = new ArrayList();
    Iterator it = call.iterateArgs();
    while (it.hasNext())
    {
      Expression exp = (Expression)it.next();
      exp.accept(this);

      // if something like i.f(cast_word8_to_int8(1));
      // and f waits for non const argument (that is we pass them by address)
      if (exp instanceof Call)
      {
         Call argCall = (Call)exp;
         if (!argCall.getMethod().isSub() && isPrimitive(argCall.getMethod().getResultType().getTypeName()))
         {
           // insert a temporary
           Variable var = new VariableImpl(
             argCall.getMethod().getResultType().getTypeName(),
             env.getNG().generateTemporaryName()
          );
          insertBefore(new InitImpl(var, argCall));
          newArgs.add(var);
        }
        else
        {
          newArgs.add(argCall);
        }
      }
      else
      {
        newArgs.add(exp);
      }
    }

    call.setArgs(newArgs);

    /**
     * Example :
     *   int i;
     *   i.max().min();
     *   ==>
     *   int_min(&int_max(&i)); // equivalent c code
     *
     * because 'this' of primitive types are passed by address
     * (as I'm making this remark, I wonder if this visitor should not go in the production package ...)
     * (because if I do this kind of remark, it demonstrate this visitor has nothing to do here)
     *
     * I can't write &int_max (forbidden c practice)
     *
     * Do it with reference type as well for proper garbage collecting with temporaries on result type
     */
    if (call.hasSubscript() && call.getSubscript() instanceof Call)
    {
      insertTemporaryToReplaceSubscript(call);
    }

    /**
     * Example :
     * 0.cast_net_nosica_lang_int32();
     * ==>
     * net_nosica_lang_int32___cast(&0);  // wrong equivalent c code
     */
    if (call.hasSubscript() && call.getSubscript() instanceof Literal)
    {
      insertTemporaryToReplaceSubscript(call);
    }

    /**
     * Example :
     * tab[i].f();
     * ==>
     * f(&tab[i]);  // wrong equivalent c code (because tab[i] can be a method call instead)
     */
    if (call.hasSubscript() && call.getSubscript() instanceof ArrayGet)
    {
      insertTemporaryToReplaceSubscript(call);
    }
  }

  public void visit(ArrayGet arrayGet) throws VisitorException
  {
    // visitor implementation
    arrayGet.getSubscript().accept(this);
    Iterator it = arrayGet.iterateArgs();
    while (it.hasNext())
    {
      Expression exp = (Expression)it.next();
      exp.accept(this);
    }

    /**
     * Example :
     * tab[i][j];
     * ==>
     * tab[i][j];  // equivalent c code
     */
    if (arrayGet.getSubscript() instanceof ArrayGet)
    {
      insertTemporaryToReplaceSubscript(arrayGet);
    }

    /**
     * Example :
     * f()[i]
     * =>
     * f()[i]
     */
    if (arrayGet.getSubscript() instanceof Call)
    {
      insertTemporaryToReplaceSubscript(arrayGet);
    }
  }

  public void visit(MemberAccess memberAccess) throws VisitorException
  {
    // visitor implementation
    if (memberAccess.hasSubscript())
    {
      memberAccess.getSubscript().accept(this);
    }

    /**
     * Example :
     * tab[i].member;
     * ==>
     * tab[i].member;  // equivalent c code
     */
    if (memberAccess.hasSubscript() && memberAccess.getSubscript() instanceof ArrayGet)
    {
      insertTemporaryToReplaceSubscript(memberAccess);
    }

    /**
     * Example :
     * f().member
     * =>
     * f().member
     */
    if (memberAccess.getSubscript() instanceof Call)
    {
      insertTemporaryToReplaceSubscript(memberAccess);
    }
  }

  private void insertTemporaryToReplaceSubscript(SubscriptableExpression exp) throws VisitorException
  {
    Expression subscript = exp.getSubscript();

    // first create one
    Variable var = new VariableImpl(
      subscript.getTypeName(),
      env.getNG().generateTemporaryName()
    );
    // translate it
    Statement stmt = new InitImpl(var, subscript);

    // insert assignment
    insertBefore(stmt);

    // replace subscript by temporary
    exp.setSubscript(var);

    // add a scopeLeave
    if (!env.getTE().getType(var.getTypeName()).isPrimitive())
    {
      insertAfter(new VariableScopeLeaveImpl(var));
    }
  }
}