/*
  $Id: Accessible.java,v 1.6 2004/03/04 22:16:16 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.compiler.typechecker;

/**
 * \brief a base interface for all accessible symbol
 * That is, for symbol that needs access privilege
 */

public interface Accessible
{
  /**
   * \brief is symbol public
   * \return true if symbol is publicly accessible
   */
  boolean isPublic();

  /**
   * \brief is symbol protected
   * \return true if symbol is protected
   */
  boolean isProtected();

  /**
   * \brief is symbol private
   * \return true if symbol is private
   */
  boolean isPrivate();

  /**
   * \brief is symbol package
   * \return true if symbol is accessible only by its package
   */
  boolean isPackage();

  /**
   * \brief is symbol static
   * \return true if symbol is accessible without a live instance
   */
  boolean isStatic();

  /**
   * \brief set this symbol as public
   * This is used in GenericitySolver to transform all methods of an interface to public
   */
  void setPublic();
}