/*
  $Id: ExceptionEnvironment.java,v 1.6 2004/03/04 22:16:16 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.compiler.typechecker;

import net.nosica.parser.node.util.var.TypeName;

import net.nosica.symbol.SymbolException;

import java.util.Set;

/**
 * \brief Manage the list of live caught exceptions during type check
 *
 * Note : The save/restore methods act pretty like a stack<br>
 *        You must call save before calling restore<br>
 *        With these two methods you can manage try block during type check<br>
 */

public interface ExceptionEnvironment
{
  /**
   * \brief Save the state of the environment
   */
  void save();

  /**
   * \brief Restore the state of the environment to the last 'save' state
   *
   * \return Set<TypeName> set of exceptions that have not been marked as really launched
   */
  Set restore();

  /**
   * \brief Add an exception to the environment
   *
   * \param typeName the exception to add to the environment
   */
  void     add(TypeName typeName) throws SymbolException, TypeCheckerException;

  /**
   * \brief query environment for actives catch site
   *
   * \param typeName type of exception
   * \return true if it exists a live catch site for this exception
   * \throw SymbolException if a typeName is not in SymbolEnvironment
   */
  boolean caught(TypeName typeName) throws SymbolException;

  /**
   * \brief mark a previously stored exception as being really launched
   *
   * \param typeName exception launched
   */
  void    markExceptionAsLaunched(TypeName typeName) throws SymbolException;

  /**
   * \brief query environment for actives catch site and mark catch site as used if exception found
   *
   * \param typeName type of exception
   * \return true if it exists a live catch site for this exception
   * \throw SymbolException if a typeName is not in SymbolEnvironment
   */
  boolean caughtAndMarkExceptionAsLaunched(TypeName typeName) throws SymbolException;
}