/*
  $Id: ExceptionEnvironmentImpl.java,v 1.7 2004/03/04 22:16:16 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.compiler.typechecker;

import net.nosica.parser.node.util.var.TypeName;
import net.nosica.compiler.application.Environment;
import net.nosica.symbol.SymbolException;
import net.nosica.util.Pair;
import net.nosica.util.SettableBoolean;

import java.util.Set;
import java.util.Stack;
import java.util.Iterator;

public class ExceptionEnvironmentImpl implements ExceptionEnvironment
{
  private Stack exceptions = new Stack();  // ArrayList<Pair<TypeName, Boolean>>
  private Stack blocks     = new Stack();  // Stack<Integer>
  private int nbModificationsSinceLastSave = 0;
  private Environment env = Environment.instance();

  public void save()
  {
    blocks.push(new Integer(nbModificationsSinceLastSave));
    nbModificationsSinceLastSave = 0;
  }

  public Set restore()
  {
    Set unlaunchedException = new java.util.HashSet();

    for (int i = 0; i < nbModificationsSinceLastSave; ++i)
    {
      Pair pair = (Pair)exceptions.pop();
      if (!((SettableBoolean)pair.second).booleanValue())
        unlaunchedException.add(pair.first);
    }

    nbModificationsSinceLastSave = ((Integer)blocks.pop()).intValue();

    return unlaunchedException;
  }

  public void     add(TypeName typeName) throws SymbolException, TypeCheckerException
  {
    // let's verify that exception is not already active in current block
    for (int i = 0; i < nbModificationsSinceLastSave; ++i)
    {
      TypeName tn = (TypeName)(((Pair)exceptions.get(i)).first);

      if (
          tn.equals(typeName) || // same typeName
          env.getTE().doesInherits(typeName, tn)
         )
        throw new TypeCheckerException("Exception " + typeName + " is already active");
    }

    // OK, exception's not active, let's register it !
    exceptions.push(new Pair(typeName, new SettableBoolean(false)));
    ++nbModificationsSinceLastSave;
  }

  public boolean caught(TypeName typeName) throws SymbolException
  {
    Iterator it = exceptions.iterator();
    while (it.hasNext())
    {
      Pair pair = (Pair)it.next();
      TypeName tn = (TypeName)pair.first;

      if (
          tn.equals(typeName) || // same typeName
          env.getTE().doesInherits(typeName, tn) // typeName inherits from tn
         )
        return true;
    }

    return false;
  }

  public void    markExceptionAsLaunched(TypeName typeName) throws SymbolException
  {
    Iterator it = exceptions.iterator();
    while (it.hasNext())
    {
      Pair pair = (Pair)it.next();
      TypeName tn = (TypeName)pair.first;

      if (
          tn.equals(typeName) || // same typeName
          env.getTE().doesInherits(typeName, tn) // typeName inherits from tn
         )
      {
        ((SettableBoolean)pair.second).setValue(true);

        return;
      }
    }

    throw new SymbolException("Unknown exception " + typeName);
  }

  public boolean caughtAndMarkExceptionAsLaunched(TypeName typeName) throws SymbolException
  {
    Iterator it = exceptions.iterator();
    while (it.hasNext())
    {
      Pair pair = (Pair)it.next();
      TypeName tn = (TypeName)pair.first;

      if (
          tn.equals(typeName) || // same typeName
          env.getTE().doesInherits(typeName, tn) // typeName inherits from tn
         )
      {
        ((SettableBoolean)pair.second).setValue(true);

        return true;
      }

    }

    return false;
  }
}