/*
  $Id: Field.java,v 1.6 2004/03/04 22:16:16 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.compiler.typechecker;

import net.nosica.parser.node.util.var.TypeName;
import net.nosica.parser.node.util.var.VariableInitializer;

public interface Field extends Accessible
{
  /**
   * \brief Return the complete type of this argument
   *
   * \return type of argument
   */
  TypeName getTypeName();

  /**
   * \brief set the typeName for this variable
   *
   * \param typeName the new type name of this variable
   */
  void setTypeName(TypeName typeName);

  /**
   * brief Returns the name of the argument
   *
   * \return the name of this argument
   */
  String getName();

  /**
   * \brief returns the constness of the variable
   *
   * \return true if variable is const
   */
  boolean isConst();

  /**
   * \brief returns if variable was initialized
   *
   * \return true if variable was initialized
   */
  boolean isInitialized();


  /**
   * \brief is this field mutable
   *
   * \return true if field is mutable
   */
  boolean isMutable();

  /**
   * \brief is this field  transient ?
   *
   * \return true if field is transient
   */
  boolean isTransient();

  /**
   * \brief return field initializer
   *
   * \return initializer of field
   */
  VariableInitializer getVarInit();

  /**
   * \brief return the type from which belongs this field
   *
   * \return enclosing type
   */
  TypeName getType();
}