/*
  $Id: FieldContext.java,v 1.6 2004/03/04 22:16:16 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.compiler.typechecker;

import net.nosica.parser.node.util.var.TypeName;
import java.util.Iterator;

/**
 * \brief modelize field use during typeCheck
 *
 * A field can be seen as the field of the super type, or the inherited field.<br>
 * A field can be non const, but can be seen as const when type checked.<br>
 * This is the purpose of this interface : modelize how is seen an underlying field.
 */
public interface FieldContext extends Variable
{
  /**
   * \brief return the field underlying this field context
   *
   * \return underlying field
   */
  Field getField();

  /**
   * \brief insert an enclosing type to this field context
   *
   * \param enclosingType enclosing type
   */
  void insertEnclosing(TypeName enclosingType);

  /**
   * \brief insert an super type to this field context
   *
   * \param superType super type
   */
  void insertSuper(TypeName superType);

  /**
   * \brief iterate subscript
   * That is the list of enclosing types or super types to add in front of subscript
   *
   * \return Iterator<Pair<Boolean, TypeName>>. The type is an enclosing if
   *         the boolean is true, a super if it is false.
   */
  Iterator iterateSubscript();

  /**
   * \brief change the constness of this field context
   *
   * \param isConst new constess of this field within this context
   */
  void setConst(boolean isConst);
}
