/*
  $Id: MethodCallContext.java,v 1.11 2004/03/04 22:16:16 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.compiler.typechecker;

import net.nosica.parser.node.util.var.TypeName;
import net.nosica.framework.Localizable;

import java.util.Iterator;

/**
 * \brief an interface to modelize a method call during type check
 *
 * A MethodCallContext is mainly created by MethodResolver
 * MethodResolver try to resolve call using several means :
 * first it tries to localize method in type
 * if it fails, it tries to resolve it using all super types of type
 * if it fails, it tries to resolve it using enclosing type then all super type of enclosing type
 */
public interface MethodCallContext
{
  /**
   * \brief return the method to use to perform call
   *
   * \return method used
   */
  Method getMethod();

  /**
   * \brief return an iterator over arguments used to perform call
   *
   * \return Iterator<MethodCallContext>
   */
  Iterator iterateArgumentCasts();

  /**
   * \brief add weight to this method call context
   * A weight is a measurement of the match of the method
   * The smaller the weight, the better the method is matched
   *
   * \param weight weight to add to method
   */
  void addWeight(int weight);

  /**
   * \brief returns the weight associated with this method
   *
   * \return weight of method
   */
  int getWeight();

  /**
   * \brief insert an enclosing type to this call
   *
   * \param enclosingType enclosing type
   */
  void insertEnclosing(TypeName enclosingType);

  /**
   * \brief insert a super type to this call
   *
   * \param superType super type
   */
  void insertSuper(TypeName superType);

  /**
   * \brief iterate subscript
   * That is the list of enclosing type to add in front of subscript !
   *
   * if Pair.first == true  ==> means this is an enclosing subscript
   * if Pair.first == false ==> means this is a super subscript
   *
   * \return Iterator<Pair<Boolean, TypeName>>
   */
  Iterator iterateSubscript();

  /**
   * \brief return localization of this method call in source code
   *
   * \return localizable node
   */
  Localizable getLocalizable();
}
