/*
  $Id: MethodCallContextImpl.java,v 1.10 2004/03/04 22:16:16 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.compiler.typechecker;

import net.nosica.parser.node.util.var.TypeName;
import net.nosica.util.Pair;
import net.nosica.framework.Localizable;

import java.util.ArrayList;
import java.util.Stack;
import java.util.Iterator;

public class MethodCallContextImpl implements MethodCallContext
{
  private Localizable localizable;
  private Method method;
  private ArrayList argumentCasts; // ArrayList<MethodCallContext>
  private int weight = 0;
  private Stack subscript = new Stack();   // Stack<Pair<Boolean, TypeName>> (Boolean means "isEnclosing")

  /**
   * \brief constructs a method call context
   *
   * \param localizable a localizable object for error reporting
   * \param method the method to use to perform call
   * \param argumentCasts ArrayList<MethodCallContext> : cast operator branches
   * \note argumentCasts.size() == method.getArguments().size()
   * \note some cell of variables can be null indicating that no cast operator was required
   */
  public MethodCallContextImpl(Localizable localizable, Method method, ArrayList argumentCasts)
  {
    this.localizable = localizable;
    this.method = method;
    this.argumentCasts = argumentCasts;
  }

  /**
   * \brief constructs a method call context, on a method without arguments
   *
   * \param localizable a localizable object for error reporting
   * \param method the method to use to perform call
   */
  public MethodCallContextImpl(Localizable localizable, Method method)
  {
    this(localizable, method, new ArrayList());

    // add null cast methods to arguments
    if (method != null)
    {
      for (int i = 0; i < method.getArguments().size(); ++i)
      {
        argumentCasts.add(null);
      }
    }
  }

  public Method getMethod() { return method; }
  public Iterator iterateArgumentCasts() { return argumentCasts.iterator(); }
  public void addWeight(int weight) { this.weight += weight; }
  public int getWeight() { return weight; }
  public void insertEnclosing(TypeName enclosingType)
  {
    subscript.push(new Pair(new Boolean(true), enclosingType));
  }
  public void insertSuper(TypeName superType)
  {
    subscript.push(new Pair(new Boolean(false), superType));
  }
  public Iterator iterateSubscript() { return subscript.iterator(); }
  public Localizable getLocalizable() { return localizable; }
}
