/*
  $Id: StatementVisitor.java,v 1.66 2004/04/18 00:52:03 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.compiler.typechecker;

import net.nosica.framework.VisitorException;

import net.nosica.application.MessageLevel;
import net.nosica.application.NativeTypes;
import net.nosica.compiler.application.Environment;
import net.nosica.compiler.application.GenericityException;
import net.nosica.compiler.application.TypeCheckerErrorExpressionException;

import net.nosica.symbol.SymbolException;
import net.nosica.symbol.MethodKindEnum;

import net.nosica.parser.node.AbstractNodeVisitor;
import net.nosica.parser.node.Node;
import net.nosica.parser.node.NodeVisitor;

import net.nosica.parser.node.util.var.Arguments;
import net.nosica.parser.node.util.var.ArgumentsImpl;
import net.nosica.parser.node.util.var.ArgumentListImpl;
import net.nosica.parser.node.util.var.FormalParameters;
import net.nosica.parser.node.util.var.FormalParameter;
import net.nosica.parser.node.util.var.TypeNameComponent;
import net.nosica.parser.node.util.var.TypeNameComponentImpl;
import net.nosica.parser.node.util.var.TypeName;
import net.nosica.parser.node.util.var.TypeNameImpl;
import net.nosica.parser.node.util.var.VariableDeclarator;
import net.nosica.parser.node.util.operator.OperatorImpl;
import net.nosica.parser.node.util.operator.OperatorEnum;

import net.nosica.parser.node.typedeclaration.ConstructorDeclaration;
import net.nosica.parser.node.typedeclaration.ExplicitConstructorInvocation;
import net.nosica.parser.node.typedeclaration.ExplicitConstructorInvocationImpl;
import net.nosica.parser.node.typedeclaration.DestructorDeclaration;
import net.nosica.parser.node.typedeclaration.OperatorDeclaration;
import net.nosica.parser.node.typedeclaration.CastOperatorDeclaration;
import net.nosica.parser.node.typedeclaration.PropertyDeclaration;
import net.nosica.parser.node.typedeclaration.MethodDeclaration;
import net.nosica.parser.node.typedeclaration.Initializer;
import net.nosica.parser.node.typedeclaration.Deinitializer;

import net.nosica.parser.node.typedeclaration.expression.PrimaryPrefixImpl;
import net.nosica.parser.node.typedeclaration.expression.PrimaryExpressionImpl;

import net.nosica.parser.node.typedeclaration.statement.Block;
import net.nosica.parser.node.typedeclaration.statement.BlockStatement;
import net.nosica.parser.node.typedeclaration.statement.BreakStatement;
import net.nosica.parser.node.typedeclaration.statement.CatchStatement;
import net.nosica.parser.node.typedeclaration.statement.ContinueStatement;
import net.nosica.parser.node.typedeclaration.statement.DoStatement;
import net.nosica.parser.node.typedeclaration.statement.ForInit;
import net.nosica.parser.node.typedeclaration.statement.ForStatement;
import net.nosica.parser.node.typedeclaration.statement.ForUpdate;
import net.nosica.parser.node.typedeclaration.statement.IfStatement;
import net.nosica.parser.node.typedeclaration.statement.LabeledStatement;
import net.nosica.parser.node.typedeclaration.statement.LocalVariableDeclaration;
import net.nosica.parser.node.typedeclaration.statement.ReturnStatement;
import net.nosica.parser.node.typedeclaration.statement.StatementExpression;
import net.nosica.parser.node.typedeclaration.statement.StatementExpressionTerminator;
import net.nosica.parser.node.typedeclaration.statement.StatementExpressionList;
import net.nosica.parser.node.typedeclaration.statement.SwitchLabel;
import net.nosica.parser.node.typedeclaration.statement.SwitchStatement;
import net.nosica.parser.node.typedeclaration.statement.ThrowStatement;
import net.nosica.parser.node.typedeclaration.statement.TryStatement;
import net.nosica.parser.node.typedeclaration.statement.WhileStatement;

import net.nosica.compiler.intermediate.Expression;
import net.nosica.compiler.intermediate.Statement;

import net.nosica.util.Pair;
import net.nosica.util.SettableInteger;

import java.util.Iterator;
import java.util.ArrayList;
import java.util.Map;
import java.util.HashMap;
import java.util.Set;
import java.util. HashSet;
import java.util.Stack;

public class StatementVisitor extends AbstractNodeVisitor implements NodeVisitor
{
  private TypeCheckVisitor visitor;
  private Environment env = Environment.instance();
  private Statement statement = null;

  /**
   * Each time a statement is entered, a pair of label is generated.<br>
   * a label will be pushed into the activeLabels stack. It will be called the begin Label.<br>
   * a label will be pushed into the endLabels stack. It will be called the end Label.<br>
   * These labels will not be pushed into the labelRepository.<br>
   * <br>
   * If the statement is a LabeledStatement, the label name will be changed, and the beginLabel
   * will be pushed into the LabelRepository.<br>
   * If the statement is a Loop Statement, the beginLabel will be pushed into the LabelRepository.<br>
   * <br>
   * If a BreakStatement is encountered, the first element of the endLabel stack that is also in the LabelRepository will be the label to jump to.<br>
   * If the label is named, then the stack will have to be searched.<br>
   * If a ContinueStatement is encountered, the first element of the activeLabel stack that is also in the LabelRepository  will be the label to jump to.<br>
   * If the label is named, then the stack will have to be searched.<br>
   * <br>
   * Once the statement is finished, the stack are popped
   */
  private Stack activeLabels = new Stack();      // Stack<Label>
  private Stack endLabels = new Stack();         // stack<Label>
  private Set userDefinedLabels = new HashSet(); // Set<String>
  private Map scopeLabelCorresp = new HashMap(); // Map<String, Integer>

  private net.nosica.compiler.intermediate.Variable returnValue = null;
  private Label endMethodLabel = null;

  public StatementVisitor(
    TypeCheckVisitor typeCheckVisitor,
    Label endMethodLabel,
    net.nosica.compiler.intermediate.Variable returnValue
  )
  {
    this.visitor = typeCheckVisitor;
    this.endMethodLabel = endMethodLabel;
    this.returnValue = returnValue;
  }

  public Statement getStatement() { return statement; }

  public void enterBlock()
  {
    this.visitor.getVE().save();
    this.visitor.getLabelRepository().save();
  }

  public void leaveBlock()
  {
    ArrayList deadLabels = this.visitor.getLabelRepository().restore();
    ArrayList deadVariables = this.visitor.getVE().restore();

    statement = this.visitor.getIRT().translateDeadVariableDestruction(
      statement,
      deadVariables
    );

    // remove labels from scope correspondance hashmap
    Iterator it = deadLabels.iterator();
    while (it.hasNext())
    {
      Label label = (Label)it.next();
      if (scopeLabelCorresp.containsKey(label.getName()))
        scopeLabelCorresp.remove(label.getName());
      else
        env.terminate("Internal error : could not find label [" + label.getName() + "]");
    }
  }

  public void visit(ConstructorDeclaration constructorDecl)
  {
    if (!constructorDecl.hasBody())
      return;

    ExplicitConstructorInvocation explicitConstructorInvocation = null;

    // if there is an explicit constructor invocation
    if (constructorDecl.hasExplicitConstructorInvocation())
      explicitConstructorInvocation = constructorDecl.getExplicitConstructorInvocation();
    else
    {
      Iterator it = visitor.getType().iterateExtends();
      while (it.hasNext())
      {
        TypeName typeName = (TypeName)it.next();

        ArrayList arguments = new ArrayList();

        if (!typeName.equals(NativeTypes.get_net_nosica_lang_Object()))
        {
          // create one with the same parameter as the parameters of this constructor
          FormalParameters formalParams = constructorDecl.getFormalParameters();
          Iterator j = formalParams.iterateParameters();
          while (j.hasNext())
          {
            FormalParameter param = (FormalParameter)j.next();
            TypeNameComponent component = new TypeNameComponentImpl(param.getVariable().getId(), null);
            ArrayList components = new ArrayList();
            components.add(component);

            ArrayList primaryComponents = new ArrayList();
            primaryComponents.add(new PrimaryPrefixImpl(new TypeNameImpl(components)));

            arguments.add(new PrimaryExpressionImpl(primaryComponents));
          }
        }

        Arguments args = new ArgumentsImpl(new ArgumentListImpl(arguments));
        explicitConstructorInvocation = new ExplicitConstructorInvocationImpl(constructorDecl.localization(), true, args);
      }
    }

    boolean constructorCalled = false;
    // super call or this call
    if (explicitConstructorInvocation != null)
    {
      if (!explicitConstructorInvocation.isSuperCall())
      {
        // call to a constructor
        // this constructor will take care of initializing fields
        constructorCalled = true;
      }

      try
      {
        explicitConstructorInvocation.accept(this);
      }
      catch (TypeCheckerErrorExpressionException e)
      {
        // normal end of type checking
      }
    }

    if (!constructorCalled)
    {
      // no constructor was called
      // initialization of non static fields
      try
      {
        // 1 : retrieve non static fields
        ArrayList nonStaticFields = getNonStaticFields();

        // 2 : insert initialization
        insertFieldInitialization(nonStaticFields);
      }
      catch (TypeCheckerErrorExpressionException e)
      {
        // interrupt type checking of current expression
      }
    }

    enterBlock();

    try
    {
      // type check rest of body
      Iterator it = constructorDecl.iterateBlockStatements();
      while (it.hasNext())
        ((Node)it.next()).accept(this);
    }
    finally
    {
      leaveBlock();
    }
  }

  private ArrayList getNonStaticFields()
  {
    ArrayList nonStaticFields = new ArrayList();

    Iterator it = this.visitor.getType().iterateFields();
    while (it.hasNext())
    {
      Field field = (Field)it.next();

      if (field.isStatic())
        continue;

      nonStaticFields.add(field);
    }

    return nonStaticFields;
  }

  private ArrayList getStaticFields()
  {
    ArrayList staticFields = new ArrayList();

    Iterator it = this.visitor.getType().iterateFields();
    while (it.hasNext())
    {
      Field field = (Field)it.next();

      if (!field.isStatic())
        continue;

      staticFields.add(field);
    }

    return staticFields;
  }

  private void insertFieldInitialization(ArrayList fields)
  {
    // 1 : default initialization
    statement = this.visitor.getIRT().translateFieldInitialization(
      statement,
      fields
    );

    // 2 : user initialization
    Iterator it = fields.iterator();
    while (it.hasNext())
    {
      Field field = (Field)it.next();
      if (field.isInitialized())
      {
        ArrayList deadVariables = null;
        this.visitor.getVE().save();
        try
        {
          // construct IR expression of initialization and type check it
          field.getVarInit().accept(visitor);
        }
        finally
        {
          deadVariables = this.visitor.getVE().restore();
        }
        Expression translatedExp = this.visitor.getTranslatedExp();
        TypeCheckerResultType resultType = this.visitor.getResultType();

        // find assign operator
        MethodCallContext match = TypeCheckerHelper.findAssignOperator(
          visitor.getType().getName(),
          new TypeCheckerResultTypeImpl(
            field.getTypeName(),
            field.isConst()
          ),
          resultType,
          new OperatorImpl(
            OperatorEnum.assign
          ),
          true,
          field.getVarInit()
        );

        // translate assignment
        translatedExp = this.visitor.getIRT().translateAssignment(
          match,
          this.visitor.getIRT().translateMemberAccess(
            this.visitor.getIRT().translateThis(),
            new FieldContextImpl(
              field
            )
          ),
          translatedExp,
          true
        );

        // and add it to statements
        statement = this.visitor.getIRT().translateSequence(
          statement,
          translatedExp
        );

        // add temporary destruction
        statement = this.visitor.getIRT().translateDeadVariableDestruction(statement, deadVariables);
      }
    }
  }

  public void visit(ExplicitConstructorInvocation explicitConstructorInvocation)
  {
    if (explicitConstructorInvocation.isSuperCall())
    {
      // super call
      TypeName extendsTypeName = null;

      Iterator k = visitor.getType().iterateExtends();
      if (k.hasNext())
        extendsTypeName = (TypeName)k.next();
      else
      {
        env.terminate("Internal error, there should be an extend");
        return;
      }

      if (k.hasNext())
      {
        env.terminate("Internal error, there should be only one extend");
        return;
      }

      if (extendsTypeName.equals(NativeTypes.get_net_nosica_lang_Object()))
        return;

      ArrayList args = new ArrayList();
      ArrayList argsTranslated = new ArrayList();
      ArrayList argsTemporaries = new ArrayList();
      visitor.visitArguments(
        explicitConstructorInvocation.getArguments(),
        args,
        argsTranslated
      );

      Method signature = new MethodImpl(
        false,
        "constructor",
        MethodKindEnum.constructor,
        args,
        false,
        null
      );

      // this will find and/or report error on constructor
      MethodCallContext mcc = this.visitor.findMethod(
        new ResultTypeImpl(
          extendsTypeName,
          false
        ),
        signature,
        explicitConstructorInvocation
      );

      // translate super call
      argsTranslated.add(
        0,
        this.visitor.getIRT().translateSuper()
      );

      statement = this.visitor.getIRT().translateSequence(
        statement,
        this.visitor.getIRT().translateMethodCall(
          mcc,
          argsTranslated
        )
      );

      // add temporary destruction
      statement = this.visitor.getIRT().translateDeadVariableDestruction(statement, argsTemporaries);
    }
    else
    { // this(args)

      // get arguments
      ArrayList args = new ArrayList();
      ArrayList argsTranslated = new ArrayList();
      ArrayList argsTemporaries = new ArrayList();
      visitor.visitArguments(
        explicitConstructorInvocation.getArguments(),
        args,
        argsTranslated
      );

      Method signature = new MethodImpl(
        false,
        "constructor",
        MethodKindEnum.constructor,
        args,
        false,
        null
      );

      // this will find and/or report error on constructor
      MethodCallContext mcc = visitor.findMethod(
        new ResultTypeImpl(
          visitor.getType().getName(),
          false
        ),
        signature,
        explicitConstructorInvocation
      );

      // translate this call
      argsTranslated.add(
        0,
        this.visitor.getIRT().translateThis()
      );

      statement = this.visitor.getIRT().translateSequence(
        statement,
        this.visitor.getIRT().translateMethodCall(
          mcc,
          argsTranslated
        )
      );

      // add temporary destruction
      statement = this.visitor.getIRT().translateDeadVariableDestruction(statement, argsTemporaries);
    }
  }

  public void visit(DestructorDeclaration destructorDecl)
  {
    if (destructorDecl.hasBlock())
      destructorDecl.getBlock().accept(this);

    // destroy all non static fields
    try
    {
      ArrayList nonStaticFields = getNonStaticFields();
      statement = this.visitor.getIRT().translateDeadVariableDestruction(
        statement,
        nonStaticFields
      );
    }
    catch (TypeCheckerErrorExpressionException e)
    {
      // interrupt type checking of current expression
    }

    // don't forget to call parent destructor if any
    Iterator it = visitor.getType().iterateExtends();
    while (it.hasNext())
    {
      TypeName typeName = (TypeName)it.next();

      if (!typeName.equals(NativeTypes.get_net_nosica_lang_Object()))
      {
        try
        {
          Type parent = env.getTE().getType(typeName);
          ArrayList arguments = new ArrayList();
          arguments.add(this.visitor.getIRT().translateThis());

          statement = this.visitor.getIRT().translateSequence(
            statement,
            this.visitor.getIRT().translateMethodCall(
              new MethodCallContextImpl(
                null,
                parent.getDestructor()
              ),
              arguments
            )
          );
        }
        catch (SymbolException e)
        {
          env.terminate("Internal error : " + e.getMessage());
        }
      }
    }
  }

  public void visit(OperatorDeclaration operatorDecl)
  {
    if (operatorDecl.hasBlock())
      operatorDecl.getBlock().accept(this);
  }

  public void visit(CastOperatorDeclaration castOperatorDecl)
  {
    if (castOperatorDecl.hasBlock())
      castOperatorDecl.getBlock().accept(this);
  }

  public void visit(PropertyDeclaration propDecl)
  {
    if (propDecl.hasBlock())
      propDecl.getBlock().accept(this);
  }

  public void visit(MethodDeclaration methodDecl)
  {
    if (methodDecl.hasBlock())
      methodDecl.getBlock().accept(this);
  }

  public void visit(Initializer initializer)
  {
    // initialization of static fields
    try
    {
      // 1 : retrieve static fields
      ArrayList staticFields = getStaticFields();

      // 2 : insert initialization
      insertFieldInitialization(staticFields);
    }
    catch (TypeCheckerErrorExpressionException e)
    {
      // interrupt type checking of current expression
    }

    initializer.getBlock().accept(this);
  }

  public void visit(Deinitializer deinitializer)
  {
    deinitializer.getBlock().accept(this);

    // destroy all static fields
    try
    {
      ArrayList staticFields = getStaticFields();
      statement = this.visitor.getIRT().translateDeadVariableDestruction(
        statement,
        staticFields
      );
    }
    catch (TypeCheckerErrorExpressionException e)
    {
      // interrupt type checking of current expression
    }
  }

  public void visit(Block block)
  {
    enterBlock();

    try
    {
      Iterator it = block.iterateStatements();
      while (it.hasNext())
      {
        BlockStatement blockStatement = (BlockStatement)it.next();

        blockStatement.accept(this);
      }
    }
    finally
    {
      leaveBlock();
    }
  }

  public void visit(BlockStatement blockStatement)
  {
    try
    {
      if (blockStatement.isLocalVariable())
      {
        // type check local variable declaration
        blockStatement.getLocalVariable().accept(this);
      }
      else
      if (blockStatement.isStatement())
      {
        // type check statement
        Label beginLabel = this.visitor.getLabelRepository().generateLabel();
        Label endLabel = this.visitor.getLabelRepository().generateLabel();

        // generate a pair beginLabel/endLabel
        activeLabels.push(beginLabel);
        endLabels.push(endLabel);

        blockStatement.getStatement().accept(this);

        endLabels.pop();
        activeLabels.pop();
      }
    }
    catch (TypeCheckerErrorExpressionException e)
    {
      // interrupt type checking of current expression
    }
  }

  public Label getBeginLabel(String labelName)
  {
    Iterator it = activeLabels.iterator();
    while (it.hasNext())
    {
      Label label = (Label)it.next();
      if (label.getName().equals(labelName) && this.visitor.getLabelRepository().get(labelName) != null)
        return label;
    }

    return null;
  }

  public Label getBeginLabel()
  {
    Iterator it = activeLabels.iterator();
    while (it.hasNext())
    {
      Label label = (Label)it.next();
      if (this.visitor.getLabelRepository().get(label.getName()) != null)
        return label;
    }

    return null;
  }

  public Label getEndLabel(String labelName)
  {
    Iterator it = activeLabels.iterator();
    Iterator j = endLabels.iterator();
    while (it.hasNext() && j.hasNext())
    {
      Label beginLabel = (Label)it.next();
      Label endLabel =   (Label)j.next();

      if (beginLabel.getName().equals(labelName) && this.visitor.getLabelRepository().get(labelName) != null)
        return endLabel;
    }

    return null;
  }

  public Label getEndLabel()
  {
    Iterator it = activeLabels.iterator();
    Iterator j = endLabels.iterator();
    while (it.hasNext() && j.hasNext())
    {
      Label beginLabel = (Label)it.next();
      Label endLabel =   (Label)j.next();

      if (this.visitor.getLabelRepository().get(beginLabel.getName()) != null)
        return endLabel;
    }

    return null;
  }

  public void visit(BreakStatement breakStatement)
  {
    Label endLabel = null;
    Label beginLabel = null;

    if (breakStatement.hasLabel())
    {
      endLabel = getEndLabel(breakStatement.getLabel());
      beginLabel = getBeginLabel(breakStatement.getLabel());

      if (endLabel == null)
      {
        env.getMS().report(MessageLevel.ERR, "TypeCheck051", breakStatement.getLabel(), breakStatement);

        throw new TypeCheckerErrorExpressionException();
      }
    }
    else
    {
      endLabel = getEndLabel();
      beginLabel = getBeginLabel();

      if (endLabel == null)
      {
        env.getMS().report(MessageLevel.ERR, "TypeCheck052", breakStatement);

        throw new TypeCheckerErrorExpressionException();
      }
    }

    statement = this.visitor.getIRT().translateDeadVariableDestruction(
      statement,
      getVariableInScope(beginLabel)
    );

    statement = this.visitor.getIRT().translateSequence(
      statement,
      this.visitor.getIRT().translateJump(endLabel)
    );
  }

  public void visit(CatchStatement catchStatement)
  {
    try
    {
      enterBlock();

      // do not add variable as it has been added in TryStatement
      // type check catch statement
      catchStatement.getBlock().accept(this);
    }
    finally
    {
      leaveBlock();
    }
  }

  public void visit(ContinueStatement continueStatement)
  {
    Label beginLabel = null;

    if (continueStatement.hasLabel())
    {
      beginLabel = getBeginLabel(continueStatement.getLabel());

      if (beginLabel == null)
      {
        env.getMS().report(MessageLevel.ERR, "TypeCheck053", continueStatement.getLabel(), continueStatement);

        throw new TypeCheckerErrorExpressionException();
      }
    }
    else
    {
      beginLabel = getBeginLabel();

      if (beginLabel == null)
      {
        env.getMS().report(MessageLevel.ERR, "TypeCheck054", continueStatement);

        throw new TypeCheckerErrorExpressionException();
      }
    }

    statement = this.visitor.getIRT().translateDeadVariableDestruction(
      statement,
      getVariableInScope(beginLabel)
    );

    statement = this.visitor.getIRT().translateSequence(
      statement,
      this.visitor.getIRT().translateJump(beginLabel)
    );
  }

  private void saveLabelScope(Label label, int scope)
  {
    scopeLabelCorresp.put(label.getName(), new Integer(scope));
  }

  private ArrayList getVariableInScope(Label label)
  {
    if (!scopeLabelCorresp.containsKey(label.getName()))
    {
      env.terminate("Internal error : no label named [" + label.getName() + "] registered");
    }

    try
    {
      int nbScope = this.visitor.getVE().getNumberOfActiveScope();
      nbScope -= ((Integer)scopeLabelCorresp.get(label.getName())).intValue();
      return visitor.getVE().variableInScope(nbScope);
    }
    catch (TypeCheckerException e)
    {
      env.terminate("Internal error : " + e.getMessage());
      return null;
    }
  }

  public void visit(DoStatement doStatement)
  {
    /*
    do
    {
      doStatement
    }
    while (cond)
    else
    {
      elseStatement
    }

    =>

    beginLabel :
      enterBlock
      doStatement
      leaveBlock
    ConditionalJump(cond, beginLabel, elseLabel, temporariesOfCondition)
    elseLabel :
      enterBlock
      elseStatement
      leaveBlock
    endLabel :

---

    do
    {
      doStatement
    }
    while (cond)

    =>

    beginLabel :
      enterBlock
      doStatement
      leaveBlock
    ConditionalJump(cond, beginLabel, endLabel, temporariesOfCondition)
    endLabel :
    */

    Label beginLabel = null;
    Label elseLabel = null;
    Label endLabel = null;
    try
    {
      beginLabel = (Label)activeLabels.peek();
      endLabel = (Label)endLabels.peek();
    }
    catch (java.util.EmptyStackException e)
    {
      env.terminate("Internal error : " + e.getMessage());
    }

    if (doStatement.hasElseStatement())
      elseLabel = this.visitor.getLabelRepository().generateLabel();
    else
      elseLabel = endLabel;

    boolean isUserLabel = false;
    try
    {
      this.visitor.getLabelRepository().add(beginLabel);
      saveLabelScope(beginLabel, this.visitor.getLabelRepository().getNumberOfActiveScope());
    }
    catch (TypeCheckerException e)
    {
      // can happen, because label has already been added by LabeledStatement
      isUserLabel = true;
    }


    if (!isUserLabel)
    {
      // beginLabel :
      statement = this.visitor.getIRT().translateSequence(
        statement,
        this.visitor.getIRT().translateLabel(beginLabel)
        );
    }

    enterBlock();
    try
    {
      doStatement.getStatement().accept(this);
    }
    finally
    {
      leaveBlock();
    }

    ArrayList deadVariables;
    this.visitor.getVE().save();
    try
    {
      // typecheck condition
      doStatement.getCondition().accept(visitor);

      // report error
      TypeCheckerResultType exp = this.visitor.getResultType();
      if (exp.isSub())
        env.getMS().report(MessageLevel.ERR, "TypeCheck021", doStatement.getCondition());
      else
      {
        if (!exp.getTypeName().equals(NativeTypes.get_net_nosica_lang_bool()))
          env.getMS().report(MessageLevel.ERR, "TypeCheck020", exp.getTypeName().toString(), doStatement.getCondition());
      }
    }
    finally
    {
      deadVariables = this.visitor.getVE().restore();
    }

    net.nosica.compiler.intermediate.Variable var = translateVariableAssignmentAndDeadVariableDestruction(this.visitor.getTranslatedExp(), deadVariables);

    // ConditionalJump(cond, beginLabel, elseLabel, temporaries)
    statement = this.visitor.getIRT().translateSequence(
      statement,
      this.visitor.getIRT().translateConditionalJump(
        var,
        beginLabel,
        elseLabel
      )
    );

    // if else statement
    if (doStatement.hasElseStatement())
    {
      // ElseLabel
      statement = this.visitor.getIRT().translateSequence(
        statement,
        this.visitor.getIRT().translateLabel(elseLabel)
      );

      enterBlock();
      try
      {
        doStatement.getElseStatement().accept(this);
      }
      finally
      {
        leaveBlock();
      }
    }

    if (!isUserLabel)
    {
      statement = this.visitor.getIRT().translateSequence(
        statement,
        this.visitor.getIRT().translateLabel(endLabel)
        );
    }
  }

  public void visit(ForInit forInit)
  {
    if (forInit.isLocalVariable())
      forInit.getLocalVariable().accept(this);
    else
      forInit.getStatements().accept(this);
  }

  public void visit(ForStatement forStatement)
  {
    /*
      Here's the structure :

      for ([initialisation]; [condition]; [update])
      {
        forStatement
      }
      [
      else
      {
        elseStatement
      }
      ]

      enterBlock
      [initialization]
      [jump conditionLabel]
      beginLabel :
        enterBlock
        statement
        leaveBlock
        [update]
      [
        conditionLabel :
        this.visitor.getVE().save();
        if (condition)
        {
          deadVariable = this.visitor.getVE().restore();
          detroy deadVariables
          goto beginLabel
        }
        else
        {
          deadVariable = this.visitor.getVE().restore();
          detroy deadVariables
          goto elseLabel (if exists, endLabel otherwise)
        }
      ]
      goto beginLabel
      [
        elseLabel :
        enterBlock
        elseStatement
        leaveBlock
      ]
      endLabel :
      leaveBlock
    */

    Label beginLabel = null;
    Label endLabel = null;
    Label elseLabel = null;

    Label conditionLabel = null;

    try
    {
      beginLabel = (Label)activeLabels.peek();
      endLabel = (Label)endLabels.peek();
    }
    catch (java.util.EmptyStackException e)
    {
      env.terminate("Internal error : " + e.getMessage());
    }

    if (forStatement.hasElseStatement())
      elseLabel = this.visitor.getLabelRepository().generateLabel();
    else
      elseLabel = endLabel;

    enterBlock();
    try
    {
      // [Initialization]
      if (forStatement.hasInitialization())
        forStatement.getInitialization().accept(this);

      if (forStatement.hasCondition())
      {
        conditionLabel = this.visitor.getLabelRepository().generateLabel();

        // jump conditionLabel
        statement = this.visitor.getIRT().translateSequence(
          statement,
          this.visitor.getIRT().translateJump(
            conditionLabel
          )
        );
      }

      boolean isUserLabel = false;
      try
      {
        this.visitor.getLabelRepository().add(beginLabel);
        saveLabelScope(beginLabel, this.visitor.getLabelRepository().getNumberOfActiveScope());
      }
      catch (TypeCheckerException e)
      {
        // can happen, because label has already been added by LabeledStatement
        isUserLabel = true;
      }

      if (!isUserLabel)
      {
        // beginLabel
        statement = this.visitor.getIRT().translateSequence(
            statement,
            this.visitor.getIRT().translateLabel(beginLabel)
        );
      }

      enterBlock();
      try
      {
        forStatement.getStatement().accept(this);
      }
      finally
      {
        leaveBlock();
      }

      // [update]
      if (forStatement.hasUpdate())
        forStatement.getUpdate().accept(this);

      /*
        [
          conditionLabel :
          if (!condition)
            goto elseLabel
      */
      if (forStatement.hasCondition())
      {
        // conditionLabel :
        statement = this.visitor.getIRT().translateSequence(
          statement,
          this.visitor.getIRT().translateLabel(conditionLabel)
        );

        ArrayList deadVariables;
        this.visitor.getVE().save();
        try
        {
          forStatement.getCondition().accept(visitor);

          // typecheck condition
          TypeCheckerResultType exp = this.visitor.getResultType();
          if (exp.isSub())
            env.getMS().report(MessageLevel.ERR, "TypeCheck021", forStatement.getCondition());
          else
          {
            if (!exp.getTypeName().equals(NativeTypes.get_net_nosica_lang_bool()))
              env.getMS().report(MessageLevel.ERR, "TypeCheck020", exp.getTypeName().toString(), forStatement.getCondition());
          }
        }
        finally
        {
          deadVariables = this.visitor.getVE().restore();
        }

        net.nosica.compiler.intermediate.Variable var = translateVariableAssignmentAndDeadVariableDestruction(this.visitor.getTranslatedExp(), deadVariables);

        // conditionalJump(cond, beginLabel, elseLabel)
        statement = this.visitor.getIRT().translateSequence(
          statement,
          this.visitor.getIRT().translateConditionalJump(
            var,
            beginLabel,
            elseLabel
          )
        );
      }
      else // for statement hasn't a condition
      {
        // goto internalBeginLoopLabel
        statement = this.visitor.getIRT().translateSequence(
          statement,
          this.visitor.getIRT().translateJump(beginLabel)
        );
      }

      /* [elseLabel :]
        [elseStatement]
      */
      if (forStatement.hasElseStatement())
      {
        // elseLabel :
        statement = this.visitor.getIRT().translateSequence(
          statement,
          this.visitor.getIRT().translateLabel(elseLabel)
        );

        enterBlock();
        try
        {
          forStatement.getElseStatement().accept(this);
        }
        finally
        {
          leaveBlock();
        }
      }

      if (!isUserLabel)
      {
        // endLabel :
        statement = this.visitor.getIRT().translateSequence(
          statement,
          this.visitor.getIRT().translateLabel(endLabel)
        );
      }
    }
    finally
    {
      leaveBlock();
    }
  }

  public void visit(ForUpdate forUpdate)
  {
    forUpdate.getStatements().accept(this);
  }

  public void visit(IfStatement ifStatement)
  {
    /**
     * if (condition)
     * {
     *   trueStatement
     * }
     * [
     * else
     * {
     * falseStatement
     * }
     * ]
     * =>
     * condJump(condition, trueLabel, falseLabel, temporariesOfCondition)
     * trueLabel :
     *  trueStatement
     * [
     *  goto endLabel
     * ]
     * falseLabel :
     * [
     *  falseStatement
     * endLabel :
     * ]
     */
    Label trueLabel = this.visitor.getLabelRepository().generateLabel();
    Label falseLabel = this.visitor.getLabelRepository().generateLabel();
    Label endLabel = null;

    if (ifStatement.hasElseStatement())
      endLabel = this.visitor.getLabelRepository().generateLabel();

    ArrayList deadVariables;
    this.visitor.getVE().save();
    try
    {
      // typecheck condition
      ifStatement.getCondition().accept(visitor);

      // typecheck condition
      TypeCheckerResultType exp = this.visitor.getResultType();
      if (exp.isSub())
        env.getMS().report(MessageLevel.ERR, "TypeCheck021", ifStatement.getCondition());
      else
      {
        if (!exp.getTypeName().equals(NativeTypes.get_net_nosica_lang_bool()))
          env.getMS().report(MessageLevel.ERR, "TypeCheck020", exp.getTypeName().toString(), ifStatement.getCondition());
      }
    }
    finally
    {
      deadVariables = this.visitor.getVE().restore();
    }

    net.nosica.compiler.intermediate.Variable var = translateVariableAssignmentAndDeadVariableDestruction(this.visitor.getTranslatedExp(), deadVariables);

    // condJump(condition, trueLabel, falseLabel)
    statement = this.visitor.getIRT().translateSequence(
      statement,
      this.visitor.getIRT().translateConditionalJump(
        var,
        trueLabel,
        falseLabel
      )
    );

    // trueLabel :
    statement = this.visitor.getIRT().translateSequence(
      statement,
      this.visitor.getIRT().translateLabel(
        trueLabel
      )
    );

    enterBlock();
    try
    {
      ifStatement.getStatement().accept(this);
    }
    finally
    {
      leaveBlock();
    }

    // if there is an else statement
    if (ifStatement.hasElseStatement())
    {
      // jump endLabel :
      statement = this.visitor.getIRT().translateSequence(
        statement,
        this.visitor.getIRT().translateJump(endLabel)
      );
    }

    // falseLabel :
    statement = this.visitor.getIRT().translateSequence(
      statement,
      this.visitor.getIRT().translateLabel(
        falseLabel
      )
    );

    // if there is an else statement
    if (ifStatement.hasElseStatement())
    {
      enterBlock();
      try
      {
        ifStatement.getElseStatement().accept(this);
      }
      finally
      {
        leaveBlock();
      }

      // endLabel :
      statement = this.visitor.getIRT().translateSequence(
        statement,
        this.visitor.getIRT().translateLabel(
          endLabel
        )
      );
    }
  }

  public void visit(LabeledStatement labeledStatement)
  {
    if (userDefinedLabels.contains(labeledStatement.getLabel()))
    {
      env.getMS().report(MessageLevel.ERR, "TypeCheck050", labeledStatement.getLabel(), labeledStatement);
    }
    userDefinedLabels.add(labeledStatement.getLabel());

    Label label = new LabelImpl(labeledStatement.getLabel());
    //  remove the auto generated label
    activeLabels.pop();
    // and replace it with the user provided one
    activeLabels.push(label);

    try
    {
      this.visitor.getLabelRepository().add(label);
      saveLabelScope(label, this.visitor.getLabelRepository().getNumberOfActiveScope());
    }
    catch (TypeCheckerException e)
    {
      // should not happen
      env.terminate("Internal error : " + e.getMessage());
    }

    statement = this.visitor.getIRT().translateSequence(
      statement,
      this.visitor.getIRT().translateLabel(label)
    );
    labeledStatement.getStatement().accept(this);
    statement = this.visitor.getIRT().translateSequence(
      statement,
      this.visitor.getIRT().translateLabel((Label)endLabels.peek())
    );
  }

  private void addCallToDefaultConstructor(
    net.nosica.compiler.intermediate.Variable var,
    Node node
  ) throws SymbolException
  {
    // add call to constructor
    ArrayList possibleConstructors = new ArrayList();
    Method defaultConstructor = env.getMR().getDefaultConstructor(
      var.getTypeName(),
      this.visitor.getType().getName(),
      possibleConstructors
    );

    // error reporting
    if (defaultConstructor == null)
    {
      Method signature = new MethodImpl(false, "constructor", MethodKindEnum.constructor, new ArrayList(), false, null);
      TypeCheckerHelper.reportMethodError(
        new ResultTypeImpl(
          var.getTypeName(),
          false
        ),
        signature,
        new ArrayList(),
        possibleConstructors,
        node
      );
      throw new TypeCheckerErrorExpressionException();
    }

    // translation to ir
    ArrayList args = new ArrayList();
    args.add(var);
    net.nosica.compiler.intermediate.Expression exp = this.visitor.getIRT().translateMethodCall(
      new MethodCallContextImpl(
        node,
        defaultConstructor,
        new ArrayList()
      ),
      args
    );

    // add ir to list of translated statements
    statement = this.visitor.getIRT().translateSequence(
      statement,
      exp
    );
  }

  public void visit(LocalVariableDeclaration localVar)
  {
    try
    {
      // ensures type is loaded
      TypeName varTypeName = env.getTR().resolve(localVar.getTypeName(), this.visitor.getAliasTable(), env.getFL());
      Type varType = env.getTL().getType(varTypeName);
      visitor.getTypeCallBack().useType(varTypeName);

      Iterator it = localVar.iterateDeclarations();

      while (it.hasNext())
      {
        VariableDeclarator varDecl = (VariableDeclarator)it.next();
        Variable var = null;

        try
        {
          var = new VariableImpl(
            localVar.isConst(),
            varTypeName,
            varDecl.getId(),
            varDecl.hasInitializer()
          );

          visitor.getVE().add(var);
        }
        catch (TypeCheckerException e)
        {
          env.getMS().report(MessageLevel.ERR, "TypeCheck005", varDecl.getId(), varDecl);
        }

        if (varDecl.hasInitializer())
        {
          enterBlock();

          try
          {
            net.nosica.compiler.intermediate.Variable IRVar = visitor.getIRT().translateVariable(var);
            visitor.setAlternativeExp(IRVar);
            visitor.setState(TypeCheckState.PARSING_VARIABLE_INITIALIZER);
            varDecl.getVariableInitializer().accept(visitor);

            TypeCheckerResultType right = this.visitor.getResultType();
            net.nosica.compiler.intermediate.Expression rightExp = null;

            ArrayList deadVariables = null;
            this.visitor.getVE().save();
            try
            {
               rightExp = this.visitor.getTranslatedExp();
            }
            finally
            {
              deadVariables = this.visitor.getVE().restore();
            }

            // check right is not sub
            if (right.isSub())
            {
              env.getMS().report(MessageLevel.ERR, "TypeCheck021", varDecl.getVariableInitializer());

              throw new TypeCheckerErrorExpressionException();
            }

            net.nosica.compiler.intermediate.Expression translatedExp = null;

            if (visitor.getState() != TypeCheckState.PARSED_PRIMITIVE_CONSTRUCTION)
            {
              // the type checker did not parsed a primitive construction

              // if primitive type, add a call to default constructor
              if (varType.isPrimitive())
                addCallToDefaultConstructor(IRVar, varDecl.getVariableInitializer());

              // try to find an assign operator for this variable

              // find method call context
              MethodCallContext mcc = TypeCheckerHelper.findAssignOperator(
                visitor.getType().getName(),
                new TypeCheckerResultTypeImpl(
                  varTypeName,
                  var.isConst()
                ),
                right,
                new OperatorImpl(OperatorEnum.assign),
                true,
                varDecl
              );

              // translate assignment @TODO@ use an INIT statement instead of a MOVE one
              translatedExp = this.visitor.getIRT().translateAssignment(
                mcc,
                this.visitor.getIRT().translateVariable(
                  var
                ),
                rightExp,
                true
              );
            }
            else
            {
              // the type checker found a primtive construction initialization
              // translatedExp contains var.constructor(args)
              translatedExp = rightExp;
            }

            // and add it to list of statements
            statement = this.visitor.getIRT().translateSequence(
              statement,
              translatedExp
            );

            // dead variables
            statement = this.visitor.getIRT().translateDeadVariableDestruction(statement, deadVariables);
          }
          finally
          {
            visitor.noState();
            leaveBlock();
          }
        }
        else
        if (varType.isPrimitive())
        {
          // add a call to default constructor
          addCallToDefaultConstructor(this.visitor.getIRT().translateVariable(var), varDecl);
        }
        else
        {
          // add an initialisation to null
          statement = this.visitor.getIRT().translateSequence(
            statement,
            this.visitor.getIRT().translateInit(
              this.visitor.getIRT().translateVariable(var),
              this.visitor.getIRT().translateLiteral(
                NativeTypes.get_net_nosica_lang_None(),
                "null"
              )
            )
          );
        }
      }
    }
    catch (SymbolException e)
    {
      env.getMS().report(MessageLevel.ERR, "Symbol031", localVar.getTypeName().toString() + " ("+e.getMessage()+")", localVar);
    }
    catch (GenericityException e)
    {
      env.getMS().report(MessageLevel.ERR, "Symbol031", localVar.getTypeName().toString() + " ("+e.getMessage()+")", localVar);
    }
  }

  private ArrayList getVariablesToDestroyForReturn()
  {
    try
    {
      return this.visitor.getVE().variableInScope(
        this.visitor.getVE().getNumberOfActiveScope()
      );
    }
    catch (TypeCheckerException e)
    {
      env.terminate("Internal error : " + e.getMessage());
      return null;
    }
  }

  public void visit(ReturnStatement returnStatement)
  {
    TypeCheckerResultType exp = null;
    net.nosica.compiler.intermediate.Expression trExp = null;

    // type checking
    if (returnStatement.hasExpression())
    {
      // typecheck expression
      returnStatement.getExpression().accept(visitor);

      exp = this.visitor.getResultType();
      trExp = this.visitor.getTranslatedExp();
    }
    else
    {
      exp = new TypeCheckerResultTypeIsSub();
    }

    // sub / resultType checking
    ResultType methodResultType = this.visitor.getMethod().getResultType();
    Method castOperator = null;
    boolean isPrimitive = false;

    if (this.visitor.getMethod().isSub() != exp.isSub())
    {
      if (exp.isSub())
        env.getMS().report(MessageLevel.ERR, "TypeCheck056", methodResultType.getTypeName().toString(), returnStatement.getExpression());
      else
        env.getMS().report(MessageLevel.ERR, "TypeCheck057", returnStatement.getExpression());

      throw new TypeCheckerErrorExpressionException();
    }
    else
    {
      if (!exp.isSub())
      {
        // if returned result is primitive, then act as if it was declared as const
        try
        {
          Type type = env.getTE().getType(methodResultType.getTypeName());
          if (type.isPrimitive())
          {
            isPrimitive = true;
            methodResultType = new ResultTypeImpl(methodResultType.getTypeName(), true);
          }
        }
        catch (SymbolException e)
        {
          env.terminate("Internal error : " + e.getMessage());
        }

        SettableInteger weight = new SettableInteger(0);

        try
        {
          castOperator = env.getMR().match(methodResultType, exp, weight);
        }
        catch (TypeMismatchException e)
        {
          env.getMS().report(MessageLevel.ERR, "TypeCheck058", new String[]{exp.toString(), methodResultType.toString()}, returnStatement.getExpression());
        }
        catch (SymbolException e)
        {
          env.terminate("Internal error : " + e.getMessage());
        }
      }
    }

    // translation
    if (returnStatement.hasExpression())
    {
      // method returns something
      if (isPrimitive)
      {
        if (castOperator != null)
        {
          // add call to cast operator .. (primitive type)
          ArrayList castArgs = new ArrayList();
          castArgs.add(trExp);

          trExp = this.visitor.getIRT().translateMethodCall(
            new MethodCallContextImpl(
              returnStatement.getExpression(),
              castOperator,
              new ArrayList()
            ),
            castArgs
          );
        }

        // translate returnValue = castOperator
        statement = this.visitor.getIRT().translateSequence(
          statement,
          this.visitor.getIRT().translateInit(
            returnValue,
            trExp
          )
        );
      }
      else
      {
        // reference type
        statement = this.visitor.getIRT().translateSequence(
          statement,
          this.visitor.getIRT().translateReferenceAssignment(
            returnValue,
            this.visitor.getTranslatedExp()
          )
        );
      }

      // dead variables destruction
      statement = this.visitor.getIRT().translateDeadVariableDestruction(
        statement,
        getVariablesToDestroyForReturn()
      );

      // jump to epilogue of function
      statement = this.visitor.getIRT().translateSequence(
        statement,
        this.visitor.getIRT().translateJump(
          endMethodLabel
        )
      );
    }
    else
    {
      // method returns, no result
      statement = this.visitor.getIRT().translateDeadVariableDestruction(
        statement,
        getVariablesToDestroyForReturn()
      );

      statement = this.visitor.getIRT().translateSequence(
        statement,
        this.visitor.getIRT().translateJump(
          endMethodLabel
        )
      );
    }
  }

  public void visit(StatementExpressionTerminator statementExpression)
  {
    statementExpression.getStatementExpression().accept(this);
  }

  public void visit(StatementExpression statementExpression)
  {
    enterBlock();

    try
    {
      if (statementExpression.isPrefix())
      {
        Node node = statementExpression.getPrefix();
        node.accept(visitor);
      }
      else
      if (statementExpression.isPrimaryAssignment())
      {
        Node node = statementExpression.getPrimaryAssignment();
        node.accept(visitor);
      }
      else
      if (statementExpression.isPrimary())
      {
        Node node = statementExpression.getPrimaryExpression();
        node.accept(visitor);
      }
      else
      if (statementExpression.isStream())
      {
        Node node = statementExpression.getStream();
        node.accept(visitor);
      }

      statement = this.visitor.getIRT().translateSequence(
        statement,
        this.visitor.getTranslatedExp()
      );
    }
    finally
    {
      leaveBlock();
    }
  }

  public void visit(StatementExpressionList statementExpressionList)
  {
    Iterator it = statementExpressionList.iterateStatements();
    while (it.hasNext())
      ((net.nosica.parser.node.typedeclaration.statement.Statement)it.next()).accept(this);
  }

  public void visit(SwitchStatement switchStatement)
  {
    /*
      This one is quite difficult.
      Here's the structure :

      switch (switchExpression)
      {
      case cond1_1, cond2_1, ... : (blocks_1)*
      case cond1_2, cond2_2, ... : (blocks_2)*
      }
      [
      else
      elseStatement
      ]

      we transform it in :

      TemporaryVariable tv = switchExpression;
      if (tv == cond1_1)
        goto blocks_1;
      else
      if (tv == cond2_1)
        goto blocks_1;
      else ...
      else
      if (tv == cond1_2)
        goto blocks_2;
      else
      if (tv == cond2_2)
        goto blocks_2;
      else ...
      [elseStatement]
      goto endSwitch
      blocks_1 :
        blocks
      goto endSwitch
      blocks_2 :
        blocks
      goto endSwitch
      endSwitch :
    */

    // typecheck expression
    boolean switchExpressionTypeCheckError = false;
    TypeCheckerResultType exp = null;

    ArrayList deadVariables;
    this.visitor.getVE().save();
    try
    {
      switchStatement.getExpression().accept(visitor);
      exp = this.visitor.getResultType();

      // report error
      if (exp.isSub())
      {
        env.getMS().report(MessageLevel.ERR, "TypeCheck021", switchStatement.getExpression());
        switchExpressionTypeCheckError = true;
      }
    }
    finally
    {
      deadVariables = this.visitor.getVE().restore();
    }

    // create a temporary variable
    net.nosica.compiler.intermediate.Variable switchExp = translateVariableAssignmentAndDeadVariableDestruction(this.visitor.getTranslatedExp(), deadVariables);

    boolean isExpPrimitive = false;
    try
    {
      isExpPrimitive = env.getTE().getType(exp.getTypeName()).isPrimitive();
    }
    catch (SymbolException e)
    {
      env.terminate("Internal error : " + e.getMessage());
    }

    Label trueLabel = null;
    Label endLabel = this.visitor.getLabelRepository().generateLabel();
    ArrayList blockLabels = new ArrayList();

    boolean switchConditionTypeCheckError = false;
    // iterate through switch label
    Iterator it = switchStatement.iterateLabels();
    while (it.hasNext())
    {
      SwitchLabel switchLabel = (SwitchLabel)it.next();

      trueLabel = this.visitor.getLabelRepository().generateLabel();
      blockLabels.add(trueLabel);

      // iterator through conditions of label
      Iterator j = switchLabel.iterateConditions();
      while (j.hasNext())
      {
        net.nosica.parser.node.typedeclaration.expression.Expression cond =
          (net.nosica.parser.node.typedeclaration.expression.Expression)j.next();

        ArrayList temporaries = null;
        this.visitor.getVE().save();
        try
        {
          cond.accept(visitor);
        }
        finally
        {
          temporaries = this.visitor.getVE().restore();
        }

        TypeCheckerResultType condExp = this.visitor.getResultType();

        // condition cannot be sub
        if (condExp.isSub())
        {
          switchConditionTypeCheckError = true;
          env.getMS().report(MessageLevel.ERR, "TypeCheck021", cond);
        }

        if (!switchExpressionTypeCheckError && !switchConditionTypeCheckError)
        {
          net.nosica.compiler.intermediate.Variable var = translateVariableAssignmentAndDeadVariableDestruction(this.visitor.getTranslatedExp(), temporaries);

          // means : translate only if no errors was reported before ...
          Label nextLabel = this.visitor.getLabelRepository().generateLabel();

          // find comparison using typeCheckVisitor services
          MethodCallContext mcc = this.visitor.findBinaryOperator(
            exp,
            condExp,
            new OperatorImpl(isExpPrimitive ? OperatorEnum.ieq : OperatorEnum.eq),
            cond
          );

          // translate it
          this.visitor.translateBinaryOperator(
            switchExp,
            var,
            mcc
          );

          net.nosica.compiler.intermediate.Variable boolVar = translateVariableAssignmentAndDeadVariableDestruction(this.visitor.getTranslatedExp(), deadVariables);

          // condJump(cond, blockLabel, nextLabel)
          statement = this.visitor.getIRT().translateSequence(
            statement,
            this.visitor.getIRT().translateConditionalJump(
              boolVar,
              trueLabel,
              nextLabel
            )
          );

          // nextLabel :
          statement = this.visitor.getIRT().translateSequence(
            statement,
            this.visitor.getIRT().translateLabel(
              nextLabel
            )
          );
        }
      }
    }

    // if else statement exists
    if (switchStatement.hasElseStatement())
    {
      // type check else statement (and produce ir code)
      switchStatement.elseStatement().accept(this);

      // goto endLabel :
      statement = this.visitor.getIRT().translateSequence(
        statement,
        this.visitor.getIRT().translateJump(endLabel)
      );
    }

    it = switchStatement.iterateLabels();
    Iterator j = blockLabels.iterator();
    while (it.hasNext() && j.hasNext())
    {
      SwitchLabel switchLabel = (SwitchLabel)it.next();
      Label label = (Label)j.next();

      // label :
      statement = this.visitor.getIRT().translateSequence(
        statement,
        this.visitor.getIRT().translateLabel(label)
      );

      // type check block and produce ir code
      switchLabel.getBlock().accept(this);

      // goto endLabel :
      statement = this.visitor.getIRT().translateSequence(
        statement,
        this.visitor.getIRT().translateJump(endLabel)
      );
    }

    // endLabel :
    statement = this.visitor.getIRT().translateSequence(
      statement,
      this.visitor.getIRT().translateLabel(endLabel)
    );
  }

  private void checkExceptionType(TypeCheckerResultType temp, net.nosica.parser.node.typedeclaration.expression.Expression throwExpression)
  {
    if (temp.isSub())
    {
      env.getMS().report(MessageLevel.ERR, "TypeCheck021", throwExpression);
      throw new TypeCheckerErrorExpressionException();
    }

    // check expression inherits from net.nosica.lang.Throwable
    try
    {
      // load Throwable !!
      try
      {
        env.getTL().getType(NativeTypes.get_net_nosica_lang_Throwable());
        this.visitor.getTypeCallBack().useType(NativeTypes.get_net_nosica_lang_Throwable());
      }
      catch (GenericityException e)
      {
        env.terminate("Internal error : " + e.getMessage());
      }
      catch (VisitorException e)
      {
        env.terminate("Internal error : " + e.getMessage());
      }

      if (!env.getTE().doesInherits(temp.getTypeName(), NativeTypes.get_net_nosica_lang_Throwable()))
      {
        env.getMS().report(MessageLevel.ERR, "TypeCheck061", temp.getTypeName().toString(), throwExpression);

        throw new TypeCheckerErrorExpressionException();
      }
    }
    catch (SymbolException e)
    {
      env.terminate("Internal error : " + e.getMessage());
    }
  }

  public void visit(ThrowStatement throwStatement)
  {
    enterBlock();
    try
    {
      // typecheck thrown exception
      throwStatement.getExpression().accept(visitor);

      TypeCheckerResultType temp = this.visitor.getResultType();
      statement = this.visitor.getIRT().translateSequence(
        statement,
        this.visitor.getIRT().translateThrow(
        this.visitor.getTranslatedExp()
        )
      );

      checkExceptionType(temp, throwStatement.getExpression());

      this.visitor.checkException(temp.getTypeName(), throwStatement);
    }
    finally
    {
      leaveBlock();
    }
  }

  private void registerActiveCatchSite(Iterator it)
  {
    // now add handlers to exception environment
    while (it.hasNext())
    {
      CatchStatement catchStatement = (CatchStatement)it.next();

      TypeName typeName = catchStatement.getTypeName();

      try
      {
        // load type
        typeName = env.getTR().resolve(typeName, this.visitor.getAliasTable(), env.getFL());
        env.getTL().getType(typeName);
        this.visitor.getTypeCallBack().useType(typeName);
      }
      catch (SymbolException e)
      {
        env.getMS().report(MessageLevel.ERR, "Symbol031", typeName.toString(), catchStatement);
        throw new TypeCheckerErrorExpressionException();
      }
      catch (GenericityException e)
      {
        throw new TypeCheckerErrorExpressionException();
      }

      // set typeName so that is it no longer needed to resolve it
      catchStatement.setTypeName(typeName);

      // register catch site
      try
      {
        this.visitor.getEE().add(typeName);
      }
      catch (SymbolException e)
      {
        env.terminate("Internal error : " + e.getMessage());
      }
      catch (TypeCheckerException e)
      {
        env.getMS().report(MessageLevel.ERR, "TypeCheck059", typeName.toString(), catchStatement);
      }
    }
  }

  private void reportUnusedCatchSite(Set unusedActiveException, Iterator j)
  {
    Iterator it = unusedActiveException.iterator();
    while (it.hasNext())
    {
      TypeName unusedTypeName = (TypeName)it.next();

      // let's find unused catch statement
      boolean foundCatchSite = false;
      while (j.hasNext())
      {
        CatchStatement catchStatement = (CatchStatement)j.next();

        TypeName caughtException = catchStatement.getTypeName();

        if (caughtException.equals(unusedTypeName))
        {
          // report only if exception does not derive from RuntimException
          try
          {
            if (
                !caughtException.equals(NativeTypes.get_net_nosica_lang_RuntimeException()) &&
                !env.getTE().doesInherits(caughtException, NativeTypes.get_net_nosica_lang_RuntimeException())
               )
            env.getMS().report(MessageLevel.ERR, "TypeCheck060", unusedTypeName.toString(), catchStatement);
          }
          catch (SymbolException e)
          {
            env.terminate("Internal error : " + e.getMessage());
          }

          foundCatchSite = true;
          break;
        }
      }

      if (!foundCatchSite)
        env.terminate("Internal error : could not assign exception and catch site");
    }
  }

  public void visit(TryStatement tryStatement)
  {
    /*
     * Here's the structure :
     * try
     * {
     *   tryBlock
     * }
     * (
     *   catch (Exception_i e)
     *   {
     *     catchBlock_i
     *   }
     * )*
     * [
     *   finally
     *   {
     *     finallyBlock
     *   }
     * ]
     *
     * and how we transform it :
     *
     * TryRegister(
     *   Map<Exception_i, Variable>,
     *   Map<Exception_i, catchLabel_i>,
     *   finallyStatement
     * )
     * tryBlock
     * TryUnregister(
     * )
     * [finallyBlock}
     * jump endTry
     * catchLabel_0 :
     *   catchBlock_0
     * jump endTry
     * catchLabel_1 :
     *   catchBlock_1
     * jump endTry
     * ...
     * catchLabel_n
     *   try_Block_n
     * jump endTry
     *
     * endTry :
     *
     * TryRegister will receive a thrown exception, then will
     * have to clean up live variables, then
     * It will have to either :
     *   jump to corresponding catch block, or
     *   rethrow exception if it cannot be handled here
     */

    // type check finally statement if any
    Statement savedStatement = statement;
    statement = null;
    Statement finallyStatement = null;

    if (tryStatement.hasFinally())
    {
      enterBlock();
      try
      {
        tryStatement.getFinallyBlock().accept(this);
      }
      finally
      {
        leaveBlock();
      }
    }

    finallyStatement = statement;
    statement = savedStatement;

    // let's register which exceptions are caught
    if (tryStatement.hasCatch())
    {
      // let's save exception environment
      this.visitor.getEE().save();
      registerActiveCatchSite(tryStatement.iterateCatch());
    }

    // add a try register statement
    Map exceptionMap = new HashMap();  // will be completed later
    statement = this.visitor.getIRT().translateSequence(
      statement,
      this.visitor.getIRT().translateTryRegister(
        exceptionMap,
        finallyStatement
      )
    );

    // while analysing try block, all thrown exceptions will be
    // marked as launched in the EE
    enterBlock();
    try
    {
      // type check tryBlock
      tryStatement.getTryBlock().accept(this);
    }
    finally
    {
      leaveBlock();
    }

    // add a TryUnregister statement
    statement = this.visitor.getIRT().translateSequence(
      statement,
      this.visitor.getIRT().translateTryUnregister()
    );

    if (tryStatement.hasCatch())
    {
      // It is now time to restore EE and get all exceptions
      // that have not been marked as launched ...
      Set unusedActiveException = visitor.getEE().restore();

      // error reporting
      reportUnusedCatchSite(unusedActiveException, tryStatement.iterateCatch());

      // generate endTryLabel
      Label endTryLabel = this.visitor.getLabelRepository().generateLabel();

      // now type check each catchStatement
      Iterator it = tryStatement.iterateCatch();
      while (it.hasNext())
      {
        CatchStatement catchStatement = (CatchStatement)it.next();

        // generate label
        Label label = this.visitor.getLabelRepository().generateLabel();

        enterBlock();
        try
        {
          // add label to statements
          statement = this.visitor.getIRT().translateSequence(
            statement,
            this.visitor.getIRT().translateLabel(
              label
            )
          );

          Variable catchVar = new VariableImpl(
            catchStatement.isConst(),
            catchStatement.getTypeName(),
            catchStatement.getVarName(),
            false
          );

          try
          {
            this.visitor.getVE().add(catchVar);
          }
          catch (TypeCheckerException e)
          {
            env.getMS().report(MessageLevel.ERR, "TypeCheck005", catchVar.getName(), catchStatement);
          }

          // type check catch block
          catchStatement.accept(this);

          // complete exceptionMap
          exceptionMap.put(
            catchStatement.getTypeName(),
            new Pair(
              label,
              this.visitor.getIRT().translateVariable(
                catchVar
              )
            )
          );
        }
        finally
        {
          leaveBlock();
        }

        //			jump endTry
    statement = this.visitor.getIRT().translateSequence(
       statement,
       this.visitor.getIRT().translateJump(
         endTryLabel
       )
    );
      }

      // add label endTryLabel to statements
      statement = this.visitor.getIRT().translateSequence(
        statement,
        this.visitor.getIRT().translateLabel(
          endTryLabel
        )
      );
    }
  }

  public void visit(WhileStatement whileStatement)
  {
    /*
      Here's the structure
      while (cond)
      {
        statements
      }
      [
      else
      elseStatement
      ]

      here's how we transform it :

      jump conditionLabel
      beginLabel :
        enterBlock
        statements
        leaveBlock
      conditionLabel :
      ConditionalJump(cond, beginLabel, endLabel, temporariesOfCondition)
      [
        elseLabel :
          enterBlock
          elseStatement
          leaveBlock
      ]
      endLabel :
    */

    Label beginLabel = null;
    Label endLabel = null;
    Label elseLabel = null;
    Label conditionLabel = this.visitor.getLabelRepository().generateLabel();

    try
    {
      beginLabel = (Label)activeLabels.peek();
      endLabel = (Label)endLabels.peek();
    }
    catch (java.util.EmptyStackException e)
    {
      env.terminate("Internal error : " + e.getMessage());
    }

    if (whileStatement.hasElseStatement())
      elseLabel = this.visitor.getLabelRepository().generateLabel();
    else
      elseLabel = endLabel;

    // jump conditionLabel
    statement = this.visitor.getIRT().translateSequence(
      statement,
      this.visitor.getIRT().translateJump(conditionLabel)
    );

    boolean isUserLabel = false;
    try
    {
      this.visitor.getLabelRepository().add(beginLabel);
      saveLabelScope(beginLabel, this.visitor.getLabelRepository().getNumberOfActiveScope());
    }
    catch (TypeCheckerException e)
    {
      // can happen, because label has already been added by LabeledStatement
      isUserLabel = true;
    }

    if (!isUserLabel)
    {
      // beginLabel :
      statement = this.visitor.getIRT().translateSequence(
        statement,
        this.visitor.getIRT().translateLabel(beginLabel)
        );
    }

    enterBlock();
    try
    {
      whileStatement.getStatement().accept(this);
    }
    finally
    {
      leaveBlock();
    }

    // conditionLabel :
    statement = this.visitor.getIRT().translateSequence(
      statement,
      this.visitor.getIRT().translateLabel(conditionLabel)
    );

    ArrayList deadVariables;
    this.visitor.getVE().save();
    try
    {
      // typecheck condition
      whileStatement.getCondition().accept(visitor);

      // typecheck condition
      TypeCheckerResultType exp = this.visitor.getResultType();
      if (exp.isSub())
        env.getMS().report(MessageLevel.ERR, "TypeCheck021", whileStatement.getCondition());
      else
      {
        if (!exp.getTypeName().equals(NativeTypes.get_net_nosica_lang_bool()))
          env.getMS().report(MessageLevel.ERR, "TypeCheck020", exp.getTypeName().toString(), whileStatement.getCondition());
      }
    }
    finally
    {
      deadVariables = this.visitor.getVE().restore();
    }

    net.nosica.compiler.intermediate.Variable var = translateVariableAssignmentAndDeadVariableDestruction(this.visitor.getTranslatedExp(), deadVariables);

    // condJump(cond, internalBeginLoopLabel, elseLabel, temporaries)
    statement = this.visitor.getIRT().translateSequence(
      statement,
      this.visitor.getIRT().translateConditionalJump(
        var,
        beginLabel,
        elseLabel
      )
    );

    // if else statement
    if (whileStatement.hasElseStatement())
    {
      // elseLabel :
      statement = this.visitor.getIRT().translateSequence(
        statement,
        this.visitor.getIRT().translateLabel(elseLabel)
      );

      enterBlock();
      try
      {
        whileStatement.getElseStatement().accept(this);
      }
      finally
      {
        leaveBlock();
      }
    }

    if (!isUserLabel)
    {
      // endLabel :
      statement = this.visitor.getIRT().translateSequence(
        statement,
        this.visitor.getIRT().translateLabel(endLabel)
        );
    }
  }

  private net.nosica.compiler.intermediate.Variable translateVariableAssignmentAndDeadVariableDestruction(
    net.nosica.compiler.intermediate.Expression exp,
    ArrayList deadVariables
  )
  {
    net.nosica.compiler.intermediate.Variable var = null;

    if (exp instanceof net.nosica.compiler.intermediate.Variable)
    {
      // OK
      var = (net.nosica.compiler.intermediate.Variable)exp;
    }
    else
    {
      Variable temporary = new VariableImpl(false, exp.getTypeName(), env.getNG().generateTemporaryName(), true);
      var = this.visitor.getIRT().translateVariable(temporary);
      statement = this.visitor.getIRT().translateSequence(
        statement,
        this.visitor.getIRT().translateInit(var, exp)
      );

      statement = this.visitor.getIRT().translateDeadVariableDestruction(statement, deadVariables);
    }

    return var;
  }
}
