/*
  $Id: TypeCheckerResultType.java,v 1.12 2004/03/04 22:16:16 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.compiler.typechecker;

import net.nosica.parser.node.util.var.TypeName;
import net.nosica.parser.node.Node;

import java.util.ArrayList;

public interface TypeCheckerResultType extends ResultType
{
  /**
   * \brief is this result type a sub result type ?
   *
   * \return true if this result type is sub
   */
  boolean isSub();

  /**
   * \brief is this result type in a static context ?
   *
   * \return true if this result type represents a static context
   */
  boolean isStatic();

  /**
   * \brief is this result type an unresolved property set ?
   *
   * \return true if unresolved property set, false otherwise
   */
  boolean isPropertySet();

  /**
   * \brief returns the dimension arguments translated for a property set array
   *
   * \return ArrayList<Expression>
   */
  ArrayList getTranslatedPropertySetArguments();

  /**
   * \brief create an assign operator for ths result type
   *
   * \param right the right argument
   * \param node for error reporting
   */
  Method createAssignOperator(TypeCheckerResultType right, Node node);

  /**
   * \brief resolve this property set
   *
   * requires : isPropertySet()
   * \param right the right argument
   * \param accessor the type which call this property set
   * \param matchingNames ArrayList<Method> the list of matching methods by name
   * \param node for error reporting
   */
  ArrayList resolvePropertySet(
    TypeCheckerResultType right,
    TypeName accessor,
    ArrayList matchingNames,
    Node node
  );
}