/*
  $Id: TypeCheckerResultTypeSetProperty.java,v 1.14 2004/03/18 23:15:20 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.compiler.typechecker;

import net.nosica.application.MessageLevel;
import net.nosica.compiler.application.Environment;
import net.nosica.compiler.application.TypeCheckerErrorExpressionException;

import net.nosica.parser.node.Node;
import net.nosica.parser.node.util.var.TypeName;

import net.nosica.symbol.SymbolException;

import java.util.ArrayList;
import java.util.Iterator;

public class TypeCheckerResultTypeSetProperty implements TypeCheckerResultType
{
  private ArrayList unresolvedMatchingMethods; // ArrayList<Method>
  private ArrayList args;                      // ArrayList<Argument>
  private ArrayList propertySetArguments;      // ArrayList<Expression>

  public TypeCheckerResultTypeSetProperty(
    ArrayList unresolvedMatchingMethods,
    ArrayList args,
    ArrayList propertySetArguments
  )
  {
    this.unresolvedMatchingMethods = unresolvedMatchingMethods;
    this.args = args;
    this.propertySetArguments = propertySetArguments;
  }

  public boolean isSub() { return true; }
  public boolean isStatic() { return false; }
  public TypeName getTypeName() { return null; }
  public boolean isConst() { return true; }
  public boolean isPropertySet() { return true; }
  public ArrayList getTranslatedPropertySetArguments() { return propertySetArguments; }
  public Method createAssignOperator(TypeCheckerResultType right, Node node)
  {
    Method m = (Method)unresolvedMatchingMethods.get(0);

    ArrayList arguments = new ArrayList();
    arguments.addAll(args);
    arguments.add(new ArgumentImpl(right.isConst(), right.getTypeName(), ""));

    return new MethodImpl(
      m.getResultType(),
      m.getName(),
      m.getKind(),
      arguments,
      m.isConst(),
      m.getType(),
      m.getAliasTable()
    );
  }
  public ArrayList resolvePropertySet(
    TypeCheckerResultType right,
    TypeName accessor,
    ArrayList matchingNames,
    Node node
  )
  {
    ArrayList matchingMethods = new ArrayList();  // ArrayList<MethodCallContext>

    Iterator it = unresolvedMatchingMethods.iterator();
    while (it.hasNext())
    {
      Method m = (Method)it.next();

      // ArrayList<Argument>
      ArrayList arguments = new ArrayList();
      arguments.addAll(args);
      arguments.add(new ArgumentImpl(right.isConst(), right.getTypeName(), ""));

      Method method = new MethodImpl(
        m.getResultType(),
        m.getName(),
        m.getKind(),
        arguments,
        m.isConst(),
        m.getType(),
        m.getAliasTable()
      );

      try
      {
        ArrayList deltaMatchingNames = new ArrayList();

        ArrayList deltaMatchingMethods = Environment.instance().getMR().findMatchingMethods(
          new ResultTypeImpl(method.getType(), false),
          method,
          accessor,
          deltaMatchingNames
        );

        addMethodCallContextArrayList(matchingMethods, deltaMatchingMethods);
        addMethodArrayList(matchingNames, deltaMatchingNames);
      }
      catch (SymbolException e)
      {
        Environment.instance().getMS().report(MessageLevel.ERR, "Symbol031", m.getType().toString(), node);
        throw new TypeCheckerErrorExpressionException();
      }
    }

    return matchingMethods;
  }

  private void addMethodCallContextArrayList(ArrayList repository, ArrayList delta)
  {
    Iterator it = delta.iterator();

searchLoop :
    while (it.hasNext())
    {
      MethodCallContext m = (MethodCallContext)it.next();

      Iterator j = repository.iterator();
      while (j.hasNext())
      {
        MethodCallContext mr = (MethodCallContext)j.next();
        if (mr.getMethod().hasSameSignature(m.getMethod()))
          continue searchLoop;
      }

      repository.add(m);
    }
  }

  private void addMethodArrayList(ArrayList repository, ArrayList delta)
  {
    Iterator it = delta.iterator();

searchLoop :
    while (it.hasNext())
    {
      Method m = (Method)it.next();

      Iterator j = repository.iterator();
      while (j.hasNext())
      {
        Method mr = (Method)j.next();
        if (mr.hasSameSignature(m))
          continue searchLoop;
      }

      repository.add(m);
    }
  }
}