/*
  $Id: VariableEnvironment.java,v 1.8 2004/03/04 22:16:17 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.compiler.typechecker;

import java.util.ArrayList;


/**
 * \brief Manage the list of live variable during type check
 *
 * Note : The save/restore methods act pretty like a stack
 *        You must call save before calling restore
 *        With these two methods you can manage the scope during type check
 */

public interface VariableEnvironment
{
  /**
   * \brief Save the state of the environment
   */
  void save();

  /**
   * \brief Restore the state of the environment to the last 'save' state
   * \return ArrayList<Variable> list of variable that were removed from VariableEnvironment
   */
  ArrayList restore();

  /**
   * \brief return the number of active scope
   *
   * \return number of active scope
   */
  int getNumberOfActiveScope();

  /**
   * \brief return all variables contained in last nbScope scope
   *
   * \param nbScope number of scope to look in
   * \return ArrayList<Variable> list of variables that are in those scopes
   * \throw TypeCheckerException is asked nbScope is greater or equal than currently registered scope
   */
  ArrayList variableInScope(int nbScope) throws TypeCheckerException;

  /**
   * \brief Add a variable to the environment
   *
   * \param variable the variable to add to the environment
   */
  void     add(Variable variable) throws TypeCheckerException;

  /**
   * \brief Get a variable by its name
   *
   * \param name the name of the variable to return
   * \return the Variable (if any) (null if not found)
   */
  Variable get(String name);
}
