/*
  $Id: VariableEnvironmentImpl.java,v 1.9 2004/03/04 22:16:17 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.compiler.typechecker;

import java.util.Map;
import java.util.HashMap;
import java.util.Stack;
import java.util.ArrayList;


public class VariableEnvironmentImpl implements VariableEnvironment
{
  private Map variables   = new HashMap();  // Map<String, Variable>
  private Stack scopes    = new Stack();    // Stack<Integer>
  private Stack undoStack = new Stack();    // Stack<String>
  private int nbModificationsSinceLastSave = 0;

  public void save()
  {
    scopes.push(new Integer(nbModificationsSinceLastSave));
    nbModificationsSinceLastSave = 0;
  }

  public ArrayList restore()
  {
    ArrayList removedVariable = new ArrayList();

    for (int i = 0; i < nbModificationsSinceLastSave; ++i)
    {
      String variableName = (String)undoStack.pop();
      removedVariable.add(variables.remove(variableName));
    }

    nbModificationsSinceLastSave = ((Integer)scopes.pop()).intValue();

    return removedVariable;
  }

  public int getNumberOfActiveScope()
  {
    return scopes.size();
  }

  public ArrayList variableInScope(int nbScope) throws TypeCheckerException
  {
    ArrayList removedVariable = new ArrayList();
    int tmpNbMod = nbModificationsSinceLastSave;
    int index = 0;

    if (nbScope > scopes.size())
      throw new TypeCheckerException("Too much scope asked : asked [" + nbScope + "], got [" + scopes.size() + "] at most");

    for (int i = 0; i < nbScope; ++i)
    {
      for (int j = 0; j < tmpNbMod; ++j, ++index)
      {
        String variableName = (String)undoStack.get(undoStack.size() - 1 - index);
        removedVariable.add(variables.get(variableName));
      }

      tmpNbMod = ((Integer)scopes.get(scopes.size() - 1 - i)).intValue();
    }

    return removedVariable;
  }

  public void add(Variable variable) throws TypeCheckerException
  {
    // check duplicate names
    if (variables.containsKey(variable.getName()))
      throw new TypeCheckerException("A variable with same " + variable.getName() + " is already active");

    ++nbModificationsSinceLastSave;

    undoStack.push(variable.getName());
    variables.put(variable.getName(), variable);
  }

  public Variable get(String name)
  {
    return (Variable)variables.get(name);
  }
}
