/*
  $Id: JavaLikeClassGenerator.java,v 1.11 2004/03/27 22:37:25 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.doc.production.html;

import net.nosica.application.NativeTypes;
import net.nosica.application.SymbolEnvironment;
import net.nosica.application.SymbolEnvironmentImpl;
import net.nosica.application.ApplicationContext;
import net.nosica.application.ApplicationContextImpl;

import net.nosica.doc.application.ReverseEnvironment;
import net.nosica.doc.application.ReverseEnvironmentImpl;
import net.nosica.doc.application.CommentHelper;

import net.nosica.symbol.GenericType;
import net.nosica.symbol.GenericMethod;
import net.nosica.symbol.Field;
import net.nosica.symbol.SymbolException;
import net.nosica.symbol.MethodKindEnum;
import net.nosica.symbol.Argument;
import net.nosica.symbol.Accessible;
import net.nosica.symbol.GenericSymbol;

import net.nosica.parser.node.util.var.TypeName;
import net.nosica.parser.node.typedeclaration.Symbol;
import net.nosica.parser.node.util.generic.GenericParameter;

import net.nosica.util.IndentedPrintWriter;
import net.nosica.util.FileHelper;
import net.nosica.util.XHTMLWriter;
import net.nosica.util.XHTMLEntity;
import net.nosica.util.XHTMLHelper;

import java.util.Map;
import java.util.HashMap;
import java.util.Set;
import java.util.HashSet;
import java.util.ArrayList;
import java.util.Iterator;

import java.io.File;
import java.io.FileOutputStream;

import java.net.URLEncoder;

/**
 * \brief generate class information "a la" Java
 */
public class JavaLikeClassGenerator implements ClassGenerator
{
  private ApplicationContext context = ApplicationContextImpl.instance();
  private SymbolEnvironment symbolEnv = SymbolEnvironmentImpl.instance();
  private ReverseEnvironment reverseEnv = ReverseEnvironmentImpl.instance();
  private String outputDirectory;
  private static final String encoding = "UTF-8";

  public JavaLikeClassGenerator(String outputDirectory)
  {
    this.outputDirectory = outputDirectory;
  }

  /**
   * \brief generate an all-classes file and one html file per class
   */
  public void generate()
  {
    File file = FileHelper.getFile(outputDirectory + "/all-classes.html");

    FileOutputStream outputStream = null;

    try
    {
      outputStream = new FileOutputStream(file);
      IndentedPrintWriter writer = new IndentedPrintWriter(outputStream);
      XHTMLWriter xhtml = new XHTMLWriter(writer);
      XHTMLEntity html = XHTMLHelper.printHeader(xhtml, "classes", "nosicadoc.css");

      xhtml.beginTag("body");

      xhtml.beginTag("table", "border=\"0\" cellspacing=\"0\" cellpadding=\"0\"");

      Iterator it = symbolEnv.iterateTypeName();
      while (it.hasNext())
      {
        TypeName typeName = (TypeName)it.next();
        try
        {
          GenericType type = symbolEnv.getGenericType(typeName);

          XHTMLEntity tr = xhtml.beginTag("tr");
          xhtml.beginTag("td");

          try
          {
            xhtml.beginTag("a", "href=\"" + URLEncoder.encode(type.getName().toString() + ".html", encoding) + "\" target=\"mainFrame\"");
            xhtml.print(type.getName().getLastComponent().getPackageElement());
          }
          catch (java.io.UnsupportedEncodingException e)
          {
            context.terminate("Internal error : " + e.getMessage());
          }

          xhtml.endTag(tr);

          generateType(type);
        }
        catch (SymbolException e)
        {
          context.terminate("Internal error : " + e.getMessage());
        }
      }

      xhtml.endTagLn(html);
      writer.flush();
      outputStream.close();
    }
    catch (java.io.FileNotFoundException e)
    {
      context.terminate(e.getMessage());
    }
    catch (java.io.IOException e)
    {
      context.terminate(e.getMessage());
    }
    finally
    {
      try
      {
        outputStream.close();
      }
      catch (java.io.IOException e)
      {
        context.terminate(e.getMessage());
      }
    }
  }

  /**
   * \brief generate documentation on a member
   *
   * \param comment the comment containing the doc
   * \param token the token to look for when generating the doc
   * \param cssClassDoc the css class to use for param for example
   * \param cssClassDocContent the css class to use for the content
   * \param xhtml the writer
   * \return true if token was found in comment and the doc was generated, false otherwise
   */
  private boolean generateHTMLDoc(String comment, String token, String cssClassDoc, String cssClassDocContent, XHTMLWriter xhtml)
  {
    // search token
    int pos = comment.indexOf(token);
    if (pos != -1)
    {
      // found
      comment = comment.substring(pos + token.length()).trim();

      // if we have a doc member that needs a name
      String firstWord = "";
      if (cssClassDoc.length() != 0)
      {
        int whitespace = comment.indexOf(' ');
        if (whitespace == -1)
          whitespace = comment.indexOf('\t');
        if (whitespace != -1)
        {
          firstWord = comment.substring(0, whitespace);
          comment = comment.substring(whitespace + 1);
        }
      }

      if (firstWord.length() != 0)
      {
        // highlight name with the given css
        XHTMLEntity divDoc = xhtml.beginTag("div", "class=\"" + cssClassDoc + "\"");
        xhtml.print(firstWord);
        xhtml.endTag(divDoc);
      }

      // highlight content with given css
      XHTMLEntity divDocContent = xhtml.beginTag("div", "class=\"" + cssClassDocContent + "\"");
      xhtml.print(comment);
      xhtml.endTag(divDocContent);

      return true;
    }

    return false;
  }

  /**
   * \brief generate documentation for a given symbol
   *
   * \param symbol the symbol to generate doc for
   * \param xhtml the writer
   */
  private void generateDoc(Symbol symbol, XHTMLWriter xhtml)
  {
    if (!symbol.hasComments())
      return;

    ArrayList comments = CommentHelper.filterComments(symbol.getComments());
    Iterator it = comments.iterator();
    while (it.hasNext())
    {
      String comment = (String)it.next();
      if (!generateHTMLDoc(comment, "\\brief", "", "brief_content", xhtml))
      {
        if (!generateHTMLDoc(comment, "\\param", "param", "param_content", xhtml))
        {
          if (!generateHTMLDoc(comment, "\\return", "", "return_content", xhtml))
          {
            if (!generateHTMLDoc(comment, "\\exception", "exception", "exception_content", xhtml))
            {
              XHTMLEntity divDocContent = xhtml.beginTag("div", "class=\"doc_content\"");
              xhtml.print(comment);
              xhtml.endTag(divDocContent);
            }
          }
        }
      }
    }
  }

  /**
   * \brief generate the html file documenation for the given type
   *
   * \param type the type to generate doc to
   */
  public void generateType(GenericType type)
  {
    File file = FileHelper.getFile(outputDirectory + "/" + type.getName().toString() + ".html");

    FileOutputStream outputStream = null;

    try
    {
      outputStream = new FileOutputStream(file);
      IndentedPrintWriter writer = new IndentedPrintWriter(outputStream);
      XHTMLWriter xhtml = new XHTMLWriter(writer);
      XHTMLEntity html = XHTMLHelper.printHeader(xhtml, "type" + type.getName(), "nosicadoc.css");

      xhtml.beginTag("body");

      {
        XHTMLEntity p = xhtml.beginTag("p", "class=\"package_name\"");
        xhtml.print(type.getName().getFirstComponents().toString());
        xhtml.endTag(p);
      }
      {
        XHTMLEntity p = xhtml.beginTag("p", "class=\"name\"");
        xhtml.print(type.getName().getLastComponent().getPackageElement());
        xhtml.endTag(p);
      }

      xhtml.printTag("hr");
      recGenerateInheritanceTree(type.getName(), xhtml);
      generateGenericInformation(type, xhtml);
      generateClassSpecificInformation(type, xhtml);
      generateInterfaceSpecificInformation(type, xhtml);
      generateSubClassesInformation(type, xhtml);
      xhtml.printTag("hr");

      {
        XHTMLEntity p = xhtml.beginTag("p");
        generateDoc(type.getNode(), xhtml);
        xhtml.endTag(p);
      }

      xhtml.printTag("hr");

      ArrayList constructors = new ArrayList();
      ArrayList properties   = new ArrayList();
      ArrayList operators    = new ArrayList();
      ArrayList methods      = new ArrayList();
      extractMethodsByType(type, constructors, properties, operators, methods);

      generateFieldsSummary(type, xhtml);
      xhtml.printTag("hr");
      generateMethodsSummary(type, constructors, properties, operators, methods, xhtml);

      generateInitializer(type, xhtml);
      generateDeinitializer(type, xhtml);

      generateFields(type, xhtml);
      generateMethods(constructors.iterator(), xhtml);
      generateDestructor(type, xhtml);
      generateMethods(properties.iterator(), xhtml);
      generateMethods(operators.iterator(), xhtml);
      generateMethods(methods.iterator(), xhtml);
      generateMethods(type.getCastOperators().values().iterator(), xhtml);

      xhtml.endTagLn(html);
      writer.flush();
      outputStream.close();
    }
    catch (java.io.FileNotFoundException e)
    {
      context.terminate(e.getMessage());
    }
    catch (java.io.IOException e)
    {
      context.terminate(e.getMessage());
    }
    finally
    {
      try
      {
        outputStream.close();
      }
      catch (java.io.IOException e)
      {
        context.terminate(e.getMessage());
      }
    }
  }

  /**
   * \brief emit html code for a typeName
   *
   * if typeName can be resolved, a link is created, otherwise, only text is emited
   *
   * \param typeName the typeName to emit doc fog
   * \param symbol the symbol in which this typeName is used (in case it is a generic one)
   * \param generateCompleteTypeName will generate only className if false, complete typeName with package if true
   * \param xhtml the writer
   */
  private void generateTypeName(TypeName typeName, GenericSymbol symbol, boolean generateCompleteTypeName, XHTMLWriter xhtml)
  {
    if (typeName.isArray())
    {
      generateTypeName(typeName.getArrayElementType(), symbol, generateCompleteTypeName, xhtml);
      int dim = typeName.getDim();
      xhtml.print("[");
      for (int i = 1; i < dim; ++i)
        xhtml.print(",");
      xhtml.print("]");

      return;
    }

    if (symbolEnv.genericTypeExists(typeName))
    {
      try
      {
        Map genericAssociation = new HashMap();
        GenericType type = symbolEnv.getBestSuitableGenericType(typeName, genericAssociation);
        TypeName realTypeName = type.getName();

        XHTMLEntity a = xhtml.beginTag("a", "href=\"" + URLEncoder.encode(realTypeName.toString() + ".html", encoding) + "\"");
        if (generateCompleteTypeName)
          xhtml.print(typeName.toString());
        else
          xhtml.print(typeName.getLastComponent().getPackageElement());
        xhtml.endTagLn(a);
      }
      catch (java.io.UnsupportedEncodingException e)
      {
        context.terminate("Internal error : " + e.getMessage());
      }
      catch (SymbolException e)
      {
        context.terminate("Internal error : " + e.getMessage());
      }
    }
    else
    {
      // this type could not be found in SymbolEnvironment
      // can be a generic parameter : let's try to find it

      GenericSymbol mySymbol = symbol;
      TypeName enclosingTypeName = null;
      for (;;)
      {
        if (mySymbol.isGenericParameter(typeName))
        {
          // found
          try
          {
            String innerLink = URLEncoder.encode(typeName.toString(), encoding);

            if (enclosingTypeName != null)
            {
              String htmlFile = URLEncoder.encode(enclosingTypeName.toString() + ".html", encoding);

              XHTMLEntity a = xhtml.beginTag("a", "href=\"" + htmlFile + "#" + innerLink + "\"");
              xhtml.print(typeName.toString());
              xhtml.endTagLn(a);
            }
            else
            {
              XHTMLEntity a = xhtml.beginTag("a", "href=\"#" + innerLink + "\"");
              xhtml.print(typeName.toString());
              xhtml.endTagLn(a);
            }
          }
          catch (java.io.UnsupportedEncodingException e)
          {
            context.terminate("Internal error : " + e.getMessage());
          }

          return;
        }
        else // try to find it in enclosing symbol of this one
        if (mySymbol.getEnclosing() != null)
        {
          Map genericAssociation = new HashMap();

          try
          {
            enclosingTypeName = mySymbol.getEnclosing();
            mySymbol = symbolEnv.getBestSuitableGenericType(mySymbol.getEnclosing(), genericAssociation);
          }
          catch (SymbolException e)
          {
            context.terminate("Internal error : " + e.getMessage());
          }
        }
        else
        {
          // we have reached top level enclosing symbol without finding it
          // just print the name
          xhtml.print(typeName.toString());
          return;
        }
      }
    }
  }

  private void recGetSubTypeList(TypeName typeName, Set subTypes)
  {
    Iterator it = reverseEnv.iterateSubTypes(typeName);
    while (it.hasNext())
    {
      TypeName subType = (TypeName)it.next();
      subTypes.add(subType);
      recGetSubTypeList(subType, subTypes);
    }
  }

  /**
   * \brief emit html containing the list of sub classes of typeName
   *
   * \param type the type from which we want to generate the list of sub types
   * \param xhtml the writer
   */
  private void generateSubTypesList(GenericType type, XHTMLWriter xhtml)
  {
    Set result = new HashSet(); // Set<TypeName>
    recGetSubTypeList(type.getName(), result);
    generateTypeNameList(result.iterator(), type, "subclasses", xhtml);
  }

  /**
   * \brief generate a java like inheritance tree
   */
  private String recGenerateInheritanceTree(TypeName typeName, String indent, XHTMLWriter xhtml)
  {
    try
    {
      Map genericAssociation = new HashMap();
      GenericType type = symbolEnv.getBestSuitableGenericType(typeName, genericAssociation);

      TypeName parent = null;

      // get parent of type
      Iterator it = type.iterateExtends();
      if (it.hasNext())
      {
        parent = (TypeName)it.next();
      }
      else
      if (!type.isPrimitive() && !type.getName().equals(NativeTypes.get_net_nosica_lang_Object()))
      {
        parent = NativeTypes.get_net_nosica_lang_Object();
      }

      if (parent != null)
      {
        // generate parent tree recursively
        if (!parent.equals(NativeTypes.get_net_nosica_lang_Object()))
        {
          indent = recGenerateInheritanceTree(parent, indent, xhtml);
        }

        // generate this inheritance link
        generateTypeName(parent, type, true, xhtml);
        xhtml.println(indent + "  |");
        xhtml.print(indent + "  +--");

        return indent + "     ";
      }
    }
    catch (SymbolException e)
    {
      context.terminate("Internal error : " + e.getMessage());
    }

    return indent;
  }

  private void recGenerateInheritanceTree(TypeName typeName, XHTMLWriter xhtml)
  {
    XHTMLEntity pre = xhtml.beginTag("pre");
    recGenerateInheritanceTree(typeName, "", xhtml);
    xhtml.println(typeName.toString());
    xhtml.endTag(pre);
  }

  /**
   * \brief emit html for a list of typeName
   *
   * \param it Iterator<TypeName>
   * \param symbol the symbol where this list of typeName is declared (in case one of them is generic)
   * \param cssClassName name of the css class to output
   * \param xhtml the writer
   */
  private void generateTypeNameList(Iterator it, GenericSymbol symbol, String cssClassName, XHTMLWriter xhtml)
  {
    XHTMLEntity p = xhtml.beginTag("p", "class=\"" + cssClassName + "\"");

    while (it.hasNext())
    {
      TypeName typeName = (TypeName)it.next();
      generateTypeName(typeName, symbol, false, xhtml);
      if (it.hasNext())
      {
        xhtml.print(" ");
      }
    }

    xhtml.endTag(p);
  }

  /**
   * \brief emit the class specific information (extends, implements, primitive, native, abstract, ...)
   *
   * \param type the type to generate
   * \param xhtml the writer
   */
  private void generateClassSpecificInformation(GenericType type, XHTMLWriter xhtml)
  {
    if (!type.isClass())
    {
      return;
    }

    if (type.iterateImplements().hasNext())
    {
      XHTMLEntity t1 = xhtml.beginTag("t1");
      xhtml.print("All implemented interfaces");
      xhtml.endTag(t1);

      generateTypeNameList(type.iterateImplements(), type, "implements", xhtml);
    }
  }

  /**
   * \brief emit interface specific information (extends, ...)
   */
  private void generateInterfaceSpecificInformation(GenericType type, XHTMLWriter xhtml)
  {
    if (!type.isInterface())
    {
      return;
    }

    if (type.iterateExtends().hasNext())
    {
      XHTMLEntity t1 = xhtml.beginTag("t1");
      xhtml.print("All implemented interfaces");
      xhtml.endTag(t1);

      generateTypeNameList(type.iterateExtends(), type, "extends", xhtml);
    }
  }

  private void generateSubClassesInformation(GenericType type, XHTMLWriter xhtml)
  {
    if (reverseEnv.iterateSubTypes(type.getName()).hasNext())
    {
      XHTMLEntity t1 = xhtml.beginTag("t1");
      xhtml.print("All known sub classes");
      xhtml.endTag(t1);

      generateSubTypesList(type, xhtml);
    }
  }

  private void generateGenericInformation(GenericSymbol symbol, XHTMLWriter xhtml)
  {
    Iterator it = symbol.iterateGenericParameters();
    if (!it.hasNext())
    {
      return;
    }

    {
      XHTMLEntity t1 = xhtml.beginTag("t1");
      xhtml.print("Generic parameters");
      xhtml.endTag(t1);
    }

    XHTMLEntity ul = xhtml.beginTag("ul", "class=\"generics\"");

    while (it.hasNext())
    {
      GenericParameter parm = (GenericParameter)it.next();

      XHTMLEntity li = xhtml.beginTag("li", "class=\"generic\"");

      try
      {
        XHTMLEntity a = xhtml.beginTag("a", "name=\"" + URLEncoder.encode(parm.getName().toString(), encoding) +"\"");
        xhtml.print(parm.getName().toString());
        xhtml.endTag(a);
      }
      catch (java.io.UnsupportedEncodingException e)
      {
        context.terminate("Internal error : " + e.getMessage());
      }
      if (parm.hasImplements())
      {
        xhtml.print(" implements ");
        generateTypeName(parm.getImplements(), symbol, false, xhtml);
      }
      if (parm.hasDefaultType())
      {
        xhtml.print(" = ");
        generateTypeName(parm.getDefaultType(), symbol, false, xhtml);
      }

      xhtml.endTag(li);
    }

    xhtml.endTag(ul);
  }

  /**
   * \brief extract from a type, the methods by kind
   *
   * \param type the type from which to extract methods
   * \param constructors will contain constructors as output
   * \param properties will contain properties as output
   * \param operators will contain operators as output
   * \param methods will contain methods as output
   */
  private void extractMethodsByType(GenericType type, ArrayList constructors, ArrayList properties, ArrayList operators, ArrayList methods)
  {
    Iterator it = type.getOverloadables().values().iterator();
    while (it.hasNext())
    {
      ArrayList ms = (ArrayList)it.next();
      Iterator j = ms.iterator();
      while (j.hasNext())
      {
        GenericMethod method = (GenericMethod)j.next();

        if (method.getKind() == MethodKindEnum.constructor)
        {
          constructors.add(method);
        }
        else
        if (method.getKind() == MethodKindEnum.property ||
            method.getKind() == MethodKindEnum.propertyArray
        )
        {
          properties.add(method);
        }
        else
        if (method.getKind() == MethodKindEnum.method)
        {
          methods.add(method);
        }
        else
        {
          operators.add(method);
        }
      }
    }
  }

  /**
   * \brief generate a unique URL based on method signature
   *
   * \param method the method to emit an url
   * \return url
   */
  private String generateMethodURL(GenericMethod method)
  {
    StringBuffer url = new StringBuffer();
    url.append(getMethodName(method).replace(' ', '_'));
    Iterator it = method.iterateArguments();
    while (it.hasNext())
    {
      Argument arg = (Argument)it.next();
      if (arg.isConst())
        url.append("_const");
      url.append("_").append(arg.getTypeName().toString());
    }
    if (method.isConst())
      url.append("_const");

    return url.toString();
  }

  /**
   * \brief return name of a method
   *
   * \param method the method to look
   * \param human readable name
   */
  private String getMethodName(GenericMethod method)
  {
    if (method.getKind() == MethodKindEnum.castOperator)
      return "cast " + method.getName();
    else
    if (method.getKind() == MethodKindEnum.infix)
      return "infix " + method.getName();
    else
    if (method.getKind() == MethodKindEnum.postfix)
      return "postfix " + method.getName();
    else
    if (method.getKind() == MethodKindEnum.prefix)
      return "prefix " + method.getName();
    else
    if (method.getKind() == MethodKindEnum.property)
      return "property " + method.getName();
    else
    if (method.getKind() == MethodKindEnum.propertyArray)
      return "property []";
    else
    if (method.getKind() == MethodKindEnum.constructor)
      return "constructor";
    else
    if (method.getKind() == MethodKindEnum.destructor)
      return "destructor";
    else
    if (method.getKind() == MethodKindEnum.initializer)
      return "initializer";
    else
    if (method.getKind() == MethodKindEnum.deinitializer)
      return "deinitializer";
    else
      return method.getName();
  }

  /**
   * \brief generate signature of a method
   *
   * \param method the method to generate
   * \param isTableSummary are we generating a table summary
   * \param xhtml the writer
   */
  private void generateMethodSignature(GenericMethod method, boolean isTableSummary, XHTMLWriter xhtml)
  {
    XHTMLEntity div = null;
    if (!isTableSummary)
    {
      div = xhtml.beginTag("div", "class=\"method\"");
    }
    generateProtection(method, xhtml);
    xhtml.print(" ");
    if (method.isSub())
      xhtml.print("sub");
    else
    {
      if (method.getResultType().isConst())
        xhtml.print("const ");
      if (!isTableSummary)
      {
        generateTypeName(method.getResultType().getTypeName(), method, false, xhtml);
      }
      else
      {
        xhtml.print(method.getResultType().getTypeName().getLastComponent().getPackageElement());
      }
    }
    xhtml.print(" ");
    if (!isTableSummary)
    {
      xhtml.print(getMethodName(method));
    }
    else
    {
      try
      {
        XHTMLEntity a = xhtml.beginTag("a", "href=\"#" + URLEncoder.encode(generateMethodURL(method), encoding) + "\"");
        xhtml.print(getMethodName(method));
        xhtml.endTag(a);
      }
      catch (java.io.UnsupportedEncodingException e)
      {
        context.terminate("Internal error : " + e.getMessage());
      }
    }
    xhtml.print("(");
    {
      Iterator it = method.iterateArguments();
      while (it.hasNext())
      {
        Argument arg = (Argument)it.next();
        if (arg.isConst())
          xhtml.print("const ");
        if (!isTableSummary)
          generateTypeName(arg.getTypeName(), method, false, xhtml);
        else
          xhtml.print(arg.getTypeName().toString());
        xhtml.print(" ");
        xhtml.print(arg.getName());
        if (it.hasNext())
          xhtml.print(", ");
      }
    }
    xhtml.print(")");
    if (method.isConst())
    {
      xhtml.print(" const");
    }
    if (!isTableSummary)
    {
      xhtml.endTag(div);
    }
  }

  /**
   * \brief generate a table as the sumarry of a list method
   *
   * \param method the one method
   * \param tableDescription title of table
   * \param cssClass css class to use
   * \param xhtml the writer
   */
  private void generateMethodsSummary(GenericMethod method, String tableDescription, String cssClass, XHTMLWriter xhtml)
  {
    XHTMLEntity table = generateTableLayout(tableDescription, cssClass, xhtml);
    xhtml.beginTag("tr");
    xhtml.beginTag("td");
    generateMethodSignature(method, true, xhtml);
    xhtml.endTag(table);
  }

  /**
   * \brief generate a table as the sumarry of a list method
   *
   * \param it Iterator<GenericMethod>
   * \param tableDescription title of table
   * \param cssClass css class to use
   * \param xhtml the writer
   */
  private void generateMethodsSummary(Iterator it, String tableDescription, String cssClass, XHTMLWriter xhtml)
  {
    if (!it.hasNext())
      return;

    XHTMLEntity table = generateTableLayout(tableDescription, cssClass, xhtml);
    while (it.hasNext())
    {
      GenericMethod m = (GenericMethod)it.next();

      XHTMLEntity tr = xhtml.beginTag("tr");
      xhtml.beginTag("td");
      generateMethodSignature(m, true, xhtml);
      xhtml.endTag(tr);
    }
    xhtml.endTag(table);
    xhtml.printTag("hr");
  }

  /**
   * \brief emit protection information
   *
   * \param accessible accessible information
   * \paramp xhtml the writer
   */
  private void generateProtection(Accessible accessible, XHTMLWriter xhtml)
  {
    if (accessible.isStatic())
      xhtml.print("static ");
    if (accessible.isPublic())
      xhtml.print("public");
    if (accessible.isProtected())
      xhtml.print("protected");
    if (accessible.isPrivate())
      xhtml.print("private");
    if (accessible.isPackage())
      xhtml.print("package");
  }

  /**
   * \brief generate a field unique URL
   *
   * \param field the field to generate
   * \return unique url
   */
  private String generateFieldURL(Field field)
  {
    return field.getName();
  }

  /**
   * \brief generate signature from field
   *
   * \param field the field to generate signature
   * \param enclosing parent of field
   * \param isTableSummary are we creating table summary
   * \param xhtml the writer
   */
  private void generateFieldSignature(Field field, GenericType enclosing, boolean isTableSummary, XHTMLWriter xhtml)
  {
    XHTMLEntity div = null;
    if (!isTableSummary)
    {
      div = xhtml.beginTag("div", "class=\"method\"");
    }

    generateProtection(field, xhtml);

    if (field.isConst())
      xhtml.print(" const");

    xhtml.print(" ");

    if (!isTableSummary)
    {
      generateTypeName(field.getTypeName(), enclosing, false, xhtml);
      xhtml.print(" " + field.getName());
    }
    else
    {
      xhtml.print(field.getTypeName().getLastComponent().getPackageElement() + " ");
      try
      {
        XHTMLEntity a = xhtml.beginTag("a", "href=\"#" + URLEncoder.encode(generateFieldURL(field), encoding) + "\"");
        xhtml.print(field.getName());
        xhtml.endTag(a);
      }
      catch (java.io.UnsupportedEncodingException e)
      {
        context.terminate("Internal error : " + e.getMessage());
      }
    }

    if (!isTableSummary)
    {
      xhtml.endTag(div);
    }
  }

  /**
   * \brief generate a simple table with a css name, and a description in a th element
   *
   * \param description name of the table
   * \param cssClass name of css to use
   * \param xhtml the printer
   */
  private XHTMLEntity generateTableLayout(String description, String cssClass, XHTMLWriter xhtml)
  {
    XHTMLEntity table = xhtml.beginTag("table", "class=\"" + cssClass + "\"");
    {
      XHTMLEntity th = xhtml.beginTag("th");
      xhtml.print(description);
      xhtml.endTag(th);
    }

    return table;
  }

  /**
   * \brief generate field summary
   *
   * the summary consists of a table with the signature of all fields
   *
   * \param type from which to output summary
   * \param xhtml the writer
   */
  private void generateFieldsSummary(
    GenericType type,
    XHTMLWriter xhtml
  )
  {
    if (!type.iterateFields().hasNext())
      return;

    XHTMLEntity table = generateTableLayout("Field summary", "field_summary", xhtml);

    Iterator it = type.iterateFields();
    while (it.hasNext())
    {
      Field field = (Field)it.next();
      XHTMLEntity tr = xhtml.beginTag("tr");
      xhtml.beginTag("td");
      generateFieldSignature(field, type, true, xhtml);
      xhtml.endTag(tr);
    }
    xhtml.endTag(table);
  }

  /**
   * \brief generate methods summary
   *
   * The summary consists of a list of table containing each a list of methods.
   * Tables are sorted by method's kind
   *
   * \param type the type to generate summary
   * \param constructors ArrayList<GenericMethod> the list of constructors
   * \param properties ArrayList<GenericMethod> the list of properties
   * \param operators ArrayList<GenericMethod> the list of operators
   * \param methods ArrayList<GenericMethod> the list of methods
   * \param xhtml the writer
   */
  private void generateMethodsSummary(
    GenericType type,
    ArrayList constructors,
    ArrayList properties,
    ArrayList operators,
    ArrayList methods,
    XHTMLWriter xhtml
  )
  {
    if (type.hasInitializer())
      generateMethodsSummary(type.getInitializer(), "Initializer summary", "initializer_summary", xhtml);
    if (type.hasDeinitializer())
      generateMethodsSummary(type.getDeinitializer(), "Deinitializer summary", "deinitializer_summary", xhtml);
    generateMethodsSummary(constructors.iterator(), "Constructors summary", "constructor_summary", xhtml);
    if (type.hasDestructor())
      generateMethodsSummary(type.getDestructor(), "Destructor summary", "destructor_summary", xhtml);
    generateMethodsSummary(properties.iterator(), "Properties summary", "property_summary", xhtml);
    generateMethodsSummary(operators.iterator(), "Operators summary", "operator_summary", xhtml);
    generateMethodsSummary(type.getCastOperators().values().iterator(), "Cast operators summary", "cast_operator_summary", xhtml);
    generateMethodsSummary(methods.iterator(), "Methods summary", "method_summary", xhtml);
  }

  /**
   * \brief generate documentation for initializer if any
   *
   * \param type the type to generate
   * \param xhtml the writer
   */
  private void generateInitializer(GenericType type, XHTMLWriter xhtml)
  {
    if (type.hasInitializer())
    {
      generateMethod(type.getInitializer(), xhtml);
    }
  }

  /**
   * \brief generate documentation for deinitializer if any
   *
   * \param type the type to generate
   * \param xhtml the writer
   */
  private void generateDeinitializer(GenericType type, XHTMLWriter xhtml)
  {
    if (type.hasDeinitializer())
    {
      generateMethod(type.getDeinitializer(), xhtml);
    }
  }

  /**
   * \brief generate documentation for fields of type
   *
   * \param type the type to generate
   * \param xhtml the writer
   */
  private void generateFields(GenericType type, XHTMLWriter xhtml)
  {
    Iterator it = type.iterateFields();
    while (it.hasNext())
    {
      Field field = (Field)it.next();
      generateField(field, type, xhtml);
      xhtml.printTag("hr");
    }
  }

  /**
   * \brief generate documentation for a list of methods
   *
   * \param it Iterator<GenericMethod>
   * \param xhtml the writer
   */
  private void generateMethods(Iterator it, XHTMLWriter xhtml)
  {
    while (it.hasNext())
    {
      GenericMethod method = (GenericMethod)it.next();
      generateMethod(method, xhtml);
      xhtml.printTag("hr");
    }
  }

  /**
   * \brief generate documentation for destructor if any
   *
   * \param type the type to generate
   * \param xhtml the writer
   */
  private void generateDestructor(GenericType type, XHTMLWriter xhtml)
  {
    if (type.hasDestructor())
    {
      generateMethod(type.getDestructor(), xhtml);
    }
  }

  /**
   * \brief generate documentation for one method
   *
   * \param method the method to generate
   * \param xhtml the writer
   */
  private void generateMethod(GenericMethod method, XHTMLWriter xhtml)
  {
    {
      XHTMLEntity div = xhtml.beginTag("div", "class=\"method_brief\"");
      {
        XHTMLEntity t1 = xhtml.beginTag("t1");
        try
        {
          xhtml.beginTag("a", "name=\"" + URLEncoder.encode(generateMethodURL(method), encoding) + "\"");
          xhtml.print(getMethodName(method));
        }
        catch (java.io.UnsupportedEncodingException e)
        {
          context.terminate("Internal error : " + e.getMessage());
        }
        xhtml.endTag(t1);
      }

      generateMethodSignature(method, false, xhtml);
      xhtml.endTag(div);
    }

    generateGenericInformation(method, xhtml);

    XHTMLEntity div = xhtml.beginTag("div", "class=\"method_content\"");
    generateDoc(method.getNode(), xhtml);
    xhtml.endTag(div);
  }

  /**
   * \brief generate documentation for field
   *
   * \param field the field to generate
   * \param enclosing enclosing type of field
   * \param xhtml the writer
   */
  private void generateField(Field field, GenericType enclosing, XHTMLWriter xhtml)
  {
    {
      XHTMLEntity div = xhtml.beginTag("div", "class=\"field_brief\"");
      {
        XHTMLEntity t1 = xhtml.beginTag("t1");
        try
        {
          xhtml.beginTag("a", "name=\"" + URLEncoder.encode(generateFieldURL(field), encoding) + "\"");
          xhtml.print(field.getName());
        }
        catch (java.io.UnsupportedEncodingException e)
        {
          context.terminate("Internal error : " + e.getMessage());
        }
        xhtml.endTag(t1);
      }

      generateFieldSignature(field, enclosing, false, xhtml);
      xhtml.endTag(div);
    }

    XHTMLEntity div = xhtml.beginTag("div", "class=\"field_content\"");
    generateDoc(field.getNode(), xhtml);
    xhtml.endTag(div);
  }
}