/*
  $Id: JavaLikeHTMLGenerator.java,v 1.6 2004/03/04 22:16:17 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.doc.production.html;

import net.nosica.application.ApplicationContext;
import net.nosica.application.ApplicationContextImpl;
import net.nosica.util.*;
import net.nosica.util.IndentedPrintWriter;

import net.nosica.util.FileHelper;

import java.io.File;
import java.io.FileOutputStream;

/**
 * \brief generate a frame with a summary, all classes and all packages
 */
public class JavaLikeHTMLGenerator implements HTMLGenerator
{
  private ApplicationContext context = ApplicationContextImpl.instance();
  private String outputDirectory;

  public JavaLikeHTMLGenerator(String outputDirectory)
  {
    this.outputDirectory = outputDirectory;
  }

  public void generate()
  {
    File file = FileHelper.getFile(outputDirectory + "/summary.html");

    try
    {
      FileOutputStream outputStream = new FileOutputStream(file);
      IndentedPrintWriter writer = new IndentedPrintWriter(outputStream);
      XHTMLWriter xhtml = new XHTMLWriter(writer);
      XHTMLEntity html = XHTMLHelper.printHeader(xhtml, "classes");

      xhtml.beginTag("body");
      xhtml.beginTag("t1");
      xhtml.print("A summary");

      xhtml.endTagLn(html);
      writer.flush();
      outputStream.close();
    }
    catch (java.io.FileNotFoundException e)
    {
      context.terminate(e.getMessage());
    }
    catch (java.io.IOException e)
    {
      context.terminate(e.getMessage());
    }

    {
      IndexGenerator gen = new JavaLikeIndexGenerator(outputDirectory);
      gen.generate();
    }
    {
      PackageGenerator gen = new JavaLikePackageGenerator(outputDirectory);
      gen.generate();
    }
    {
      ClassGenerator gen = new JavaLikeClassGenerator(outputDirectory);
      gen.generate();
    }
  }
}