/*
  $Id: JavaLikePackageGenerator.java,v 1.11 2004/03/27 22:37:25 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.doc.production.html;

import net.nosica.application.ApplicationContext;
import net.nosica.application.ApplicationContextImpl;
import net.nosica.application.SymbolEnvironment;
import net.nosica.application.SymbolEnvironmentImpl;
import net.nosica.symbol.GenericType;
import net.nosica.symbol.SymbolException;
import net.nosica.util.*;
import net.nosica.util.IndentedPrintWriter;
import net.nosica.util.FileHelper;
import net.nosica.parser.node.util.var.TypeName;

import java.io.File;
import java.io.FileOutputStream;

import java.util.ArrayList;
import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;

import java.net.URLEncoder;

/**
 * \brief generate package information : all packages and one file per package
 */
public class JavaLikePackageGenerator implements PackageGenerator
{
  private ApplicationContext context = ApplicationContextImpl.instance();
  private SymbolEnvironment symbolEnv = SymbolEnvironmentImpl.instance();
  private String outputDirectory;
  private static final String encoding = "UTF-8";

  public JavaLikePackageGenerator(String outputDirectory)
  {
    this.outputDirectory = outputDirectory;
  }

  public void generate()
  {
    Map packages = new HashMap(); // HashMap<TypeName, ArrayList<GenericType> >

    Iterator it = symbolEnv.iterateTypeName();
    while (it.hasNext())
    {
      TypeName typeName = (TypeName)it.next();
      try
      {
        GenericType type = symbolEnv.getGenericType(typeName);

        if (!type.isNested() && !type.isInner())
        {
          TypeName packageTypeName = type.getName().getFirstComponents();
          if (packages.containsKey(packageTypeName))
          {
            ArrayList types = (ArrayList)packages.get(packageTypeName);
            types.add(type);
          }
          else
          {
            ArrayList types = new ArrayList();
            types.add(type);
            packages.put(packageTypeName, types);
          }
        }
      }
      catch (SymbolException e)
      {
        context.terminate("Internal error : " +e.getMessage());
      }
    }

    generateAllPackages(packages);
  }

  /**
   * \brief generate the all-packages file listing all available packages
   *
   * \param packages Map<TypeName, ArrayList<GenericType> >
   */
  private void generateAllPackages(Map packages)
  {
    File file = FileHelper.getFile(outputDirectory + "/all-packages.html");

    FileOutputStream outputStream = null;

    try
    {
      outputStream = new FileOutputStream(file);
      IndentedPrintWriter writer = new IndentedPrintWriter(outputStream);
      XHTMLWriter xhtml = new XHTMLWriter(writer);
      XHTMLEntity html = XHTMLHelper.printHeader(xhtml, "packages", "nosicadoc.css");

      xhtml.beginTag("body");

      try
      {
        XHTMLEntity a = xhtml.beginTag("a", "href=\"" + URLEncoder.encode("all-classes.html", encoding) + "\" target=\"classesFrame\"");
        xhtml.print("All classes");
        xhtml.endTag(a);
      }
      catch (java.io.UnsupportedEncodingException e)
      {
        context.terminate("Internal error : " + e.getMessage());
      }

      xhtml.printTag("br");
      xhtml.printTag("br");

      xhtml.print("packages");

      xhtml.beginTag("table", "border=\"0\" cellspacing=\"0\" cellpadding=\"0\"");

      Iterator it = packages.entrySet().iterator();
      while (it.hasNext())
      {
        Map.Entry entry = (Map.Entry)it.next();
        TypeName packageName = (TypeName)entry.getKey();
        ArrayList types = (ArrayList)entry.getValue();

        XHTMLEntity tr = xhtml.beginTag("tr");
        xhtml.beginTag("td");
        try
        {
          xhtml.beginTag("a", "href=\"" + URLEncoder.encode(packageName.toString() + ".html", encoding) + "\" target=\"classesFrame\"");
          xhtml.print(packageName.toString());
        }
        catch (java.io.UnsupportedEncodingException e)
        {
          context.terminate("Internal error : " + e.getMessage());
        }

        xhtml.endTag(tr);

        generatePackage(packageName, types);
      }

      xhtml.endTagLn(html);
      writer.flush();
      outputStream.close();
    }
    catch (java.io.FileNotFoundException e)
    {
      context.terminate(e.getMessage());
    }
    catch (java.io.IOException e)
    {
      context.terminate(e.getMessage());
    }
    finally
    {
      try
      {
        outputStream.close();
      }
      catch (java.io.IOException e)
      {
        context.terminate(e.getMessage());
      }
    }
  }

  /**
   * \brief generate file for a package
   *
   * \param packageName name of the package to generate
   * \param types ArrayList<GenericType>
   */
  void generatePackage(TypeName packageName, ArrayList types)
  {
    File file = FileHelper.getFile(outputDirectory + "/" + packageName + ".html");

    try
    {
      FileOutputStream outputStream = new FileOutputStream(file);
      IndentedPrintWriter writer = new IndentedPrintWriter(outputStream);
      XHTMLWriter xhtml = new XHTMLWriter(writer);
      XHTMLEntity html = XHTMLHelper.printHeader(xhtml, "package" + packageName, "nosicadoc.css");

      xhtml.beginTag("body");

      XHTMLEntity t1 = xhtml.beginTag("t1");
      xhtml.print("Package " + packageName);
      xhtml.endTag(t1);

      xhtml.printTag("br");
      xhtml.printTag("br");

      xhtml.beginTag("table", "border=\"0\" cellspacing=\"0\" cellpadding=\"0\"");

      Iterator it = types.iterator();
      while (it.hasNext())
      {
        GenericType type = (GenericType)it.next();

        XHTMLEntity tr = xhtml.beginTag("tr");
        xhtml.beginTag("td");
        try
        {
          xhtml.beginTag("a", "href=\"" + URLEncoder.encode(type.getName().toString() + ".html", encoding) + "\" target=\"mainFrame\"");
          xhtml.print(type.getName().toString());
        }
        catch (java.io.UnsupportedEncodingException e)
        {
          context.terminate("Internal error : " + e.getMessage());
        }

        xhtml.endTag(tr);
      }

      xhtml.endTagLn(html);
      writer.flush();
      outputStream.close();
    }
    catch (java.io.FileNotFoundException e)
    {
      context.terminate(e.getMessage());
    }
    catch (java.io.IOException e)
    {
      context.terminate(e.getMessage());
    }
  }
}