/*
  $Id: ConfigurationFileGenerator.java,v 1.1 2004/04/01 21:27:55 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.ni;

import net.nosica.compiler.application.Environment;
import net.nosica.compiler.production.c.CNameConverter;
import net.nosica.compiler.production.c.CNameConverterImpl;

import net.nosica.parser.node.util.var.TypeName;

import net.nosica.symbol.GenericMethod;
import net.nosica.symbol.GenericType;
import net.nosica.symbol.SymbolException;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;

import java.util.Iterator;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import java.util.Vector;

public class ConfigurationFileGenerator
{
  private static Environment env = Environment.instance();

  public static Map generate(String outputDir) throws SymbolException, FileNotFoundException, IOException
  {
    Map typeConfiguration = new HashMap(); // Map<GenericType, Properties>

    Iterator it = env.getSE().iterateTypeName();
    while (it.hasNext())
    {
      TypeName typeName = (TypeName)it.next();
      GenericType type = env.getSE().getGenericType(typeName);

      if (type.isNative())
      {
        Properties p = generateConfiguration(outputDir, type);
        typeConfiguration.put(type, p);
      }
    }

    return typeConfiguration;
  }

  private static Properties generateConfiguration(String outputDir, GenericType type) throws FileNotFoundException, IOException
  {
    File path = new File(outputDir, "include");
    CNameConverter nameConverter = new CNameConverterImpl();

    String configurationFileName = nameConverter.cRadixFileName(type.getName()) + ".properties";

    // generate missing parent directory
    File file = new File(path, configurationFileName);
    file.getParentFile().mkdirs();

    Properties p = new Properties();

    if (file.exists())
    {
      p.load(new FileInputStream(file));
    }

    updateProperties(file, p, nameConverter, type);

    StringBuffer header = new StringBuffer();
    header.append("This file was generated by nosicani. ");
    header.append("Syntax is 'methodName = [source|inline]'");

    p.store(new FileOutputStream(file), header.toString());

    return p;
  }

  private static void updateProperties(File f, Properties p, CNameConverter nameConverter, GenericType type)
  {
    Iterator it = type.iterateMethods();
    while (it.hasNext())
    {
      GenericMethod m = (GenericMethod)it.next();

      String methodName = nameConverter.cMethodName(MethodHelper.instantiateMethod(m));
      String impl = p.getProperty(methodName);

      if (impl == null)
      {
        p.setProperty(methodName, "source");
      }
      else
      {
        if (!impl.equals("source") && !impl.equals("inline"))
        {
          StringBuffer msg = new StringBuffer();
          msg.append(f.getAbsolutePath())
             .append(" : \n");
          msg.append("Incorrect value for method ")
             .append(methodName)
             .append(". Found ")
             .append(impl)
             .append(", was waited for [source|impl]");

          env.terminate(msg.toString());
        }
      }
    }
  }
}