/*
  $Id: Main.java,v 1.4 2004/04/01 21:27:55 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.ni;

import net.nosica.application.ApplicationContext;
import net.nosica.application.ApplicationContextImpl;
import net.nosica.application.PropertyException;
import net.nosica.application.NosicaPropertySetup;
import net.nosica.application.NativeTypes;
import net.nosica.application.SymbolEnvironment;
import net.nosica.application.SymbolEnvironmentImpl;

import net.nosica.compiler.application.Environment;

import net.nosica.compiler.production.c.CNameConverter;
import net.nosica.compiler.production.c.CNameConverterImpl;

import net.nosica.framework.VisitorException;

import net.nosica.parser.node.util.var.TypeName;

import net.nosica.symbol.GenericType;
import net.nosica.symbol.GenericTypeImpl;
import net.nosica.symbol.GenericMethod;
import net.nosica.symbol.SymbolException;

import net.nosica.compiler.typechecker.Argument;
import net.nosica.compiler.typechecker.ArgumentImpl;
import net.nosica.compiler.typechecker.Method;
import net.nosica.compiler.typechecker.MethodImpl;
import net.nosica.compiler.typechecker.ResultType;
import net.nosica.compiler.typechecker.ResultTypeImpl;
import net.nosica.compiler.typechecker.Type;
import net.nosica.compiler.typechecker.TypeImpl;

import net.nosica.util.CommandLineAnalysis;
import net.nosica.util.CommandLineFlag;
import net.nosica.util.CommandLineOption;
import net.nosica.util.LightResolver;
import net.nosica.util.LightParser;
import net.nosica.util.IndentedPrintWriter;

import net.nosica.compiler.Config;

import java.io.File;
import java.io.FileOutputStream;
import java.io.FileNotFoundException;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.HashSet;

public class Main
{
  private ApplicationContext context = ApplicationContextImpl.instance();
  private Environment env = Environment.instance();

  private String systemPropertyFile;
  private String userPropertyFile;
  private String output;
  private String arch;
  private String[] files;

  static public void main(String[] args)
  {
    try
    {
      Main main = new Main(args);
      main.run();
    }
    catch (Throwable e)
    {
      e.printStackTrace();
      System.exit(2);
    }
  }

  public Main(String[] args)
  {
    try
    {
      NosicaPropertySetup.setup(context);
    }
    catch (PropertyException e)
    {
      context.terminate("Internal error : " + e.getMessage());
    }

    commandLineAnalysis(args);

    try
    {
      File systemProperties = new File(systemPropertyFile);
      if (systemProperties.exists())
      {
        System.out.println("A system property file has been found, loading from [" + systemProperties.getAbsolutePath() + "]");
        context.loadPropertiesFromFile(systemProperties);
      }
      else
      {
        context.terminate("Could not load system property file from " + systemPropertyFile);
      }

      File userProperties = new File(userPropertyFile);
      if (userProperties.exists())
      {
        System.out.println("A user property file has been found, loading from [" + userProperties.getAbsolutePath() + "]");
        context.loadPropertiesFromFile(userProperties);
      }
    }
    catch (PropertyException e)
    {
      context.terminate(e.getMessage());
    }
    catch (java.io.IOException e)
    {
      context.terminate(e.getMessage());
    }
  }

  private void commandLineAnalysis(String[] args)
  {
    CommandLineAnalysis cmd = new CommandLineAnalysis(
      "nosicani",
      "[options] files",
      "",
      Config.VERSION
    );

    CommandLineOption cl_s = new CommandLineOption(cmd, 's', "system",  "System property file", Config.SYSTEM_PROPERTY_FILE);
    CommandLineOption cl_p = new CommandLineOption(cmd, 'p', "user",    "User property file", "");
    CommandLineOption cl_o = new CommandLineOption(cmd, 'o', "output",  "directory output", ".");
    CommandLineOption cl_a = new CommandLineOption(cmd, 'a', "arch",    "directory arch", "");
    CommandLineFlag   cl_h = new CommandLineFlag(cmd,   'h', "help",    "Display this help message", false);
    CommandLineFlag   cl_v = new CommandLineFlag(cmd,   'v', "version", "Print version information and exit", false);

    files = cmd.process(args);

    if (cl_h.getValue())
    {
      context.terminate(cmd.help());
    }
    if (cl_v.getValue())
    {
      context.terminate(cmd.version());
    }

    systemPropertyFile = cl_s.getValue();
    userPropertyFile   = cl_p.getValue();
    output             = cl_o.getValue();
    arch               = cl_a.getValue();

    if (arch.length() == 0)
    {
      context.terminate("No architecture specified");
    }

    if (files.length == 0)
    {
      StringBuffer message = new StringBuffer("No files were given on command line\n");
      message.append(cmd.help());
      context.terminate(message.toString());
    }
  }

  private void run()
  {
    for (int i = 0; i < files.length; ++i)
    {
      File file = new File(files[i]);

      if (!file.exists())
      {
        context.terminate("Path " + files[i] + " does not exist");
      }

      // parse files given on command line
      LightParser parser = new LightParser();
      try
      {
        if (file.isDirectory())
        {
          parser.handleDirectory(file);
        }
        else
        {
          parser.handleFile(file);
        }
      }
      catch (java.io.IOException e)
      {
        context.terminate("I/O error : " + e.getMessage());
      }
    }

    // resolve typename
    LightResolver resolver = new LightResolver();
    resolver.resolveTypes();

    // now generate native interface
    try
    {
      TypeEnvironmentPopulator.populate();

      // Map<GenericType, Properties>
      Map typeConfiguration = ConfigurationFileGenerator.generate(output);

      NativeInterfaceGenerator .generate(typeConfiguration, output);
      NativeInlineImplGenerator.generate(typeConfiguration, output, arch);
      NativeSourceImplGenerator.generate(typeConfiguration, output, arch);
    }
    catch (Throwable e)
    {
      context.terminate("Internal error : " + e.getMessage());
    }
  }
}