/*
  $Id: NativeInterfaceGenerator.java,v 1.4 2004/04/18 00:52:03 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.ni;

import net.nosica.compiler.application.Environment;

import net.nosica.compiler.production.c.CNameConverter;
import net.nosica.compiler.production.c.CNameConverterImpl;

import net.nosica.symbol.GenericMethod;
import net.nosica.symbol.GenericMethodImpl;
import net.nosica.symbol.GenericType;
import net.nosica.symbol.MethodKindEnum;
import net.nosica.symbol.SymbolException;

import net.nosica.parser.node.util.var.TypeName;

import net.nosica.compiler.typechecker.ArgumentImpl;
import net.nosica.compiler.typechecker.Method;
import net.nosica.compiler.typechecker.MethodImpl;
import net.nosica.compiler.typechecker.ResultType;
import net.nosica.compiler.typechecker.ResultTypeImpl;

import net.nosica.framework.VisitorException;

import net.nosica.util.IndentedPrintWriter;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;

public class NativeInterfaceGenerator
{
  private static Environment env = Environment.instance();
  private static final String[] protectedZonesName = {"header", "type", "footer"};

  public static void generate(Map typeConfiguration, String outputDir) throws SymbolException, FileNotFoundException, IOException, VisitorException
  {
    Iterator it = env.getSE().iterateTypeName();
    while (it.hasNext())
    {
      TypeName typeName = (TypeName)it.next();
      GenericType type = env.getSE().getGenericType(typeName);

      if (type.isNative())
      {
        generateNativeInterface(outputDir, type, (Properties)typeConfiguration.get(type));
      }
    }
  }

  private static void generateNativeInterface(String outputDir, GenericType type, Properties p) throws FileNotFoundException, IOException, VisitorException
  {
    System.out.println("Generating type " + type.getName().toString());

    File path = new File(outputDir, "include");
    CNameConverter nameConverter = new CNameConverterImpl();

    // generate missing parent directory
    File file = new File(path, nameConverter.cIncludeTypeName(type.getName()));
    file.getParentFile().mkdirs();

    // retrieve protected zone
    Map protectedZones = new HashMap(); // Map<String, String>
    if (file.exists())
    {
      ZoneProtection.extractProtectedZones(file, protectedZones);
      ZoneProtection.backupFile(file);
    }

    for (int i = 0; i < protectedZonesName.length; ++i)
    {
      if (!protectedZones.containsKey(protectedZonesName[i]))
      {
        protectedZones.put(protectedZonesName[i], "");
      }
    }

    IndentedPrintWriter writer = new IndentedPrintWriter(new FileOutputStream(file));

    String macroDefineName = nameConverter.cMacroDefineName(type.getName());
    writeHeader(macroDefineName, writer);

    ZoneProtection.writeProtectedArea(writer, protectedZones, "header");
    writer.println();
    ZoneProtection.writeProtectedArea(writer, protectedZones, "type");
    writer.println();

    Iterator it = type.iterateMethods();
    while (it.hasNext())
    {
      GenericMethod m = (GenericMethod)it.next();
      generateNativeMethod(writer, nameConverter, m, p);
    }

    writer.println();
    writer.println("#include \"" + nameConverter.cRadixFileName(type.getName()) + ".inl\"");

    writer.println();
    ZoneProtection.writeProtectedArea(writer, protectedZones, "footer");
    writeFooter(macroDefineName, writer);
    writer.close();
  }

  private static void generateNativeMethod(
    IndentedPrintWriter writer,
    CNameConverter nameConverter,
    GenericMethod method,
    Properties p
  ) throws VisitorException
  {
    if (!MethodHelper.isNative(method))
      return;

    Method m = MethodHelper.instantiateMethod(method);
    String methodName = nameConverter.cMethodName(m);
    String impl = (String)p.get(methodName);

    writer.println();
    if (impl.equals("inline"))
    {
      writer.println("static inline");
    }

    writer.print(nameConverter.cMethodSignature(m));
    writer.println(";");
  }

  private static void writeHeader(String macroDefineName, IndentedPrintWriter writer)
  {
    writer.println("// This file was generated by nosicani");
    writer.println("// only edit in protected zones");
    writer.println("// all other zone will be lost by next generation");
    writer.println();
    writer.println("#ifndef " + macroDefineName);
    writer.println("#define " + macroDefineName);
    writer.println();
  }

  private static void writeFooter(String macroDefineName, IndentedPrintWriter writer)
  {
    writer.println();
    writer.println("#endif // " + macroDefineName);
  }
}