/*
  $Id: NativeSourceImplGenerator.java,v 1.3 2004/04/18 00:52:03 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.ni;

import net.nosica.compiler.application.Environment;
import net.nosica.compiler.production.c.CNameConverter;
import net.nosica.compiler.production.c.CNameConverterImpl;
import net.nosica.compiler.typechecker.Method;

import net.nosica.symbol.GenericMethod;
import net.nosica.symbol.GenericType;
import net.nosica.symbol.SymbolException;

import net.nosica.parser.node.util.var.TypeName;

import net.nosica.framework.VisitorException;

import net.nosica.util.IndentedPrintWriter;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.FileOutputStream;

import java.util.Iterator;
import java.util.Map;
import java.util.HashMap;
import java.util.Properties;

public class NativeSourceImplGenerator
{
  private static Environment env = Environment.instance();
  private static final String[] protectedZonesName = {"preheader", "postheader", "footer"};

  public static void generate(Map typeConfiguration, String outputDir, String arch) throws SymbolException, FileNotFoundException, IOException, VisitorException
  {
    Iterator it = env.getSE().iterateTypeName();
    while (it.hasNext())
    {
      TypeName typeName = (TypeName)it.next();
      GenericType type = env.getSE().getGenericType(typeName);

      if (type.isNative())
      {
        generateNativeSource(outputDir, arch, type, (Properties)typeConfiguration.get(type));
      }
    }
  }

  private static void generateNativeSource(String outputDir, String arch, GenericType type, Properties p) throws FileNotFoundException, IOException, VisitorException
  {
    System.out.println("Generating type " + type.getName().toString());

    File path = new File(outputDir, arch);
    CNameConverter nameConverter = new CNameConverterImpl();

    // generate missing parent directory
    File file = new File(path, nameConverter.cRadixFileName(type.getName()) + ".c");
    file.getParentFile().mkdirs();

    // retrieve protected zone
    Map protectedZones = new HashMap(); // Map<String, String>
    if (file.exists())
    {
      ZoneProtection.extractProtectedZones(file, protectedZones);
      ZoneProtection.backupFile(file);
    }

    for (int i = 0; i < protectedZonesName.length; ++i)
    {
      if (!protectedZones.containsKey(protectedZonesName[i]))
      {
        protectedZones.put(protectedZonesName[i], "");
      }
    }

    IndentedPrintWriter writer = new IndentedPrintWriter(new FileOutputStream(file));

    writeHeader(writer);

    ZoneProtection.writeProtectedArea(writer, protectedZones, "preheader");
    writer.println();

    writer.println("#include \"" + nameConverter.cIncludeTypeName(type.getName()) + "\"");
    writer.println();

    ZoneProtection.writeProtectedArea(writer, protectedZones, "postheader");
    writer.println();

    Iterator it = type.iterateMethods();
    while (it.hasNext())
    {
      GenericMethod method = (GenericMethod)it.next();
      if (!MethodHelper.isNative(method))
        continue;

      Method m = MethodHelper.instantiateMethod(method);
      String methodName = nameConverter.cMethodName(m);

      String impl = p.getProperty(methodName);
      if (impl.equals("inline"))
        continue;

      if (!protectedZones.containsKey(methodName))
      {
        protectedZones.put(methodName, "");
      }

      generateNativeMethod(writer, nameConverter, m, protectedZones);
    }

    writer.println();
    ZoneProtection.writeProtectedArea(writer, protectedZones, "footer");
    writer.println();

    if (protectedZones.size() != 0)
    {
      writer.println("// find below unrecognized areas");
      ZoneProtection.writeAllProtectedArea(writer, protectedZones);
    }

    writer.close();
  }

  private static void generateNativeMethod(
    IndentedPrintWriter writer,
    CNameConverter nameConverter,
    Method method,
    Map protectedZones
  ) throws VisitorException
  {
    String methodName = nameConverter.cMethodName(method);

    writer.println();
    writer.println(nameConverter.cMethodSignature(method));
    writer.println("{");
    ZoneProtection.writeProtectedArea(writer, protectedZones, methodName);
    writer.println("}");
  }

  private static void writeHeader(IndentedPrintWriter writer)
  {
    writer.println("// This file was generated by nosicani");
    writer.println("// only edit in protected zones");
    writer.println("// all other zone will be lost by next generation");
    writer.println();
  }
}