/*
  $Id: TypeEnvironmentPopulator.java,v 1.4 2004/04/18 00:52:03 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.ni;

import net.nosica.parser.node.util.var.TypeName;

import net.nosica.symbol.GenericType;
import net.nosica.symbol.GenericTypeImpl;
import net.nosica.symbol.GenericMethod;
import net.nosica.symbol.GenericMethodImpl;
import net.nosica.symbol.MethodKindEnum;
import net.nosica.symbol.SymbolException;

import net.nosica.compiler.typechecker.Type;
import net.nosica.compiler.typechecker.TypeImpl;

import net.nosica.application.NativeTypes;
import net.nosica.application.SymbolEnvironment;

import net.nosica.compiler.application.Environment;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

public class TypeEnvironmentPopulator
{
  private static Environment env = Environment.instance();

  public static void populate() throws SymbolException
  {
    // create fake types so that basic calls to Type Environment like
    // env.getTE().getType(typeName).isNative() works
    Iterator it = env.getSE().iterateTypeName();
    while (it.hasNext())
    {
      TypeName typeName = (TypeName)it.next();
      GenericType genType = env.getSE().getGenericType(typeName);

      if (genType.isNative())
      {
        populateType(genType);
      }
    }
  }

  private static void populateType(GenericType type) throws SymbolException
  {
    // create missing methods if needed
    createInitializerIfNeeded(type);
    createDeinitializerIfNeeded(type);
    createDestructorIfNeeded(type);
    createConstructorIfNeeded(type);

    // populate
    populateOneType(type);

    Iterator it = type.iterateMethods();
    while (it.hasNext())
    {
      GenericMethod m = (GenericMethod)it.next();
      populateType(m);
    }
  }

  private static void populateType(GenericMethod m) throws SymbolException
  {
    if (!m.isSub())
    {
      populateOneType(m.getResultType().getTypeName());
    }

    Iterator it = m.iterateArguments();
    while (it.hasNext())
    {
      net.nosica.symbol.Argument arg = (net.nosica.symbol.Argument)it.next();
      populateOneType(arg.getTypeName());
    }
  }

  private static void populateOneType(TypeName typeName) throws SymbolException
  {
    if (env.getTE().typeExists(typeName))
      return;

    if (typeName.isArray())
    {
      populateOneType(typeName.getArrayElementType());

      GenericType arrayElementType = env.getSE().getGenericType(typeName.getArrayElementType());

      Set implementsSet = new HashSet();
      implementsSet.add(NativeTypes.get_net_nosica_lang_Array());

      GenericType createdGenType = new GenericTypeImpl(
        null,
        true,
        false,
        false,
        new ArrayList(),
        null,
        false,
        true,
        false,
        false,
        typeName,
        java.util.Collections.EMPTY_SET,
        implementsSet,
        arrayElementType.getAliasTable(),
        java.util.Collections.EMPTY_MAP
      );

      env.getSE().addGenericType(createdGenType);

      Type type = new TypeImpl(
        createdGenType,
        createdGenType.getName(),
        createdGenType.getExtends(),
        createdGenType.getImplements(),
        new ArrayList(),
        java.util.Collections.EMPTY_MAP,
        java.util.Collections.EMPTY_MAP,
        null,
        null,
        null,
        createdGenType.getEnclosing(),
        createdGenType.getAliasTable(),
        java.util.Collections.EMPTY_MAP
      );

      env.getTE().addType(type);
    }
    else
    {
      populateOneType(env.getSE().getGenericType(typeName));
    }
  }

  private static void populateOneType(GenericType genType) throws SymbolException
  {
    if (!env.getTE().typeExists(genType.getName()))
    {
      Type type = new TypeImpl(
        genType,
        genType.getName(),
        genType.getExtends(),
        genType.getImplements(),
        new ArrayList(),
        java.util.Collections.EMPTY_MAP,
        java.util.Collections.EMPTY_MAP,
        null,
        null,
        null,
        genType.getEnclosing(),
        genType.getAliasTable(),
        java.util.Collections.EMPTY_MAP
      );

      env.getTE().addType(type);
    }
  }

  private static void createInitializerIfNeeded(GenericType type)
  {
    if (type.hasInitializer())
      return;

    GenericMethod initializer = new GenericMethodImpl(
      null,
      true,
      false,
      false,
      new ArrayList(),
      type.getName(),
      true,
      false,
      false,
      false,
      null,
      MethodKindEnum.initializer,
      "initializer",
      new ArrayList(),
      java.util.Collections.EMPTY_SET,
      false
    );

    type.setInitializer(initializer);
  }

  private static void createDeinitializerIfNeeded(GenericType type)
  {
    if (type.hasDeinitializer())
      return;

    GenericMethod deinitializer = new GenericMethodImpl(
      null,
      true,
      false,
      false,
      new ArrayList(),
      type.getName(),
      true,
      false,
      false,
      false,
      null,
      MethodKindEnum.deinitializer,
      "deinitializer",
      new ArrayList(),
      java.util.Collections.EMPTY_SET,
      false
    );

    type.setDeinitializer(deinitializer);
  }

  private static void createDestructorIfNeeded(GenericType type)
  {
    if (type.hasDestructor())
      return;

    if (typeHasOnlyStaticMethods(type))
      return;

    GenericMethod destructor = new GenericMethodImpl(
      null,
      true,
      false,
      false,
      new ArrayList(),
      type.getName(),
      false,
      false,
      false,
      false,
      null,
      MethodKindEnum.destructor,
      "destructor",
      new ArrayList(),
      java.util.Collections.EMPTY_SET,
      false
    );

    type.addMethod(destructor);
  }

  private static void createConstructorIfNeeded(GenericType type)
  {
    if (type.getOverloadables().get("constructor") != null)
      return;

    if (typeHasOnlyStaticMethods(type))
      return;

    GenericMethod constructor = new GenericMethodImpl(
      null,
      true,
      false,
      false,
      new ArrayList(),
      type.getName(),
      false,
      false,
      false,
      false,
      null,
      MethodKindEnum.constructor,
      "constructor",
      new ArrayList(),
      java.util.Collections.EMPTY_SET,
      false
    );

    type.addMethod(constructor);
  }

  private static boolean typeHasOnlyStaticMethods(GenericType type)
  {
    Iterator it = type.iterateMethods();
    while (it.hasNext())
    {
      GenericMethod m = (GenericMethod)it.next();
      if (!m.isStatic())
        return false;
    }

    return true;
  }
}