/*
  $Id: ZoneProtection.java,v 1.2 2004/04/18 00:52:03 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.ni;

import net.nosica.util.IndentedPrintWriter;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.FileNotFoundException;
import java.io.IOException;
//import java.nio.channels.FileChannel;

import java.util.Iterator;
import java.util.Map;

public class ZoneProtection
{
  private static final String protectString = "//##protect##";

  public static void backupFile(File file) throws FileNotFoundException, IOException
  {
    if (!file.exists())
    {
      return;
    }

    int counter = 0;
    File backupFile = null;
    do
    {
      backupFile = new File(file.getAbsolutePath() + "." + counter);
      ++counter;
    }
    while (backupFile.exists());

/*  more efficient, but internal error with gcj
    FileChannel src = new FileInputStream(file).getChannel();
    FileChannel dst = new FileOutputStream(backupFile).getChannel();
    dst.transferFrom(src, 0, src.size());
    src.close();
    dst.close();
*/
    BufferedReader reader = new BufferedReader(new FileReader(file));
    BufferedWriter writer = new BufferedWriter(new FileWriter(file));

    String line;
    while ((line = reader.readLine()) != null)
    {
      writer.write(line);
      writer.newLine();
    }

    reader.close();
    writer.close();
  }

  public static void writeProtectedArea(IndentedPrintWriter writer, Map protectedZones, String name)
  {
    String zone = (String)protectedZones.get(name);
    protectedZones.remove(name);

    if (zone == null)
      return;

    writeArea(writer, name, zone);
  }

  public static void writeAllProtectedArea(IndentedPrintWriter writer, Map protectedZones)
  {
    Iterator it = protectedZones.entrySet().iterator();

    while (it.hasNext())
    {
      Map.Entry entry = (Map.Entry)it.next();
      String name = (String)entry.getKey();
      String zone = (String)entry.getValue();

      writeArea(writer, name, zone);
    }
  }

  private static void writeArea(IndentedPrintWriter writer, String name, String zone)
  {
    writer.println("//##protect##" + name);
    if (zone.length() != 0)
    {
      writer.println(zone);
    }
    writer.println("//##protect##" + name);
  }

  /**
   * \brief extract from a file the content in protected zone
   *
   * a zone is protected by two commentary beginning with ##protect##
   *
   * \param file the file to read
   * \param protectedZones Map<String, String> the list of protected zone and their content
   */
  public static void extractProtectedZones(File file, Map protectedZones) throws FileNotFoundException, IOException
  {
    FileReader reader = new FileReader(file);
    BufferedReader stream = new BufferedReader(reader);

    String line = null;
    String protectedName = null;
    StringBuffer zone = null;
    int state = 0; // 0 : we're looking from a protected zone, thus discarding all lines
                   // 1 : we're in a protected zone, thus pushing all lines into 'zone'
    while ((line = stream.readLine()) != null)
    {
      int pos;
      if ((pos = line.indexOf(protectString)) == 0)
      {
        String remaining = line.substring(protectString.length());

        if (state == 0)
        {
          protectedName = remaining;
          zone = new StringBuffer();
          state = 1;
        }
        else
        {
          protectedZones.put(protectedName, zone.toString());
          state = 0;
        }
      }
      else
      if (state == 1)
      {
        if (zone.length() != 0)
          zone.append("\n");
        zone.append(line);
      }
    }

    if (state == 1)
    {
      protectedZones.put(protectedName, zone.toString());
    }

    reader.close();
  }
}