/*
  $Id: NodeVisitor.java,v 1.4 2004/03/04 22:16:17 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.parser.node;

import  net.nosica.framework.Visitor;

import  net.nosica.parser.node.filestructure.*;
import  net.nosica.parser.node.typedeclaration.*;
import  net.nosica.parser.node.typedeclaration.expression.*;
import  net.nosica.parser.node.typedeclaration.statement.*;
import  net.nosica.parser.node.util.generic.*;
import  net.nosica.parser.node.util.var.*;

/**
 * Interface for all visitors in Nosica<BR>
 * Here are detailed all method a visitor must be able to deal with
 */

public interface NodeVisitor extends Visitor
{
  // compilation unit
  void visit(CompilationUnit compilationUnit);

  // package
  void visit(PackageDeclaration packageDecl);

  // import
  void visit(ImportDeclaration importDecl);

  // type declaration
  void visit(CastOperatorDeclaration castOpDecl);
  void visit(ClassBody classBody);
  void visit(ClassBodyDeclaration classBodyDecl);
  void visit(ClassDeclaration classDecl);
  void visit(ConstructorDeclaration constructorDecl);
  void visit(DestructorDeclaration destructorDecl);
  void visit(ExplicitConstructorInvocation constructorInvocation);
  void visit(FieldDeclaration fieldDecl);
  void visit(Initializer init);
  void visit(Deinitializer deinit);
  void visit(InterfaceDeclaration interfaceDecl);
  void visit(InterfaceMemberDeclaration interfaceMemberDecl);
  void visit(MethodDeclaration methodDecl);
  void visit(MethodDeclarator methodDecl);
  void visit(NestedClassDeclaration nestedClassDecl);
  void visit(NestedInterfaceDeclaration nestedInterfaceDecl);
  void visit(OperatorDeclaration opDecl);
  void visit(PropertyDeclaration opDecl);
  void visit(TypeDeclaration typeDecl);
  void visit(AliasDeclaration aliasDecl);

  // expression
  void visit(AdditiveExpression addExp);
  void visit(AllocationExpression allocExp);
  void visit(AssignExpression assignExp);
  void visit(CastExpression castExp);
  void visit(ConditionalAndExpression condAndExp);
  void visit(ConditionalExpression condExp);
  void visit(ConditionalOrExpression condOrExp);
  void visit(EqualityExpression eqExpression);
  void visit(ImpliesExpression implyExp);
  void visit(InstanceOfExpression instanceOfExp);
  void visit(MultiplicativeExpression mulExp);
  void visit(PowerExpression powerExp);
  void visit(PrefixExpression prefixExp);
  void visit(PrimaryExpression primaryExp);
  void visit(PrimaryComponent primaryComponent);
  void visit(RelationalExpression relationalExp);
  void visit(UnaryExpression unaryExp);
  void visit(UnaryExpressionNotPlusMinus unaryExp);
  void visit(StreamExpression streamExp);

  // statement
  void visit(Block block);
  void visit(BlockStatement blockStatement);
  void visit(BreakStatement breakStatement);
  void visit(CatchStatement catchStatement);
  void visit(ContinueStatement continueStatement);
  void visit(DoStatement doStatement);
  void visit(EmptyStatement emptyStatement);
  void visit(ForInit forInit);
  void visit(ForStatement forStatement);
  void visit(ForUpdate forUpdate);
  void visit(IfStatement ifStatement);
  void visit(LabeledStatement labeledStatement);
  void visit(LocalVariableDeclaration localVar);
  void visit(ReturnStatement returnStatement);
  void visit(StatementExpressionTerminator statementExpression);
  void visit(StatementExpression statementExpression);
  void visit(StatementExpressionList statementExpressionList);
  void visit(SwitchLabel switchLabel);
  void visit(SwitchStatement switchStatement);
  void visit(ThrowStatement throwStatement);
  void visit(TryStatement tryStatement);
  void visit(WhileStatement whileStatement);

  // generic
  void visit(GenericDeclaration genericDecl);
  void visit(GenericParameter genericParam);

  // var
  void visit(ArgumentList argList);
  void visit(Arguments args);
  void visit(ArrayDimsAndInits arrayDimsAndInits);
  void visit(ArrayInitializer arrayInit);
  void visit(FormalParameter formalParm);
  void visit(FormalParameters formalParms);
  void visit(TypeNameComponent typeNameComponent);
  void visit(TypeName typeName);
  void visit(TypeNameList typeNameList);
  void visit(ResultType resultType);
  void visit(VariableDeclarator varDecl);
  void visit(VariableInitializer varInit);
}
