/*
  $Id: CompilationUnit.java,v 1.4 2004/03/04 22:16:17 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.parser.node.filestructure;

import  java.util.Iterator;
import  net.nosica.parser.node.Node;

/**
 * \brief node that represents a CompilationUnit
 *
 * e.g : a nosica file by itself is a CompilationUnit<br>
 * it must defines first the package (if any)<br>
 * then all import declarations<br>
 * then the classes defined by CompilationUnit<br>
 * <br>
 * e.g :
 * <pre>
 * package test;
 *
 * import net.nosica.io.Console;
 *
 * class Test<T>
 * {
 * }
 *
 * class Test<int>
 * {
 * }
 *
 * </pre>
 * will defines a CompilationUnit containing :
 * <ol>
 *   <li>a PackageDeclaration containing Test
 *   <li>only one ImportDeclaration containing net.nosica.io.Console
 *   <li>a list of types containing generic type Test<T>, and type Test<int>
 * </ol>
 * <br>
 * grammar :<br>
 *   CompilationUnit ::=<br>
 *    ( PackageDeclaration )? ( ImportDeclaration )* ( TypeDeclaration )* <EOF>
 */
public interface CompilationUnit extends Node
{
  /**
   * \brief has this CompilationUnit a PackageDeclaration ?
   *
   * a PackageDeclaration is optional only at the root of the source path
   *
   * \return true if there is a PackageDeclaration
   */
  boolean hasPackageDeclaration();

  /**
   * \brief return the PackageDeclaration
   *
   * \remarks need hasPackageDeclaration to return true
   * \return PackageDeclaration
   */
  PackageDeclaration getPackageDeclaration();

  /**
   * \brief iterates through list of import declaration
   *
   * \return Iterator<ImportDeclaration>
   */
  Iterator iterateImportDeclaration();

  /**
   * \brief iterates through list of declared types
   *
   * \return Iterator<TypeDeclaration>
   */
  Iterator iterateTypesDeclaration();
}
