/*
  $Id: AliasDeclaration.java,v 1.4 2004/03/04 22:16:17 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.parser.node.typedeclaration;

import  net.nosica.parser.node.util.var.TypeName;

/**
 * \brief declares a new Alias
 *
 * e.g. : in source code : <br>
 * <pre>
 * interface A
 * {
 *   sub f();
 * }
 *
 * interface B
 * {
 *   sub f();
 * }
 *
 * class Toto implements A, B
 * {
 *   alias A.f af;
 *   alias B.f bf;
 *
 *   sub af() {}
 *   sub bf() {}
 * }
 * </pre>
 * <br>
 * An other example :<br>
 * <pre>
 * class Test
 * {
 *   import com.mycompany.sub1.Toto sub1Toto;
 *   import com.mycompany.sub2.Toto sub2Toto;
 * }
 * </pre>
 * <br>
 * First example enables somebody to "rename" a method because of a name conflict.<br>
 * Second example enables somebody to "alias" a type and gives it another name because of a name conflict.<br>
 * <br>
 * grammar :<br>
 *   AliasDeclaration ::= ( "alias" | "import" ) TypeName <IDENTIFIER> ";"
 */

public interface AliasDeclaration extends ClassBodyDeclaration, InterfaceMemberDeclaration
{
  /**
   * \brief returns alias
   *
   * \return alias
   */
  String getAlias();

  /**
   * \brief indicates if this AliasDeclaration is a "method alias" or an "import and alias"
   *
   * \return true if this is an import, false otherwise
   */
  boolean isImport();

  /**
   * \brief return aliased type or method
   *
   * In a parsing view, A.f is not different from com.mycompany.sub1.Toto : it is a TypeName<br>
   * Following result of isImport() we have to take care of the real signification of getTypeName() !
   *
   * \return aliased
   */
  TypeName getType();
}
