/*
  $Id: CastOperatorDeclaration.java,v 1.5 2004/03/25 22:54:39 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.parser.node.typedeclaration;

import  net.nosica.parser.node.util.var.TypeName;
import  net.nosica.parser.node.util.var.TypeNameList;
import  net.nosica.parser.node.typedeclaration.statement.Block;

/**
 * \brief a node that represents the declaration of a cast operator
 *
 * e.g. : in source code :
 * <pre>
 * primitive class Test
 * {
 *   public cast Toto() { ... }
 * }
 * </pre>
 * will define a CastOperatorDeclaration.<br>
 * <br>
 * as we can see from grammar, a cast operator is mandatory public.<br>
 * it is only legal in a primitive type.<br>
 * it can only have a primitive type as the target of the cast.<br>
 *
 * grammar :<br>
 *   CastOperatorDeclaration ::=
 *     "public" "cast" TypeName "(" ")" ( "throws" TypeNameList )? ( Block | ";" )
 */
public interface CastOperatorDeclaration extends Method, ClassBodyDeclaration
{
  /**
   * \brief return the result type of the method as the "casted type"
   *
   * \return casted type
   */
  TypeName getCastedType();
}
