/*
  $Id: ClassDeclarationImpl.java,v 1.6 2004/03/04 22:16:17 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.parser.node.typedeclaration;

import  net.nosica.framework.Visitor;
import  net.nosica.parser.node.NodeVisitor;
import  net.nosica.parser.node.util.generic.GenericDeclaration;
import  net.nosica.parser.node.util.var.TypeName;
import  net.nosica.parser.node.util.var.TypeNameList;

import java.util.ArrayList;

public class ClassDeclarationImpl extends AbstractSymbol implements ClassDeclaration
{
  private boolean isAbstract = false;
  private boolean isFinal = false;
  private boolean isPublic = false;
  private boolean isProtected = false;
  private boolean isPrivate = false;
  private boolean isPrimitive = false;
  private boolean isNative = false;
  private boolean isInner = false;

  private String identifier;
  private GenericDeclaration genericDecl;
  private TypeName extend;
  private TypeNameList implementList;
  private ClassBody classBody;

  public ClassDeclarationImpl(
    String identifier,
    GenericDeclaration genericDecl,
    TypeName extend,
    TypeNameList implement,
    ClassBody classBody
   )
  {
    this("", null, identifier, genericDecl, extend, implement, classBody);
  }

  public ClassDeclarationImpl(
    String localization,
    ArrayList components,
    String identifier,
    GenericDeclaration genericDecl,
    TypeName extend,
    TypeNameList implement,
    ClassBody classBody
   )
  {
    super(localization, components);

    this.identifier = identifier;
    this.genericDecl = genericDecl;
    this.extend = extend;
    this.implementList = implement;
    this.classBody = classBody;
  }

  public void setAbstract(boolean isAbstract) { this.isAbstract = isAbstract; }
  public boolean isAbstract() {return isAbstract;}
  public void setFinal(boolean isFinal) { this.isFinal = isFinal; }
  public boolean isFinal() {return isFinal;}
  public void setPublic(boolean isPublic) { this.isPublic = isPublic; }
  public boolean isPublic() {return isPublic;}
  public void setProtected(boolean isProtected) { this.isProtected = isProtected; }
  public boolean isProtected() {return isProtected;}
  public void setPrivate(boolean isPrivate) { this.isPrivate = isPrivate; }
  public boolean isPrivate() {return isPrivate;}
  public void setPrimitive(boolean isPrimitive) { this.isPrimitive = isPrimitive; }
  public boolean isPrimitive() {return isPrimitive;}
  public void setNative(boolean isNative) { this.isNative = isNative; }
  public boolean isNative() {return isNative;}
  public void setInner(boolean isInner) { this.isInner = isInner; }
  public boolean isInner() {return isInner;}

  public String getIdentifier() {return identifier;}
  public boolean hasGenericDeclaration() {return genericDecl != null;}
  public GenericDeclaration getGenericDeclaration() {return genericDecl;}
  public boolean hasExtend() {return extend != null;}
  public TypeName getExtend() {return extend;}
  public boolean hasImplementList() {return implementList != null;}
  public TypeNameList getImplementList() {return implementList;}
  public ClassBody getClassBody() {return classBody;}

  public void accept(Visitor visitor)
  {
    ((NodeVisitor)visitor).visit(this);
  }
}
