/*
  $Id: ConstructorDeclarationImpl.java,v 1.10 2004/04/18 00:52:03 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.parser.node.typedeclaration;

import  java.util.ArrayList;
import  java.util.Iterator;
import  net.nosica.framework.Visitor;
import  net.nosica.parser.node.NodeVisitor;
import  net.nosica.parser.node.typedeclaration.statement.Block;
import  net.nosica.parser.node.typedeclaration.statement.BlockImpl;
import  net.nosica.parser.node.util.generic.GenericDeclaration;
import  net.nosica.parser.node.util.var.FormalParameters;
import  net.nosica.parser.node.util.var.TypeNameList;

public class ConstructorDeclarationImpl extends AbstractSymbol implements ConstructorDeclaration
{
  private GenericDeclaration genericDecl = null;
  private FormalParameters formalParameters;
  private TypeNameList throwList = null;
  private ExplicitConstructorInvocation explicitConstructorInvocation = null;
  private ArrayList blockStatements = null;
  private boolean isPublic = false;
  private boolean isProtected = false;
  private boolean isPrivate = false;

  public ConstructorDeclarationImpl(
    GenericDeclaration genericDecl,
    FormalParameters formalParameters,
    TypeNameList throwList,
    ExplicitConstructorInvocation explicitConstructorInvocation,
    ArrayList blockStatements,
    boolean isPublic,
    boolean isProtected,
    boolean isPrivate
  )
  {
    this("", null, genericDecl, formalParameters, throwList, explicitConstructorInvocation, blockStatements, isPublic, isProtected, isPrivate);
  }

  public ConstructorDeclarationImpl(
    String localization,
    ArrayList comments,
    GenericDeclaration genericDecl,
    FormalParameters formalParameters,
    TypeNameList throwList,
    ExplicitConstructorInvocation explicitConstructorInvocation,
    ArrayList blockStatements,
    boolean isPublic,
    boolean isProtected,
    boolean isPrivate
  )
  {
    super(localization, comments);

    this.genericDecl = genericDecl;
    this.formalParameters = formalParameters;
    this.throwList = throwList;
    this.explicitConstructorInvocation = explicitConstructorInvocation;
    this.blockStatements = blockStatements;
    this.isPublic = isPublic;
    this.isProtected = isProtected;
    this.isPrivate = isPrivate;
  }

  public boolean hasGenericDeclaration() { return genericDecl != null; }
  public GenericDeclaration getGenericDeclaration() { return genericDecl; }
  public FormalParameters getFormalParameters() { return formalParameters; }
  public boolean hasThrowList() { return throwList != null; }
  public TypeNameList getThrowList() { return throwList; }
  public boolean hasExplicitConstructorInvocation() { return explicitConstructorInvocation != null; }
  public ExplicitConstructorInvocation getExplicitConstructorInvocation() { return explicitConstructorInvocation; }
  public Iterator iterateBlockStatements() { return blockStatements.iterator(); }
  public boolean hasBlock() { return hasBody(); }
  public Block getBlock() { return new BlockImpl(blockStatements); }
  public boolean isPublic() {return isPublic;}
  public boolean isProtected() {return isProtected;}
  public boolean isPrivate() {return isPrivate;}

  public boolean hasBody()
  {
    if (explicitConstructorInvocation != null) return true;
    return blockStatements != null;
  }

  public void accept(Visitor visitor)
  {
    ((NodeVisitor)visitor).visit(this);
  }
}
