/*
  $Id: FieldDeclaration.java,v 1.4 2004/03/04 22:16:17 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.parser.node.typedeclaration;

import  net.nosica.parser.node.util.var.TypeName;
import  java.util.Iterator;

/**
 * \brief node that represents a FieldDeclaration
 *
 * e.g. : in source code :
 * <pre>
 * class Test
 * {
 *   public int i, j;
 * }
 * </pre>
 * will define a FieldDeclaration. Fields will be public of type int.<br>
 * there will be two VariableDeclarator containing i and j.<br>
 * <br>
 *
 * as defined by grammar, a field can be public, protected, private, package<br>
 * It can also be static, const or mutable, or transient.<br>
 * <br>
 * grammar :<br>
 *   FieldDeclaration ::=<br>
 *     ( "public" | "protected" | "private" | "static" | "const" | "mutable" | "transient" )*
 *     TypeName
 *     VariableDeclarator
 *     ( "," VariableDeclarator )* ";"
 */
public interface FieldDeclaration extends ClassBodyDeclaration
{
  /**
   * \brief is field public ?
   *
   * \return true if field is public
   */
  boolean isPublic();

  /**
   * \brief is field protected ?
   *
   * \return true if field is protected
   */
  boolean isProtected();

  /**
   * \brief is field private ?
   *
   * \return true if field is private
   */
  boolean isPrivate();

  /**
   * \brief is field static ?
   *
   * \return true if field is static
   */
  boolean isStatic();

  /**
   * \brief is field const ?
   *
   * \return true if field is const
   */
  boolean isConst();

  /**
   * \brief is field mutable ?
   *
   * \return true if field is mutable
   */
  boolean isMutable();

  /**
   * \brief is field transient ?
   *
   * \remarks for now, transient is unused
   * \return true if field is transient
   */
  boolean isTransient();

  /**
   * \brief returns type of fields
   *
   * \return type of fields
   */
  TypeName getTypeName();

  /**
   * \brief return variable declarator
   *
   * \return Iterator<VariableDeclarator>
   */
  Iterator iterateVariableDeclarator();
}
