/*
  $Id: Initializer.java,v 1.5 2004/03/25 22:54:39 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.parser.node.typedeclaration;

import  net.nosica.parser.node.typedeclaration.statement.Block;

/**
 * \brief node that represents an initializer
 *
 * e.g : in source code :<br>
 * <pre>
 * class Test
 * {
 *   static
 *   {
 *     ...
 *   }
 * }
 * </pre>
 * will define a static initializer<br>
 * <br>
 * a static initializer is a piece of code that is called before the main method of the executed program<br>
 * several static initializer will be called in a random order, so no one can make asumptions in which order
 * they will be called.<br>
 * a static initializer is considered as a static method.<br>
 * so one can access static methods and static fields of type.<br>
 * However, no one should access static fields of another types as it is possible that they are not statically initialized yet.<br>
 * <br>
 *
 * grammar :<br>
 *   Initializer ::=<br>
 *     ( "staticconstructor" Block )
 */
public interface Initializer extends Method, ClassBodyDeclaration
{
}
