/*
  $Id: InterfaceDeclarationImpl.java,v 1.6 2004/03/04 22:16:17 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.parser.node.typedeclaration;

import  java.util.ArrayList;
import  java.util.Iterator;
import  net.nosica.framework.Visitor;
import  net.nosica.parser.node.NodeVisitor;
import  net.nosica.parser.node.util.generic.GenericDeclaration;
import  net.nosica.parser.node.util.var.TypeNameList;

public class InterfaceDeclarationImpl extends AbstractSymbol implements InterfaceDeclaration
{
  private String name;
  private GenericDeclaration genericDecl;
  private TypeNameList extendList;
  private ArrayList interfaceMemberDecls;

  private boolean isPublic = false;
  private boolean isPrivate = false;
  private boolean isProtected = false;

  public InterfaceDeclarationImpl(
    String name,
    GenericDeclaration genericDecl,
    TypeNameList extendList,
    ArrayList interfaceMemberDecls
  )
  {
    this("", null, name, genericDecl, extendList, interfaceMemberDecls);
  }

  public InterfaceDeclarationImpl(
    String localization,
    ArrayList comments,
    String name,
    GenericDeclaration genericDecl,
    TypeNameList extendList,
    ArrayList interfaceMemberDecls
  )
  {
    super(localization, comments);

    this.name = name;
    this.genericDecl = genericDecl;
    this.extendList = extendList;
    this.interfaceMemberDecls = interfaceMemberDecls;
  }

  public void setPublic(boolean isPublic) { this.isPublic = isPublic; }
  public void setProtected(boolean isProtected) { this.isProtected = isProtected; }
  public void setPrivate(boolean isPrivate) { this.isPrivate = isPrivate; }
  public String getName() {return name;}
  public boolean hasGenericDeclaration() {return genericDecl != null;}
  public GenericDeclaration getGenericDeclaration() {return genericDecl;}
  public boolean hasExtendList() {return extendList != null;}
  public TypeNameList getExtendList() {return extendList;}
  public Iterator iterateInterfaceMemberDeclaration() {return interfaceMemberDecls.iterator();}
  public boolean isPublic() {return isPublic;}
  public boolean isProtected() {return isProtected;}
  public boolean isPrivate() {return isPrivate;}

  public void accept(Visitor visitor)
  {
    ((NodeVisitor)visitor).visit(this);
  }
}
