/*
  $Id: Method.java,v 1.5 2004/05/06 19:47:00 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.parser.node.typedeclaration;

import  net.nosica.parser.node.util.var.TypeNameList;
import  net.nosica.parser.node.typedeclaration.statement.Block;

/**
 * \brief a common interface for MethodDeclaration, CastOperatorDeclaration, OperatorDeclaration, PropertyDeclaration
 */
public interface Method extends Symbol
{
  /**
   * \brief has this method a declared thrown list of exception ?
   *
   * \return true if method throws exceptions
   */
  boolean hasThrowList();

  /**
   * \brief has this method an implementation ?
   *
   * An interface can declare a method without providing implementation<br>
   * A native class can also declare a method without providing implementation<br>
   *
   * \return true if method has a block
   */
  boolean hasBlock();

  /**
   * \brief return the list of thrown exceptions
   *
   * \remarks need hasThrowList() to return true
   * \return implementation (if any)
   */
  TypeNameList getThrowList();

  /**
   * \brief return implementation of method (if any)
   *
   * \remarks need hasBlock() to return true
   * \return implementation
   */
  Block getBlock();
}