/*
  $Id: MethodDeclaration.java,v 1.4 2004/03/04 22:16:17 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.parser.node.typedeclaration;

import  net.nosica.parser.node.util.var.ResultType;

/**
 * \brief node that represents a Method Declaration
 *
 * e.g : in source code :<br>
 * <pre>
 * clas Test
 * {
 *   sub f() {}
 * }
 * </pre>
 * will define a method.<br>
 * <br>
 * grammar :<br>
 *   MethodDeclaration ::=<br>
 *     ( "public" | "protected" | "private" | "static" | "final" | "callback" )*
 *     ResultType MethodDeclarator
 *     ( "const" )?
 *     ( "throws" TypeNameList )?
 *     ( Block | ";" ) OptionalBlock
 */
public interface MethodDeclaration extends Method, ClassBodyDeclaration, InterfaceMemberDeclaration
{
  // attributes
  boolean isPublic();
  boolean isProtected();
  boolean isPrivate();
  boolean isStatic();
  boolean isFinal();
  boolean isCallback();
  boolean isMethodConst();

  // sub nodes
  ResultType getResultType();
  MethodDeclarator getMethodDeclarator();
}
