/*
  $Id: MethodDeclarationImpl.java,v 1.6 2004/03/04 22:16:17 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.parser.node.typedeclaration;

import  net.nosica.framework.Visitor;
import  net.nosica.parser.node.NodeVisitor;
import  net.nosica.parser.node.util.var.ResultType;
import  net.nosica.parser.node.util.var.TypeNameList;
import  net.nosica.parser.node.typedeclaration.statement.Block;

import java.util.ArrayList;

public class MethodDeclarationImpl extends AbstractSymbol implements MethodDeclaration
{
  private boolean isPublic = false;
  private boolean isProtected = false;
  private boolean isPrivate = false;
  private boolean isStatic = false;
  private boolean isFinal = false;
  private boolean isCallback = false;

  private ResultType result;
  private MethodDeclarator methodDecl;
  private boolean methodIsConst = false;
  private TypeNameList throwList = null;
  private Block block = null;

  public MethodDeclarationImpl(
    ResultType result,
    MethodDeclarator methodDecl,
    boolean methodIsConst,
    TypeNameList throwList,
    Block block,
    boolean isPublic,
    boolean isProtected,
    boolean isPrivate,
    boolean isStatic,
    boolean isFinal,
    boolean isCallback
  )
  {
    this("", null, result, methodDecl, methodIsConst, throwList, block, isPublic, isProtected, isPrivate, isStatic, isFinal, isCallback);
  }

  public MethodDeclarationImpl(
    String localization,
    ArrayList comments,
    ResultType result,
    MethodDeclarator methodDecl,
    boolean methodIsConst,
    TypeNameList throwList,
    Block block,
    boolean isPublic,
    boolean isProtected,
    boolean isPrivate,
    boolean isStatic,
    boolean isFinal,
    boolean isCallback
  )
  {
    super(localization, comments);

    this.result = result;
    this.methodDecl = methodDecl;
    this.methodIsConst = methodIsConst;
    this.throwList = throwList;
    this.block = block;
    this.isPublic = isPublic;
    this.isProtected = isProtected;
    this.isPrivate = isPrivate;
    this.isStatic = isStatic;
    this.isFinal = isFinal;
    this.isCallback = isCallback;
  }

 public ResultType getResultType() {return result;}
 public MethodDeclarator getMethodDeclarator() {return methodDecl;}
 public boolean isMethodConst() {return methodIsConst;}
 public boolean hasThrowList() {return throwList != null;}
 public TypeNameList getThrowList() {return throwList;}
 public boolean hasBlock() {return block != null;}
 public Block getBlock() {return block;}
 public boolean isPublic() {return isPublic;}
 public boolean isProtected() {return isProtected;}
 public boolean isPrivate() {return isPrivate;}
 public boolean isStatic() {return isStatic;}
 public boolean isFinal() {return isFinal;}
 public boolean isCallback() {return isCallback;}

 public void accept(Visitor visitor)
 {
   ((NodeVisitor)visitor).visit(this);
 }
}
