/*
  $Id: NestedClassDeclaration.java,v 1.4 2004/03/04 22:16:17 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.parser.node.typedeclaration;

/**
 * \brief node that represents a nested class
 *
 * e.g : in source code :
 * <pre>
 * class Test
 * {
 *   class NestedType
 *   {
 *   }
 * }
 * </pre>
 * will define a nested class, whereas
 * <pre>
 * class Test
 * {
 *   inner class NestedType
 *   {
 *   }
 * }
 * </pre>
 * will define an inner class.<br>
 * <br>
 * differences are that a nested class is an independant class that defines a fancy namespace<br>
 * whereas an inner class is a class that can only be created in the context of it's enclosing parent and keeps
 * a link to it's enclosing parent so that it has access to fields and methods of it's parent as if they were
 * part of it.<br>
 * <br>
 * grammar :<br>
 *   NestedClassDeclaration ::=<br>
 *     ( "abstract" | "final" | "public" | "protected" | "private" | "primitive" | "native" | "inner" )*
 *     UnmodifiedClassDeclaration
 */
public interface NestedClassDeclaration extends ClassBodyDeclaration, InterfaceMemberDeclaration
{
  /**
   * \brief return underlying class declaration
   *
   * \return class declaration
   */
  ClassDeclaration getClassDeclaration();
}
