/*
  $Id: OperatorDeclarationImpl.java,v 1.6 2004/03/04 22:16:17 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.parser.node.typedeclaration;

import  net.nosica.framework.Visitor;
import  net.nosica.parser.node.NodeVisitor;
import  net.nosica.parser.node.util.var.ResultType;
import  net.nosica.parser.node.util.operator.Operator;
import  net.nosica.parser.node.util.generic.GenericDeclaration;
import  net.nosica.parser.node.util.var.FormalParameters;
import  net.nosica.parser.node.util.var.TypeNameList;
import  net.nosica.parser.node.typedeclaration.statement.Block;

import java.util.ArrayList;

public class OperatorDeclarationImpl extends AbstractSymbol implements OperatorDeclaration
{
  private ResultType type;
  private boolean isInfix = false;
  private boolean isPrefix = false;
  private boolean isPostfix = false;
  private Operator op;
  private GenericDeclaration genericDecl = null;
  private FormalParameters formalParameters;
  private boolean methodIsConst = false;
  private TypeNameList throwList = null;
  private Block block = null;
  private boolean isPublic = false;
  private boolean isProtected = false;
  private boolean isPrivate = false;
  private boolean isFinal = false;

  public OperatorDeclarationImpl(
    ResultType type,
    boolean isInfix,
    boolean isPrefix,
    boolean isPostfix,
    Operator op,
    GenericDeclaration genericDecl,
    FormalParameters formalParameters,
    boolean methodIsConst,
    TypeNameList throwList,
    Block block,
    boolean isPublic,
    boolean isProtected,
    boolean isPrivate,
    boolean isFinal
  )
  {
    this("", null, type, isInfix, isPrefix, isPostfix, op, genericDecl, formalParameters, methodIsConst, throwList, block, isPublic, isProtected, isPrivate, isFinal);
  }

  public OperatorDeclarationImpl(
    String localization,
    ArrayList comments,
    ResultType type,
    boolean isInfix,
    boolean isPrefix,
    boolean isPostfix,
    Operator op,
    GenericDeclaration genericDecl,
    FormalParameters formalParameters,
    boolean methodIsConst,
    TypeNameList throwList,
    Block block,
    boolean isPublic,
    boolean isProtected,
    boolean isPrivate,
    boolean isFinal
  )
  {
    super(localization, comments);

    this.type = type;
    this.isInfix = isInfix;
    this.isPrefix = isPrefix;
    this.isPostfix = isPostfix;
    this.op = op;
    this.genericDecl = genericDecl;
    this.formalParameters = formalParameters;
    this.methodIsConst = methodIsConst;
    this.throwList = throwList;
    this.block = block;
    this.isPublic = isPublic;
    this.isProtected = isProtected;
    this.isPrivate = isPrivate;
    this.isFinal = isFinal;
  }

  public ResultType getType() {return type;}
  public boolean isInfix() {return isInfix;}
  public boolean isPrefix() {return isPrefix;}
  public boolean isPostfix() {return isPostfix;}
  public Operator getOperator() {return op;}
  public boolean hasGenericDeclaration() {return genericDecl != null;}
  public GenericDeclaration getGenericDeclaration() {return genericDecl;}
  public FormalParameters getFormalParameters() {return formalParameters;}
  public boolean isMethodConst() {return methodIsConst;}
  public boolean hasThrowList() {return throwList != null;}
  public TypeNameList getThrowList() {return throwList;}
  public boolean hasBlock() {return block != null;}
  public Block getBlock() {return block;}
  public boolean isPublic() {return isPublic;}
  public boolean isProtected() {return isProtected;}
  public boolean isPrivate() {return isPrivate;}
  public boolean isFinal() {return isFinal;}

  public void accept(Visitor visitor)
  {
    ((NodeVisitor)visitor).visit(this);
  }
}
