/*
  $Id: PropertyDeclarationImpl.java,v 1.6 2004/03/04 22:16:17 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.parser.node.typedeclaration;

import net.nosica.parser.node.NodeVisitor;
import net.nosica.parser.node.util.var.ResultType;
import net.nosica.parser.node.util.var.TypeNameList;
import net.nosica.parser.node.util.var.FormalParameters;
import net.nosica.parser.node.util.generic.GenericDeclaration;
import net.nosica.parser.node.typedeclaration.statement.Block;

import net.nosica.framework.Visitor;

import java.util.ArrayList;

public class PropertyDeclarationImpl extends AbstractSymbol implements PropertyDeclaration
{
  private ResultType resultType;
  private String name;
  private boolean isPropertyArray;
  private GenericDeclaration genericDecl;
  private FormalParameters formalParameters;
  private boolean methodIsConst;
  private TypeNameList throwList;
  private Block block;
  private boolean isPublic;
  private boolean isProtected;
  private boolean isPrivate;
  private boolean isFinal;
  private boolean isStatic;

  public PropertyDeclarationImpl(
    ResultType resultType,
    String name,
    boolean isPropertyArray,
    GenericDeclaration genericDecl,
    FormalParameters formalParameters,
    boolean methodIsConst,
    TypeNameList throwList,
    Block block,
    boolean isPublic,
    boolean isProtected,
    boolean isPrivate,
    boolean isFinal,
    boolean isStatic
  )
  {
    this("", null, resultType, name, isPropertyArray, genericDecl, formalParameters, methodIsConst, throwList, block, isPublic, isProtected, isPrivate, isFinal, isStatic);
  }

  public PropertyDeclarationImpl(
    String localization,
    ArrayList comments,
    ResultType resultType,
    String name,
    boolean isPropertyArray,
    GenericDeclaration genericDecl,
    FormalParameters formalParameters,
    boolean methodIsConst,
    TypeNameList throwList,
    Block block,
    boolean isPublic,
    boolean isProtected,
    boolean isPrivate,
    boolean isFinal,
    boolean isStatic
  )
  {
    super(localization, comments);

    this.resultType = resultType;
    this.name = name;
    this.isPropertyArray = isPropertyArray;
    this.genericDecl = genericDecl;
    this.formalParameters = formalParameters;
    this.methodIsConst = methodIsConst;
    this.throwList = throwList;
    this.block = block;
    this.isPublic = isPublic;
    this.isProtected = isProtected;
    this.isPrivate = isPrivate;
    this.isFinal = isFinal;
    this.isStatic = isStatic;
  }

  public ResultType getResultType() { return resultType; }
  public String getName() { return name; }
  public boolean isPropertyArray() { return isPropertyArray; }
  public boolean hasGenericDeclaration() { return genericDecl != null; }
  public GenericDeclaration getGenericDeclaration() { return genericDecl; }
  public FormalParameters getFormalParameters() { return formalParameters; }
  public boolean isMethodConst() { return methodIsConst; }
  public boolean hasThrowList() { return throwList != null; }
  public TypeNameList getThrowList() { return throwList; }
  public boolean hasBlock() { return block != null; }
  public Block getBlock() { return block; }
  public boolean isPublic() { return isPublic; }
  public boolean isProtected() { return isProtected; }
  public boolean isPrivate() { return isPrivate; }
  public boolean isFinal() { return isFinal; }
  public boolean isStatic() { return isStatic; }

  public void accept(Visitor visitor)
  {
    ((NodeVisitor)visitor).visit(this);
  }
}
