/*
  $Id: TypeDeclaration.java,v 1.4 2004/03/04 22:16:17 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.parser.node.typedeclaration;

import  net.nosica.parser.node.Node;

/**
 * \brief a node that represents a type
 *
 * e.g : in source code :
 * <pre>
 * class Test
 * {
 * }
 * </pre>
 * will create a node TypeDeclaration that contains a ClassDeclaration<br>
 * whereas :<br>
 * <pre>
 * interface Toto
 * {
 * }
 * </pre>
 * will create a node TypeDeclaration that contains an InterfaceDeclaration
 * grammar :<br>
 *   TypeDeclaration ::= ClassDeclaration | InterfaceDeclaration<br>
 */

public interface TypeDeclaration extends Node
{
  /**
   * \brief is this is a ClassDeclaration ?
   *
   * \return true if this is a ClassDeclaration
   */
  boolean isClassDeclaration();

  /**
   * \brief is this is an InterfaceDeclaration ?
   *
   * \return true if this is a InterfaceDeclaration
   */
  boolean isInterfaceDeclaration();

  /**
   * \brief returns underlying ClassDeclaration
   *
   * \remarks need isClassDeclaration() to return true
   * \return underlying ClassDeclaration
   */
  ClassDeclaration getClassDeclaration();

  /**
   * \brief returns underlying InterfaceDeclaration
   *
   * \remarks need isInterfaceDeclaration() to return true
   * \return underlying InterfaceDeclaration
   */
  InterfaceDeclaration getInterfaceDeclaration();
}
