/*
  $Id: PrimaryPrefixImpl.java,v 1.4 2004/03/04 22:16:17 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.parser.node.typedeclaration.expression;

import  net.nosica.framework.Visitor;
import  net.nosica.parser.node.AbstractNode;
import  net.nosica.parser.node.NodeVisitor;
import  net.nosica.parser.node.util.literal.Literal;
import  net.nosica.parser.node.util.var.TypeName;
import  net.nosica.parser.node.util.var.Arguments;

import java.util.Iterator;

public class PrimaryPrefixImpl extends AbstractNode implements PrimaryComponent
{
  private Literal literal = null;
  private Expression expression;
  private AllocationExpression allocationExpression = null;
  private TypeName typeName = null;
  private boolean isThis = false;
  private boolean isSuper = false;

  public PrimaryPrefixImpl(Literal literal)
  {
    this("", literal);
  }

  public PrimaryPrefixImpl(boolean isThis, boolean isSuper)
  {
    this("", isThis, isSuper);
  }

  public PrimaryPrefixImpl(Expression expression)
  {
    this("", expression);
  }

  public PrimaryPrefixImpl(AllocationExpression allocationExp)
  {
    this("", allocationExp);
  }

  public PrimaryPrefixImpl(TypeName typeName)
  {
    this("", typeName);
  }

  public PrimaryPrefixImpl(String localization, Literal literal)
  {
    super(localization);

    this.literal = literal;
  }

  public PrimaryPrefixImpl(String localization, boolean isThis, boolean isSuper)
  {
    super(localization);

    this.isThis = isThis;
    this.isSuper = isSuper;
  }

  public PrimaryPrefixImpl(String localization, Expression expression)
  {
    super(localization);

    this.expression = expression;
  }

  public PrimaryPrefixImpl(String localization, AllocationExpression allocationExp)
  {
    super(localization);

    this.allocationExpression = allocationExp;
  }

  public PrimaryPrefixImpl(String localization, TypeName typeName)
  {
    super(localization);

    this.typeName = typeName;
  }

  public boolean isLiteral() { return literal != null; }
  public boolean isThis() { return isThis; }
  public boolean isSuper() { return isSuper; }
  public boolean isExpression() { return expression != null; }
  public boolean isAllocation() { return allocationExpression != null; }
  public boolean isTypeName() { return typeName != null; }
  public boolean isArrayAccess() { return false; }
  public boolean isMember() { return false; }
  public boolean isArguments() { return false; }

  public Literal getLiteral() { return literal; }
  public Expression getExpression() { return expression; }
  public AllocationExpression getAllocation() { return allocationExpression; }
  public TypeName getTypeName() { return typeName; }
  public String getMember() { throw new UnsupportedOperationException(); }
  public Iterator iterateDimensions() { throw new UnsupportedOperationException(); }
  public Arguments getArguments() { throw new UnsupportedOperationException(); }

  public void accept(Visitor visitor)
  {
    ((NodeVisitor)visitor).visit(this);
  }
}
