/*
  $Id: ArrayDimsAndInitsImpl.java,v 1.5 2004/03/04 22:16:17 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.parser.node.util.var;

import  java.util.ArrayList;
import  net.nosica.parser.node.AbstractNode;
import  java.util.Iterator;
import  java.util.ListIterator;
import  net.nosica.framework.Visitor;
import  net.nosica.parser.node.NodeVisitor;

public class ArrayDimsAndInitsImpl extends AbstractNode implements ArrayDimsAndInits
{
  private ArrayList initializedDims = null;      // ArrayList<ArrayList<Expression>>
  private ArrayList unitializedDims = null;      // ArrayList<Integer>

  private ArrayInitializer arrayInit = null;
  private ArrayList dimensions = null;           // ArrayList<Integer>

  public ArrayDimsAndInitsImpl(ArrayList initializedDims, ArrayList unitializedDims)
  {
    this("", initializedDims, unitializedDims);
  }

  public ArrayDimsAndInitsImpl(ArrayList dimensions, ArrayInitializer arrayInit)
  {
    this("", dimensions, arrayInit);
  }

  public ArrayDimsAndInitsImpl(String localization, ArrayList initializedDims, ArrayList unitializedDims)
  {
    super(localization);

    this.initializedDims = initializedDims;
    this.unitializedDims = unitializedDims;
  }

  public ArrayDimsAndInitsImpl(String localization, ArrayList dimensions, ArrayInitializer arrayInit)
  {
    super(localization);

    this.dimensions = dimensions;
    this.arrayInit = arrayInit;
  }

  public Iterator iterateDimensions()
  {
    if (dimensions != null)
      return dimensions.iterator();
    else
    {
      ArrayList temp = initializedDims;
      temp.addAll(unitializedDims);

      return temp.iterator();
    }
  }

  public ListIterator reverseIterateDimensions()
  {
    if (dimensions != null)
      return dimensions.listIterator(dimensions.size());
    else
    {
      ArrayList temp = initializedDims;
      temp.addAll(unitializedDims);

      return temp.listIterator(temp.size());
    }
  }

  public boolean hasInitializedDims()
  {
    return initializedDims != null;
  }

  public Iterator iterateInitializedDims()
  {
    return initializedDims.iterator();
  }

  public ListIterator reverseIterateInitializedDims()
  {
    return initializedDims.listIterator(initializedDims.size());
  }

  public Iterator iterateUnitializedDims()
  {
    return unitializedDims.iterator();
  }

  public ListIterator reverseIterateUnitializedDims()
  {
    return unitializedDims.listIterator(unitializedDims.size());
  }

  public boolean hasArrayInitializer()
  {
    return arrayInit != null;
  }

  public ArrayInitializer getArrayInitializer()
  {
    return arrayInit;
  }

  public void accept(Visitor visitor)
  {
    ((NodeVisitor)visitor).visit(this);
  }
}
