/*
  $Id: TypeName.java,v 1.5 2004/03/04 22:16:17 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.parser.node.util.var;

import net.nosica.parser.node.Node;
import java.util.Iterator;
import java.util.ArrayList;

/**
 * \brief TypeName representation
 * TypeName can be th representation of the name of a type as in net.nosica.lang.int<BR>
 * It can also be the representation of a method call as in net.nosica.MySingleton.instance<BR>
 * It can also be the representation of a member access as in net.nosica.myField<BR>
 * In short, it can be a lot of things ...<BR>
 * It also contains generic information and array information is needed
 */

public interface TypeName extends Node
{
  /**
   * \brief Is this type name the representation of an array ?
   *
   * \return true if this type name as an array extension
   */
  boolean isArray();

  /**
   * \brief Is this type name a scalar<BR>
   * requires : !isArray()
   *
   * \return true is this type is a scalar
   */
  boolean isScalar();

  /**
   * \brief Is this type name an empty type name ?
   *
   * \return true if empty
   */
  boolean isEmpty();

  /**
   * \brief Return the array element type
   * requires : isArray()
   */
  TypeName getArrayElementType();

  /**
   * \brief Iterate through all components of the type name
   *
   * \return Iterator<TypeNameComponent>
   */
  Iterator iterateComponents();

  /**
   * \brief Returns all components of type name
   *
   * \return ArrayList<TypeNameComponent>
   */
  ArrayList getComponents();

  /**
   * \brief Return the first components except last one
   *
   * \return a new type name as the first components except last one
   */
  TypeName getFirstComponents();

  /**
   * \brief get last component of TypeName
   *
   * \return last component of TypeName (if any), null if empty
   */
  TypeNameComponent getLastComponent();

  /**
   * \brief return true if one component is generic
   *
   * \return true if one component at least is generic
   */
  boolean isGeneric();

  /**
   * \brief Return the number of dimension of this array
   *
   * \return number of dimension
   */
  int getDim();
}