/*
  $Id: TypeNameComponentImpl.java,v 1.5 2004/03/04 22:16:17 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.parser.node.util.var;

import  net.nosica.parser.node.AbstractNode;
import  net.nosica.framework.Visitor;
import  net.nosica.parser.node.NodeVisitor;
import java.util.ArrayList;
import java.util.Iterator;

public class TypeNameComponentImpl extends AbstractNode implements TypeNameComponent
{
  private String packageElement;
  private ArrayList typeNameGenericExtension;

  private String stringCache = null;

  /**
   * Constructs a TypeNameComponent
   *
   * \param packageElement component of path
   * \param typeNameGenericExtension a ArrayList<TypeName> that may be null
   */
  public TypeNameComponentImpl(String packageElement, ArrayList typeNameGenericExtension)
  {
    this("", packageElement, typeNameGenericExtension);
  }

  /**
   * Constructs a TypeNameComponent
   *
   * \param localization localization of this TypeNameComponent in source code
   * \param packageElement component of path
   * \param typeNameGenericExtension a ArrayList<TypeName> that may be null
   */
  public TypeNameComponentImpl(String localization, String packageElement, ArrayList typeNameGenericExtension)
  {
    super(localization);

    this.packageElement = packageElement;
    this.typeNameGenericExtension = typeNameGenericExtension;
  }

  public String getPackageElement() { return packageElement; }
  public boolean hasGenericExtension() { return typeNameGenericExtension != null; }
  public Iterator iterateGenericExtension() { return typeNameGenericExtension.iterator(); }
  public ArrayList getGenericExtension() { return typeNameGenericExtension; }

  public void accept(Visitor visitor)
  {
    ((NodeVisitor)visitor).visit(this);
  }

  synchronized public String toString()
  {
    if (stringCache == null)
    {
      StringBuffer tmp = new StringBuffer(packageElement);
      if (typeNameGenericExtension != null && typeNameGenericExtension.size() != 0)
      {
        tmp.append('<');
        Iterator it = typeNameGenericExtension.iterator();
        while (it.hasNext())
        {
          TypeName typeName = (TypeName)it.next();
          tmp.append(typeName.toString());
          if (it.hasNext())
            tmp.append(',');
        }
        tmp.append('>');
      }

      stringCache = tmp.toString();
    }

    return stringCache;
  }
}