/*
  $Id: TypeNameImpl.java,v 1.5 2004/03/04 22:16:17 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.parser.node.util.var;

import  net.nosica.parser.node.AbstractNode;
import  net.nosica.framework.Visitor;
import  net.nosica.parser.node.NodeVisitor;
import  net.nosica.compiler.application.Environment;
import java.util.Iterator;
import java.util.ArrayList;

public class TypeNameImpl extends AbstractNode implements TypeName
{
  private ArrayList components = null;
  private int dim = 0;
  private TypeName arrayElementType = null;

  private String stringCache = null;

  /**
   * Create a new scalar TypeName
   *
   * \param components the components of the TypeName
   */
  public TypeNameImpl(ArrayList components)
  {
    this("", components);
  }

  /**
   * Create a new scalar TypeName
   *
   * \param localization a string describing the start of the typeName in the source file
   * \param components the components of the TypeName
   */
  public TypeNameImpl(String localization, ArrayList components)
  {
    super(localization);

    Environment.check(components != null, "components should not be null");
    this.components = components;
  }

  /**
   * Create a new array TypeName
   *
   * \param dim number of dimension of the array
   * \param arrayElementType the type of element of the array
   */
  public TypeNameImpl(int dim, TypeName arrayElementType)
  {
    super(arrayElementType.localization());

    this.dim = dim;
    this.arrayElementType = arrayElementType;
  }

  /**
   * Create an empty typeName
   */
  public TypeNameImpl() { super(""); components = new ArrayList(); }

  public boolean isArray()  { return dim != 0; }
  public boolean isScalar() { return dim == 0; }
  public boolean isEmpty() { return dim == 0 && components.isEmpty(); }
  public TypeName getArrayElementType() { return arrayElementType; }
  public Iterator iterateComponents() { return components.iterator(); }
  public ArrayList getComponents() { return components; }

  public void accept(Visitor visitor)
  {
    ((NodeVisitor)visitor).visit(this);
  }

  public TypeName getFirstComponents()
  {
    if (isArray())
      return arrayElementType.getFirstComponents();
    else
    if (isEmpty())
      return null;
    else
    if (components.size() == 1)
      return new TypeNameImpl(new ArrayList());
    else
    {
      ArrayList tmp = new ArrayList();
      tmp.addAll(components);
      tmp.remove(tmp.size() - 1);

      return new TypeNameImpl(tmp);
    }
  }

  public TypeNameComponent getLastComponent()
  {
    if (isArray())
      return arrayElementType.getLastComponent();
    else
    if (isEmpty())
      return null;
    else
      return (TypeNameComponent)components.get(components.size() - 1);
  }

  public boolean isGeneric()
  {
    if (isArray())
      return arrayElementType.isGeneric();
    else
    {
      Iterator it = components.iterator();
      while (it.hasNext())
      {
        TypeNameComponent component = (TypeNameComponent)it.next();
        if (component.hasGenericExtension())
          return true;
      }

      return false;
    }
  }

  public int getDim()
  {
    return dim;
  }

  public synchronized String toString()
  {
    if (stringCache == null)
    {
      StringBuffer result = new StringBuffer();

      if (isArray())
      {
        TypeName temp = this;

        while (temp.isArray())
        {
          result.append('[');
          for (int i = 1; i < temp.getDim(); ++i)
            result.append(',');
          result.append(']');

          temp = temp.getArrayElementType();
        }

        result.insert(0, temp.toString());
      }
      else
      {
        Iterator it = components.iterator();
        while (it.hasNext())
        {
          TypeNameComponent typeNameComponent = (TypeNameComponent)it.next();
          result.append(typeNameComponent.toString());

          if (it.hasNext())
            result.append('.');
        }
      }

      stringCache = result.toString();
    }

    return stringCache;
  }

  public boolean equals(Object obj)
  {
    if (!(obj instanceof TypeNameImpl))
      return false;

    TypeNameImpl typeName = (TypeNameImpl)obj;

    return toString().equals(typeName.toString());
  }

  public int hashCode() { return toString().hashCode(); }
}