/*
  $Id: Accessible.java,v 1.4 2004/03/04 22:16:17 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.symbol;


/**
 * \brief base interface for all accessible Symbols
 * That is all symbols that have restrictions on access
 */

public interface Accessible
{
  /**
   * \brief is symbol public
   * \return true if symbol is publicly accessible
   */
  boolean isPublic();

  /**
   * \brief is symbol protected
   * \return true if symbol is protected
   */
  boolean isProtected();

  /**
   * \brief is symbol private
   * \return true if symbol is private
   */
  boolean isPrivate();

  /**
   * \brief is symbol package
   * \return true if symbol is accessible only by its package
   */
  boolean isPackage();

  /**
   * \brief is symbol static
   * \return true if symbol is accessible without a live instance
   */
  boolean isStatic();

  /**
   * \brief set the staticness of this symbol
   *
   * \param isStatic true if symbol is static, false otherwise
   */
  void setStatic(boolean isStatic);

  /**
   * \brief set the symbol as public
   */
  void setPublic();
}