/*
  $Id: AccessibleGenericSymbolImpl.java,v 1.6 2004/03/04 22:16:17 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.symbol;

import net.nosica.parser.node.util.var.TypeName;
import net.nosica.parser.node.util.generic.GenericParameter;

import java.util.ArrayList;
import java.util.Iterator;

abstract public class AccessibleGenericSymbolImpl extends AccessibleImpl implements GenericSymbol
{
  private ArrayList genericParameterList;   // ArrayList<GenericParameter>
  private boolean isPartial = false;
  private net.nosica.parser.node.typedeclaration.Symbol node;
  private TypeName enclosing;

  /**
   * \param node node where was define the symbol we are constructing
   * \param isPublic true if symbol is public
   * \param isProtected true if symbol is protected
   * \param isPrivate true if symbol is private
   * \param genericParameterList a ArrayList<GenericParameter>
   */
  public AccessibleGenericSymbolImpl(
    net.nosica.parser.node.typedeclaration.Symbol node,
    boolean isPublic,
    boolean isProtected,
    boolean isPrivate,
    ArrayList genericParameterList,
    TypeName enclosing
  )
  {
    super(isPublic, isProtected, isPrivate);
    this.node = node;
    this.genericParameterList = genericParameterList;
    this.enclosing = enclosing;
  }

  public Iterator iterateGenericParameters()
  {
    return genericParameterList.iterator();
  }

  public ArrayList getGenericParameters()
  {
    return genericParameterList;
  }

  public GenericParameter getGenericParameter(TypeName typeName) throws SymbolException
  {
    Iterator i = genericParameterList.iterator();
    while (i.hasNext())
    {
      GenericParameter genericParm = (GenericParameter)i.next();
      if (genericParm.getName().equals(typeName))
        return genericParm;
    }

    throw new SymbolException("Wasn't able to resolve generic parameter named [" + typeName + "]");
  }

  public boolean isGenericParameter(TypeName typeName)
  {
    Iterator i = genericParameterList.iterator();
    while (i.hasNext())
    {
      GenericParameter genericParm = (GenericParameter)i.next();
      if (genericParm.getName().equals(typeName))
        return true;
    }

    return false;
  }

  public void setGenericParameters(ArrayList genericParameters)
  {
    this.genericParameterList = genericParameters;
  }

  public boolean hasGenericParameter() { return !genericParameterList.isEmpty(); }
  public boolean isPartial() { return isPartial; }
  public void setPartial(boolean isPartial) { this.isPartial = isPartial; }
  public net.nosica.parser.node.typedeclaration.Symbol getNode() { return node; }
  public void setNode(net.nosica.parser.node.typedeclaration.Symbol symbol) { this.node = symbol; }
  public TypeName getEnclosing() { return enclosing; }
  public void setEnclosing(TypeName enclosing) { this.enclosing = enclosing; }
}