/*
  $Id: AccessibleImpl.java,v 1.4 2004/03/04 22:16:17 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.symbol;


public class AccessibleImpl implements Accessible
{
  private boolean isPublic;
  private boolean isProtected;
  private boolean isPrivate;
  private boolean isPackage;
  private boolean isStatic;

  /**
   * Constructs an accessible instance<BR>
   * By default, a symbol is not static<BR>
   * A symbol can be only public, or only protected, or only private, or only package.<BR>
   * It cannot be a mix of these kind of access
   *
   * \param isPublic true if symbol is public
   * \param isProtected true if symbol is protected
   * \param isPrivate true if symbol is private
   */
  public AccessibleImpl(
                         boolean isPublic,
                         boolean isProtected,
                         boolean isPrivate
                        )
  {
    this(isPublic, isProtected, isPrivate, false);
  }

  /**
   * Constructs an accessible instance<BR>
   * A symbol can be only public, or only protected, or only private, or only package.<BR>
   * It cannot be a mix of these kind of access
   *
   * \param isPublic true if symbol is public
   * \param isProtected true if symbol is protected
   * \param isPrivate true if symbol is private
   * \param isStatic true if symbol is static
   */
  public AccessibleImpl(
                         boolean isPublic,
                         boolean isProtected,
                         boolean isPrivate,
                         boolean isStatic
                       )
  {
    this(isPublic, isProtected, isPrivate, !isPublic && !isProtected && !isPrivate, isStatic);
  }

  /**
   * Constructs an accessible instance<BR>
   * A symbol can be only public, or only protected, or only private, or only package.<BR>
   * It cannot be a mix of these kind of access
   *
   * \param isPublic true if symbol is public
   * \param isProtected true if symbol is protected
   * \param isPrivate true if symbol is private
   * \param isPackage true if symbol is package
   * \param isStatic true if symbol is static
   */
  public AccessibleImpl(
                         boolean isPublic,
                         boolean isProtected,
                         boolean isPrivate,
                         boolean isPackage,
                         boolean isStatic
                       )
  {
    this.isPublic     = isPublic;
    this.isProtected  = isProtected;
    this.isPrivate    = isPrivate;
    this.isPackage    = isPackage;
    this.isStatic     = isStatic;
  }

  public boolean isPublic()     { return isPublic; }
  public boolean isProtected()  { return isProtected; }
  public boolean isPrivate()    { return isPrivate; }
  public boolean isPackage()    { return isPackage; }
  public boolean isStatic()     { return isStatic; }
  public void    setStatic(boolean isStatic)       { this.isStatic = isStatic; }
  public void    setPublic()
  {
    isPublic = true;
    isProtected = false;
    isPrivate = false;
    isPackage = false;
  }
}