/*
  $Id: AliasTable.java,v 1.4 2004/03/04 22:16:17 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.symbol;

import net.nosica.parser.node.util.var.TypeName;

import java.util.Iterator;

/**
 * \brief manages aliases for a GenericType
 */

public interface AliasTable
{
  /**
   * \brief add an alias to this AliasTable
   *
   * \param alias alias to a Type or a method
   * \param aliased Type or Method to alias
   */
  void addAlias(TypeName alias, TypeName aliased) throws SymbolException;

  /**
   * \brief add an alias to this AliasTable
   *
   * \param alias alias to a Type or a method
   * \param aliased Type or Method to alias
   * \throws SymbolException if alias does not exists
   */
  void replaceAlias(TypeName alias, TypeName aliased) throws SymbolException;

  /**
   * \brief Return the TypeName associated with this alias
   * If no alias was found but table alias has an enclosing alias table, then request enclosing alias table
   *
   * \param alias the alias to resolve
   * \return the TypeName if it is aliased, null otherwise
   */
  TypeName resolveAlias(TypeName alias);

  /**
   * \brief has this table an enclosing table ?
   *
   * \return true if thsi table has an enclosing table
   */
  boolean hasEnclosing();

  /**
   * \brief Return the enclosing table of this table (if any)
   * requires : hasEnclosing()
   *
   * \return the enclosing table if it exists, null otherwise
   */
  AliasTable getEnclosing();

  /**
   * \brief Returns an iterator on all aliases
   *
   * \return Iterator<Map.Entry<TypeName, TypeName> >
   */
  Iterator iterateAliases();

  /**
   * \brief Check if alias exists
   *
   * \param alias the alias to search for
   * \return true if alias is already referenced
   */
  boolean aliasExists(TypeName alias);
}