/*
  $Id: AliasTableImpl.java,v 1.4 2004/03/04 22:16:17 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.symbol;

import net.nosica.parser.node.util.var.TypeName;

import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;

public class AliasTableImpl implements AliasTable
{
  private AliasTable enclosing;
  private Map aliases = new HashMap();  // Map<TypeName, TypeName>

  /**
   * Create a new empty alias table
   */
  public AliasTableImpl() {}

  /**
   * Create a new empty aliasa table<BR>
   * Table has an enclosing alias table : all aliases of enclosing table are also aliases of this table
   *
   * \param enclosing the enclosing table to use
   */
  public AliasTableImpl(AliasTable enclosing) { this.enclosing = enclosing; }

  public void addAlias(TypeName alias, TypeName aliased) throws SymbolException
  {
    if (aliasExists(alias))
      throw new SymbolException("Alias [" + alias + "] was already added");
    aliases.put(alias, aliased);
  }

  public void replaceAlias(TypeName alias, TypeName aliased) throws SymbolException
  {
   if (!aliasExists(alias))
      throw new SymbolException("Alias [" + alias + "] cannot be replaced : it does not exist");
    aliases.put(alias, aliased);
  }

  public TypeName resolveAlias(TypeName alias)
  {
    if (aliases.containsKey(alias))
      return (TypeName)aliases.get(alias);
    else
    if (enclosing != null)
      return enclosing.resolveAlias(alias);
    else
      return null;
  }

  public boolean aliasExists(TypeName alias)
  {
    return aliases.containsKey(alias) || (enclosing != null && enclosing.aliasExists(alias));
  }

  public boolean hasEnclosing() { return enclosing != null; }
  public AliasTable getEnclosing() { return enclosing; }
  public Iterator iterateAliases() { return aliases.entrySet().iterator(); }
}