/*
  $Id: FeedSymbolVisitorHelper.java,v 1.6 2004/03/04 22:16:17 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.symbol;

import net.nosica.parser.node.util.var.FormalParameters;
import net.nosica.parser.node.util.var.FormalParameter;
import net.nosica.parser.node.util.var.TypeNameList;
import net.nosica.parser.node.util.var.TypeName;

import java.util.ArrayList;
import java.util.Set;
import java.util.HashSet;
import java.util.Iterator;

public class FeedSymbolVisitorHelper
{
  /**
   * \brief Helper function used to convert a FormalParameters data into a ArrayList<Argument>
   * If two Arguments have the same name, then this method will throw<BR>
   * Check also that an argument cannot have an initializer
   *
   * \param formalParameters data to convert
   * \return ArrayList<Argument>
   * \throw SymbolException if 2 arguments have the same name, or if an argument has an initializer
   */
  static public ArrayList formalParameters2Arguments(FormalParameters formalParameters)
  {
    ArrayList result = new ArrayList();

    Iterator it = formalParameters.iterateParameters();

    while (it.hasNext())
    {
      FormalParameter formalParm = (FormalParameter)it.next();

      Argument arg = new ArgumentImpl(formalParm.isConst(), formalParm.getTypeName(), formalParm.getVariable().getId());
      result.add(arg);
    }

    return result;
  }

  /**
   * \brief Helper function to convert a TypeNameList into a Set<TypeName>
   *
   * \param typeNameList a list of TypeName
   * \return a Set<TypeName>
   */
  static public Set typeNameList2ThrowList(TypeNameList typeNameList)
  {
    Set result = new HashSet();

    if (typeNameList == null)
      return result;

    Iterator it = typeNameList.iterateTypeNames();

    while (it.hasNext())
    {
      TypeName typeName = (TypeName)it.next();
      result.add(typeName);
    }

    return result;
  }

  /**
   * \brief Helper function to convert a ResultType from parser to a ResultType from symbol
   *
   * \param resultType node to convert
   * \return symbol ResultType
   */
  static public ResultType resultType2ResultType(net.nosica.parser.node.util.var.ResultType resultType)
  {
    if (resultType.isSub())
      return null;

    return new ResultTypeImpl(resultType.isConst(), resultType.getTypeName());
  }
}