/*
  $Id: FieldImpl.java,v 1.6 2004/03/04 22:16:17 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.symbol;

import net.nosica.parser.node.util.var.TypeName;
import net.nosica.parser.node.util.var.VariableInitializer;

public class FieldImpl extends ArgumentImpl implements Field
{
  private Accessible accessible;
  private boolean isMutable;
  private boolean isTransient;
  private VariableInitializer varInit;
  private net.nosica.parser.node.typedeclaration.Symbol node;
  private TypeName enclosing;

  /**
   * \brief Constructs a new field
   * A field cannot be both const and mutable<br>
   * A field cannot be both const and static<br>
   * A field cannot be both static and transient<br>
   *
   * \param node node that defines Field
   * \param isPublic is field public ?
   * \param isProtected is field protected ?
   * \param isPrivate is field private ?
   * \param isConst is field const ?
   * \param isMutable is field mutable ?
   * \param isTransient is field transient ?
   * \param isStatic is field static ?
   * \param typeName TypeName of field
   * \param name name of field
   * \param varInit initializer of field, null if none
   * \param enclosing enclosing type
   */
  public FieldImpl(
    net.nosica.parser.node.typedeclaration.Symbol node,
    boolean isPublic,
    boolean isProtected,
    boolean isPrivate,
    boolean isConst,
    boolean isMutable,
    boolean isTransient,
    boolean isStatic,
    TypeName typeName,
    String name,
    VariableInitializer varInit,
    TypeName enclosing
  )
  {
    super(isConst, typeName, name);
    this.node = node;
    accessible = new AccessibleImpl(isPublic, isProtected, isPrivate);
    accessible.setStatic(isStatic);
    this.isMutable = isMutable;
    this.isTransient = isTransient;
    this.varInit = varInit;
    this.enclosing = enclosing;
  }

  public boolean isMutable() { return isMutable; }
  public boolean isTransient() { return isTransient; }
  public boolean isInitialized() { return varInit != null; }
  public VariableInitializer getVarInit() { return varInit; }
  public void setStatic(boolean isStatic) { accessible.setStatic(isStatic); }
  public boolean isPublic() { return accessible.isPublic(); }
  public boolean isProtected() { return accessible.isProtected(); }
  public boolean isPrivate() { return accessible.isPrivate(); }
  public boolean isPackage() { return accessible.isPackage(); }
  public boolean isStatic() { return accessible.isStatic(); }
  public void    setPublic() { accessible.setPublic(); }
  public net.nosica.parser.node.typedeclaration.Symbol getNode() { return node; }
  public void setNode(net.nosica.parser.node.typedeclaration.Symbol symbol) { this.node = symbol; }
  public TypeName getEnclosing() { return enclosing; }
  public void setEnclosing(TypeName enclosing) { this.enclosing = enclosing; }
}