/*
  $Id: GenericMethod.java,v 1.7 2004/05/06 19:47:00 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.symbol;

import java.util.Iterator;
import java.util.ArrayList;
import java.util.Set;

/**
 * \brief interface for a generic method
 */

public interface GenericMethod extends GenericSymbol
{
  /**
   * \brief return the Method's node in the abstract syntax tree (AST) that defines this symbol
   *
   * \return the method's node of the AST
   */
  net.nosica.parser.node.typedeclaration.Method getMethod();

  /**
   * \brief is method const ?
   * requires : !isStatic()
   *
   * \return true if method is const
   */
  boolean isConst();

  /**
   * \brief is method final ?
   * requires : !isStatic()
   *
   * \return true id method is final
   */
  boolean isFinal();

  /**
   * \brief is method a Native Code callback ?
   *
   * \return true if method is a callback
   */
  boolean isCallback();

  /**
   * \brief is method implemented in the source code (non abstract?)
   *
   * \return true if the method is implemented in the source code
   */
  boolean isImplemented();

  /**
   * \brief iterate through all parameters of this method
   *
   * \return Iterator<Argument>
   */
  Iterator iterateArguments();

  /**
   * \brief Return arguments
   *
   * \return ArrayList<Argument>
   */
  ArrayList getArguments();

  /**
   * \brief is method sub ?
   *
   * \return true id method is sub
   */
  boolean isSub();

  /**
   * \brief return the resultType of this method
   * requires : !isSub()
   *
   * \return the resultType of this method
   */
  ResultType getResultType();

  /**
   * \brief return the name of this method
   *
   * \return name of the method
   */
  String getName();

  /**
   * \brief return the kind of this method
   *
   * \return kind of the method
   */
  MethodKindEnum getKind();

  /**
   * \brief iterate through list of exception declared to be thrown by method
   *
   * \return Iterator<TypeName>
   */
  Iterator iterateThrowList();

  /**
   * \brief return exceptions thrown by method
   *
   *  \return Set<TypeName>
   */
  Set getThrowList();

  /**
   * \brief set exceptions thrown by method
   *
   *  \param thrownSet Set<TypeName>
   */
  void setThrowList(Set thrownSet);
}