/*
  $Id: GenericMethodImpl.java,v 1.9 2004/05/06 19:47:00 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.symbol;

import net.nosica.parser.node.util.var.TypeName;

import java.util.ArrayList;
import java.util.Set;
import java.util.Iterator;

public class GenericMethodImpl extends AccessibleGenericSymbolImpl implements GenericMethod
{
  private boolean isConst;
  private boolean isFinal;
  private boolean isCallback;
  private boolean isImplemented;
  private ResultType resultType;
  private MethodKindEnum kind;
  private String name;
  private ArrayList arguments;         // ArrayList<Argument>
  private Set throwList;            // Set<TypeName>

  /**
   * Creates a new generic method<BR>
   * A method can't be both const and static
   * A method can't be both static and final
   *
   * \param node node that defines this method
   * \param isPublic is this method public ?
   * \param isProtected is this method protected ?
   * \param isPrivate is this method private ?
   * \param genericParameters a ArrayList<GenericParameter> of the generic parameters introduced by this method
   * \param enclosing enclosing type of this method
   * \param isStatic is this method static ?
   * \param isConst is this method const ?
   * \param isFinal is this method final ?
   * \param isCallback  is this method a callback from native methods ?
   * \param resultType resultType of this method, null otherwise
   * \param kind kind of method, see MethodKindEnum for available method type
   * \param name name of method, can be an alias
   * \param arguments a ArrayList<Artument>
   * \param throwList a Set<TypeName>
   * \param isImplemented is this method implemented ?
   */
  public GenericMethodImpl(
    net.nosica.parser.node.typedeclaration.Method method,
    boolean isPublic,
    boolean isProtected,
    boolean isPrivate,
    ArrayList genericParameters,
    TypeName enclosing,
    boolean isStatic,
    boolean isConst,
    boolean isFinal,
    boolean isCallback,
    ResultType resultType,
    MethodKindEnum kind,
    String name,
    ArrayList arguments,
    Set throwList,
    boolean isImplemented
  )
  {
    super(method, isPublic, isProtected, isPrivate, genericParameters, enclosing);
    setStatic(isStatic);

    this.isConst    = isConst;
    this.isFinal    = isFinal;
    this.isCallback = isCallback;
    this.resultType = resultType;
    this.kind       = kind;
    this.name       = name;
    this.arguments  = arguments;
    this.throwList  = throwList;
    this.isImplemented = isImplemented;
  }

  public net.nosica.parser.node.typedeclaration.Method getMethod() { return (net.nosica.parser.node.typedeclaration.Method)getNode(); }
  public boolean isConst() { return isConst; }
  public boolean isFinal() { return isFinal; }
  public boolean isCallback() { return isCallback; }
  public boolean isImplemented() { return isImplemented; }
  public Iterator iterateArguments() { return arguments.iterator(); }
  public ArrayList getArguments() { return arguments; }
  public boolean isSub() { return resultType == null; }
  public ResultType getResultType() { return resultType; }
  public String getName() { return name; }
  public MethodKindEnum getKind() { return kind; }
  public Iterator iterateThrowList() { return throwList.iterator(); }
  public Set getThrowList() { return throwList; }
  public void setThrowList(Set thrownSet) { this.throwList = thrownSet; }
}