/*
  $Id: GenericSymbol.java,v 1.6 2004/03/04 22:16:17 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.symbol;

import net.nosica.parser.node.util.var.TypeName;
import net.nosica.parser.node.util.generic.GenericParameter;

import java.util.Iterator;
import java.util.ArrayList;

/**
 * \brief base interface for all generic symbols
 * Basically, only types and methods can be generic
 */
public interface GenericSymbol extends Symbol, Accessible
{
  /**
   * \brief Iterate through all generic parameters this method introduces
   *
   * \return Iterator<GenericParameter>
   */
  Iterator iterateGenericParameters();

  /**
   * \brief Returns all generic parameters
   *
   * \return ArrayList<GenericParameter>
   */
  ArrayList getGenericParameters();

  /**
   * \brief returns GenericParameter if it exists
   *
   * \param typeName name of the generic parameter to retrieve
   * \return GenericParameter if it exists
   */
  GenericParameter getGenericParameter(TypeName typeName) throws SymbolException;

  /**
   * \brief look into generic parameter to see if typeName is a generic parameter
   *
   * \return true if it is a generic parameter
   */
  boolean isGenericParameter(TypeName typeName);

  /**
   * \brief set the generic parameters of this symbol
   *
   * \param genericParameters the new generic parameters to use (ArrayList<GenericParameter>)
   */
  void setGenericParameters(ArrayList genericParameters);

  /**
   * \brief does this symbol has a generic symbol ?
   * That is : has this symbol some generic parameter ?
   *
   * \return true if symbol introduces new generic parameters
   */
  boolean hasGenericParameter();

  /**
   * \brief does this symbol is partially generic ?
   *
   * \return true if type is partially generic
   */
  boolean isPartial();

  /**
   * \brief set this symbol as partially generic
   *
   * \param isPartial new value
   */
  void setPartial(boolean isPartial);
}