/*
  $Id: CommandLineAnalysis.java,v 1.1 2004/03/18 23:15:20 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.util;

import java.util.ArrayList;
import java.util.Iterator;

public class CommandLineAnalysis
{
  private ArrayList options = new ArrayList(); // ArrayList<CommandLineOption>
  private ArrayList flags   = new ArrayList(); // ArrayList<CommandLineFlag>

  private String progName;
  private String shortOptionDescription;
  private String additionalComment;
  private String version;

  private static final int NONE = 0;
  private static final int WAITING_VALUE = 1;

  public CommandLineAnalysis(String progName, String shortOptionDescription, String additionalComment, String version)
  {
    this.progName               = progName;
    this.shortOptionDescription = shortOptionDescription;
    this.additionalComment      = additionalComment;
    this.version                = version;
  }

  public void addOption(CommandLineOption option)
  {
    options.add(option);
  }

  public void addFlag(CommandLineFlag flag)
  {
    flags.add(flag);
  }

  public String help()
  {
    StringBuffer sb = new StringBuffer();

    sb.append("Usage: ").append(progName)
      .append(" ").append(shortOptionDescription)
      .append("\n");

    Iterator it = flags.iterator();
    while (it.hasNext())
    {
      CommandLineFlag item = (CommandLineFlag)it.next();
      sb.append(argHelp(item));
    }

    it = options.iterator();
    while (it.hasNext())
    {
      CommandLineOption item = (CommandLineOption)it.next();
      sb.append(argHelp(item));
    }

    return sb.toString();
  }

  public String version()
  {
    return progName + " version " + version;
  }

  private String argHelp(CommandLineFlag item)
  {
    StringBuffer sb = new StringBuffer();
    sb.append("  -").append(item.getShortName())
      .append(" --").append(item.getLongName())
      .append(" : ").append(item.getDescription())
      .append("\n");
    return sb.toString();
  }

  private String argHelp(CommandLineOption item)
  {
    StringBuffer sb = new StringBuffer();
    sb.append("  -").append(item.getShortName())
      .append(" value --").append(item.getLongName())
      .append(" value : ").append(item.getDescription())
      .append("\n");
    return sb.toString();
  }

  public String[] process(String[] args)
  {
    int state = NONE;
    CommandLineOption optionValue = null;
    ArrayList unrecognizedArgs = new ArrayList(); // ArrayList<String>

ARG_LOOP :
    for (int i = 0; i < args.length; ++i)
    {
      String originalArg = args[i];
      String arg = new String(originalArg);

      if (state == WAITING_VALUE)
      {
        optionValue.setValue(arg);
        state = NONE;
        continue ARG_LOOP;
      }

      if (arg.indexOf("--") == 0)
      {
        // found a '--'
        arg = arg.substring(2);

        Iterator it = options.iterator();
        while (it.hasNext())
        {
          CommandLineOption option = (CommandLineOption)it.next();
          if (option.getLongName().equals(arg))
          {
            state = WAITING_VALUE;
            optionValue = option;
            continue ARG_LOOP;
          }
        }

        it = flags.iterator();
        while (it.hasNext())
        {
          CommandLineFlag flag = (CommandLineFlag)it.next();
          if (flag.getLongName().equals(arg))
          {
            flag.set();
            continue ARG_LOOP;
          }
        }
      }
      else
      if (arg.indexOf('-') == 0)
      {
        // found a '-'
        arg = arg.substring(1);

        char shortName;
        if (arg.length() > 0)
        {
          shortName = arg.charAt(0);
          arg = arg.substring(1);
        }
        else
        {
          // unrecognized
          unrecognizedArgs.add(originalArg);
          continue ARG_LOOP;
        }

        Iterator it = options.iterator();
        while (it.hasNext())
        {
          CommandLineOption option = (CommandLineOption)it.next();
          if (option.getShortName() == shortName)
          {
            if (arg.length() > 0)
            {
              optionValue.setValue(arg);
              continue ARG_LOOP;
            }

            state = WAITING_VALUE;
            optionValue = option;
            continue ARG_LOOP;
          }
        }

        it = flags.iterator();
        while (it.hasNext())
        {
          CommandLineFlag flag = (CommandLineFlag)it.next();
          if (flag.getShortName() == shortName)
          {
            if (arg.length() > 0)
            {
              // unrecognized
              unrecognizedArgs.add(originalArg);
              continue ARG_LOOP;
            }

            flag.set();
            continue ARG_LOOP;
          }
        }
      }

      unrecognizedArgs.add(originalArg);
    }

    if (state != NONE)
    {
      unrecognizedArgs.add("--" + optionValue.getLongName());
    }

    return (String[])unrecognizedArgs.toArray(new String[0]);
  }
}