/*
  $Id: LightParser.java,v 1.1 2004/03/18 23:15:20 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.util;

import net.nosica.parser.NosicaParser;
import net.nosica.parser.node.filestructure.CompilationUnit;

import net.nosica.application.ApplicationContext;
import net.nosica.application.ApplicationContextImpl;
import net.nosica.application.FileLoaderImpl;
import net.nosica.application.SymbolEnvironment;
import net.nosica.application.SymbolEnvironmentImpl;

import net.nosica.symbol.FeedSymbolVisitor;
import net.nosica.symbol.GenericType;
import net.nosica.symbol.NullSymbolCallBackImpl;
import net.nosica.symbol.SymbolException;

import java.util.ArrayList;
import java.util.Iterator;

import java.io.File;
import java.io.FileInputStream;

public class LightParser
{
  // the parser
  private static NosicaParser nosicaParser = null;

  private SymbolEnvironment symbolEnv = SymbolEnvironmentImpl.instance();
  private ApplicationContext context  = ApplicationContextImpl.instance();

  public void handleDirectory(File file) throws java.io.IOException
  {
    File[] files = file.listFiles(new NosicaFileFilter(true));

    // recurse into subdirectory first
    for (int i = 0; i < files.length; ++i)
    {
      if (files[i].isDirectory())
      {
        System.out.println("Entering directory " + files[i].getCanonicalPath());
        handleDirectory(files[i]);
        System.out.println("Leaving directory " + files[i].getCanonicalPath());
      }
    }

    // then into files
    for (int i = 0; i < files.length; ++i)
    {
      if (files[i].isFile())
      {
        handleFile(files[i]);
      }
    }
  }

  public void handleFile(File file) throws java.io.FileNotFoundException
  {
    // parse the file !
    if (nosicaParser == null)
    {
      nosicaParser = new NosicaParser(new FileInputStream(file));
      NosicaParser.extractCommentary = true;
    }
    else
    {
      NosicaParser.ReInit(new FileInputStream(file));
    }

    NosicaParser.setCurrentFile(file);

    try
    {
      System.out.println("Parsing file " + file.getName());
      CompilationUnit compilUnit = NosicaParser.CompilationUnit();
      FeedSymbolVisitor visitor = new FeedSymbolVisitor(
        FileLoaderImpl.instance(),
        new NullSymbolCallBackImpl()
      );
      compilUnit.accept(visitor);

      ArrayList types = visitor.getTypes();
      Iterator it = types.iterator();
      while (it.hasNext())
      {
        try
        {
          symbolEnv.addGenericType( (GenericType) it.next());
        }
        catch (SymbolException e)
        {
          System.err.println("Warning : " + e.getMessage());
        }
      }
    }
    catch (net.nosica.parser.ParseException e)
    {
      context.terminate(e.getMessage());
    }
    catch (net.nosica.parser.node.EOFParseException e)
    {
      context.terminate(e.getMessage());
    }
  }
}