/*
  $Id: SettableInteger.java,v 1.5 2004/03/04 22:16:17 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.util;

public class SettableInteger
{
  private int value;

  public SettableInteger(int value)
  {
    this.value = value;
  }

  public SettableInteger(String s)
  {
    value = Integer.valueOf(s).intValue();
  }

  public byte byteValue()
  {
    return new Integer(value).byteValue();
  }

  public int compareTo(SettableInteger anotherInteger)
  {
    if (value > anotherInteger.value)
      return 1;
    else
    if (value < anotherInteger.value)
      return -1;
    else
      return 0;
  }

  public int compareTo(Object o)
  {
    if (o instanceof SettableInteger)
      return compareTo((SettableInteger)o);
    else
      throw new ClassCastException("Object should be a SettableInteger");
  }

  public static SettableInteger decode(String nm)
  {
    return new SettableInteger(Integer.decode(nm).intValue());
  }

  public double doubleValue()
  {
    return value;
  }

  public boolean equals(Object obj)
  {
    if (!(obj instanceof SettableInteger))
      return false;

    SettableInteger setInt = (SettableInteger)obj;

    return value == setInt.value;
  }

  public float floatValue()
  {
    return value;
  }

  public static SettableInteger getInteger(String nm)
  {
    return new SettableInteger(Integer.getInteger(nm).intValue());
  }

  public static SettableInteger getInteger(String nm, int val)
  {
    return new SettableInteger(Integer.getInteger(nm, val).intValue());
  }

  public static SettableInteger getInteger(String nm, Integer val)
  {
    return new SettableInteger(Integer.getInteger(nm, val).intValue());
  }

  public int hashCode()
  {
    return value;
  }

  public int intValue()
  {
    return value;
  }

  public void setValue(int value)
  {
    this.value = value;
  }

  public SettableInteger inc()
  {
    ++value;
    return this;
  }

  public SettableInteger dec()
  {
    --value;
    return this;
  }

  public SettableInteger inc(int inc)
  {
    value += inc;
    return this;
  }

  public SettableInteger dec(int dec)
  {
    value -= dec;
    return this;
  }

  public long longValue()
  {
    return value;
  }

  public static int parseInt(String s)
  {
    return Integer.parseInt(s);
  }

  public static int parseInt(String s, int radix)
  {
    return Integer.parseInt(s, radix);
  }

  public short shortValue()
  {
    return (short)value;
  }

  public static String toBinaryString(int i)
  {
    return Integer.toBinaryString(i);
  }

  public static String toHexString(int i)
  {
    return Integer.toHexString(i);
  }

  public static String toOctalString(int i)
  {
    return Integer.toOctalString(i);
  }

  public String toString()
  {
    return Integer.toString(value);
  }

  public static String toString(int i)
  {
    return Integer.toString(i);
  }

  public static String toString(int i, int radix)
  {
    return Integer.toString(i, radix);
  }

  public static SettableInteger valueOf(String s)
  {
    return new SettableInteger(s);
  }

  public static SettableInteger valueOf(String s, int radix)
  {
    return new SettableInteger(Integer.valueOf(s, radix).intValue());
  }
}
