/*
  $Id: XHTMLHelper.java,v 1.4 2004/03/04 22:16:17 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.nosica.util;

/**
 * \brief an helper class for XHTML writing
 */
public class XHTMLHelper
{
  /**
   * \brief write a xhtml header
   *
   * write xml version.
   * write dtd.
   * write beginning of html tag.
   * write head tag containing a title
   *
   * \param xhtml the writer
   * \param title title of xhtml document
   * \return html tag
   */
  public static XHTMLEntity printHeader(XHTMLWriter xhtml, String title)
  {
    xhtml.printXMLVersion();
    xhtml.printDTD();
    XHTMLEntity html = xhtml.beginTagLn("html", "xmlns=\"http://www.w3.org/1999/xhtml\"");
    xhtml.addIndent();

    XHTMLEntity head = xhtml.beginTag("head");
    xhtml.beginTag("title");
    xhtml.print(title);
    xhtml.endTagLn(head);

    return html;
  }

  /**
   * \brief write a xhtml header
   *
   * write xml version.
   * write dtd.
   * write beginning of html tag.
   * write head tag containing a title and a link to a css file
   *
   * \param xhtml the writer
   * \param title title of xhtml document
   * \param cssFile cssFile to include
   * \return html tag
   */
  public static XHTMLEntity printHeader(XHTMLWriter xhtml, String title, String cssFile)
  {
    xhtml.printXMLVersion();
    xhtml.printDTD();
    XHTMLEntity html = xhtml.beginTagLn("html", "xmlns=\"http://www.w3.org/1999/xhtml\"");
    xhtml.addIndent();

    XHTMLEntity head = xhtml.beginTag("head");
    {
      XHTMLEntity titleTag = xhtml.beginTag("title");
      xhtml.print(title);
      xhtml.endTag(titleTag);
    }
    xhtml.beginTag("link",  "rel=\"stylesheet\" href=\"" + cssFile + "\" type=\"text/css\" charset=\"utf-8\"");
    xhtml.endTagLn(head);

    return html;
  }
}