#!/usr/bin/perl -w

use strict;

# test for command line arguments
if ($#ARGV == -1)
{
  print "errorcoverage.pl message_file\n";
  print "\twhere message_file is the path to nosica message property file\n";
  exit(1);
}

my $messageFile = $ARGV[0];
my @errorCodes = ();

# retrieve error codes in message file
open(MSG_HDL, '<', $messageFile) || die "cannot open $messageFile";
while (<MSG_HDL>)
{
  if (/^#/)
  {
    next;
  }

  if (/^(\w*)\s*;/)
  {
    my $errorCode = $1;
    
    if ($errorCode =~ /Compil/)
    {
      next;
    }

    push @errorCodes, $1;
  }
}
close MSG_HDL;

my $currentDir=`pwd`;
chomp $currentDir;
$currentDir .= "/";

my %retrievedErrorCodes;
retrieveErrorCodesFromReferenceFile($currentDir, 0);

#print "retrieved error codes\n";
#foreach my $key (keys %retrievedErrorCodes)
#{
#  print $key."\n";
#}

my @missingErrorCodes = ();
foreach (@errorCodes)
{
  if (! (exists $retrievedErrorCodes{$_}))
  {
    push @missingErrorCodes, $_;
  }
}

if ($#missingErrorCodes != -1)
{
  print "There is missing error codes : error code coverage is not complete !\n";
  foreach (@missingErrorCodes)
  {
    print $_."\n";
  }
}

sub retrieveErrorCodesFromReferenceFile
{
  my $dir = shift;
  local *DIR_HDL;

  # first, let's find directory and recurse into it
  opendir DIR_HDL, $dir || die "cannot open '$dir'";
  while (my $file = readdir DIR_HDL)
  {
    if ((-d $dir.$file) && ($file ne ".") && ($file ne "..") && ($file ne "CVS"))
    {
      retrieveErrorCodesFromReferenceFile($dir.$file."/");
    }
  }
  closedir(DIR_HDL);

  # now, only find nosica files
  opendir DIR_HDL, $dir || die "cannot open '$dir'";
  while (my $file = readdir DIR_HDL)
  {
    if ($file eq "." || $file eq ".." | $file eq "CVS")
    {
      next;
    }

    if (-f $dir.$file)
    {
      if ($file =~ /\.ref$/)
      {
        my $completeFileName = $dir.$file;
        my $errorCode = `cat $completeFileName`;
        chomp $errorCode;
        $retrievedErrorCodes{$errorCode} = 1;
      }
    }
  }

  closedir(DIR_HDL);
}
