<?

  $output = fopen("book.tex", "wt");
  handleFile("../book.xml");
  fclose($output);

  // --- Handlers

  function emitText($data)
  {
    global $output;
    fwrite($output, $data . "\n");
  }

  global $source;

  function emitSource($data)
  {
    global $source;
    $source = $source . $data . "\n";
  }

  // --- Parsing Framework

  $depth = 0;
  $content = "";

  function noHandler($data)
  {
    global $content;
    $content = $content . $data;
  }

  function startTag($parser, $name, $attrs)
  {
    global $depth;
    global $content;
    global $contentHandler;
    global $title, $subtitle, $version, $author, $authorEmail;
    global $output;

    // handle special situation: include files
    if ($name == "INCLUDE")
    {
      $fname = $attrs["HREF"];
      handleFile("../$fname");
      return;
    }

    //
    $depth++;
    switch($name) {
    case "BODY":
      $contentHandler[$depth] = noHandler;
      fwrite($output,
            "\\documentclass[10pt,a4paper]{manual}\n".
            "\\usepackage{fancyvrb}\n".
            "\\usepackage{color}\n".
            
            "\\title{ $title \\\\\n$subtitle}\n".
            "\\author{ $author - <$authorEmail>}\n".
            "\\release{ $version}\n".

            "\\font\\noscode   = cmtt10 scaled 900\n".
            "\\definecolor{c_comments}{rgb}{0.8, 0.2, 0.2}\n".
            "\\definecolor{c_keywords}{rgb}{  0, 0.8, 0  }\n".
            "\\definecolor{c_symbols} {rgb}{  0,   0, 0.8}\n".

            "\\renewcommand{\\FancyVerbFormatLine}[1]{%\n".
            "  {\\noscode{}\\ \\ \\ \\ \\ \\ #1}}\n".

            "\\DefineVerbatimEnvironment%\n".
            "  {NosVerbatim}{Verbatim}\n".
            "  {numbers=left, commandchars=\\\\}\n".

            "\\begin{document}\n".
            "\\maketitle\n".
            "\\tableofcontents\n".
            "\\input{license}\n".
            "\\vspace{5cm}\n".
            
            "This document was processed using the \\LaTeX{} typesetting ".
            "system and the \\code{manual} document style developed as part ".
            "of the \\code{Python} documentation project. The package ".
            "\\code{fancyvrb} has been used for source code formatting.\n"
      );
      break;
    case "CHAPTER":
      $contentHandler[$depth] = emitText;
      fwrite($output, "\\chapter{".$attrs["NAME"]."}\n");
      break;
    case "SECTION":
      $contentHandler[$depth] = emitText;
      fwrite($output, "\\section{".$attrs["NAME"]."}\n");
      break;
    case "SOURCE":
      $contentHandler[$depth] = emitSource;
      $source = "";
      break;
    default:
      $content = "";
      $contentHandler[$depth] = noHandler;
    }
  }

  function endTag($parser, $name)
  {
    global $depth;
    global $content;
    global $title, $subtitle, $version, $author, $authorEmail;
    global $output, $source;

    // handle special situation: include files
    if ($name == "INCLUDE")
    {
      return;
    }

    //
    switch($name) {
    case "TITLE":        $title       = $content; break;
    case "SUBTITLE":     $subtitle    = $content; break;
    case "VERSION":      $version     = $content; break;
    case "AUTHOR":       $author      = $content; break;
    case "AUTHOR-EMAIL": $authorEmail = $content; break;
    case "SOURCE":
      $t = fopen("/tmp/xml2tex_nos.tmp", "wt");
      fwrite($t, $source);
      fclose($t);
      
      system("perl ../../../tools/syntax_highlight/latex/highlight.pl ../../.. /tmp/xml2tex_nos.tmp /tmp/xml2tex_tex.tmp");

      $source = join('', file("/tmp/xml2tex_tex.tmp"));
      
      fwrite($output, $source);
      break;
    case "BODY":
      fwrite($output, "\\end{document}\n");
      break;
    }

    //
    $depth--;
  }

  function content($parser, $data)
  {
    global $depth;
    global $contentHandler;
    $trimmed = trim($data);
    if (!empty($trimmed)) $contentHandler[$depth]($data);
  }

  function handleFile($fname)
  {
    $parser = xml_parser_create();
    xml_set_element_handler($parser, "startTag", "endTag");
    xml_set_character_data_handler($parser, "content");

    $f = fopen($fname, "rt");
    if (!$f) error("Cannot open XML file [$fname]");
    while($data = fread($f, 4096))
    {
      xml_parse($parser, $data, feof($f));
    }
    fclose($f);

    xml_parser_free($parser);
  }

  function error($message)
  {
    echo "ERROR: $message";
    exit();
  }

?>
