/*
  $Id: narrow.c,v 1.1 2004/04/18 00:52:04 djobet Exp $
  Copyright 2001-2002 David Jobet (david.jobet@free.fr) and Eric Nicolas (enicolas@dvdfr.com)
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "types.h"

// From INT8

word8 narrow_int8_to_word8(int8 value) {
  ASSERT(value >= 0, "int8 value of out bound when narrowing to a word8");
  return (word8)value;
}
word16 narrow_int8_to_word16(int8 value) {
  ASSERT(value >= 0, "int8 value of out bound when narrowing to a word16");
  return (word16)value;
}
word32 narrow_int8_to_word32(int8 value) {
  ASSERT(value >= 0, "int8 value of out bound when narrowing to a word32");
  return (word32)value;
}
word64 narrow_int8_to_word64(int8 value) {
  ASSERT(value >= 0, "int8 value of out bound when narrowing to a word64");
  return (word64)value;
}

// From INT16

int8 narrow_int16_to_int8(int16 value) {
  ASSERT(value >= int8_MIN, "int16 value of out bound when narrowing to a int8");
  ASSERT(value <= int8_MAX, "int16 value of out bound when narrowing to a int8");
  return (int8)value;
}
word8 narrow_int16_to_word8(int16 value) {
  ASSERT(value >= 0, "int16 value of out bound when narrowing to a word8");
  ASSERT(value <= word8_MAX, "int16 value of out bound when narrowing to a word8");
  return (word8)value;
}
word16 narrow_int16_to_word16(int16 value) {
  ASSERT(value >= 0, "int16 value of out bound when narrowing to a word16");
  return (word16)value;
}
word32 narrow_int16_to_word32(int16 value) {
  ASSERT(value >= 0, "int16 value of out bound when narrowing to a word32");
  return (word32)value;
}
word64 narrow_int16_to_word64(int16 value) {
  ASSERT(value >= 0, "int16 value of out bound when narrowing to a word64");
  return (word64)value;
}

// From INT32

int8 narrow_int32_to_int8(int32 value) {
  ASSERT(value >= int8_MIN, "int32 value of out bound when narrowing to a int8");
  ASSERT(value <= int8_MAX, "int32 value of out bound when narrowing to a int8");
  return (int8)value;
}
int16 narrow_int32_to_int16(int32 value) {
  ASSERT(value >= int16_MIN, "int32 value of out bound when narrowing to a int16");
  ASSERT(value <= int16_MAX, "int32 value of out bound when narrowing to a int16");
  return (int16)value;
}
word8 narrow_int32_to_word8(int32 value) {
  ASSERT(value >= 0, "int32 value of out bound when narrowing to a word8");
  ASSERT(value <= word8_MAX, "int32 value of out bound when narrowing to a word8");
  return (word8)value;
}
word16 narrow_int32_to_word16(int32 value) {
  ASSERT(value >= 0, "int32 value of out bound when narrowing to a word16");
  ASSERT(value <= word16_MAX, "int32 value of out bound when narrowing to a word16");
  return (word16)value;
}
word32 narrow_int32_to_word32(int32 value) {
  ASSERT(value >= 0, "int32 value of out bound when narrowing to a word32");
  return (word32)value;
}
word64 narrow_int32_to_word64(int32 value) {
  ASSERT(value >= 0, "int32 value of out bound when narrowing to a word64");
  return (word64)value;
}
float32 narrow_int32_to_float32(int32 value) {
  ASSERT(value >= float32_MIN, "int32 value of out bound when narrowing to a float32");
  ASSERT(value <= float32_MAX, "int32 value of out bound when narrowing to a float32");
  return (float32)value;
}

// From INT64

int8 narrow_int64_to_int8(int64 value) {
  ASSERT(value >= int8_MIN, "int64 value of out bound when narrowing to a int8");
  ASSERT(value <= int8_MAX, "int64 value of out bound when narrowing to a int8");
  return (int8)value;
}
int16 narrow_int64_to_int16(int64 value) {
  ASSERT(value >= int16_MIN, "int64 value of out bound when narrowing to a int16");
  ASSERT(value <= int16_MAX, "int64 value of out bound when narrowing to a int16");
  return (int16)value;
}
int32 narrow_int64_to_int32(int64 value) {
  ASSERT(value >= int32_MIN, "int64 value of out bound when narrowing to a int32");
  ASSERT(value <= int32_MAX, "int64 value of out bound when narrowing to a int32");
  return (int32)value;
}
word8 narrow_int64_to_word8(int64 value) {
  ASSERT(value >= 0, "int64 value of out bound when narrowing to a word8");
  ASSERT(value <= word8_MAX, "int64 value of out bound when narrowing to a word8");
  return (word8)value;
}
word16 narrow_int64_to_word16(int64 value) {
  ASSERT(value >= 0, "int64 value of out bound when narrowing to a word16");
  ASSERT(value <= word16_MAX, "int64 value of out bound when narrowing to a word16");
  return (word16)value;
}
word32 narrow_int64_to_word32(int64 value) {
  ASSERT(value >= 0, "int64 value of out bound when narrowing to a word32");
  ASSERT(value <= word32_MAX, "int64 value of out bound when narrowing to a word32");
  return (word32)value;
}
word64 narrow_int64_to_word64(int64 value) {
  ASSERT(value >= 0, "int64 value of out bound when narrowing to a word64");
  return (word64)value;
}
float32 narrow_int64_to_float32(int64 value) {
  ASSERT(value >= float32_MIN, "int64 value of out bound when narrowing to a float32");
  ASSERT(value <= float32_MAX, "int64 value of out bound when narrowing to a float32");
  return (float32)value;
}
float64 narrow_int64_to_float64(int64 value) {
  ASSERT(value >= float64_MIN, "int64 value of out bound when narrowing to a float64");
  ASSERT(value <= float64_MAX, "int64 value of out bound when narrowing to a float64");
  return (float64)value;
}

// From WORD8

int8 narrow_word8_to_int8(word8 value) {
  ASSERT(value <= int8_MAX, "word8 value of out bound when narrowing to a int8");
  return (int8)value;
}

// From WORD16

int8 narrow_word16_to_int8(word16 value) {
  ASSERT(value <= int8_MAX, "word16 value of out bound when narrowing to a int8");
  return (int8)value;
}
int16 narrow_word16_to_int16(word16 value) {
  ASSERT(value <= int16_MAX, "word16 value of out bound when narrowing to a int16");
  return (int16)value;
}
word8 narrow_word16_to_word8(word16 value) {
  ASSERT(value <= word8_MAX, "word16 value of out bound when narrowing to a word8");
  return (word8)value;
}

// From WORD32

int8 narrow_word32_to_int8(word32 value) {
  ASSERT(value <= int8_MAX, "word32 value of out bound when narrowing to a int8");
  return (int8)value;
}
int16 narrow_word32_to_int16(word32 value) {
  ASSERT(value <= int16_MAX, "word32 value of out bound when narrowing to a int16");
  return (int16)value;
}
int32 narrow_word32_to_int32(word32 value) {
  ASSERT(value <= int32_MAX, "word32 value of out bound when narrowing to a int32");
  return (int32)value;
}
word8 narrow_word32_to_word8(word32 value) {
  ASSERT(value <= word8_MAX, "word32 value of out bound when narrowing to a word8");
  return (word8)value;
}
word16 narrow_word32_to_word16(word32 value) {
  ASSERT(value <= word16_MAX, "word32 value of out bound when narrowing to a word16");
  return (word16)value;
}
float32 narrow_word32_to_float32(word32 value) {
  ASSERT(value <= float32_MAX, "word32 value of out bound when narrowing to a float32");
  return (float32)value;
}

// From WORD64

int8 narrow_word64_to_int8(word64 value) {
  ASSERT(value <= int8_MAX, "word64 value of out bound when narrowing to a int8");
  return (int8)value;
}
int16 narrow_word64_to_int16(word64 value) {
  ASSERT(value <= int16_MAX, "word64 value of out bound when narrowing to a int16");
  return (int16)value;
}
int32 narrow_word64_to_int32(word64 value) {
  ASSERT(value <= int32_MAX, "word64 value of out bound when narrowing to a int32");
  return (int32)value;
}
int64 narrow_word64_to_int64(word64 value) {
  ASSERT(value <= int64_MAX, "word64 value of out bound when narrowing to a int64");
  return (int64)value;
}
word8 narrow_word64_to_word8(word64 value) {
  ASSERT(value <= word8_MAX, "word64 value of out bound when narrowing to a word8");
  return (word8)value;
}
word16 narrow_word64_to_word16(word64 value) {
  ASSERT(value <= word16_MAX, "word64 value of out bound when narrowing to a word16");
  return (word16)value;
}
word32 narrow_word64_to_word32(word64 value) {
  ASSERT(value <= word32_MAX, "word64 value of out bound when narrowing to a word32");
  return (word32)value;
}
float32 narrow_word64_to_float32(word64 value) {
  ASSERT(value <= float32_MAX, "word64 value of out bound when narrowing to a float32");
  return (float32)value;
}
float64 narrow_word64_to_float64(word64 value) {
  ASSERT(value <= float64_MAX, "word64 value of out bound when narrowing to a float64");
  return (float64)value;
}

// From FLOAT32

int8 narrow_float32_to_int8(float32 value) {
  ASSERT(value >= int8_MIN, "float32 value of out bound when narrowing to a int8");
  ASSERT(value <= int8_MAX, "float32 value of out bound when narrowing to a int8");
  ASSERT(fabs(value - floor(value)) >= float32_EPSILON, "float32 value not an natural number when narrowing to a int8");
  return (int8)value;
}
int16 narrow_float32_to_int16(float32 value) {
  ASSERT(value >= int16_MIN, "float32 value of out bound when narrowing to a int16");
  ASSERT(value <= int16_MAX, "float32 value of out bound when narrowing to a int16");
  ASSERT(fabs(value - floor(value)) >= float32_EPSILON, "float32 value not an natural number when narrowing to a int16");
  return (int16)value;
}
int32 narrow_float32_to_int32(float32 value) {
  ASSERT(value >= int32_MIN, "float32 value of out bound when narrowing to a int32");
  ASSERT(value <= int32_MAX, "float32 value of out bound when narrowing to a int32");
  ASSERT(fabs(value - floor(value)) >= float32_EPSILON, "float32 value not an natural number when narrowing to a int32");
  return (int32)value;
}
int64 narrow_float32_to_int64(float32 value) {
  ASSERT(value >= int64_MIN, "float32 value of out bound when narrowing to a int64");
  ASSERT(value <= int64_MAX, "float32 value of out bound when narrowing to a int64");
  ASSERT(fabs(value - floor(value)) >= float32_EPSILON, "float32 value not an natural number when narrowing to a int64");
  return (int64)value;
}
word8 narrow_float32_to_word8(float32 value) {
  ASSERT(value >= word8_MIN, "float32 value of out bound when narrowing to a word8");
  ASSERT(value <= word8_MAX, "float32 value of out bound when narrowing to a word8");
  ASSERT(fabs(value - floor(value)) >= float32_EPSILON, "float32 value not an natural number when narrowing to a word8");
  return (word8)value;
}
word16 narrow_float32_to_word16(float32 value) {
  ASSERT(value >= word16_MIN, "float32 value of out bound when narrowing to a word16");
  ASSERT(value <= word16_MAX, "float32 value of out bound when narrowing to a word16");
  ASSERT(fabs(value - floor(value)) >= float32_EPSILON, "float32 value not an natural number when narrowing to a word16");
  return (word16)value;
}
word32 narrow_float32_to_word32(float32 value) {
  ASSERT(value >= word32_MIN, "float32 value of out bound when narrowing to a word32");
  ASSERT(value <= word32_MAX, "float32 value of out bound when narrowing to a word32");
  ASSERT(fabs(value - floor(value)) >= float32_EPSILON, "float32 value not an natural number when narrowing to a word32");
  return (word32)value;
}
word64 narrow_float32_to_word64(float32 value) {
  ASSERT(value >= word64_MIN, "float32 value of out bound when narrowing to a word64");
  ASSERT(value <= word64_MAX, "float32 value of out bound when narrowing to a word64");
  ASSERT(fabs(value - floor(value)) >= float32_EPSILON, "float32 value not an natural number when narrowing to a word64");
  return (word64)value;
}

// From FLOAT64

int8 narrow_float64_to_int8(float64 value) {
  ASSERT(value >= int8_MIN, "float64 value of out bound when narrowing to a int8");
  ASSERT(value <= int8_MAX, "float64 value of out bound when narrowing to a int8");
  ASSERT(fabs(value - floor(value)) >= float64_EPSILON, "float64 value not an natural number when narrowing to a int8");
  return (int8)value;
}
int16 narrow_float64_to_int16(float64 value) {
  ASSERT(value >= int16_MIN, "float64 value of out bound when narrowing to a int16");
  ASSERT(value <= int16_MAX, "float64 value of out bound when narrowing to a int16");
  ASSERT(fabs(value - floor(value)) >= float64_EPSILON, "float64 value not an natural number when narrowing to a int16");
  return (int16)value;
}
int32 narrow_float64_to_int32(float64 value) {
  ASSERT(value >= int32_MIN, "float64 value of out bound when narrowing to a int32");
  ASSERT(value <= int32_MAX, "float64 value of out bound when narrowing to a int32");
  ASSERT(fabs(value - floor(value)) >= float64_EPSILON, "float64 value not an natural number when narrowing to a int32");
  return (int32)value;
}
int64 narrow_float64_to_int64(float64 value) {
  ASSERT(value >= int64_MIN, "float64 value of out bound when narrowing to a int64");
  ASSERT(value <= int64_MAX, "float64 value of out bound when narrowing to a int64");
  ASSERT(fabs(value - floor(value)) >= float64_EPSILON, "float64 value not an natural number when narrowing to a int64");
  return (int64)value;
}
word8 narrow_float64_to_word8(float64 value) {
  ASSERT(value >= word8_MIN, "float64 value of out bound when narrowing to a word8");
  ASSERT(value <= word8_MAX, "float64 value of out bound when narrowing to a word8");
  ASSERT(fabs(value - floor(value)) >= float64_EPSILON, "float64 value not an natural number when narrowing to a word8");
  return (word8)value;
}
word16 narrow_float64_to_word16(float64 value) {
  ASSERT(value >= word16_MIN, "float64 value of out bound when narrowing to a word16");
  ASSERT(value <= word16_MAX, "float64 value of out bound when narrowing to a word16");
  ASSERT(fabs(value - floor(value)) >= float64_EPSILON, "float64 value not an natural number when narrowing to a word16");
  return (word16)value;
}
word32 narrow_float64_to_word32(float64 value) {
  ASSERT(value >= word32_MIN, "float64 value of out bound when narrowing to a word32");
  ASSERT(value <= word32_MAX, "float64 value of out bound when narrowing to a word32");
  ASSERT(fabs(value - floor(value)) >= float64_EPSILON, "float64 value not an natural number when narrowing to a word32");
  return (word32)value;
}
word64 narrow_float64_to_word64(float64 value) {
  ASSERT(value >= word64_MIN, "float64 value of out bound when narrowing to a word64");
  ASSERT(value <= word64_MAX, "float64 value of out bound when narrowing to a word64");
  ASSERT(fabs(value - floor(value)) >= float64_EPSILON, "float64 value not an natural number when narrowing to a word64");
  return (word64)value;
}
float32 narrow_float64_to_float32(float64 value) {
  ASSERT(value >= float32_MIN, "float64 value of out bound when narrowing to a float32");
  ASSERT(value <= float32_MAX, "float64 value of out bound when narrowing to a float32");
  return (float32)value;
}
