/*
  $Id: num_limits.h,v 1.5 2004/03/04 22:31:51 djobet Exp $
  Copyright 2001-2002 David Jobet (david.jobet@free.fr) and Eric Nicolas (enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language.

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// Nosica standard library / native implementation ----------------------------

#ifndef _NUM_LIMITS_
#define _NUM_LIMITS_

// --- word8

#define word8_IS_SIGNED          false
#define word8_IS_EXACT           true
#define word8_IS_INTEGER         true
#define word8_RADIX              2
#define word8_DIGITS             8
#define word8_DIGITS10           3
#define word8_MIN                ___U8(0)
#define word8_MAX                ___U8(255)

// --- word16

#define word16_IS_SIGNED         false
#define word16_IS_EXACT          true
#define word16_IS_INTEGER        true
#define word16_RADIX             2
#define word16_DIGITS            16
#define word16_DIGITS10          5
#define word16_MIN               ___U16(0)
#define word16_MAX               ___U16(65535)

// --- word32

#define word32_IS_SIGNED         false
#define word32_IS_EXACT          true
#define word32_IS_INTEGER        true
#define word32_RADIX             2
#define word32_DIGITS            32
#define word32_DIGITS10          10
#define word32_MIN               ___U32(0)
#define word32_MAX               ___U32(4294967295)

// --- word64

#define word64_IS_SIGNED         false
#define word64_IS_EXACT          true
#define word64_IS_INTEGER        true
#define word64_RADIX             2
#define word64_DIGITS            64
#define word64_DIGITS10          20
#define word64_MIN               ___U64(0)
#define word64_MAX               ___U64(18446744073709551615)

// --- int8

#define int8_IS_SIGNED           true
#define int8_IS_EXACT            true
#define int8_IS_INTEGER          true
#define int8_RADIX               2
#define int8_DIGITS              8
#define int8_DIGITS10            3
#define int8_MIN                 ___I8(-127)-1
#define int8_MAX                 ___I8( 127)

// --- int16

#define int16_IS_SIGNED          true
#define int16_IS_EXACT           true
#define int16_IS_INTEGER         true
#define int16_RADIX              2
#define int16_DIGITS             16
#define int16_DIGITS10           5
#define int16_MIN                ___I16(-32767)-1
#define int16_MAX                ___I16( 32767)

// --- int32

#define int32_IS_SIGNED          true
#define int32_IS_EXACT           true
#define int32_IS_INTEGER         true
#define int32_RADIX              2
#define int32_DIGITS             32
#define int32_DIGITS10           10
#define int32_MIN                ___I32(-2147483647)-1
#define int32_MAX                ___I32( 2147483647)

// --- int64

#define int64_IS_SIGNED          true
#define int64_IS_EXACT           true
#define int64_IS_INTEGER         true
#define int64_RADIX              2
#define int64_DIGITS             64
#define int64_DIGITS10           20
#define int64_MIN                ___I64(-9223372036854775807)-1
#define int64_MAX                ___I64( 9223372036854775807)

// --- float32

#define float32_IS_SIGNED        true
#define float32_IS_EXACT         false
#define float32_IS_INTEGER       false
#define float32_RADIX            2
#define float32_DIGITS           32
#define float32_DIGITS10         6
#define float32_MIN              1.17549435e-38F
#define float32_MAX              3.40282347e+38F
#define float32_INFINITY         0.0F /*@TODO@*/
#define float32_EPSILON          1.19209290e-07F
#define float32_QUIETNAN         0.0F /*@TODO@*/
#define float32_NAN              0.0F /*@TODO@*/
#define float32_MAXEXPONENT      128
#define float32_MAXEXPONENT10    38
#define float32_MINEXPONENT      -125
#define float32_MINEXPONENT10    -37

// --- float64

#define float64_IS_SIGNED        true
#define float64_IS_EXACT         false
#define float64_IS_INTEGER       false
#define float64_RADIX            2
#define float64_DIGITS           64
#define float64_DIGITS10         15
#define float64_MIN              2.2250738585072014e-308
#define float64_MAX              1.7976931348623157e+308
#define float64_INFINITY         0.0F /*@TODO@*/
#define float64_EPSILON          2.2204460492503131e-16
#define float64_QUIETNAN         0.0F /*@TODO@*/
#define float64_NAN              0.0F /*@TODO@*/
#define float64_MAXEXPONENT      1024
#define float64_MAXEXPONENT10    308
#define float64_MINEXPONENT      -1021
#define float64_MINEXPONENT10    -307

#endif // _NUM_LIMITS_
