 /*
  $Id: types.h,v 1.9 2004/05/01 16:04:10 djobet Exp $
  Copyright 2001-2002 David Jobet(david.jobet@free.fr) and Eric Nicolas(enicolas@dvdfr.com).
  Copyright 2003-2004 David Jobet(david.jobet@free.fr).
  This file is part of Nosica, a compiler for the Nosica language

  Nosica is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Nosica is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Nosica; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// Nosica standard library / native implementation ----------------------------

#ifndef _TYPES_
#define _TYPES_

#include "types0.h"
//#define PATHDEBUG
#define MEMORYDEBUG

// forward declaration of Object
struct ___net_nosica_lang_Object;

#define net_nosica_lang_bool    bool
#define net_nosica_lang_int8    int8
#define net_nosica_lang_int16   int16
#define net_nosica_lang_int32   int32
#define net_nosica_lang_int64   int64
#define net_nosica_lang_word8   word8
#define net_nosica_lang_word16  word16
#define net_nosica_lang_word32  word32
#define net_nosica_lang_word64  word64
#define net_nosica_lang_float32 float32
#define net_nosica_lang_float64 float64

// --- Pointer to destructor
typedef void (*pdtor)(struct ___net_nosica_lang_Object* this);

// --- Runtime type information
typedef struct
{
  word32  ___typeCode;
  pdtor   ___dtor;
  char *  ___name;
  word32  ___size;
} ___RTTI;

// forward declaration of RTTI information
extern ___RTTI rtti[];

// forward declaration of inherit data information
extern word32 *___inheritData[];
extern word32 ___inheritDataEntries;

// --- Tiny necessary helpers

#include "Object.h"

void* mallocAndInit(word32 size, int typeCode, char* file, int line);

void* memMalloc(word32 size, char *file, int line);
void  memFree  (void* block);
void  memMallocInit();
void  memMallocDone();

// an helper function that returns wether child inherits from parent by looking into ___inheritData
int ___inherits(word32 childTid, word32 fatherTid);

// an helper function that returns a typecode from a typename
___RTTI *getRTTIByTypeCode(word32 typeCode);
___RTTI *getRTTIByName(const char *name);

// extern TLS pointer to exception variable
extern net_nosica_lang_Object *___e;

#define FORWARD_NOSICA_DECL(typename) typedef net_nosica_lang_Object * (typename);

#ifdef PATHDEBUG
#  define ENTER(name) fprintf(stdout, "<ENTER:%s\n", name);
#  define LEAVE(name) fprintf(stdout, " LEAVE:%s>\n", name);
#else
#  define ENTER(name)
#  define LEAVE(name)
#endif

#ifdef MEMORYDEBUG
#  define Malloc(s,f,l)   memMalloc(s, f, l)
#  define Free(a)         memFree(a)
#else
#  define Malloc(s, f, l) malloc(s)
#  define Free(a)         free(a)
#  define memMallocInit()
#  define memMallocDone()
#endif

#define ABORT(message)                                                     \
  {                                                                        \
    fprintf(stderr, "%s\n", message);                                      \
    abort();                                                               \
  }

#define ASSERT(condition, message)                                         \
  if (!(condition))  {                                                     \
    fprintf(stderr, "Assertion failed %s:\n%s\n", #condition, message);    \
    abort();                                                               \
  }

#define INCREF(object)                                                     \
  if (object != null) {                                                    \
    ++((net_nosica_lang_Object*)object)->___nbRef;                         \
  }

#define DECREF(object)                                                     \
  if (object != null) {                                                    \
    --((net_nosica_lang_Object*)object)->___nbRef;                         \
    if ((( net_nosica_lang_Object*)object)->___nbRef == 0) {               \
      ((net_nosica_lang_Object*)object)->___rtti->___dtor((net_nosica_lang_Object*)object); \
      Free(object);                                                        \
    }                                                                      \
  }

#define NBREF(object) (((net_nosica_lang_Object*)object)->___nbRef)

// --- Inclusion of basic data types (net.nosica.lang)

#include "num_limits.h"
#include "narrows.h"

#include "net/nosica/lang/bool.h"
#include "net/nosica/lang/int8.h"
#include "net/nosica/lang/int16.h"
#include "net/nosica/lang/int32.h"
#include "net/nosica/lang/int64.h"
#include "net/nosica/lang/word8.h"
#include "net/nosica/lang/word16.h"
#include "net/nosica/lang/word32.h"
#include "net/nosica/lang/word64.h"
#include "net/nosica/lang/float32.h"
#include "net/nosica/lang/float64.h"
#include "net/nosica/lang/char.h"
#include "net/nosica/lang/char8.h"

#define ARRAY_1D_GET(elt, array, dim0) (array->data[dim0])
#define ARRAY_2D_GET(elt, array, dim0, dim1) (array->data[dim0 * array->sizeOfDim[0] / sizeOf(elt) + dim1])

// --- Generic array type for access from native code

typedef struct {
  net_nosica_lang_Object super;
  void* data;
  net_nosica_lang_word32 size;
  net_nosica_lang_word32 nbDim;		// total number of bytes in data
  net_nosica_lang_word32 sizeOfDim[0];	// total number of bytes in each dimension
} ___Array;

#define ARRAY_SIZE(a)         ((___Array*)a)->size
#define ARRAY_NBDIM(a)        ((___Array*)a)->nbDim
#define ARRAY_DATA(a, T)      ((T*)(((___Array*)a)->data))
#define ARRAY_SIZEOFDIM(a, n) ((___Array*)a)->sizeOfDim[n]

// --- C String manipulation for native code

char* makeCString(char* src, word32 size, char* file, int line);

#define CSTRINGctor(dst, src)  \
  dst=makeCString(ARRAY_DATA(src, char), ARRAY_SIZE(src), __FILE__, __LINE__);

#define CSTRINGdtor(str) \
  Free(str);

#endif // _TYPES_
