#!/usr/bin/perl -w

use strict;

# generate most primitive types in net.nosica.lang
# usage:
#   in nosica/
#   perl tools/make_primitive.pl

my    $nos_basepath = "stdlib/net/nosica/lang";
my  $c_int_basepath = "nativestdlib/c/include/net/nosica/lang";
my $c_impl_basepath = "nativestdlib/c/generic/net/nosica/lang";

my $message     = "";
open(MSG, "statement.txt") || die "Cannot open statement.txt";
while(<MSG>)
{
  $message = $message . $_;
}
close(MSG);

my $floatingpoint = 0;

## --- int8

my $type      = "int8";
my $mtype     = "int16";
my $interface = "Numeric";
my $minidoc   = "signed 8-bit integer";
my $signed    = 1;
my @casts     = ( 0, 1, 1, 1, 0, 0, 0, 0, 1, 1 );
my @narrows   = ( 0, 0, 0, 0, 1, 1, 1, 1, 0, 0 );
output();

## --- int16

$type      = "int16";
$mtype     = "int32";
$interface = "Numeric";
$minidoc   = "signed 16-bit integer";
$signed    = 1;
@casts     = ( 0, 0, 1, 1, 0, 0, 0, 0, 1, 1 );
@narrows   = ( 1, 0, 0, 0, 1, 1, 1, 1, 0, 0 );
output();

## --- int32

$type      = "int32";
$mtype     = "int32";
$interface = "Numeric";
$minidoc   = "signed 32-bit integer";
$signed    = 1;
@casts     = ( 0, 0, 0, 1, 0, 0, 0, 0, 0, 1 );
@narrows   = ( 1, 1, 0, 0, 1, 1, 1, 1, 1, 0 );
output();

## --- int64

$type      = "int64";
$mtype     = "int64";
$interface = "Numeric";
$minidoc   = "signed 64-bit integer";
$signed    = 1;
@casts     = ( 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 );
@narrows   = ( 1, 1, 1, 0, 1, 1, 1, 1, 1, 1 );
output();

## --- word8

$type      = "word8";
$mtype     = "word16";
$interface = "Numeric";
$minidoc   = "unsigned 8-bit integer";
$signed    = 0;
@casts     = ( 0, 1, 1, 1, 0, 1, 1, 1, 1, 1 );
@narrows   = ( 1, 0, 0, 0, 0, 0, 0, 0, 0, 0 );
output();

## --- word16

$type      = "word16";
$mtype     = "word32";
$interface = "Numeric";
$minidoc   = "unsigned 16-bit integer";
$signed    = 0;
@casts     = ( 0, 0, 1, 1, 0, 0, 1, 1, 1, 1 );
@narrows   = ( 1, 1, 0, 0, 1, 0, 0, 0, 0, 0 );
output();

## --- word32

$type      = "word32";
$mtype     = "word32";
$interface = "Numeric";
$minidoc   = "unsigned 32-bit integer";
$signed    = 0;
@casts     = ( 0, 0, 0, 1, 0, 0, 0, 1, 0, 1 );
@narrows   = ( 1, 1, 1, 0, 1, 1, 0, 0, 1, 0 );
output();

## --- word64

$type      = "word64";
$mtype     = "word64";
$interface = "Numeric";
$minidoc   = "unsigned 64-bit integer";
$signed    = 0;
@casts     = ( 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 );
@narrows   = ( 1, 1, 1, 1, 1, 1, 1, 0, 1, 1 );
output();

$floatingpoint = 1;

## --- float32

$type      = "float32";
$mtype     = "float32";
$interface = "FloatingPoint";
$minidoc   = "32generic-bit floating point number.";
$signed    = 1;
@casts     = ( 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 );
@narrows   = ( 1, 1, 1, 1, 1, 1, 1, 1, 0, 0 );
output();

## --- float32

$type      = "float64";
$mtype     = "float64";
$interface = "FloatingPoint";
$minidoc   = "64-bit floating point number.";
$signed    = 1;
@casts     = ( 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 );
@narrows   = ( 1, 1, 1, 1, 1, 1, 1, 1, 1, 0 );
output();


sub output {
  my $typeR  = $type;
  my $mtypeR = $mtype;
  while(length($typeR)  < 7) { $typeR  = $typeR  . " "; }
  while(length($mtypeR) < 7) { $mtypeR = $mtypeR . " "; }

  # CREATE THE NOSICA SOURCE FILE

  my $filename="$nos_basepath/$type.nos";
  unlink($filename);
  open(OUT, ">$filename") || die "cannot create file $filename";

  print OUT $message;

  print OUT "// Nosica standard library ----------------------------------------------------\n";
  print OUT "\n";
  print OUT "package net.nosica.lang;\n";
  print OUT "\n";
  print OUT "// The native $type data type : $minidoc.\n";
  print OUT "\n";
  print OUT "public primitive native class $type implements $interface {\n";
  print OUT "\n";
  print OUT "  // --- Object standard methods\n";
  print OUT "\n";
  print OUT "  public constructor();\n";
  print OUT "\n";
  print OUT "  public const $typeR  infix~  (const $type rhs);\n";
  print OUT "  public const bool    infix~~ (const $type rhs) const;\n";
  print OUT "  public const bool    infix!~ (const $type rhs) const;\n";
  print OUT "\n";
  print OUT "  // --- Implement Comparable\n";
  print OUT "\n";
  print OUT "  public const bool    infix< (const $type rhs) const;\n";
  print OUT "  public const bool    infix<=(const $type rhs) const;\n";
  print OUT "  public const bool    infix> (const $type rhs) const;\n";
  print OUT "  public const bool    infix>=(const $type rhs) const;\n";
  print OUT "  public const int     compare(const $type rhs) const;\n";
  print OUT "  public const bool    inRange(const $type lower, const $type upper) const;\n";
  print OUT "  public const $typeR min(const $type rhs) const;\n";
  print OUT "  public const $typeR max(const $type rhs) const;\n";
  print OUT "  public const $typeR min() const;\n";
  print OUT "  public const $typeR max() const;\n";
  print OUT "\n";
  print OUT "  // --- Implements RingElement\n";
  print OUT "\n";
  print OUT "  public const $typeR infix+(const $type rhs) const;\n";
  print OUT "  public const $typeR infix-(const $type rhs) const;\n";
  print OUT "  public const $mtypeR infix*(const $type rhs) const;\n";
  print OUT "  public const $typeR infix/(const $type rhs) const;\n";
  print OUT "  public const $typeR infix%(const $type rhs) const;\n";
  print OUT "  public const $typeR infix^(const $type exponent) const;\n";
  print OUT "\n";
  if ($signed)
  {
    print OUT "  public const $typeR prefix-() const;\n";
    print OUT "\n";
  }
  print OUT "  public const $typeR infix+=(const $type rhs);\n";
  print OUT "  public const $typeR infix-=(const $type rhs);\n";
  print OUT "  public const $typeR infix*=(const $type rhs);\n";
  print OUT "  public const $typeR infix/=(const $type rhs);\n";
  print OUT "  public const $typeR infix%=(const $type rhs);\n";
  print OUT "  public const $typeR infix^=(const $type exponent);\n";
  print OUT "\n";
  print OUT "  public const bool   divisible(const $type rhs) const;\n";
  print OUT "  public const $typeR one()  const;\n";
  print OUT "  public const $typeR zero() const;\n";
  print OUT "\n";
  print OUT "  // --- Implements Numeric\n";
  print OUT "\n";
  print OUT "  public const $typeR  prefix++();\n";
  print OUT "  public const $typeR  prefix--();\n";
  print OUT "  public const int     sign()      const;\n";
  print OUT "  public const bool    isSigned()  const;\n";
  print OUT "  public const bool    isExact()   const;\n";
  print OUT "  public const bool    isInteger() const;\n";
  print OUT "  public const word    radix()     const;\n";
  print OUT "  public const word    digits()    const;\n";
  print OUT "  public const word    digits10()  const;\n";

  if ($floatingpoint)
  {
    print OUT "\n";
    print OUT "  // --- Implements FloatingPoint\n";
    print OUT "\n";
    print OUT "  public const $typeR  infinity() const;\n";
    print OUT "  public const $typeR  epsilon()  const;\n";
    print OUT "  public const $typeR  quietNaN() const;\n";
    print OUT "  public const $typeR  NaN()      const;\n";
    print OUT "  public const word    maxExponent()   const;\n";
    print OUT "  public const word    maxExponent10() const;\n";
    print OUT "  public const int     minExponent()   const;\n";
    print OUT "  public const int     minExponent10() const;\n";
    print OUT "  public const word32  truncToWord32() const;\n";
    print OUT "  public const word64  truncToWord64() const;\n";
  }

  print OUT "\n";
  print OUT "  // --- Static version of some information\n";
  print OUT "\n";
  print OUT "  public static const $typeR  min(const $type first, const $type second);\n";
  print OUT "  public static const $typeR  max(const $type first, const $type second);\n";
  print OUT "  public static const $typeR  min();\n";
  print OUT "  public static const $typeR  max();\n";
  print OUT "  public static const $typeR  one();\n";
  print OUT "  public static const $typeR  zero();\n";
  print OUT "  public static const bool    isSigned();\n";
  print OUT "  public static const bool    isExact();\n";
  print OUT "  public static const bool    isInteger();\n";
  print OUT "  public static const word    radix();\n";
  print OUT "  public static const word    digits();\n";
  print OUT "  public static const word    digits10();\n";

  if ($floatingpoint)
  {
    print OUT "  public static $typeR  infinity();\n";
    print OUT "  public static $typeR  epsilon();\n";
    print OUT "  public static $typeR  quietNaN();\n";
    print OUT "  public static $typeR  NaN();\n";
    print OUT "  public static int     maxExponent();\n";
    print OUT "  public static int     maxExponent10();\n";
    print OUT "  public static int     minExponent();\n";
    print OUT "  public static int     minExponent10();\n";
  }

  print OUT "\n";
  print OUT "  // --- Automatic conversions\n";
  print OUT "\n";
  if ($casts[0]) { print OUT "  public cast int8();\n"; }
  if ($casts[1]) { print OUT "  public cast int16();\n"; }
  if ($casts[2]) { print OUT "  public cast int32();\n"; }
  if ($casts[3]) { print OUT "  public cast int64();\n"; }
  if ($casts[4]) { print OUT "  public cast word8();\n"; }
  if ($casts[5]) { print OUT "  public cast word16();\n"; }
  if ($casts[6]) { print OUT "  public cast word32();\n"; }
  if ($casts[7]) { print OUT "  public cast word64();\n"; }
  if ($casts[8]) { print OUT "  public cast float32();\n"; }
  if ($casts[9]) { print OUT "  public cast float64();\n"; }
  print OUT "\n";
  print OUT "  // --- Checked conversions\n";
  print OUT "\n";
  if ($narrows[0]) { print OUT "  public const int8    narrowToInt8()    const;\n"; }
  if ($narrows[1]) { print OUT "  public const int16   narrowToInt16()   const;\n"; }
  if ($narrows[2]) { print OUT "  public const int32   narrowToInt32()   const;\n"; }
  if ($narrows[3]) { print OUT "  public const int64   narrowToInt64()   const;\n"; }
  if ($narrows[4]) { print OUT "  public const word8   narrowToWord8()   const;\n"; }
  if ($narrows[5]) { print OUT "  public const word16  narrowToWord16()  const;\n"; }
  if ($narrows[6]) { print OUT "  public const word32  narrowToWord32()  const;\n"; }
  if ($narrows[7]) { print OUT "  public const word64  narrowToWord64()  const;\n"; }
  if ($narrows[8]) { print OUT "  public const float32 narrowToFloat32() const;\n"; }
  if ($narrows[9]) { print OUT "  public const float64 narrowToFloat64() const;\n"; }
  print OUT "\n";
  print OUT "}\n";

  close OUT;


  # CREATE THE C NATIVE SOURCE FILE

  my $T  = "net_nosica_lang_$type";
  my $TN = $type;
  my $TD = "_NATIVE_NET_NOSICA_LANG_$type\_H_";
  $TD =~ tr/a-z/A-Z/;

  $filename="$c_int_basepath/$type.h";
  unlink($filename);
  open(OUT, ">$filename") || die "cannot create file $filename";

  print OUT $message;

  print OUT "// Nosica standard library ----------------------------------------------------\n";
  print OUT "// Native C implementation\n";
  print OUT "\n";
  print OUT "#ifndef $TD\n";
  print OUT "#define $TD\n";
  print OUT "\n";

  print OUT "//##protect##type\n";
  print OUT "#define $T $TN\n";
  print OUT "//##protect##type\n";
  print OUT "\n";
  print OUT "static inline void $T\_constructor($T *this);      \n";
  print OUT "static inline void $T\_destructor($T *this);       \n";
  print OUT "static inline void $T\_initializer_static();       \n";
  print OUT "static inline void $T\_deinitializer_static();     \n";
  print OUT "\n";
  print OUT "static inline $T $T\_infix_iassign_const_$T($T *this, $T rhs);               \n";
  print OUT "\n";
  print OUT "static inline net_nosica_lang_bool  $T\_infix_ieq_const_$T\_const($T *this, $T rhs); \n";
  print OUT "static inline net_nosica_lang_bool  $T\_infix_ine_const_$T\_const($T *this, $T rhs); \n";
  print OUT "\n";
  print OUT "static inline net_nosica_lang_bool  $T\_infix_lt_const_$T\_const($T *this, $T rhs);  \n";
  print OUT "static inline net_nosica_lang_bool  $T\_infix_le_const_$T\_const($T *this, $T rhs);  \n";
  print OUT "static inline net_nosica_lang_bool  $T\_infix_gt_const_$T\_const($T *this, $T rhs);  \n";
  print OUT "static inline net_nosica_lang_bool  $T\_infix_ge_const_$T\_const($T *this, $T rhs);  \n";
  print OUT "static inline net_nosica_lang_int32 $T\_method_compare_const_$T\_const($T *this, $T rhs);  \n";
  print OUT "static inline net_nosica_lang_int32 $T\_method_inRange_const_$T\_const_$T\_const($T *this, $T lower, $T upper); \n";
  print OUT "\n";
  print OUT "static inline $T $T\_infix_add_const_$T\_const($T *this, $T rhs); \n";
  print OUT "static inline $T $T\_infix_sub_const_$T\_const($T *this, $T rhs); \n";
  print OUT "static inline $T $T\_infix_mul_const_$T\_const($T *this, $T rhs); \n";
  print OUT "static inline $T $T\_infix_div_const_$T\_const($T *this, $T rhs); \n";
  print OUT "static inline $T $T\_infix_mod_const_$T\_const($T *this, $T rhs); \n";
  print OUT "static inline $T $T\_infix_pow_const_$T\_const($T *this, $T rhs); \n";
  print OUT "\n";
  if ($signed)
  {
    print OUT "static inline $T $T\_prefix_sub_const($T *this); \n";
    print OUT "\n";
  }
  print OUT "static inline $T $T\_infix_addassign_const_$T($T *this, $T rhs); \n";
  print OUT "static inline $T $T\_infix_subassign_const_$T($T *this, $T rhs); \n";
  print OUT "static inline $T $T\_infix_mulassign_const_$T($T *this, $T rhs); \n";
  print OUT "static inline $T $T\_infix_divassign_const_$T($T *this, $T rhs); \n";
  print OUT "static inline $T $T\_infix_modassign_const_$T($T *this, $T rhs); \n";
  print OUT "static inline $T $T\_infix_powassign_const_$T($T *this, $T rhs); \n";
  print OUT "\n";
  print OUT "static inline net_nosica_lang_bool $T\_method_divisible_const_$T\_const($T *this, $T rhs); \n";
  print OUT "\n";
  print OUT "static inline $T $T\_prefix_inc($T *this); \n";
  print OUT "static inline $T $T\_prefix_dec($T *this); \n";
  print OUT "\n";
  print OUT "static inline $T $T\_method_min_const_$T\_const($T *this, $T rhs); \n";
  print OUT "static inline $T $T\_method_min_const($T *this); \n";
  print OUT "static inline $T $T\_method_min_const_$T\_const_$T\_static($T lhs, $T rhs); \n";
  print OUT "static inline $T $T\_method_min_static(); \n";
  print OUT "\n";
  print OUT "static inline $T $T\_method_max_const_$T\_const($T *this, $T rhs); \n";
  print OUT "static inline $T $T\_method_max_const($T *this); \n";
  print OUT "static inline $T $T\_method_max_const_$T\_const_$T\_static($T lhs, $T rhs); \n";
  print OUT "static inline $T $T\_method_max_static(); \n";
  print OUT "\n";
  print OUT "static inline $T $T\_method_one_const($T *this); \n";
  print OUT "static inline $T $T\_method_one_static(); \n";
  print OUT "static inline $T $T\_method_zero_const($T *this); \n";
  print OUT "static inline $T $T\_method_zero_static(); \n";
  print OUT "\n";
  print OUT "static inline net_nosica_lang_int32  $T\_method_sign_const($T *this); \n";
  print OUT "static inline net_nosica_lang_bool   $T\_method_isSigned_const($T *this); \n";
  print OUT "static inline net_nosica_lang_bool   $T\_method_isExact_const($T *this); \n";
  print OUT "static inline net_nosica_lang_bool   $T\_method_isInteger_const($T *this); \n";
  print OUT "static inline net_nosica_lang_word32 $T\_method_radix_const($T *this); \n";
  print OUT "static inline net_nosica_lang_word32 $T\_method_digits_const($T *this); \n";
  print OUT "static inline net_nosica_lang_word32 $T\_method_digits10_const($T *this); \n";

  if ($floatingpoint)
  {
    print OUT "\n";
    print OUT "static inline $T $T\_method_infinity_const($T *this); \n";
    print OUT "static inline $T $T\_method_epsilon_const($T *this); \n";
    print OUT "static inline $T $T\_method_quietNaN_const($T *this); \n";
    print OUT "static inline $T $T\_method_NaN_const($T *this); \n";
    print OUT "static inline net_nosica_lang_word32 $T\_method_maxExponent_const($T *this); \n";
    print OUT "static inline net_nosica_lang_word32 $T\_method_maxExponent10_const($T *this); \n";
    print OUT "static inline net_nosica_lang_int32  $T\_method_minExponent_const($T *this); \n";
    print OUT "static inline net_nosica_lang_int32  $T\_method_minExponent10_const($T *this); \n";
    print OUT "static inline net_nosica_lang_word32 $T\_method_truncToWord32_const($T *this); \n";
    print OUT "static inline net_nosica_lang_word64 $T\_method_truncToWord64_const($T *this); \n";
  }

  print OUT "\n";
  print OUT "static inline net_nosica_lang_bool   $T\_method_isSigned_static(); \n";
  print OUT "static inline net_nosica_lang_bool   $T\_method_isExact_static(); \n";
  print OUT "static inline net_nosica_lang_bool   $T\_method_isInteger_static(); \n";
  print OUT "static inline net_nosica_lang_word32 $T\_method_radix_static(); \n";
  print OUT "static inline net_nosica_lang_word32 $T\_method_digits_static(); \n";
  print OUT "static inline net_nosica_lang_word32 $T\_method_digits10_static(); \n";

  if ($floatingpoint)
  {
    print OUT "static inline $T $T\_method_infinity_static(); \n";
    print OUT "static inline $T $T\_method_epsilon_static(); \n";
    print OUT "static inline $T $T\_method_quietNaN_static(); \n";
    print OUT "static inline $T $T\_method_NaN_static(); \n";
    print OUT "static inline net_nosica_lang_word32 $T\_method_maxExponent_static(); \n";
    print OUT "static inline net_nosica_lang_word32 $T\_method_maxExponent10_static(); \n";
    print OUT "static inline net_nosica_lang_int32  $T\_method_minExponent_static(); \n";
    print OUT "static inline net_nosica_lang_int32  $T\_method_minExponent10_static(); \n";
  }

  print OUT "\n";
  print OUT "  // --- Automatic conversions\n";
  print OUT "\n";
  if ($casts[0]) { print OUT "static inline net_nosica_lang_int8      $T\_cast_net_nosica_lang_int8_const($T *this); \n"; }
  if ($casts[1]) { print OUT "static inline net_nosica_lang_int16     $T\_cast_net_nosica_lang_int16_const($T *this); \n"; }
  if ($casts[2]) { print OUT "static inline net_nosica_lang_int32     $T\_cast_net_nosica_lang_int32_const($T *this); \n"; }
  if ($casts[3]) { print OUT "static inline net_nosica_lang_int64     $T\_cast_net_nosica_lang_int64_const($T *this); \n"; }
  if ($casts[4]) { print OUT "static inline net_nosica_lang_word8     $T\_cast_net_nosica_lang_word8_const($T *this); \n"; }
  if ($casts[5]) { print OUT "static inline net_nosica_lang_word16    $T\_cast_net_nosica_lang_word16_const($T *this); \n"; }
  if ($casts[6]) { print OUT "static inline net_nosica_lang_word32    $T\_cast_net_nosica_lang_word32_const($T *this); \n"; }
  if ($casts[7]) { print OUT "static inline net_nosica_lang_word64    $T\_cast_net_nosica_lang_word64_const($T *this); \n"; }
  if ($casts[8]) { print OUT "static inline net_nosica_lang_float32   $T\_cast_net_nosica_lang_float32_const($T *this); \n"; }
  if ($casts[9]) { print OUT "static inline net_nosica_lang_float64   $T\_cast_net_nosica_lang_float64_const($T *this); \n"; }

  print OUT "\n";
  print OUT "  // --- Checked conversions\n";
  print OUT "\n";

  if ($narrows[0]) { print OUT "static inline net_nosica_lang_int8    $T\_method_narrowToInt8_const($T *this); \n"; }
  if ($narrows[1]) { print OUT "static inline net_nosica_lang_int16   $T\_method_narrowToInt16_const($T *this); \n"; }
  if ($narrows[2]) { print OUT "static inline net_nosica_lang_int32   $T\_method_narrowToInt32_const($T *this); \n"; }
  if ($narrows[3]) { print OUT "static inline net_nosica_lang_int64   $T\_method_narrowToInt64_const($T *this); \n"; }
  if ($narrows[4]) { print OUT "static inline net_nosica_lang_word8   $T\_method_narrowToWord8_const($T *this); \n"; }
  if ($narrows[5]) { print OUT "static inline net_nosica_lang_word16  $T\_method_narrowToWord16_const($T *this); \n"; }
  if ($narrows[6]) { print OUT "static inline net_nosica_lang_word32  $T\_method_narrowToWord32_const($T *this); \n"; }
  if ($narrows[7]) { print OUT "static inline net_nosica_lang_word64  $T\_method_narrowToWord64_const($T *this); \n"; }
  if ($narrows[8]) { print OUT "static inline net_nosica_lang_float32 $T\_method_narrowToFloat32_const($T *this); \n"; }
  if ($narrows[9]) { print OUT "static inline net_nosica_lang_float64 $T\_method_narrowToFloat64_const($T *this); \n"; }

  print OUT "\n";
  print OUT "#include \"net/nosica/lang/$type.inl\"\n";

  print OUT "\n";
  print OUT "#endif // $TD\n";

  close OUT;

  $filename="$c_impl_basepath/$type.inl";
  unlink($filename);
  open(OUT, ">$filename") || die "cannot create file $filename";

  print OUT $message;

  print OUT "// Nosica standard library ----------------------------------------------------\n";
  print OUT "// Native C implementation\n";
  print OUT "\n";
  print OUT "static inline void $T\_constructor($T *this)\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_constructor\n";
  print OUT "//##protect##$T\_constructor\n";
  print OUT "}\n";
  print OUT "static inline void $T\_destructor($T *this)\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_destructor\n";
  print OUT "//##protect##$T\_destructor\n";
  print OUT "}\n";
  print OUT "static inline void $T\_initializer_static()\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_initializer_static\n";
  print OUT "//##protect##$T\_initializer_static\n";
  print OUT "}\n";
  print OUT "static inline void $T\_deinitializer_static()\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_deinitializer_static\n";
  print OUT "//##protect##$T\_deinitializer_static\n";
  print OUT "}\n";
  print OUT "\n";
  print OUT "static inline $T $T\_infix_iassign_const_$T($T *this, $T rhs)\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_infix_iassign_const_$T\n";
  print OUT "return *this = rhs;\n";
  print OUT "//##protect##$T\_infix_iassign_const_$T\n";
  print OUT "}\n";
  print OUT "\n";
  print OUT "static inline net_nosica_lang_bool  $T\_infix_ieq_const_$T\_const($T *this, $T rhs)\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_infix_ieq_const_$T\_const\n";
  print OUT "return *this == rhs;\n";
  print OUT "//##protect##$T\_infix_ieq_const_$T\_const\n";
  print OUT "}\n";
  print OUT "static inline net_nosica_lang_bool  $T\_infix_ine_const_$T\_const($T *this, $T rhs)\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_infix_ine_const_$T\_const\n";
  print OUT "return *this != rhs;\n";
  print OUT "//##protect##$T\_infix_ine_const_$T\_const\n";
  print OUT "}\n";
  print OUT "\n";
  print OUT "static inline net_nosica_lang_bool  $T\_infix_lt_const_$T\_const($T *this, $T rhs)\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_infix_lt_const_$T\_const\n";
  print OUT "return *this < rhs;\n";
  print OUT "//##protect##$T\_infix_lt_const_$T\_const\n";
  print OUT "}\n";
  print OUT "static inline net_nosica_lang_bool  $T\_infix_le_const_$T\_const($T *this, $T rhs)\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_infix_le_const_$T\_const\n";
  print OUT "return *this <= rhs;\n";
  print OUT "//##protect##$T\_infix_le_const_$T\_const\n";
  print OUT "}\n";
  print OUT "static inline net_nosica_lang_bool  $T\_infix_gt_const_$T\_const($T *this, $T rhs)\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_infix_gt_const_$T\_const\n";
  print OUT "return *this > rhs;\n";
  print OUT "//##protect##$T\_infix_gt_const_$T\_const\n";
  print OUT "}\n";
  print OUT "static inline net_nosica_lang_bool  $T\_infix_ge_const_$T\_const($T *this, $T rhs)\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_infix_ge_const_$T\_const\n";
  print OUT "return *this >= rhs;\n";
  print OUT "//##protect##$T\_infix_ge_const_$T\_const\n";
  print OUT "}\n";
  print OUT "static inline net_nosica_lang_int32 $T\_method_compare_const_$T\_const($T *this, $T rhs)\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_method_compare_const_$T\_const\n";
  print OUT "if (*this < rhs) return -1;\n";
  print OUT "if (*this > rhs) return 1;\n";
  print OUT "return 0;\n";
  print OUT "//##protect##$T\_method_compare_const_$T\_const\n";
  print OUT "}\n";
  print OUT "static inline net_nosica_lang_int32 $T\_method_inRange_const_$T\_const_$T\_const($T *this, $T lower, $T upper)\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_method_inRange_const_$T\_const_$T\_const\n";
  print OUT "return (lower < *this) && (*this < upper);\n";
  print OUT "//##protect##$T\_method_inRange_const_$T\_const_$T\_const\n";
  print OUT "}\n";
  print OUT "\n";
  print OUT "static inline $T $T\_infix_add_const_$T\_const($T *this, $T rhs)\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_infix_add_const_$T\_const\n";
  print OUT "return *this + rhs;\n";
  print OUT "//##protect##$T\_infix_add_const_$T\_const\n";
  print OUT "}\n";
  print OUT "static inline $T $T\_infix_sub_const_$T\_const($T *this, $T rhs)\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_infix_sub_const_$T\_const\n";
  print OUT "return *this - rhs;\n";
  print OUT "//##protect##$T\_infix_sub_const_$T\_const\n";
  print OUT "}\n";
  print OUT "static inline $T $T\_infix_mul_const_$T\_const($T *this, $T rhs)\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_infix_mul_const_$T\_const\n";
  print OUT "return *this * rhs;\n";
  print OUT "//##protect##$T\_infix_mul_const_$T\_const\n";
  print OUT "}\n";
  print OUT "static inline $T $T\_infix_div_const_$T\_const($T *this, $T rhs)\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_infix_div_const_$T\_const\n";
  print OUT "return *this / rhs;\n";
  print OUT "//##protect##$T\_infix_div_const_$T\_const\n";
  print OUT "}\n";
  print OUT "static inline $T $T\_infix_mod_const_$T\_const($T *this, $T rhs)\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_infix_mod_const_$T\_const\n";
  if ($floatingpoint)
  {
    print OUT "//\@TODO@ : % is forbidden for float : rework RingElement interface\n";
    print OUT "return *this;\n";
  }
  else
  {
    print OUT "return *this % rhs;\n";
  }
  print OUT "//##protect##$T\_infix_mod_const_$T\_const\n";
  print OUT "}\n";
  print OUT "static inline $T $T\_infix_pow_const_$T\_const($T *this, $T exponent)\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_infix_pow_const_$T\_const\n";
  if ($floatingpoint)
  {
    print OUT "return pow(*this, exponent);\n";
  }
  else
  {
    print OUT "return *this ^ exponent;\n";
  }
  print OUT "//##protect##$T\_infix_pow_const_$T\_const\n";
  print OUT "}\n";
  print OUT "\n";
  if ($signed)
  {
    print OUT "static inline $T $T\_prefix_sub_const($T *this)\n";
    print OUT "{\n";
    print OUT "//##protect##$T\_prefix_sub_const\n";
    print OUT "return -*this;\n";
    print OUT "//##protect##$T\_prefix_sub_const\n";
    print OUT "}\n";
    print OUT "\n";
  }
  print OUT "static inline $T $T\_infix_addassign_const_$T($T *this, $T rhs)\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_infix_addassign_const_$T\n";
  print OUT "return *this += rhs;\n";
  print OUT "//##protect##$T\_infix_addassign_const_$T\n";
  print OUT "}\n";
  print OUT "static inline $T $T\_infix_subassign_const_$T($T *this, $T rhs)\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_infix_subassign_const_$T\n";
  print OUT "return *this -= rhs;\n";
  print OUT "//##protect##$T\_infix_subassign_const_$T\n";
  print OUT "}\n";
  print OUT "static inline $T $T\_infix_mulassign_const_$T($T *this, $T rhs)\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_infix_mulassign_const_$T\n";
  print OUT "return *this *= rhs;\n";
  print OUT "//##protect##$T\_infix_mulassign_const_$T\n";
  print OUT "}\n";
  print OUT "static inline $T $T\_infix_divassign_const_$T($T *this, $T rhs)\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_infix_divassign_const_$T\n";
  print OUT "return *this /= rhs;\n";
  print OUT "//##protect##$T\_infix_divassign_const_$T\n";
  print OUT "}\n";
  print OUT "static inline $T $T\_infix_modassign_const_$T($T *this, $T rhs)\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_infix_modassign_const_$T\n";
  if ($floatingpoint)
  {
    print OUT "//\@TODO@ : % is forbidden for float : rework RingElement interface\n";
    print OUT "return *this;\n";
  }
  else
  {
    print OUT "return *this %= rhs;\n";
  }
  print OUT "//##protect##$T\_infix_modassign_const_$T\n";
  print OUT "}\n";
  print OUT "static inline $T $T\_infix_powassign_const_$T($T *this, $T exponent)\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_infix_powassign_const_$T\n";
  if ($floatingpoint)
  {
    print OUT "return *this = pow(*this, exponent);\n";
  }
  else
  {
    print OUT "return *this ^= exponent;\n";
  }
  print OUT "//##protect##$T\_infix_powassign_const_$T\n";
  print OUT "}\n";
  print OUT "\n";
  print OUT "static inline net_nosica_lang_bool $T\_method_divisible_const_$T\_const($T *this, $T rhs)\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_method_divisible_const_$T\_const\n";
  if ($floatingpoint)
  {
    print OUT "//\@TODO@ : % is forbidden for float : rework RingElement interface\n";
    print OUT "return 0;\n";
  }
  else
  {
    print OUT "return (*this % rhs) == 0;\n";
  }
  print OUT "//##protect##$T\_method_divisible_const_$T\_const\n";
  print OUT "}\n";
  print OUT "\n";
  print OUT "static inline $T $T\_prefix_inc($T *this)\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_prefix_inc\n";
  print OUT "return ++*this;\n";
  print OUT "//##protect##$T\_prefix_inc\n";
  print OUT "}\n";
  print OUT "static inline $T $T\_prefix_dec($T *this)\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_prefix_dec\n";
  print OUT "return --*this;\n";
  print OUT "//##protect##$T\_prefix_dec\n";
  print OUT "}\n";
  print OUT "\n";
  print OUT "static inline $T $T\_method_min_const_$T\_const($T *this, $T rhs)\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_method_min_const_$T\_const\n";
  print OUT "return $T\_method_min_const_$T\_const_$T\_static(*this, rhs);\n";
  print OUT "//##protect##$T\_method_min_const_$T\_const\n";
  print OUT "}\n";
  print OUT "static inline $T $T\_method_min_const($T *this)\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_method_min_const\n";
  print OUT "return $TN\_MIN;\n";
  print OUT "//##protect##$T\_method_min_const\n";
  print OUT "}\n";
  print OUT "static inline $T $T\_method_min_const_$T\_const_$T\_static($T first, $T second)\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_method_min_const_$T\_const_$T\_static\n";
  print OUT "return first < second ? first : second;\n";
  print OUT "//##protect##$T\_method_min_const_$T\_const_$T\_static\n";
  print OUT "}\n";
  print OUT "static inline $T $T\_method_min_static()\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_method_min_static\n";
  print OUT "return $TN\_MIN;\n";
  print OUT "//##protect##$T\_method_min_static\n";
  print OUT "}\n";
  print OUT "\n";
  print OUT "static inline $T $T\_method_max_const_$T\_const($T *this, $T rhs)\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_method_max_const_$T\_const\n";
  print OUT "return $T\_method_max_const_$T\_const_$T\_static(*this, rhs);\n";
  print OUT "//##protect##$T\_method_max_const_$T\_const\n";
  print OUT "}\n";
  print OUT "static inline $T $T\_method_max_const($T *this)\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_method_max_const\n";
  print OUT "return $TN\_MAX;\n";
  print OUT "//##protect##$T\_method_max_const\n";
  print OUT "}\n";
  print OUT "static inline $T $T\_method_max_const_$T\_const_$T\_static($T first, $T second)\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_method_max_const_$T\_const_$T\_static\n";
  print OUT "return first > second ? first : second;\n";
  print OUT "//##protect##$T\_method_max_const_$T\_const_$T\_static\n";
  print OUT "}\n";
  print OUT "static inline $T $T\_method_max_static()\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_method_max_static\n";
  print OUT "return $TN\_MAX;\n";
  print OUT "//##protect##$T\_method_max_static\n";
  print OUT "}\n";
  print OUT "\n";
  print OUT "static inline $T $T\_method_one_const($T *this)\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_method_one_const\n";
  print OUT "return 1;\n";
  print OUT "//##protect##$T\_method_one_const\n";
  print OUT "}\n";
  print OUT "static inline $T $T\_method_one_static()\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_method_one_static\n";
  print OUT "return 1;\n";
  print OUT "//##protect##$T\_method_one_static\n";
  print OUT "}\n";
  print OUT "static inline $T $T\_method_zero_const($T *this)\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_method_zero_const\n";
  print OUT "return 0;\n";
  print OUT "//##protect##$T\_method_zero_const\n";
  print OUT "}\n";
  print OUT "static inline $T $T\_method_zero_static()\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_method_zero_static\n";
  print OUT "return 0;\n";
  print OUT "//##protect##$T\_method_zero_static\n";
  print OUT "}\n";
  print OUT "\n";
  print OUT "static inline net_nosica_lang_int32  $T\_method_sign_const($T *this)\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_method_sign_const\n";
  print OUT "return $T\_method_compare_const_$T\_const(this, 0);\n";
  print OUT "//##protect##$T\_method_sign_const\n";
  print OUT "}\n";
  print OUT "static inline net_nosica_lang_bool   $T\_method_isSigned_const($T *this)\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_method_isSigned_const\n";
  print OUT "return $TN\_IS_SIGNED;\n";
  print OUT "//##protect##$T\_method_isSigned_const\n";
  print OUT "}\n";
  print OUT "static inline net_nosica_lang_bool   $T\_method_isExact_const($T *this)\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_method_isExact_const\n";
  print OUT "return $TN\_IS_EXACT;\n";
  print OUT "//##protect##$T\_method_isExact_const\n";
  print OUT "}\n";
  print OUT "static inline net_nosica_lang_bool   $T\_method_isInteger_const($T *this)\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_method_isInteger_const\n";
  print OUT "return $TN\_IS_INTEGER;\n";
  print OUT "//##protect##$T\_method_isInteger_const\n";
  print OUT "}\n";
  print OUT "static inline net_nosica_lang_word32 $T\_method_radix_const($T *this)\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_method_radix_const\n";
  print OUT "return $TN\_RADIX;\n";
  print OUT "//##protect##$T\_method_radix_const\n";
  print OUT "}\n";
  print OUT "static inline net_nosica_lang_word32 $T\_method_digits_const($T *this)\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_method_digits_const\n";
  print OUT "return $TN\_DIGITS;\n";
  print OUT "//##protect##$T\_method_digits_const\n";
  print OUT "}\n";
  print OUT "static inline net_nosica_lang_word32 $T\_method_digits10_const($T *this)\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_method_digits10_const\n";
  print OUT "return $TN\_DIGITS10;\n";
  print OUT "//##protect##$T\_method_digits10_const\n";
  print OUT "}\n";

  if ($floatingpoint)
  {
    print OUT "\n";
    print OUT "static inline $T $T\_method_infinity_const($T *this)\n";
    print OUT "{\n";
    print OUT "//##protect##$T\_method_infinity_const\n";
    print OUT "return $TN\_INFINITY;\n";
    print OUT "//##protect##$T\_method_infinity_const\n";
    print OUT "}\n";
    print OUT "static inline $T $T\_method_epsilon_const($T *this)\n";
    print OUT "{\n";
    print OUT "//##protect##$T\_method_epsilon_const\n";
    print OUT "return $TN\_EPSILON;\n";
    print OUT "//##protect##$T\_method_epsilon_const\n";
    print OUT "}\n";
    print OUT "static inline $T $T\_method_quietNaN_const($T *this)\n";
    print OUT "{\n";
    print OUT "//##protect##$T\_method_quietNaN_const\n";
    print OUT "return $TN\_QUIETNAN;\n";
    print OUT "//##protect##$T\_method_quietNaN_const\n";
    print OUT "}\n";
    print OUT "static inline $T $T\_method_NaN_const($T *this)\n";
    print OUT "{\n";
    print OUT "//##protect##$T\_method_NaN_const\n";
    print OUT "return $TN\_NAN;\n";
    print OUT "//##protect##$T\_method_NaN_const\n";
    print OUT "}\n";
    print OUT "static inline net_nosica_lang_word32 $T\_method_maxExponent_const($T *this)\n";
    print OUT "{\n";
    print OUT "//##protect##$T\_method_maxExponent_const\n";
    print OUT "return $TN\_MAXEXPONENT;\n";
    print OUT "//##protect##$T\_method_maxExponent_const\n";
    print OUT "}\n";
    print OUT "static inline net_nosica_lang_word32 $T\_method_maxExponent10_const($T *this)\n";
    print OUT "{\n";
    print OUT "//##protect##$T\_method_maxExponent10_const\n";
    print OUT "return $TN\_MAXEXPONENT10;\n";
    print OUT "//##protect##$T\_method_maxExponent10_const\n";
    print OUT "}\n";
    print OUT "static inline net_nosica_lang_int32  $T\_method_minExponent_const($T *this)\n";
    print OUT "{\n";
    print OUT "//##protect##$T\_method_minExponent_const\n";
    print OUT "return $TN\_MINEXPONENT;\n";
    print OUT "//##protect##$T\_method_minExponent_const\n";
    print OUT "}\n";
    print OUT "static inline net_nosica_lang_int32  $T\_method_minExponent10_const($T *this)\n";
    print OUT "{\n";
    print OUT "//##protect##$T\_method_minExponent10_const\n";
    print OUT "return $TN\_MINEXPONENT10;\n";
    print OUT "//##protect##$T\_method_minExponent10_const\n";
    print OUT "}\n";
    print OUT "static inline net_nosica_lang_word32 $T\_method_truncToWord32_const($T *this)\n";
    print OUT "{\n";
    print OUT "//##protect##$T\_method_truncToWord32_const\n";
    print OUT "return (word32)*this;\n";
    print OUT "//##protect##$T\_method_truncToWord32_const\n";
    print OUT "}\n";
    print OUT "static inline net_nosica_lang_word64 $T\_method_truncToWord64_const($T *this)\n";
    print OUT "{\n";
    print OUT "//##protect##$T\_method_truncToWord64_const\n";
    print OUT "return (word64)*this;\n";
    print OUT "//##protect##$T\_method_truncToWord64_const\n";
    print OUT "}\n";
  }

  print OUT "\n";
  print OUT "static inline net_nosica_lang_bool   $T\_method_isSigned_static()\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_method_isSigned_static\n";
  print OUT "return $TN\_IS_SIGNED;\n";
  print OUT "//##protect##$T\_method_isSigned_static\n";
  print OUT "}\n";
  print OUT "static inline net_nosica_lang_bool   $T\_method_isExact_static()\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_method_isExact_static\n";
  print OUT "return $TN\_IS_EXACT;\n";
  print OUT "//##protect##$T\_method_isExact_static\n";
  print OUT "}\n";
  print OUT "static inline net_nosica_lang_bool   $T\_method_isInteger_static()\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_method_isInteger_static\n";
  print OUT "return $TN\_IS_INTEGER;\n";
  print OUT "//##protect##$T\_method_isInteger_static\n";
  print OUT "}\n";
  print OUT "static inline net_nosica_lang_word32 $T\_method_radix_static()\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_method_radix_static\n";
  print OUT "return $TN\_RADIX;\n";
  print OUT "//##protect##$T\_method_radix_static\n";
  print OUT "}\n";
  print OUT "static inline net_nosica_lang_word32 $T\_method_digits_static()\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_method_digits_static\n";
  print OUT "return $TN\_DIGITS;\n";
  print OUT "//##protect##$T\_method_digits_static\n";
  print OUT "}\n";
  print OUT "static inline net_nosica_lang_word32 $T\_method_digits10_static()\n";
  print OUT "{\n";
  print OUT "//##protect##$T\_method_digits10_static\n";
  print OUT "return $TN\_DIGITS10;\n";
  print OUT "//##protect##$T\_method_digits10_static\n";
  print OUT "}\n";

  if ($floatingpoint)
  {
    print OUT "static inline $T $T\_method_infinity_static()\n";
    print OUT "{\n";
    print OUT "//##protect##$T\_method_infinity_static\n";
    print OUT "return $TN\_INFINITY;\n";
    print OUT "//##protect##$T\_method_infinity_static\n";
    print OUT "}\n";
    print OUT "static inline $T $T\_method_epsilon_static()\n";
    print OUT "{\n";
    print OUT "//##protect##$T\_method_epsilon_static\n";
    print OUT "return $TN\_EPSILON;\n";
    print OUT "//##protect##$T\_method_epsilon_static\n";
    print OUT "}\n";
    print OUT "static inline $T $T\_method_quietNaN_static()\n";
    print OUT "{\n";
    print OUT "//##protect##$T\_method_quietNaN_static\n";
    print OUT "return $TN\_QUIETNAN;\n";
    print OUT "//##protect##$T\_method_quietNaN_static\n";
    print OUT "}\n";
    print OUT "static inline $T $T\_method_NaN_static()\n";
    print OUT "{\n";
    print OUT "//##protect##$T\_method_NaN_static\n";
    print OUT "return $TN\_NAN;\n";
    print OUT "//##protect##$T\_method_NaN_static\n";
    print OUT "}\n";
    print OUT "static inline net_nosica_lang_word32 $T\_method_maxExponent_static()\n";
    print OUT "{\n";
    print OUT "//##protect##$T\_method_maxExponent_static\n";
    print OUT "return $TN\_MAXEXPONENT;\n";
    print OUT "//##protect##$T\_method_maxExponent_static\n";
    print OUT "}\n";
    print OUT "static inline net_nosica_lang_word32 $T\_method_maxExponent10_static()\n";
    print OUT "{\n";
    print OUT "//##protect##$T\_method_maxExponent10_static\n";
    print OUT "return $TN\_MAXEXPONENT10;\n";
    print OUT "//##protect##$T\_method_maxExponent10_static\n";
    print OUT "}\n";
    print OUT "static inline net_nosica_lang_int32  $T\_method_minExponent_static()\n";
    print OUT "{\n";
    print OUT "//##protect##$T\_method_minExponent_static\n";
    print OUT "return $TN\_MINEXPONENT;\n";
    print OUT "//##protect##$T\_method_minExponent_static\n";
    print OUT "}\n";
    print OUT "static inline net_nosica_lang_int32  $T\_method_minExponent10_static()\n";
    print OUT "{\n";
    print OUT "//##protect##$T\_method_minExponent10_static\n";
    print OUT "return $TN\_MINEXPONENT10;\n";
    print OUT "//##protect##$T\_method_minExponent10_static\n";
    print OUT "}\n";
  }

  print OUT "\n";
  print OUT "  // --- Automatic conversions\n";
  print OUT "\n";
  if ($casts[0])
  {
    print OUT "static inline net_nosica_lang_int8      $T\_cast_net_nosica_lang_int8_const($T *this)\n";
    print OUT "{\n";
    print OUT "//##protect##$T\_cast_net_nosica_lang_int8_const\n";
    print OUT "return (int8)*this;\n";
    print OUT "//##protect##$T\_cast_net_nosica_lang_int8_const\n";
    print OUT "}\n";
  }
  if ($casts[1])
  {
    print OUT "static inline net_nosica_lang_int16     $T\_cast_net_nosica_lang_int16_const($T *this)\n";
    print OUT "{\n";
    print OUT "//##protect##$T\_cast_net_nosica_lang_int16_const\n";
    print OUT "return (int16)*this;\n";
    print OUT "//##protect##$T\_cast_net_nosica_lang_int16_const\n";
    print OUT "}\n";
  }
  if ($casts[2])
  {
    print OUT "static inline net_nosica_lang_int32     $T\_cast_net_nosica_lang_int32_const($T *this)\n";
    print OUT "{\n";
    print OUT "//##protect##$T\_cast_net_nosica_lang_int32_const\n";
    print OUT "return (int32)*this;\n";
    print OUT "//##protect##$T\_cast_net_nosica_lang_int32_const\n";
    print OUT "}\n";
  }
  if ($casts[3])
  {
    print OUT "static inline net_nosica_lang_int64     $T\_cast_net_nosica_lang_int64_const($T *this)\n";
    print OUT "{\n";
    print OUT "//##protect##$T\_cast_net_nosica_lang_int64_const\n";
    print OUT "return (int64)*this;\n";
    print OUT "//##protect##$T\_cast_net_nosica_lang_int64_const\n";
    print OUT "}\n";
  }
  if ($casts[4])
  {
    print OUT "static inline net_nosica_lang_word8     $T\_cast_net_nosica_lang_word8_const($T *this)\n";
    print OUT "{\n";
    print OUT "//##protect##$T\_cast_net_nosica_lang_word8_const\n";
    print OUT "return (word8)*this;\n";
    print OUT "//##protect##$T\_cast_net_nosica_lang_word8_const\n";
    print OUT "}\n";
  }
  if ($casts[5])
  {
    print OUT "static inline net_nosica_lang_word16    $T\_cast_net_nosica_lang_word16_const($T *this)\n";
    print OUT "{\n";
    print OUT "//##protect##$T\_cast_net_nosica_lang_word16_const\n";
    print OUT "return (word16)*this;\n";
    print OUT "//##protect##$T\_cast_net_nosica_lang_word16_const\n";
    print OUT "}\n";
  }
  if ($casts[6])
  {
    print OUT "static inline net_nosica_lang_word32    $T\_cast_net_nosica_lang_word32_const($T *this)\n";
    print OUT "{\n";
    print OUT "//##protect##$T\_cast_net_nosica_lang_word32_const\n";
    print OUT "return (word32)*this;\n";
    print OUT "//##protect##$T\_cast_net_nosica_lang_word32_const\n";
    print OUT "}\n";
  }
  if ($casts[7])
  {
    print OUT "static inline net_nosica_lang_word64    $T\_cast_net_nosica_lang_word64_const($T *this)\n";
    print OUT "{\n";
    print OUT "//##protect##$T\_cast_net_nosica_lang_word64_const\n";
    print OUT "return (word64)*this;\n";
    print OUT "//##protect##$T\_cast_net_nosica_lang_word64_const\n";
    print OUT "}\n";
  }
  if ($casts[8])
  {
    print OUT "static inline net_nosica_lang_float32   $T\_cast_net_nosica_lang_float32_const($T *this)\n";
    print OUT "{\n";
    print OUT "//##protect##$T\_cast_net_nosica_lang_float32_const\n";
    print OUT "return (float32)*this;\n";
    print OUT "//##protect##$T\_cast_net_nosica_lang_float32_const\n";
    print OUT "}\n";
  }
  if ($casts[9])
  {
    print OUT "static inline net_nosica_lang_float64   $T\_cast_net_nosica_lang_float64_const($T *this)\n";
    print OUT "{\n";
    print OUT "//##protect##$T\_cast_net_nosica_lang_float64_const\n";
    print OUT "return (float64)*this;\n";
    print OUT "//##protect##$T\_cast_net_nosica_lang_float64_const\n";
    print OUT "}\n";
  }

  print OUT "\n";
  print OUT "  // --- Checked conversions\n";
  print OUT "\n";

  if ($narrows[0])
  {
    print OUT "static inline net_nosica_lang_int8    $T\_method_narrowToInt8_const($T *this)\n";
    print OUT "{\n";
    print OUT "//##protect##$T\_method_narrowToInt8_const\n";
    print OUT "return narrow_$TN\_to_int8(*this);\n";
    print OUT "//##protect##$T\_method_narrowToInt8_const\n";
    print OUT "}\n";
  }
  if ($narrows[1])
  {
    print OUT "static inline net_nosica_lang_int16   $T\_method_narrowToInt16_const($T *this)\n";
    print OUT "{\n";
    print OUT "//##protect##$T\_method_narrowToInt16_const\n";
    print OUT "return narrow_$TN\_to_int16(*this);\n";
    print OUT "//##protect##$T\_method_narrowToInt16_const\n";
    print OUT "}\n";
  }
  if ($narrows[2])
  {
    print OUT "static inline net_nosica_lang_int32   $T\_method_narrowToInt32_const($T *this)\n";
    print OUT "{\n";
    print OUT "//##protect##$T\_method_narrowToInt32_const\n";
    print OUT "return narrow_$TN\_to_int32(*this);\n";
    print OUT "//##protect##$T\_method_narrowToInt32_const\n";
    print OUT "}\n";
  }
  if ($narrows[3])
  {
    print OUT "static inline net_nosica_lang_int64   $T\_method_narrowToInt64_const($T *this)\n";
    print OUT "{\n";
    print OUT "//##protect##$T\_method_narrowToInt64_const\n";
    print OUT "return narrow_$TN\_to_int64(*this);\n";
    print OUT "//##protect##$T\_method_narrowToInt64_const\n";
    print OUT "}\n";
  }
  if ($narrows[4])
  {
    print OUT "static inline net_nosica_lang_word8   $T\_method_narrowToWord8_const($T *this)\n";
    print OUT "{\n";
    print OUT "//##protect##$T\_method_narrowToWord8_const\n";
    print OUT "return narrow_$TN\_to_word8(*this);\n";
    print OUT "//##protect##$T\_method_narrowToWord8_const\n";
    print OUT "}\n";
  }
  if ($narrows[5])
  {
    print OUT "static inline net_nosica_lang_word16  $T\_method_narrowToWord16_const($T *this)\n";
    print OUT "{\n";
    print OUT "//##protect##$T\_method_narrowToWord16_const\n";
    print OUT "return narrow_$TN\_to_word16(*this);\n";
    print OUT "//##protect##$T\_method_narrowToWord16_const\n";
    print OUT "}\n";
  }
  if ($narrows[6])
  {
    print OUT "static inline net_nosica_lang_word32  $T\_method_narrowToWord32_const($T *this)\n";
    print OUT "{\n";
    print OUT "//##protect##$T\_method_narrowToWord32_const\n";
    print OUT "return narrow_$TN\_to_word32(*this);\n";
    print OUT "//##protect##$T\_method_narrowToWord32_const\n";
    print OUT "}\n";
  }
  if ($narrows[7])
  {
    print OUT "static inline net_nosica_lang_word64  $T\_method_narrowToWord64_const($T *this)\n";
    print OUT "{\n";
    print OUT "//##protect##$T\_method_narrowToWord64_const\n";
    print OUT "return narrow_$TN\_to_word64(*this);\n";
    print OUT "//##protect##$T\_method_narrowToWord64_const\n";
    print OUT "}\n";
  }
  if ($narrows[8])
  {
    print OUT "static inline net_nosica_lang_float32 $T\_method_narrowToFloat32_const($T *this)\n";
    print OUT "{\n";
    print OUT "//##protect##$T\_method_narrowToFloat32_const\n";
    print OUT "return narrow_$TN\_to_float32(*this);\n";
    print OUT "//##protect##$T\_method_narrowToFloat32_const\n";
    print OUT "}\n";
  }
  if ($narrows[9])
  {
    print OUT "static inline net_nosica_lang_float64 $T\_method_narrowToFloat64_const($T *this)\n";
    print OUT "{\n";
    print OUT "//##protect##$T\_method_narrowToFloat64_const\n";
    print OUT "return narrow_$TN\_to_float64(*this);\n";
    print OUT "//##protect##$T\_method_narrowToFloat64_const\n";
    print OUT "}\n";
  }

  print OUT "\n";

  close OUT;
}
